#include <iostream>
#include <iomanip>
#include <sstream>
#include <vector>
#include <analysis/vmedaq.h>
/**
 * Tests the reading of ADC data
 * 
 */

class MyADCData: public ADCData
{
public:
    MyADCData() {}
    ~MyADCData() {}
    void onNewData(const Module *, int evtcnt, const ChannelList &);
};

void MyADCData::onNewData(const Module *m, int evtcnt, const ChannelList &buffers)
{
    /*
     * The ADC data comes as a collection of events read from the ADC multievent
     * buffer, and each event has the usual list of channels
     */
    std::cout << "======= Event " << evtcnt << " ==================" << std::endl;
    std::cout << "Number of buffers: " << buffers.size() << std::endl;
    
    ChannelList::const_iterator ibuf;
    for (ibuf=buffers.begin(); ibuf!=buffers.end();++ibuf)
    {
        DataBuffer *B = dynamic_cast<DataBuffer *>(*ibuf);
        std::cout << "\tBuffer: gate counter " << B->gate_counter() << std::endl;
        
        ChannelList::const_iterator ip;
        for (ip=B->list().begin(); ip!=B->list().end();++ip)
        {
            std::cout << "\t\tChannel " << (*ip)->channel() << " val " << (*ip)->signal() << std::endl;
        }
    }
}

int main(int argc, char **argv)
{
    VMEdaq vmedaq;
    MyADCData obs;

    int i;
    std::vector<std::string> files;
    for (i = 1; i < argc; i++)
        files.push_back(argv[i]);

    if (files.empty())
    {
        std::cout << "I need 1 or more input files"<< std::endl;
        return 1;
    }

    vmedaq.open(files);
    vmedaq.add( &obs );

    vmedaq.loop();

    vmedaq.close();
    std::cout << "\n\nBye !!!"<< std::endl;
    return 0;
}
