#include <iostream>
#include <iomanip>
#include <sstream>
#include <vector>
#include <analysis/vmedaq.h>
/**
 * Tests the reading of TDC data + VATAGP
 * 
 */

class MyVATAData : public VATAGP
{
public:
    MyVATAData() : VATAGP(false) {}
    ~MyVATAData() {}
    
    void onNewData(const Module *, int evtcnt, const ChannelList &);
};
void MyVATAData::onNewData(const Module *m, int evtcnt, const ChannelList &data)
{
    std::cout << "======= VATA Event "<< evtcnt << " =================="<< std::endl;
    std::cout << "Number of channels: "<< data.size() << std::endl;
    ChannelList::const_iterator ip;
    for (ip = data.begin(); ip != data.end(); ++ip)
    {
        VATAChannel *ch = dynamic_cast<VATAChannel *>(*ip);
        if (ch->adj())
            continue;
        
        std::cout << "Channel " << ch->channel() << ": " << ch->signal() << std::endl;
    }
}

class MyTDCData: public TDC488Data
{
public:
    MyTDCData() {}
    ~MyTDCData() {}
    void onNewData(const Module *, int evtcnt, const ChannelList &);
};

void MyTDCData::onNewData(const Module *m, int evtcnt, const ChannelList &buffers)
{
    /*
     * The TDC data comes as a collection of events read from the TDC multievent
     * buffer, and each event has the usual list of channels
     */
    std::cout << "======= TDC Event " << evtcnt << " ==================" << std::endl;
    std::cout << "Number of buffers: " << buffers.size() << std::endl;
    
    ChannelList::const_iterator ibuf;
    for (ibuf=buffers.begin(); ibuf!=buffers.end();++ibuf)
    {
        DataBuffer *B = dynamic_cast<DataBuffer *>(*ibuf);
        std::cout << "\tBuffer: gate counter " << B->gate_counter() << std::endl;
        
        ChannelList::const_iterator ip;
        for (ip=B->list().begin(); ip!=B->list().end();++ip)
        {
            std::cout << "\t\tChannel " << (*ip)->channel() << " val " << (*ip)->signal() << std::endl;
        }
    }
}

int main(int argc, char **argv)
{
    VMEdaq vmedaq;
    MyTDCData obs;
    MyVATAData vata;

    int i;
    std::vector<std::string> files;
    for (i = 1; i < argc; i++)
        files.push_back(argv[i]);

    if (files.empty())
    {
        std::cout << "I need 1 or more input files"<< std::endl;
        return 1;
    }

    vmedaq.open(files);
    vmedaq.add( TDCCaenData, &obs );
    vmedaq.add( NormalData, &vata);
    
    vmedaq.loop();

    vmedaq.close();
    std::cout << "\n\nBye !!!"<< std::endl;
    return 0;
}
