#!/usr/bin/env python
"""
    Reads Caen TDC 488 data from vmedaq
"""
import vmedaq

class TDC(vmedaq.SimpleVMEdaqObserver):
    """ The VMEdaqObserver implementation for the TDC
    """
    def __init__(self,dtype):
        """ Initializes the instance
        """
        super(TDC, self).__init__(dtype)

    def data_valid(self, data):
        """
        Very simple tests on the validity of the data. It just
        checks that the internal data structure is OK
        """
        dsz = data[0] & 0xfff
        sz  = data[2] & 0xfff

        # Check data integrity
        if  dsz != data[dsz-1] or  dsz-sz != 4  :
            return False
        else:
            return True
        

    def DataRecord(self, data):
        """ Decodes the data
        """
        self.cnt += 1
        idd = (data[2] & 0xfff00000) >> 20
        dsz = data[0] & 0xfff
        sz  = data[2] & 0xfff

        if not self.data_valid(data):
            print 'Invalid data'
            return

        self.ndata += 1
        try:
            module = self.idlist[idd]
        except KeyError:
            print '...no existe',self.idlist
            return

        bf = []
        evcnt = 0
        for i in range(0, sz):
            dt = (data[3+i] & 0xffff)
            if dt & 0x8000 == 0:
                val  = dt & 0xfff
                chan = (dt & 0x7000)>>12
                bf.append( (evcnt, (chan, val)) )
            else:
                evcnt = dt & 0xfff

        # self.idlist does not work properly. Give just the first one
        self.onNewData(self.idlist.values()[0], data[1], bf)
