#!/usr/bin/env python

import struct

class HstData(object):
    def __init__(self,data):
        self.buffer = data
        fmt = "iiiiiiidd"
        self.offset = struct.calcsize(fmt);
        (self.size,
         self.bfsiz,
         self.nchip,
         self.nchan,
         self.nbins,
         self.nx,
         self.ny,
         self.rmax,
         self.rmin) = struct.unpack(fmt,data[:self.offset])
        self.ntot = self.nchan*self.nchip
        self.bfsiz = struct.calcsize("%dd" % self.ntot)
        self.sz2d = 0
        if self.nx > 0:
            self.sz2d = struct.calcsize("%dd" % (self.nx*self.ny))
            if self.sz2d<self.bfsiz:
                self.sz2d = self.bfsiz
        else:
            self.sz2d = self.bfsiz
            
        #print self.size,self.nchip,self.nchan,self.rmin,self.rmax,self.nbins

        self.__ped    = self.offset
        self.__noise  = self.__ped + self.bfsiz
        self.__hitmap = self.__noise + self.bfsiz
        self.__h2dmap = self.__hitmap + self.bfsiz
        self.__h2dmapc= self.__h2dmap + self.sz2d
        self.__out    = self.__h2dmapc + self.sz2d
        self.__chan   = self.__out + self.bfsiz
        self.__hsig   = self.__chan + self.bfsiz
        self.__hsig2  = self.__hsig + struct.calcsize("%dd" % (self.nchip+1)*self.nbins)

    def ped(self):
        s1  = self.__ped
        s2  = s1 + self.bfsiz
        return struct.unpack("%dd" % self.ntot,self.buffer[s1:s2])

    def noise(self):
        s1  = self.__noise
        s2  = s1 + self.bfsiz
        return struct.unpack("%dd" % self.ntot,self.buffer[s1:s2])
        
    def hsig(self):
        fmt = "%dd" % (self.nchip+1)*self.nbins
        s1  = self.__hsig
        s2  = s1 + struct.calcsize(fmt)
        return struct.unpack(fmt,self.buffer[s1:s2])

    def out(self):
        s1  = self.__out
        s2  = s1 + self.bfsiz
        return struct.unpack("%dd" % self.ntot,self.buffer[s1:s2])

    def hitmap(self):
        fmt = "%dd" % self.ntot
        s1 = self.__hitmap
        s2 = s1 + self.bfsiz
        return struct.unpack(fmt,self.buffer[s1:s2])

    def hitmap2d(self):
        fmt = "%dd" % (self.nx * self.ny)
        s1 = self.__h2dmap
        s2 = s1 + self.sz2d
        return struct.unpack(fmt,self.buffer[s1:s2])

#----------------------------------------------------------------------------------
