#!/usr/bin/env python

import ORBit
import CORBA
import string
import sys
import zlib
import struct
import HstData
import re
import StringIO

from references import find_daqpp_references
# Load the DAQcorba type library for the Observer.
ORBit.load_typelib('DAQcorba')
from DAQcorba import DAQ

def time_str(tt):
    seconds = tt % 60
    minutes = (tt/60) % 60
    hours   = (tt/3600) 

    return '%2d:%02d:%02d' % (hours,minutes,seconds)

def status_str(st):
    rate = st.rate
    rate_units = 'Hz'
    if rate>1.e6:
        rate = rate/1.e6
        rate_units = 'MHz'
    elif rate>1000.:
        rate = rate/1000.
        rate_units = 'kHz'

    s = 'Evts %7d time %s rate %8.1f %s eff %5.1f - %7.1f kb/s' % (
        st.ntrig,time_str(st.time),rate,rate_units,st.eff,st.throughput/1024./st.time)

    return s

_trim = re.compile('\s+')
class ScanVar(object):
    channel, pulse, threshold, trim = range(0,4)
    var_names=("Channel", "Pulse", "Threshold", "Trim")

    def __init__(self,type=0,first=0.,last=1.,step=1.,default=0.):
        self.type    = type
        self.first   = first
        self.last    = last
        self.step    = step
        self.default = default
        self.value   = first

    def load(self,s):
        v = _trim.sub(' ',s.strip()).split(' ')
        if len(v)!=6:
            return None

        try:
            self.type = int(v[0])
            self.first = float(v[1])
            self.last  = float(v[2])
            self.step  = float(v[3])
            self.default = float(v[4])
            self.value = float(v[5])
        except:
            try:
                ei = sys.exc_info()
                print "ScanVar.load:\n%s\n%s\n" %(ei[0],ei[1])
            finally:
                del ei
            return None

        return self

class ScanPoint(dict):
    """ A ScanPoint is a set of ScanVars that conform a scan """

    def __init__(self,s):
        self.load(s)

    def print_current(self):
        for type,scp in self.items():
            print ScanVar.var_names[type],scp.value,

        print ''

    def load(self,str):
        f = StringIO.StringIO(str)
        s=f.readline().strip()
        v=_trim.sub(' ',s).split(' ')
        if len(v)==0:
            return None

        try:
            nvar = int(v[0])
            self.clear()
            for i in range(0,nvar):
                s = f.readline()
                var = ScanVar()
                var.load(s)
                self[var.type]=var

        except:
            try:
                ei = sys.exc_info()
                print "ScanPoint.load:\n%s\n%s" %(ei[0],ei[1])
            finally:
                del ei
            return None
        
        return self
        

def main(file_ref=None):
    # Init the orb
    orb = CORBA.ORB_init(sys.argv)

    # Find the references
    daqm,observable = find_daqpp_references(orb,file_ref)
    if not daqm :
        sys.exit(1)

    runm  = daqm.findRunManager('main') 
    state = runm.getState()
    print ('\r%s' % status_str(state)),
    print ''
    try:
        any = runm.getParam('scanpoint')
    
        ss = any.value()
        sc = ScanPoint( ss[3:-4] )
        sc.print_current()
        print ''
    except DAQ.DAQObject.ParameterNotExist:
        pass
    
    Mid = daqm. listModuleIDs()
    module = daqm.findModule(Mid[0])
    state = module.getState()
    print ('%s' % status_str(state))

    data = module.getMonitorData()
    sz   = len(data)-struct.calcsize("LL");
    olen,clen,s = struct.unpack("LL %ds" % sz,data)
    print len(data),'-> orig. ',olen,'compressed ',clen,'s',len(s)

    ooo = zlib.decompress(s)
    print len(ooo)

    hd = HstData.HstData(ooo)

    

if __name__=='__main__':
    ref=None
    try:
        ref = sys.argv[1]
    except:
        pass
    
    main()

#-------------------------------------------------------------------------
