#!/usr/bin/env python
"""
Exports a few utils to determine the IORs generated by vmedaq.

"""
import ORBit
import CORBA
import urllib2
import StringIO


def find_daqpp_references(orb, file_ref=None):
    """
    This will try and contact to the NameService to find the
    references of the DAQm nager and the Observable. If it fails with
    the NameService, it will assume that the IORs of the references
    are in a file called vmedaq.ior in the cwd
    """
    # load the interface definition for the name server, and import the module
    ORBit.load_file('/usr/share/idl/orbit-2.0/CosNaming.idl')
    import CosNaming

    if file_ref == None:
        file_ref = 'http://ific.uv.es/~lacasta/vmedaq.ior'
    
    daqm = None
    observable = None
    try:
        #
        # Try with the name service
        #
        obj = orb.resolve_initial_references("NameService")
        nameserver = obj._narrow(CosNaming.NamingContext)
        
        # create the name of the server
        name = [CosNaming.NameComponent('DAQ', ''),
                CosNaming.NameComponent('DAQmanager', '')
                ]
        # resolve the name in order to get the server
        daqm = nameserver.resolve(name)

        # create the name of the server
        name = [CosNaming.NameComponent('DAQ', ''),
                CosNaming.NameComponent('Observer', '')
                ]
        # resolve the name in order to get the server
        observable = nameserver.resolve(name)

    except:
        #
        # Failed... Try with the IORs
        #
        try:
            fior = StringIO.StringIO(
                urllib2.urlopen(file_ref).read()
                )
        except:
            try:
                print 'file:', file_ref
                fior =  StringIO.StringIO( open(file_ref).read() )
            except:
                print 'Cannot access the ior'
                return None, None
            
        ior = None
        objects = {}
        for line in fior:
            ss = line[:-1].split('\t')
            if len(ss)>0:
                objects[ss[0]] = ss[1]

        if 'daqm' in objects:
            ior = objects['daqm']
            daqm = orb.string_to_object(ior)

        if 'observable' in objects:
            ior = objects['observable']
            observable = orb.string_to_object(ior)

    return daqm, observable
#-------------------------------------------------------------------------
