#!/usr/bin/env python

import sys
import time
import ORBit
import CORBA

from references import find_daqpp_references
# Load the DAQcorba type library for the Observer.
ORBit.load_typelib('DAQcorba')
from DAQcorba import DAQ


def create_any(val):
    if isinstance(val, bool):
        return CORBA.Any(CORBA.TypeCode('IDL:omg.org/CORBA/boolean:1.0'),val)
    elif isinstance(val,int) or isinstance(val,long):
        return CORBA.Any(CORBA.TypeCode('IDL:omg.org/CORBA/long:1.0'),val)
    elif isinstance(val,float):
        return CORBA.Any(CORBA.TypeCode('IDL:omg.org/CORBA/float:1.0'),val)
    elif isinstance(val,str):
        return CORBA.Any(CORBA.TypeCode('IDL:omg.org/CORBA/string:1.0'),val)
    else:
        return CORBA.Any(CORBA.TypeCode('IDL:omg.org/CORBA/string:1.0'),str(val))

def main(file_ref=None):
    # Init the orb
    orb = CORBA.ORB_init(sys.argv)

    # Find the references
    daqm,observable = find_daqpp_references(orb,file_ref)
    if not daqm or daqm._is_nil() :
        print 'Could not connect'
        sys.exit(1)

    mconf = daqm.findModule('VMEDAQconfig')
    if mconf._is_nil() :
        print 'Cannot find config module'
        sys.exit(1)

    # Now we start changing parameters
    param_list = mconf.getParamNames()
    for par in param_list:
        rc = mconf.getParam(par)
        val = rc.value()
        print par,'->',val

##     mconf.setParam('IO/run_number',create_any(2),True)
##     mconf.setParam('IO/compression', create_any(True), True)
##     mconf.setParam('IO/prefix', create_any('runXX'), True)
##     mconf.setParam('DAQ/trigger_type', create_any('external'), True)
##     mconf.setParam('DAQ/log_data', create_any(False), True)
##     mconf.setParam('DAQ/maxevents',create_any(2000),True)
##     # mconf.setParam('DAQ/log_data', create_any(True), True)
##     mconf.setParam('DAQ/pedestal_run', create_any(5000), True)

##     while True:
##         time.sleep(0.2)
##         rm = daqm.findRunManager('main')
##         state = rm.getState()
##         if state.status != DAQ.Running:
##             break

    mconf.setParam('DAQ/maxevents',create_any(2000),True)
    mconf.setParam('DAQ/Start', create_any(0), True)

    print 'Done'
    
if __name__ == '__main__':
    ref=None
    try:
        ref = sys.argv[1]
    except:
        pass
    
    main(ref)

#---------------------------------------------------------------------
