#!/usr/bin/env python
import sys,os
import gtk
import gobject
import gtkhst
import struct
import zlib
import string
import array
from gtktools import SpinSlider,RunMenu
from GladeApp import GladeApp

import ORBit
import CORBA
import MyObserver
import HstData
import time
from threading import Thread

from references import find_daqpp_references
# Load the DAQcorba type library for the Observer.
ORBit.load_typelib('DAQcorba')
from DAQcorba import DAQ

glade_dir = ""


class MyRunMenu(RunMenu):
    def __init__(self, runm):
        super(MyRunMenu,self).__init__()
        self.subscribe(runm)
        self.cmd_queue=[]
        gobject.idle_add(self.check_commands)

    def check_commands(self):
        if len(self.cmd_queue):
            cmd = self.cmd_queue.pop(0)
            self.do_command(cmd)

    def subscribe(self,runm):
        self.runm = runm
        if runm:
            runm.addObserver( self )

    def command(self,w,command):
        self.cmd_queue.append(command)

    def do_command(self,command):
        if not self.runm:
            return

        if command==self.notready or command==self.reset:
            if self.runm.isActive():
                self.runm.deActivate()

        elif command==self.getready:
            self.runm.GetReady()

        elif command==self.run:
            if not self.runm.isActive():
                self.runm.Activate()

            print "RunManager",self.runm.get_id(),": active [",self.runm.isActive(),"]"
            daqpp.startRunManager( self.runm )

        elif command==self.stop:
            daqpp.stopRunManager( self.runm )
            print self.runm.GetStatus()

        elif command==self.pause:
            self.runm.Pause()

        elif command==self.resume:
            self.runm.Continue()

        else:
            print 'MyRunMenu.command: Unhandled command',command


    def update(self,arg):
        self.set_state(arg)




def time_str(tt):
    seconds = tt % 60
    minutes = (tt/60) % 60
    hours   = (tt/3600) 

    return '%2d:%02d:%02d' % (hours,minutes,seconds)
        
def get_monitor_data(module):
    try:
        data = module.getMonitorData()
        sz   = len(data)-struct.calcsize("LL");
        olen,clen,s = struct.unpack("LL %ds" % sz,data)
        return HstData.HstData( zlib.decompress(s))
    except Exception,e:
        if e._id=="IDL:omg.org/CORBA/COMM_FAILURE:1.0":
            gtk.main_quit()

class MainWindow(GladeApp):
    def __init__(self,obs, glade_path="vmedaq_corba.glade",root="MainWindow", domain=None):
        self.obs = obs
        self.obs.connect("state_changed",self.new_state,None)
        Mid = self.obs.daqm. listModuleIDs()
        self.module = self.obs.daqm.findModule(Mid[0])
        self.hd = get_monitor_data(self.module)
        self.signal_array=array.array('d',[ x for x in self.hd.hsig()])
        if (self.hd.nx>0):
            self.hitmap_array=array.array('d',[ x for x in self.hd.hitmap2d()])
        else:
            self.hitmap_array=array.array('d',[ x for x in self.hd.hitmap()])
            
        glade_path = os.path.join(glade_dir, glade_path)
        GladeApp.__init__(self, glade_path, root, domain)

    def new(self):
        #context MainWindow.new {
        state = self.obs.runm.getState()
        self.set_state(state)
        self.lblStatus.set_text( MyObserver.RunStates[ state.status ] )
        gobject.timeout_add(20000,self.update,self)
        gobject.timeout_add(10000,self.update_state,self)

        self.on_Refresh(None)
        #context MainWindow.new }



    #context MainWindow custom methods {
    #--- Write your own methods here ---#
    def new_state(self,obj,id,data):
        self.lblStatus.set_text(id)
        self.lblStatus.queue_draw()
    
    def update_state(self,data):
        try:
            state = self.obs.runm.getState()
            self.set_state(state)
        except Exception,e:
            if e._id=="IDL:omg.org/CORBA/COMM_FAILURE:1.0":
                gtk.main_quit()
            
        return True

    def set_state(self,state):
        rate = state.rate
        rate_units = 'Hz'
        if rate>1.e6:
            rate = rate/1.e6
            rate_units = 'MHz'
        elif rate>1000.:
            rate = rate/1000.
            rate_units = 'kHz'

        self.lblDuration.set_text( time_str(state.time) )
        self.lblNtrig.set_text( '%7d' % state.ntrig )
        self.lblRate.set_text( '%8.1f %s' %(rate,rate_units) )
        self.tableState.queue_draw()

    def update(self,data):
        if not self.module:
            return True

        try:
            self.hd = get_monitor_data(self.module)
        except Exception,e:
            if e._id=="IDL:omg.org/CORBA/COMM_FAILURE:1.0":
                gtk.main_quit()
            
        hst = self.hst_signal.get_data('hst')
        i=0
        for x in self.hd.hsig()[0:self.hd.nbins]:
            self.signal_array[i] = x
            i += 1
            
        self.hst_signal.queue_draw()

        if self.hd.nx>0: v = self.hd.hitmap2d()
        else:            v = self.hd.hitmap()
        i=0
        for x in v:
            self.hitmap_array[i]=x
            i += 1

        self.hst_hitmap.queue_draw()
        
        return True
    #context MainWindow custom methods }

    def create_hst_signal(self, name, str1, str2, int1, int2):
        #context MainWindow.create_hst_signal {
        hst = gtkhst.Histogram(self.hd.nbins,self.hd.rmin,self.hd.rmax,self.signal_array)
        widget = gtkhst.Hst1D( hst )
        widget.set_data('hst',hst)
        widget.set_xtitle("ADC counts");
        widget.show_all()
        return widget
        #context MainWindow.create_hst_signal }

    def create_hst_hitmap(self, name, str1, str2, int1, int2):
        #context MainWindow.create_hst_hitmap {
        if self.hd.nx>0:
            hst = gtkhst.Histogram2D(self.hd.nx,0.,self.hd.nx,
                                     self.hd.ny,0.,self.hd.ny,
                                     self.hitmap_array
                                     )
            widget =  gtkhst.Hst2D( hst )
        else:
            hst = gtkhst.Histogram(self.hd.ntot,0.,self.hd.ntot,self.hitmap_array)
            widget = gtkhst.Hst1D( hst )

        widget.set_data('hst',hst)
        widget.show_all()
        return widget
        #context MainWindow.create_hst_hitmap }

    def on_Refresh(self, widget, *args):
        #context MainWindow.on_Refresh {
        self.update_state(None)
        self.update(None)
        #context MainWindow.on_Refresh }

def main():
    # Init the orb
    orb = CORBA.ORB_init(sys.argv)

    # Find the references
    file = None
    for x in sys.argv[1:]:
        if x[0]!='-':
            file = x
            break

##     try:
##         file=sys.argv[1]
##     except IndexError:
##         file=None
##         #file = 'file://%s/vmedaq.ior' % os.getcwd()
        
    
    daqm,observable = find_daqpp_references(orb,file)
    if not daqm or not observable:
        print 'Could not find the objects'
        print daqm,observable
        sys.exit(1)

    obs = MyObserver.MyObserver(daqm)
    obsref = obs._this()
    callback=None
    callback = observable.registerCallback( obsref )
    if not callback:
        print 'Could not get a callback'
        sys.exit(1)

    main_window = MainWindow(obs)

    try:
        main_window.run()
    except:
        print 'Tenemos una excepcion'

    try:
        observable.unregisterCallback(callback)
    except Exception,e:
        if hasattr(e,'_id'):
            print e._id

if __name__ == "__main__":
    main()
