#!/usr/bin/env python

import sys
import time
import getopt

import ROOT
import vmedaq

saves = {}
def save_root_object(O):
    saves[O.GetName()]=O
        
def print_time(t):
   if t>86400.:
       t /= 86400.
       return t,"d"
   
   elif  t>3600.:
       t/= 3600.;
       return t,"h";
   
   elif  t>60.:
       t/=60.;
       return t,"m"
   
   elif t>1:
       return t,"s";
   
   elif t>1.e-3:
       t/=1.e-3
       return t,"ms"
   
   elif t>1.e-6:
       t/=1.e-6
       return t,"us";

   else:
       t/=1.e-9
       return t,"ns"
    


class Spectrum(vmedaq.ADC):
    def __init__(self,idtype):
        super(Spectrum,self).__init__(idtype)
        self.t0 = 0
        self.hst2 = ROOT.TH2D("hBa2","Ba 2D",
                              512,0.,1024.,
                              32,-0.5,31.5)
        self.hst = ROOT.TH1D("hSpec","Spectrum",512,0.,1024.);

    def save_hst(self,fname):
        ofile = ROOT.TFile(fname,"RECREATE")
        self.hst2.Write()
        self.hst.Write()
        ofile.Write()
        ofile.Close()
    
    def __str__(self):
        return "Spectrum"

    def onMap(self,module):
        pass

    def onNewData(self,module,evtcnt,data):

        for cntr, chanlist in data:
            
            for chan,val in chanlist:
                if chan<0 or chan>=module.ntot:
                    continue

            self.hst2.Fill(val,chan)
        
        if (self.ndata % 1000)==0:
            print ('\r%8d %s/evt' % (self.ndata, '%.2f %s' % print_time(
                (time.time()-self.t0)/self.ndata) )),
            sys.stdout.flush()

    def onNewFile(self,file_name):
        if self.ndata==0:
            self.t0 = time.time()
        else:
            print 'Average time per event: %.2f %s' % print_time(
                (time.time()-self.t0)/self.ndata)
            
        str = (len(file_name)+4)*'+'
        print '\n%s\n+ %s\n%s\n' % (str,file_name,str)


def main(fflist=None):
    fname="Spectrum.root"
    try:
        opts,args=getopt.getopt(sys.argv[1:],"o:",["out=",])
    except getopt.GetoptError,msg:
        print msg
        sys.exit(1)

    for opt,arg in opts:
        if opt in ('-o','--out'):
            fname = arg
        
    file_list = [ ff for ff in args ]
    if fflist:
        for ff in fflist:
            file_list.append(ff)

    print file_list
        
    V = vmedaq.VMEdaq()
    S = Spectrum(0x40000)
    V.register( S )
    V.open(file_list)
    
    try:
        V.loop()
    except vmedaq.VMEdaqError,e:
        print "Algo ha salido mal:",e
    except KeyboardInterrupt:
        print 'Arrrrgggg !!!!!' 


    print 'End of Loop'
    V.close()

    print 'Saving Histograms in',fname
    S.save_hst(fname)
    print '...done'

if __name__=='__main__':
    try:
        main()
    except KeyboardInterrupt:
        print 'Arrrrrggggg!!!!!'
        
    print 'Adios...'
#-------------------------------------------------------------------------
