#!/usr/bin/env python

import sys
import getopt
sys.path.insert(0,'/home/lacasta/test/py-vmedaq/build/lib.linux-i686-2.3')

import ROOT
import vmedaq

saves = {}
def save_root_object(O):
    saves[O.GetName()]=O        

class Spectrum(vmedaq.VATAGP):
    def __init__(self):
        super(Spectrum,self).__init__()
        
        self.cnvs  = ROOT.TCanvas("cSpectrum","Checking sparse",400,700)
        self.hst = ROOT.TH2F("hDiff","Ped diff",
                                                 256,-0.5,255.5,
                                                 128,-64.,64.)
        self.hitmap = ROOT.TH1F("hHit","Hitmap",256,-0.5,255.5)
        self.hsignal = ROOT.TH1F("hSignal", "Signal",256,0.,1024.)

        self.cnvs.Divide(1,3)
        self.cnvs.cd(1)
        self.hst.Draw("colz")
        self.cnvs.cd(2)
        self.hitmap.Draw()
        self.cnvs.cd(3)
        self.hsignal.Draw()
        
        save_root_object(self.cnvs)
        save_root_object(self.hst)
        save_root_object(self.hitmap)
        save_root_object(self.hsignal)
    
    def __str__(self):
        return "Spectrum"

    def onNewData(self,module,evtcnt,data):
        sgchan = {}
        for  chan, val, adj in data:
            if chan<0 or chan>=module.ntot:
                return

            if not adj:
                sgchan.setdefault(chan,[]).append(val)

        xxx = 0.
        for chan, val in sgchan.items():
            if chan==0 or val[0]==0.:
                continue
            
            if len(val)>1:
                v = val[0]-val[1]
                self.hst.Fill(chan,v)

            self.hitmap.Fill(chan)
            if val[0]<1023.:
                self.hsignal.Fill(val[0])
        
        if (self.ndata%10000)==0:
            for i in range(1,4):
                pad = self.cnvs.cd(i)
                pad.Modified(1)
                pad.Update()

        if (self.ndata % 1000)==0:
            print ('\r%8d'%self.ndata),
            sys.stdout.flush()

    def onNewFile(self,file_name):
        str = (len(file_name)+4)*'+'
        print '%s\n+ %s\n%s' % (str,file_name,str)

def main():
    ROOT.gROOT.SetStyle("Plain")
    ROOT.gStyle.SetPalette(1)

    file_list = [ x for x in sys.argv[1:] ]
    S = Spectrum()
    print file_list
        
    V = vmedaq.VMEdaq()
    V.register( S )
    V.open(file_list)    

    
    while 1:
        try:
            bt = V.read()
            if bt==None:
                break
        except vmedaq.VMEdaqError,e:
            print "Algo ha salido mal:",e
            break



    print 'End of Loop'
    ROOT.gApplication.Run(1)
    print "End of file"
    V.close()


if __name__=='__main__':
    try:
        main()
    except KeyboardInterrupt:
        print 'Arrrrrggggg!!!!!'
        
    print 'Adios...'
#-------------------------------------------------------------------------
