#!/usr/bin/env python
import time
import math
from Tracer import Tracer
from random import gauss,random

import ROOT

saves = {}

def save_root_object(O):
    saves[O.GetName()]=O


cntr = 0

def fill_histogram(data,args):
    global cntr
    hst = args[0]
    cnvs= args[1]

    i=0
    for x in data:
        hst.SetBinContent(i,x)
        i += 1

    if (cntr % 10)==0:
        cnvs.Modified(1)
        cnvs.Update()

class Monitor(object):
    def __init__(self,name,title,mean,npts=1000):
        self.cntr = 0
        self.tracer = Tracer(npts)
        self.hst = ROOT.TH1F( 'h%s'%name,title,self.tracer.size,0.,self.tracer.size)
        self.hst.SetMaximum(mean+10.)
        self.hst.SetMinimum(mean-10.)
        save_root_object(self.hst)
        
        self.gr = ROOT.TGraph(self.tracer.size)
        self.gr.SetName( 'gr%s' % name )
        self.gr.SetLineColor(4)
        self.gr.SetLineWidth(2)
        save_root_object(self.gr)
        for i in xrange(0,self.gr.GetN()):
            self.gr.SetPoint(i,i,0.)

    def reset(self):
        self.cntr = 0

    def Draw(self):
        self.hst.Draw()
        self.gr.Draw("LPS")

    def fill_hst(self,data,args):
        pad = args;
        i = 0
        for x in data:
            self.gr.SetPoint(i,i,x)
            i += 1

        if (self.cntr % 5)==0:
            pad.Modified(1)
            pad.Update()

    def fill(self,x):
        self.cntr += 1
        self.tracer.fill(x)

    def register(self,args):
        self.tracer.register(self.fill_hst,args)

class MonitorPedestal(Monitor):
    def __init__(self,graph,name,title,mean,npts=1000):
        super(MonitorPedestal,self).__init__(name,title,mean,npts)
        self.orig = graph
        

    def Draw(self):
        Monitor.Draw(self)
        self.orig.gr.Draw("LPS")        

class MonitorNoise(Monitor):
    def __init__(self,name,title,mean,npts=1000):
        super(MonitorNoise,self).__init__(name,title,mean,npts)
        
def sin_changed(x,freq,ampl):
    return ampl*math.sin( 2.*math.pi*x/freq)

class RealValue(Tracer):
    def __init__(self,name,npts,average=0):
        super(RealValue,self).__init__(npts,average)
        self.gr = ROOT.TGraph(npts)
        self.gr.SetLineWidth(2)
        self.gr.SetLineColor(2)
        for i in xrange(0,self.gr.GetN()):
            self.gr.SetPoint(i,i,0.)
        self.register(self.update_graph)
        saves[name] = self.gr

    def update_graph(self,data,args):
        i = 0
        for x in data:
            self.gr.SetPoint(i,i,x)
            i += 1
        

def main():
    global cntr
    # Real values for pedestal and noise
    mean_0 = 100.
    mean = mean_0
    noise=   5.


    cnvs =  ROOT.TCanvas("cnvs")
    cnvs.Divide(1,2)
    save_root_object(cnvs)

    npts = 500
    theMean = RealValue("mean",npts)
    # prepare the pedestals stuff
    mPed = MonitorPedestal(theMean,'Ped','Pedestal',mean,npts)

    pad = cnvs.cd(1)
    pad.SetGridy(1)
    mPed.Draw()
    mPed.register(pad)
    pad.Update()

    # now the noise
    mNoise = MonitorNoise('Noise','Noise',noise,npts)
    pad = cnvs.cd(2)
    pad.SetGridy(1)
    mNoise.Draw()
    mNoise.register(pad)
    pad.Update()
    

    # Start the loop
    ped  =   0.
    rms  =   0.
    alpha = 0.001


    ped_weight = 1./32.
    noise_weight = 1./256.

    T0 = time.time()
    for i in xrange(0,10000):
        cntr += 1
        if i>150:
            mean = mean_0 + sin_changed(i,350,5.)
        else:
            mean = mean_0

        theMean.fill(mean)
        
        val = gauss(mean,noise)-ped
##         if random()>0.9:
##             val += gauss(200.,10.)


        if ped==0:
            ped = val
            continue

        corr = val*ped_weight;

        ped += corr
        mPed.fill(ped)

        if rms==0.:
            rms=1.

        xs = val/rms
        rms = rms * ( (1.-noise_weight)+xs*xs*noise_weight )
        mNoise.fill(rms)

    print 'End Of Loop'
    ROOT.gApplication.Run(1)    

if __name__ == '__main__':
    ROOT.gROOT.SetStyle("Plain")
    ROOT.gStyle.SetOptStat(0)
    try:
        main()
    except KeyboardInterrupt:
        print 'Aaaaarrrgggg !!!!'

    
    print 'Adios..'

#-------------------------------------------------------------------------------------
