#!/usr/bin/env python

import sys
import time
import getopt
import copy

import vmedaq
import vmedaq.TDC
import ROOT

def print_time(t):
   if t>86400.:
       t /= 86400.
       return "%.2f d" %t
   
   elif  t>3600.:
       t/= 3600.;
       return "%.2f h" % t
   
   elif  t>60.:
       t/=60.;
       return "%.2f m" % t
   
   elif t>1:
       return "%.2f s" % t
   
   elif t>1.e-3:
       t/=1.e-3
       return "%.2f ms" % t
   
   elif t>1.e-6:
       t/=1.e-6
       return "%.2f us" % t

   else:
       t/=1.e-9
       return "%.2f ns" %t

class DataReader(vmedaq.VMEdaq):
    """
    """
    def __init__(self):
        super(DataReader, self).__init__()
        self.t0 = time.time()
        self.cnt = 0

    def new_event(self, evt_num):
        self.cnt += 1
        if (evt_num % 1000)==0:
            print ('\r%8d Average time/event %12s' % (
                evt_num, print_time( (time.time()-self.t0)/self.cnt))),
            sys.stdout.flush()
        

class Event(object):
    """ This represents an event in which one can find many
    modules... It aims at providing a way of wynchronizing
    """
    def __init__(self, ofile):
        self.modules  = 0
        self.evt_list = {}
        self.rootFile = ROOT.TFile(ofile,"RECREATE")
        self.hTDC     = ROOT.TH1D("hTDC","TDC",175,0.,700.)
        self.hVATA    = ROOT.TH1D("hVATA","VATA",128,0.,1024.)
        self.hAll = ROOT.TH2D("hAll","All",256,0.,1024.,175,0.,700.)


    def save_histograms(self):
        print 'Writing file ',self.rootFile.GetName()
        self.rootFile.Write()

    def register(self):
        self.modules += 1

    def add_event(self, mid, evt, data):
        evt_data = self.evt_list.setdefault(evt, {})
        evt_data[mid] = data
        if len(data)==0:
            print '**** vacio !! porque !!!'

        if len(evt_data) == self.modules:
            vata = -1.
            tdc  = -1.
            for a,b in evt_data.items():
                if a==256:
                    vata = b[0][1]
                elif a==512:
                    tdc = b[0][1]

            self.hAll.Fill(vata,tdc)

            del(self.evt_list[evt])
            for key in self.evt_list.keys():
                if key < evt:
                    del(self.evt_list[key])

class Spectrum(vmedaq.VATAGP):
    def __init__(self,dtype,event):
        super(Spectrum,self).__init__(dtype)
        self.t0 = 0
        self.evt = event
        self.evt.register()
        self.data = []
        
    def __str__(self):
        return "Spectrum"

    def onMap(self,module):
        pass
    
    def onNewData(self,module,evtcnt,data):

        if len(data)>self.nadj+1:
            return

        for chan,val,adj in data:
            if chan<=0 or chan>=module.ntot:
                continue

            if not adj:
                ped = module.pedestals[chan]
                v   = val-ped
                self.evt.hVATA.Fill( v )
                self.data.append( (chan, v) )
        
    def EndOfEvent(self,mid,evt):
        if len(self.data):
            self.evt.add_event(mid,evt,self.data)
            self.data = []


class TDCana(vmedaq.TDC.TDC):
    def __init__(self, dtype, event):
        super(TDCana, self).__init__(dtype)

        self.evt      = event
        self.evt.register()
        self.data = []
        self.range = 700.

        
    def EndOfEvent(self,mid,evt):
        if len(self.data):
            self.evt.add_event(mid,evt,self.data)
            self.data = []

    def onNewData(self, module, evcnt, data):
        if len(data)==1:
            for e, item in data:
                chan, val = item
                if chan==0:
                    x = 700.*(1.-val/4096.)
                    self.data.append( (chan, x ) )
                    self.evt.hTDC.Fill(x)
            
##         if (self.ndata % 1000)==0:
##             print ('\r%8d'%self.ndata),
##             sys.stdout.flush()


def main(file_list):
        
    E = Event('tdc.root')
    V = DataReader()
#    V = vmedaq.VMEdaq()
    tdc = TDCana(0xa0000,E)
    vata = Spectrum(0x10000,E)
    V.register( tdc )
    V.register( vata )
    V.open(file_list)
    
    try:
        V.loop()
    except vmedaq.VMEdaqError,e:
        print "Algo ha salido mal:",e
    except KeyboardInterrupt:
        print 'Arrrrgggg !!!!!' 


    print 'End of Loop'
    V.close()
    E.save_histograms()

if __name__=='__main__':
    try:
        opts,args=getopt.getopt(sys.argv[1:],"o:",["out=",])
    except getopt.GetoptError,msg:
        print msg
        sys.exit(1)

    for opt,arg in opts:
        if opt in ('-o','--out'):
            fname = arg
        
    file_list = [ file for file in args ]

    try:
        main(file_list)
    except KeyboardInterrupt:
        print 'Arrrrrggggg!!!!!'
        
    print 'Adios...'
