#!/usr/bin/env python

import sys
import getopt
sys.path.insert(0,'/home/lacasta/test/py-vmedaq/build/lib.linux-i686-2.3')

import ROOT
import vmedaq
from Tracer import Tracer

saves = {}


        
class Scan(vmedaq.VATAGP):
    """
    We will assume that we have 3 scan variables: channel, pulse
    and threshold.
    """
    def __init__(self):
        global saves
        
        super(Scan,self).__init__()

        self.nevts = 0
        self.vars = []

        self.hsig = None
        self.hthr = None
        self.cnvs  = ROOT.TCanvas()
        self.cnvs.Divide(1,2)

        saves[self.cnvs.GetName()] = self.cnvs

        self.nt = 0
        self.last_chann = -1
        
    def __str__(self):
        return "MyObserver"

    def create_histograms(self,id):
        if self.hsig:
            self.hsig.IsA().Destructor( self.hsig )
        self.hsig = ROOT.TH2F("hSig","Signal Chan %d" % id,
                              128,0.,1024.,
                              10,0.05,1.05
                              )
        saves[self.hsig.GetName()] = self.hsig

        if self.hthr:
            self.hthr.IsA().Destructor( self.hthr )
        self.hthr = ROOT.TH2F("hThr",("Threshold Chan. %d" % id),
                              128,-0.5,255.5,
                              10,0.05,1.05
                              )
        saves[self.hthr.GetName()] = self.hthr

    def onNewFile(self, file_name):
        str = (len(file_name)+4)*'+'
        print '%s\n+ %s\n%s' % (str,file_name,str)

    def onNewData(self,module,evtcnt,channels):
        type,pulse = self.vars[1]
        pulse = pulse/100.
        for T in channels:
            chan,val,adj = T
            if chan<0 or chan>=module.ntot:
                return

            if not adj and chan==self.last_chann:
                self.hsig.Fill(val-module.pedestals[chan],pulse)

            break
        
    def onNewPoint(self,nevts,data):
        self.nevts = nevts
        self.vars  = data
        type,val = self.vars[0]
        if val!=self.last_chann:
            if self.hsig:
                pad = self.cnvs.cd(1)
                self.hsig.DrawCopy("colz")
                pad.Modified(1)
                pad.Update()

            if self.hthr:
                pad = self.cnvs.cd(2)
                self.hthr.DrawCopy("colz")
                pad.Modified(1)
                pad.Update()

            self.last_chann = val
            self.create_histograms(val)

        self.nt    = self.hsig.GetSumOfWeights()

##         print "-----------------------------"
##         print "New point",self.ndata,"N. var",len(data),"N. evts",nevts
##         for type,val in data:
##             print '\t',vmedaq.var_names[type],'val',val

        print '\r',
        for type,val in data:
            print vmedaq.var_names[type],val,

        self.first_in_point=-1

    def onEndPoint(self):
        type,chan  = self.vars[0]
        type,pulse = self.vars[1]
        type,thrs  = self.vars[2]
        val = (self.hsig.GetSumOfWeights()-self.nt)/float(self.nevts)

        pulse = pulse/100.
        self.hthr.Fill(thrs,pulse,val)
            

def main():
    ROOT.gROOT.SetStyle("Plain")
    ROOT.gStyle.SetOptStat(0)

    S = Scan()

    file_list = [ file for file in sys.argv[1:] ]

    print file_list
        
    V = vmedaq.VMEdaq()
    V.register( S )
    V.open(file_list)
    
    try:
        V.loop()
    except vmedaq.VMEdaqError,e:
        print "Algo ha salido mal:",e

    print '\nEnd of Loop'
    ROOT.gApplication.Run(1)
    print "End of file"
    V.close()


if __name__=='__main__':
    try:
        main()
    except KeyboardInterrupt:
        print 'Arrrrrggggg!!!!!'
        
    print 'Adios...'
#-------------------------------------------------------------------------
