#!/usr/bin/env python

import sys
import getopt
sys.path.insert(0,'/home/lacasta/test/py-vmedaq/build/lib.linux-i686-2.3')

import ROOT
import vmedaq
from Tracer import Tracer

saves = {}
def save_root_object(O):
    saves[O.GetName()]=O



class Spectrum(vmedaq.VATAGP):
    def __init__(self):
        super(Spectrum,self).__init__()
        
        self.cnvs  = ROOT.TCanvas("cnvs")
        self.hst   = None
        save_root_object(self.cnvs)
        
    
    def __str__(self):
        return "Spectrum"

    def onPedestal(self,module):
        nbin = module.ntot
        self.hst = ROOT.TProfile("hPed_%d" % module.id,
                                 "Pedestals of module %d" % module.id,
                                 nbin+1,-0.5,nbin+0.5)
        self.hst.SetMinimum(0.)
        self.hst.Draw()
        save_root_object(self.hst)
        
    def onNewData(self,module,evtcnt,data):
        for chan,val,adj in data:
            if chan<0 or chan>=module.ntot:
                continue

            self.hst.Fill(chan,val)
#            if adj:
#                self.hst.Fill(chan,val)


        
        if (self.ndata%10000)==0:
            self.cnvs.Modified(1)
            self.cnvs.Update()

        if (self.ndata % 1000)==0:
            print ('\r%8d'%self.ndata),
            sys.stdout.flush()

    def onNewFile(self,file_name):
        str = (len(file_name)+4)*'+'
        print '%s\n+ %s\n%s' % (str,file_name,str)

def Af(*data):
    print '%x' % data[0][0]

def main():
    file_list = [ file for file in sys.argv[1:] ]
    print file_list

    ROOT.gROOT.SetStyle("Plain")
    ROOT.gStyle.SetOptStat(0)

    V = vmedaq.VMEdaq()
    V.register( Spectrum() )
    V.open(file_list)
    
    try:
        V.loop()
    except vmedaq.VMEdaqError,e:
        print "Algo ha salido mal:",e
    except KeyboardInterrupt:
        print 'Arrrrgggg !!!!!' 


    print 'End of Loop'
    ROOT.gApplication.Run(1)
    print "End of file"
    V.close()


if __name__=='__main__':
    try:
        main()
    except KeyboardInterrupt:
        print 'Arrrrrggggg!!!!!'
        
    print 'Adios...'
#-------------------------------------------------------------------------
