#!/usr/bin/env python

import sys
import getopt
sys.path.insert(0,'/home/lacasta/test/py-vmedaq/build/lib.linux-i686-2.3')

import ROOT
import vmedaq
from Tracer import Tracer

saves = {}
def save_root_object(O):
    saves[O.GetName()]=O


class Scan(vmedaq.VATAGP):
    def __init__(self):
        global saves
        
        super(Scan,self).__init__()

        self.nevts = 0
        self.vars = []

        self.graph = ROOT.TGraph()
        self.cnvs  = ROOT.TCanvas()
        self.hst = ROOT.TH1F("hThrs","Threshold",10,0.,256.)
        self.hst.SetMinimum(0.)
        self.hst.SetMaximum(1.1)
        self.hst.Draw()
        self.graph.SetMarkerStyle(21)
        self.graph.SetLineColor(1)
        self.graph.SetLineWidth(2)
        self.cnvs.Update()

        saves[self.cnvs.GetName()] = self.cnvs
        saves[self.hst.GetName()] = self.hst
        saves['graph'] = self.graph
    
    def __str__(self):
        return "MyObserver"

    def onNewFile(self, file_name):
        str = (len(file_name)+4)*'+'
        print '%s\n+ %s\n%s' % (str,file_name,str)


    def onNewPoint(self,nevts,data):
        self.nevts = nevts
        self.vars  = data
##         print "-----------------------------"
##         print "New point",self.ndata,"N. var",len(data),"N. evts",nevts
##         for type,val in data:
##             print '\t',vmedaq.var_names[type],'val',val

    def onEndPoint(self):
        val = self.cnt/float(self.nevts)
        self.graph.SetPoint(self.npoint,self.vars[0][1],val)

        self.hst.Draw()
        self.graph.Draw("LPS") 

        self.cnvs.Modified(1)
        self.cnvs.Update()
        print self.npoint,self.vars[0][1],val

            
class EventSize(vmedaq.VATAGP):
    def __init__(self):
        super(EventSize,self).__init__()

        self.tracer = Tracer(500,10)
        self.tracer.callback = self.fill_histogram
        self.cnvs  = ROOT.TCanvas("cEvent")
        self.cnvs.Divide(1,2)
        self.hst = ROOT.TH1F("hEvent","EventSize",self.tracer.size+5,0.,self.tracer.size+5)

        self.cnvs.cd(1)
        self.hst.Draw()

        self.hdist = ROOT.TH1F("hD","EventSize dist.",100,0.,2000.)
        self.cnvs.cd(2)
        self.hdist.Draw()

        saves[self.cnvs.GetName()] = self.cnvs
        saves[self.hst.GetName()] = self.hst
        saves[self.hdist.GetName()] = self.hdist

    def __str__(self):
        return "EventSize"

    def DataRecord(self,data):
        self.tracer.fill( len(data) )
        self.hdist.Fill(len(data))


        if (self.ndata % 100)==0:
            print ('\r%8d'%self.ndata),
            sys.stdout.flush()

        self.ndata += 1

    def onNewFile(self,file_name):
        str = (len(file_name)+4)*'+'
        print '%s\n+ %s\n%s' % (str,file_name,str)

    def fill_histogram(self,data):
        i = 5
        for x in data:
            self.hst.SetBinContent(i,x)
            i += 1

        if (self.ndata%(self.tracer.size/4))==0:
            self.cnvs.cd(1)
            ROOT.gPad.Modified(1)
            ROOT.gPad.Update()
            
        if (self.ndata%1000)==0:
            self.cnvs.cd(2)
            ROOT.gPad.Modified(1)
            ROOT.gPad.Update()
            
        

class Spectrum(vmedaq.VATAGP):
    def __init__(self):
        super(Spectrum,self).__init__()
        
        self.cnvs  = ROOT.TCanvas("cSpectrum")
        self.hst = ROOT.TH1F("hBa","Ba spectrum",256,0.,1024.)
        self.hst.SetMinimum(0.)
        self.hst.Draw()

        saves[self.cnvs.GetName()] = self.cnvs
        saves[self.hst.GetName()] = self.hst
    
    def __str__(self):
        return "Spectrum"

    def onNewData(self,module,evtcnt,data):
        for chan,val,adj in data:
            if chan<0 or chan>=module.ntot:
                continue

            ped = module.pedestals[chan]
            v   = val-ped
            if not adj:
                self.hst.Fill(v)

        
        if (self.ndata%10000)==0:
            self.cnvs.Modified(1)
            self.cnvs.Update()

        if (self.ndata % 1000)==0:
            print ('\r%8d'%self.ndata),
            sys.stdout.flush()

    def onNewFile(self,file_name):
        str = (len(file_name)+4)*'+'
        print '%s\n+ %s\n%s' % (str,file_name,str)

def Af(*data):
    print '%x' % data[0][0]

def main():
    file_list = []
    S = None

    try:
        opts,args=getopt.getopt(sys.argv[1:],"t:",["type=",])
    except getopt.GetoptError,msg:
        print msg
        sys.exit(1)

    for opt,arg in opts:
        if opt in ('-t','--type'):
            stype = arg.lower()
            if stype=='scan':
                S=Scan()
            if stype=='event':
                S=EventSize()
            if stype=='data':
                S=Spectrum()


    if not S:
        S = Spectrum()
        
    file_list = [ file for file in args ]

    print file_list
        
    V = vmedaq.VMEdaq()
    V.register( S )
    V.open(file_list)
    
    try:
        V.loop()
    except vmedaq.VMEdaqError,e:
        print "Algo ha salido mal:",e
    except KeyboardInterrupt:
        print 'Arrrrgggg !!!!!' 


    print 'End of Loop'
    ROOT.gApplication.Run(1)
    print "End of file"
    V.close()


if __name__=='__main__':
    try:
        main()
    except KeyboardInterrupt:
        print 'Arrrrrggggg!!!!!'
        
    print 'Adios...'
#-------------------------------------------------------------------------
