# Configure paths for ROOT

dnl AM_PATH_ROOT([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND [, MODULES]]]])
dnl Test for ROOT, and define ROOT_CFLAGS and ROOT_LIBS
dnl
AC_DEFUN([AM_PATH_ROOT],
[dnl 
dnl Get the cflags and libraries from the root-config script
dnl
AC_ARG_WITH(root-prefix,[  --with-root-prefix=PFX   Prefix where ROOT is installed (optional)],
            root_config_prefix="$withval", root_config_prefix="")
AC_ARG_WITH(root-exec-prefix,[  --with-root-exec-prefix=PFX Exec prefix where ROOT is installed (optional)],
            root_config_exec_prefix="$withval", root_config_exec_prefix="")
AC_ARG_ENABLE(roottest, [  --disable-roottest       Do not try to compile and run a test ROOT program],
		    , enable_roottest=yes)

  for module in . $4
  do
      case "$module" in
         root) 
             root_config_args="$root_config_args root"
         ;;
      esac
  done

  if test x$root_config_exec_prefix != x ; then
     root_config_args="$root_config_args --exec-prefix=$root_config_exec_prefix"
     if test x${ROOT_CONFIG+set} != xset ; then
        ROOT_CONFIG=$root_config_exec_prefix/bin/root-config
     fi
  fi
  if test x$root_config_prefix != x ; then
     root_config_args="$root_config_args --prefix=$root_config_prefix"
     if test x${ROOT_CONFIG+set} != xset ; then
        ROOT_CONFIG=$root_config_prefix/bin/root-config
     fi
  fi

  AC_PATH_PROG(ROOT_CONFIG, root-config, no)
  min_root_version=ifelse([$1], ,0.99.7,$1)
  AC_MSG_CHECKING(for ROOT - version >= $min_root_version)
  no_root=""
  if test "$ROOT_CONFIG" = "no" ; then
    no_root=yes
  else
    ROOT_CFLAGS=`$ROOT_CONFIG $root_config_args --cflags`
    ROOT_LIBS=`$ROOT_CONFIG $root_config_args --libs`
    root_config_major_version=`$ROOT_CONFIG $root_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    root_config_minor_version=`$ROOT_CONFIG $root_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    root_config_micro_version=`$ROOT_CONFIG $root_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
  fi
  if test "x$no_root" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$ROOT_CONFIG" = "no" ; then
       echo "*** The root-config script installed by ROOT could not be found"
       echo "*** If ROOT was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the ROOT_CONFIG environment variable to the"
       echo "*** full path to root-config."
     fi
     ROOT_CFLAGS=""
     ROOT_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(ROOT_CFLAGS)
  AC_SUBST(ROOT_LIBS)
])
