#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_CONFIG_H
#include <config_analysis.h>
#endif
#include <cstring>
#if defined(HAS_GTK)
#include <gtk/gtk.h>
#include <gtk-hst/gtkhistogram.h>
#include <gtk-hst/histogram.h>


#include <analysis/DataFormat.h>
#include <analysis/vmedaq.h>
#include "pixmaps.h"

GtkWidget *create_widget (gchar *, gchar *, gchar *, gint , gint );
GtkWidget *create_spyData (gpointer user_data);
GtkWidget *create_pixmap(GtkWidget *widget, char **data);
void       set_event_number(int nevt);


GtkHst1D   *evt, *hsig;
GtkWidget  *event_number;
int         action = 0;
VMEdaq      vmedaq;

void hst_histogram_set_pointer(HstHistogram *h2d, gpointer ptr);

class VATAdata : public VATAGP
{
private:
    bool first_ped;
    double *buffer, *dsignal;
    std::vector<unsigned int *> Q;
    int _pos;
    bool save;
public:
    VATAdata(bool up = false)
            : VATAGP(up), first_ped(true),  buffer(0), dsignal(0), _pos(-1), save(true)
    {}

    ~VATAdata();
    void DataRecord(unsigned int *);
    void onNewRun();
    void onPedestals(const Module *m);
    void onNewData(const Module *, int evtcnt, const ChannelList &);


    void decrement_pos()
    {
        if (_pos == -1)
            _pos = Q.size() - 1;
        else
        {
            _pos--;
            if (_pos < 0)
                _pos = 0;
        }
    }
    void increment_pos()
    {
        if (_pos >= 0)
            _pos++;
    }
    bool decode_next();
};

VATAdata::~VATAdata()
{
    std::vector<unsigned int *>::iterator ip;
    for (ip = Q.begin(); ip != Q.end(); ip++)
        g_free( *ip );
}

void VATAdata::DataRecord(unsigned int *data)
{
    VATAGP::DataRecord(data);

    if (save)
    {
        int size = data[0] & 0xfffL;
        unsigned int *bf = g_new(unsigned int, size);
        memcpy(bf, data, size*sizeof(unsigned int));
        Q.push_back(bf);

        if (Q.size() > 100)
        {
            std::vector<unsigned int *>::iterator ip = Q.begin();
            bf = *ip;
            g_free(bf);
            Q.erase( ip );
        }
    }

}

bool VATAdata::decode_next()
{
    if (_pos == Q.size() || _pos < 0 )
        return false;
    else
    {
        save = false;
        DataRecord( Q[_pos] );
        save = true;
        return true;
    }
}

void VATAdata::onNewRun()
{
    std::cout << "Number of Modules: " << _idlist.size() << std::endl;
}
void VATAdata::onPedestals(const Module *md)
{
    std::cout << "Pedestals for Module id. " << md->id()
    << ": Number of chips: " << md->nchip()
    << " Chann/chip " << md->nchan()
    << std::endl;

    int i;
    if (buffer)
        g_free(buffer);

    if (dsignal)
        g_free(dsignal);

    double *v = buffer = g_new(double, md->ntot());
    double *w = dsignal = g_new(double, md->ntot());
    for (i = 0;i < md->ntot();i++)
        v[i] = w[i] = 0.;

    g_object_set( HST_HISTOGRAM(evt->func),
                  "nx", md->ntot(), "xmin", 0., "xmax", (double)md->ntot(),
                  "ptr", v,
                  NULL);
    g_object_set( HST_HISTOGRAM(hsig->func),
                  "nx", md->ntot(), "xmin", 0., "xmax", (double)md->ntot(),
                  "ptr", w,
                  NULL);
    g_object_set( G_OBJECT(evt), "hymin", -50., "hymax", 1024., NULL);
    gtk_widget_queue_draw(GTK_WIDGET(evt));

    g_object_set( G_OBJECT(hsig), "hymin", -512., "hymax", 512., NULL);
    gtk_widget_queue_draw(GTK_WIDGET(hsig));

}

void VATAdata::onNewData(const Module *md, int evtcnt, const ChannelList &data)
{
    set_event_number(evtcnt);

    int i;
    for (i = 0;i < md->ntot();i++)
        buffer[i] = dsignal[i] = 0.;

    ChannelList::const_iterator ip;
    for (ip = data.begin();ip != data.end();++ip)
    {
        Channel *ptr = *ip;
        VATAChannel *ch = dynamic_cast<VATAChannel *>(ptr);
        int cn = ch->channel();
        if (cn >= 0 && cn < md->ntot())
        {
            buffer[cn] = ch->signal();
            dsignal[cn] = ch->signal() - md->pedestal( cn );
        }
    }
    gtk_widget_queue_draw(GTK_WIDGET(evt));
    gtk_widget_queue_draw(GTK_WIDGET(hsig));
}


int main(int argc, char **argv)
{
    GtkWidget *spyData;
    VATAdata vata;

    if ( !argv[1] )
    {
        printf("I need an input file\n");
        exit(1);
    }

    std::string iFile(argv[1]);
    vmedaq.open( iFile );
    vmedaq.add( &vata );
    gtk_init (&argc, &argv);
    spyData = create_spyData ((gpointer *) & vata);
    gtk_widget_show (spyData);

    gtk_main ();
    return 0;
}

void do_action(GtkWidget *, VATAdata *vata)
{
}

gboolean do_read = FALSE;
int tmout = -1;

void
Pause_button(GtkWidget *, VATAdata *vata)
{
    do_read = FALSE;
}

void
Fwd_button(GtkWidget *, VATAdata *vata)
{
    vata->increment_pos();
    if ( !vata->decode_next() )
        while ( !IsDataRecord( vmedaq.read() ) );
}

void
Bwd_button(GtkWidget *, VATAdata *vata)
{
    vata->decrement_pos();
    vata->decode_next();
}
gboolean show_events( VATAdata *vata )
{
    Fwd_button(0, vata);
    if (!do_read)
        tmout = -1;

    return do_read;
}

void FFwd_button(GtkWidget *w, VATAdata *vata)
{
    if (tmout < 0)
    {
        tmout = g_timeout_add( 10,
                               (GSourceFunc)show_events,
                               (gpointer)vata);
        do_read = TRUE;
    }
}

GtkWidget *create_spyData (gpointer user_data)
{
    GtkWidget *spyData;
    GtkWidget *vbox1;
    GtkWidget *toolbar1;
    GtkWidget *tmp_toolbar_icon;
    GtkWidget *btnBBack;
    GtkWidget *btnBack;
    GtkWidget *btnPause;
    GtkWidget *btnFwd;
    GtkWidget *btnFFwd;
    GtkWidget *Histogram;
    HstHistogram *hst1d;

    spyData = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_object_set_data (GTK_OBJECT (spyData), "spyData", spyData);
    gtk_window_set_title (GTK_WINDOW (spyData), "spyData");

    vbox1 = gtk_vbox_new (FALSE, 0);
    gtk_widget_ref (vbox1);
    gtk_object_set_data_full (GTK_OBJECT (spyData), "vbox1", vbox1,
                              (GtkDestroyNotify) gtk_widget_unref);
    gtk_widget_show (vbox1);
    gtk_container_add (GTK_CONTAINER (spyData), vbox1);

    toolbar1 = gtk_toolbar_new ();
    gtk_toolbar_set_orientation(GTK_TOOLBAR (toolbar1),
                                GTK_ORIENTATION_HORIZONTAL);
    gtk_widget_show (toolbar1);
    gtk_box_pack_start (GTK_BOX (vbox1), toolbar1, FALSE, FALSE, 0);

    tmp_toolbar_icon = create_pixmap (spyData, stock_first_xpm);
    btnBBack = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                           GTK_TOOLBAR_CHILD_BUTTON,
                                           NULL,
                                           NULL,
                                           NULL, NULL,
                                           tmp_toolbar_icon, NULL, NULL);
    gtk_widget_show (btnBBack);

    tmp_toolbar_icon = create_pixmap (spyData, stock_left_arrow_xpm);
    btnBack = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                          GTK_TOOLBAR_CHILD_BUTTON,
                                          NULL,
                                          NULL,
                                          NULL, NULL,
                                          tmp_toolbar_icon,
                                          GTK_SIGNAL_FUNC(Bwd_button) ,
                                          user_data);
    gtk_widget_show (btnBack);

    tmp_toolbar_icon = create_pixmap (spyData, stock_stop_xpm);
    btnPause = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                           GTK_TOOLBAR_CHILD_BUTTON,
                                           NULL,
                                           NULL,
                                           NULL, NULL,
                                           tmp_toolbar_icon,
                                           GTK_SIGNAL_FUNC(Pause_button) ,
                                           user_data);
    gtk_widget_show (btnPause);

    tmp_toolbar_icon = create_pixmap (spyData, stock_right_arrow_xpm);
    btnFwd = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                         GTK_TOOLBAR_CHILD_BUTTON,
                                         NULL,
                                         NULL,
                                         NULL, NULL,
                                         tmp_toolbar_icon,
                                         GTK_SIGNAL_FUNC(Fwd_button),
                                         user_data
                                        );
    gtk_widget_show (btnFwd);

    tmp_toolbar_icon = create_pixmap (spyData, stock_last_xpm);
    btnFFwd = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                          GTK_TOOLBAR_CHILD_BUTTON,
                                          NULL,
                                          NULL,
                                          NULL, NULL,
                                          tmp_toolbar_icon,
                                          GTK_SIGNAL_FUNC(FFwd_button),
                                          user_data);
    gtk_widget_show (btnFFwd);


    event_number = gtk_label_new("      0");
    gtk_toolbar_append_widget ( GTK_TOOLBAR (toolbar1),
                                event_number,
                                NULL,
                                NULL
                              );
    gtk_widget_show (event_number);

    // Create the event display
    hst1d = hst_histogram_new(10, 0., 10., 0);
    Histogram = gtk_hst1d_new(HST_FUNCTION(hst1d));
    evt = GTK_HST1D(Histogram);
    hst_function_set_name(HST_FUNCTION(hst1d), "raw data");
    gtk_hst1d_sync_range(evt, TRUE);
    gtk_hst1d_draw_x_axis(evt, HST_AXIS_NONE);
    gtk_hst1d_draw_y_axis(evt, HST_AXIS_MIDDLE);
    g_object_set( G_OBJECT(evt), "hymin", -50., "hymax", 1024., NULL);
    gtk_hst1d_set_colors(evt, "foreground", "red", "background", "black", "axis", "green", NULL);
    gtk_widget_show (Histogram);
    gtk_box_pack_start (GTK_BOX (vbox1), Histogram, TRUE, TRUE, 0);
    GTK_WIDGET_UNSET_FLAGS (Histogram, GTK_CAN_FOCUS);
    GTK_WIDGET_UNSET_FLAGS (Histogram, GTK_CAN_DEFAULT);

    // Create the signal display
    hst1d = hst_histogram_new(10, 0., 10., 0);
    Histogram = gtk_hst1d_new(HST_FUNCTION(hst1d));
    hsig = GTK_HST1D(Histogram);
    hst_function_set_name(HST_FUNCTION(hst1d), "signal");
    gtk_hst1d_sync_range(hsig, TRUE);
    gtk_hst1d_draw_x_axis(hsig, HST_AXIS_NONE);
    gtk_hst1d_draw_y_axis(hsig, HST_AXIS_MIDDLE);
    g_object_set( G_OBJECT(hsig), "hymin", -50., "hymax", 1024., NULL);
    gtk_hst1d_set_colors(hsig, "foreground", "red", "background", "black", "axis", "green", NULL);
    gtk_widget_show (Histogram);
    gtk_box_pack_start (GTK_BOX (vbox1), Histogram, TRUE, TRUE, 0);
    GTK_WIDGET_UNSET_FLAGS (Histogram, GTK_CAN_FOCUS);
    GTK_WIDGET_UNSET_FLAGS (Histogram, GTK_CAN_DEFAULT);


    gtk_signal_connect (GTK_OBJECT (spyData), "delete_event",
                        GTK_SIGNAL_FUNC (gtk_main_quit),
                        NULL);

    return spyData;
}


GtkWidget *create_pixmap(GtkWidget *widget, char **data)
{
    GdkColormap *colormap;
    GdkPixmap *gdkpixmap;
    GdkBitmap *mask;
    GtkWidget *pixmap;
    colormap = gtk_widget_get_colormap (widget);
    gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (
                    NULL, colormap, &mask, NULL, data);
    if ( gdkpixmap == 0 )
        return 0;

    pixmap = gtk_pixmap_new (gdkpixmap, mask);
    gdk_pixmap_unref (gdkpixmap);
    gdk_bitmap_unref (mask);
    return pixmap;
}


void set_event_number(int nevt)
{
    char buff[1024];
    GtkLabel *lbl;
    sprintf(buff, "%10d", nevt);
    lbl = GTK_LABEL(event_number);
    gtk_label_set_text(lbl, buff);
}
#else
int main(int argc, char **argv) {
    printf("spydata needs GTK and GTKHST\n");
    return 1;
}

#endif
