#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <iomanip>
#include <csignal>
#include <cstring>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <getopt.h>
#include <DAQ++/ObserverModel.h>
#include <DAQ++/DAQmanager.h>
#include <daq/DAQcontroller.h>
#include <gui/MainWindow.h>
#include <gui/RunMenu.h>
#include <gui/ParamSlider.h>
#include <gui/MsgManager.h>
#include <gui/GladeApp.h>
#include <config/ConfigParser.h>
#include <config/GeneralConfig.h>
#include <config/IOConfig.h>
#include <config/PathConfig.h>
#include <python/PythonLoader.h>

#if defined(HAS_DAQcorba)
#include <DAQcorba/DAQ-utils.h>
#endif

void gui_force_load();

GMainLoop *loop=0;
void quit(int )
{

    //   signal(SIGINT,SIG_DFL);
    std::cout << std::endl << "Arrrgggg !!!!!" << std::endl;
    if (loop)
    {
        g_main_loop_quit(loop);
    }
}



static void my_log (const gchar *log_domain,
                    GLogLevelFlags log_level,
                    const gchar *message,
                    gpointer user_data)
{
    if (!strcmp(log_domain,"libglade") && strstr(message,"unknown"))
        return;
    printf("MyLogger[%s]: %s\n", log_domain, message);
    //    (*old_log)(log_domain,log_level,message,user_data);
}

void setup_log_intercept()
{
    g_log_set_handler ("GLib-GObject",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                        G_LOG_FLAG_FATAL |
                                        G_LOG_FLAG_RECURSION),
                       my_log, 0);
    g_log_set_handler ("Gtk",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                        G_LOG_FLAG_FATAL |
                                        G_LOG_FLAG_RECURSION),
                       my_log, 0);
    g_log_set_handler ("GLib",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                        G_LOG_FLAG_FATAL |
                                        G_LOG_FLAG_RECURSION),
                       my_log, 0);
    g_log_set_handler ("XPath",
                       (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                        G_LOG_FLAG_FATAL |
                                        G_LOG_FLAG_RECURSION),
                                        my_log, 0);
    g_log_set_handler ("libglade",
                        (GLogLevelFlags)(G_LOG_LEVEL_MASK |
                                         G_LOG_FLAG_FATAL |
                                         G_LOG_FLAG_RECURSION),
                                         my_log, 0);
}

int main(int argc, char **argv)
{
    char *prgnam = strrchr(argv[0], '/');
    if (!prgnam)
        prgnam = argv[0];
    else
        prgnam++;

    gui_force_load();

    std::cout << "Starting " << prgnam << std::endl;
    setup_log_intercept();

    // initialize gtk
    int nfake = 1;
    gtk_init (&nfake, &argv);
    glade_init();

    // The python loader
#if defined(HAS_PYTHON) && defined(HAS_PYGTK)
    PythonLoader &pyloader = PythonLoader::getInstance();
    pyloader.initialize(argc, argv);
#endif


    // parse the command line options
    ConfigParser *cfg = ConfigParser::theConfig();
    std::ostringstream opath;
    opath << ".:" << getenv("HOME") << "/.vmedaq";
    cfg->add_config_path(opath.str());

    int ivalidate = 0;
    int igui = 1;
    int idefault=1;
    static struct option long_options[] =
        {
            {"validate", 0, &ivalidate, 1},
            {"schema-path", 1, 0, 's'},
            {"config-path", 1, 0, 'c'},
            {"ior", 1, 0, 'i'},
            {"gui",0,&igui,1},
            {"no-gui", 0, &igui, 0},
            {"no-cfg", 0, &idefault, 0},
            {0, 0, 0, 0}
        };
    while (1)
    {
        char c;
        int option_index = 0;
        c = getopt_long (argc, argv, "s:c:i:h", long_options, &option_index);
        if ( c == -1 )
            break;
        switch (c)
        {
            case 0:
                break;
            case 's':
                cfg->add_schema_path( optarg );
                break;
            case 'c':
                cfg->add_config_path( optarg );
                break;
            case '?':
                std::cout << "A missing argument for " << long_options[option_index].name << std::endl;
                break;
            default:
                std::cout << optopt << " " << argv[optind]
                                                   << " [Ughhhh !!!!]"
                                                   << std::endl;
                break;
        }
    }

    // Get config file
    const char *xmlFile = optind >= argc ? 0 : argv[optind];
    std::string std;



    // Activate the MsgManager
    MsgManager *mmm = MsgManager::theMsgManager();

    if (!xmlFile && idefault)
    {
        // Assume we have it in the "standard" place
        std = std::string( getenv("HOME") ) + "/.vmedaq/vmedaq.xml";
        if ( access(std.c_str(), F_OK) )
        {
            std::cout << std::endl
                      << std << " not found." << std::endl
                      << "I will need a configuration file." << std::endl
                      << "usage:" << std::endl
                      << "\tvmedaq [config_file]" << std::endl
                      << std::endl;
            return 1;
        }
        xmlFile = std.c_str();
    }

    cfg->set_do_validate( ivalidate != 0 );
    cfg->reset();

    try
    {
        if (!cfg->parse(xmlFile) )
        {
            std::cout << prgnam << ": Problems parsing " << xmlFile << std::endl;
            return 1;
        }
    }
    catch (ConfigException e)
    {
        std::cout << prgnam << ": Problems parsing " << xmlFile << std::endl;
        std::cout << "..." << e.what() << std::endl;
        return 1;
    }

    // The DAQ controller
    DAQcontroller *DAQ =DAQcontroller::instance();
    DAQ->setup_daq(argc, argv);

    // The xmlrpc server
#if defined(HAS_PYTHON) && defined(HAS_PYGTK)
    pyloader.start_server(8080);
#endif


    // GUI
    MainWindow *wnd = 0;
#if defined(HAS_DAQcorba)
    try
    {
#endif
        // catch control-c
        signal(SIGINT, quit);
        if (igui)
        {
            //---------------------------------------------------------------------
            // Gtk stuff
            //---------------------------------------------------------------------

            // Create the main window
            wnd = new MainWindow();
            wnd->init("vmedaq.glade", "vmedaq");

            // Setup DAQ gui
            DAQ->set_main_window(wnd);
            gtk_main();
        }
        else
        {
            loop = g_main_loop_new (NULL, TRUE);
            g_main_loop_run (loop);
        }
#if defined(HAS_DAQcorba)
    }
    catch ( CORBA_Exception e )
    {
        std::cout << "Got a CORBA_Exception: " << e.what() << std::endl;
    }
    catch ( ... )
    {
        std::cout << "Got an unknown exception" << std::endl;
    }
#endif

    //---------------------------------------------------------------------
    // Save the configuration
    //---------------------------------------------------------------------
    std::string default_dir = std::string( getenv("HOME") ) + "/.vmedaq";
    std::string config_file = default_dir + "/vmedaq.xml";
    struct stat dfstat;
    bool failed = false;
    std::cout << "Saving current configuration in " << config_file << std::endl;
    if ( stat(default_dir.c_str(), &dfstat) )
    {
        if ( mkdir(default_dir.c_str(), 0666) )
        {
            std::cout << "Could not create " << default_dir << std::endl;
            failed = true;
        }
    }
    if (!failed)
        ConfigParser::theConfig()->save_config( config_file.c_str() );


    if (igui)
    {
        ParamSlider::delete_all();
        RunMenu::delete_all();
    }
    // Temporary patch to remove a sig abort when closing the program
    /*------
      Shuting down the ORB
      ...destroying Module 256
      pure virtual method called
      terminate called without an active exception
      Aborted

      For some reason there is still someone "registered" to the
      DAQmanager.
    */
    DAQpp::DAQmanager::theDAQmanager()->deleteObservers();
    DAQ->reset_daq();
    std::cout << "Done" << std::endl;
}
