#include "IOConfig.h"

void IOConfig::cpy(const IOConfig&x)
{
    compress = x.compress;
    max_size = x.max_size;
    path = x.path;
    prefix = x.prefix;
    run_number = x.run_number;
}

void IOConfig::operator=(const IOConfig &x)
{
    if ( &x != this) cpy(x);
}

void IOConfig::parse_node(xmlNodePtr _node)
throw (ConfigException)
{
    std::string val;
    node = _node;
    ConfigParser *cfg = ConfigParser::theConfig();
    bool compress = true;
    try
    {
        compress = cfg->get_attribute<bool>(node, "compression");
    }
    catch ( ConfigException &e )
    {
        compress = true;
    }
    set_compression( compress );

    int ival;
    std::string sval;
    try
    {
        ival =  cfg->get_attribute<int>(node, "max_size");
    }
    catch ( ConfigException &e )
    {
        ival = 50000000;
    }
    set_max_size( ival );

    try
    {
        sval = cfg->get_attribute<std::string>(node, "path");
    }
    catch ( ConfigException &e )
    {
        sval = "/tmp/";
    }
    set_output_path( sval );

    try
    {
        sval = cfg->get_attribute<std::string>(node, "prefix");
    }
    catch ( ConfigException &e )
    {
        sval = "vmedaq";
    }
    set_prefix( sval );

    try
    {
        ival = cfg->get_attribute<int>(node, "run_number");
    }
    catch ( ConfigException &e )
    {
        ival = 0;
    }
    set_run_number( ival );
}

void IOConfig::save_config()
   throw (ConfigException)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    node = cfg->get_io_node();
    if (!node)
        return;
    
    
    cfg->check_attribute<bool>(node, "compression", compress);
    cfg->check_attribute<int>(node, "max_size", max_size);
    cfg->check_attribute<int>(node, "run_number", run_number);
    cfg->check_attribute<std::string>(node, "prefix", prefix);
    cfg->check_attribute<std::string>(node, "path", path);
}
std::ostream &operator<<(std::ostream &os, const IOConfig &p)
{
    os << "IOConfig: " << std::endl
    << "   + compress: " << (p.compress ? "on" : "off" ) << std::endl
    << "   + mx. size: " << p.max_size << std::endl
    << "   + run no. : " << p.run_number << std::endl
    << "   + prefix  : " << p.prefix << std::endl
    << "   + path    : " << p.path;
    return os;
}
