#ifndef __IOConfig_h__
#define __IOConfig_h__
#include <config/ConfigParser.h>
/**
   \brief class IOConfig.

   \ingroup CONFIG
*/
class IOConfig : public SectionParser
{
private:
    bool compress;
    int max_size;
    int run_number;
    std::string path;
    std::string prefix;
    void cpy(const IOConfig &);
public:
    /// Constructor
    IOConfig()
            : compress(false), max_size(50000),
            run_number(0), path("/tmp"), prefix("run")
    {}

    /// Copy constructor
    IOConfig(const IOConfig &x)
    {
        cpy(x);
    }

    /// Asign operator
    void operator=(const IOConfig &x);

    /// Destructor
    ~IOConfig()
    {}

    bool get_compression() const
    {
        return compress;
    }
    int  get_max_size() const
    {
        return max_size;
    }
    int  get_run_number() const
    {
        return run_number;
    }
    const std::string get_output_path() const
    {
        return path;
    }
    const std::string get_prefix() const
    {
        return prefix;
    }

    void set_compression(bool x)
    {
        compress = x;
    }
    void set_max_size(int x)
    {
        max_size = x;
    }
    void set_output_path(const std::string &x)
    {
        path = x;
    }
    void set_run_number(int x)
    {
        run_number = x;
    }
    void set_prefix(const std::string &x)
    {
        prefix = x;
    }

    friend std::ostream &operator<<(std::ostream &os, const IOConfig &c);

    void parse_node(xmlNodePtr )
       throw (ConfigException);
    void save_config() throw (ConfigException);
};

#endif
