#ifndef __ModuleParser_h__
#define __ModuleParser_h__
#include <iosfwd>
#include <string>
#include <config/SectionParser.h>
//#include <gui/MonitorWindow.h>

class MonitorWindow;
/**
 *  \brief class ModuleParser.
 *  \ingroup CONFIG
 *
 *  This is a specialized descendant of SectionParser used to handle
 *  modules in the modules section. In addition to the parsing methods
 *  defined in SectionParser it defines:
 *     - config : called by the gui to change configuration parameters
 *     - monitor: called by the gui to create the monitor display
 *     - decode : called to decode the data
 *     - range_min, range_max: to retrieve the range of the monitor
 *                             histograms
 *  The hook element in the plugins section should return a pointer to
 *  ModuleParser. After processing the plugins section the ConfigParser
 *  will register itself the ModuleParser objects to parse the
 *  correspondig modules.
 */

struct _GtkWidget;
class Analysis;
class PadMap;

class ModuleParser : public SectionParser
{
    private:
        int    nbins;
        double rmin, rmax;
        std::string pedestal_file;
        PadMap *map;
        xmlNodePtr monitor_node;
    public:
        ModuleParser() : nbins(512), rmin(0.), rmax(1023.), map(0), monitor_node(0)
        {}
        virtual ~ModuleParser();

        virtual _GtkWidget *config(const std::string &id)
        {
            return 0;
        }
        virtual MonitorWindow *monitor(const std::string &id, const std::string &glade_file)
        {
            return 0;
        }

        virtual void parse_monitor(const std::string &element, const std::string &id, Analysis *);
        virtual void parse_map(const std::string &element, const std::string &id, Analysis *);
        virtual void parse_common(const std::string &element, const std::string &id);

        virtual void save_common();
        double range_min() const
        {
            return rmin;
        }
        double range_max() const
        {
            return rmax;
        }
        int    nbin() const
        {
            return nbins;
        }
        void set_range(double mn, double mx)
        {
            if ( mn < mx )
            {
                rmin = mn;
                rmax = mx;
            }
            else
            {
                rmin = mx;
                rmax = mn;
            }
        }
        void set_nbins(int n)
        {
            nbins = n;
        }

        void create_map(int nx, int ny, int ntot);
        PadMap *get_map()
        {
            return map;
        }
        const std::string &get_pedestal_file() const { return pedestal_file; }
        void set_pedestal_file(const std::string &file) { pedestal_file=file; }
};


#endif
