#include <iostream>
#include <config/ConfigParser.h>
#include <config/PluginParser.h>
#include <gui/GladeApp.h>
#include <python/PythonLoader.h>
#include "PathConfig.h"

PathConfig::PathConfig()
{
    paths[libpath] = SearchPath();
    paths[gladepath] = SearchPath();
    paths[configpath] = SearchPath();
}

void PathConfig::parse_node(xmlNodePtr _node)
    throw (ConfigException)
{
//    ConfigParser *cfg = ConfigParser::theConfig();
    xmlNodePtr var = node->xmlChildrenNode;

    for (;var;var = var->next)
    {
        if (var->type != XML_ELEMENT_NODE)
            continue;
        
        if (!var->name)
            continue;
        
        std::string block = (char *)var->name;
        PathType type;
        if (block=="libpaths")
        {
            type = libpath;
        }
        else if (block == "gladepaths")
        {
            type = gladepath;
        }
        else if (block =="configpaths")
        {
            type = configpath;
        }
        else if (block == "pythonpaths")
        {
            type = pythonpath;
        }
        else
        {
            std::cout << "Element " << block << " is invalid in <paths> section" << std::endl;
        }
        parse_block(type, var);
    }
}

void PathConfig::parse_block(PathType type, xmlNodePtr node)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    if (type==none)
        return;
    
    xmlNodePtr child  = node->xmlChildrenNode;

    for (; child; child = child->next)
    {
        if (child->type != XML_ELEMENT_NODE)
            continue;
        
        if (!child->name)
            continue;
        
        std::string name = (char *)child->name;
        if (name!="path")
            continue;
        
        std::string val = cfg->get_value<std::string>(child);
        add_path(type, val);
    }
}

void PathConfig::add_path(PathType type, const std::string &val)
{
    SearchPath &the_set = paths[type];
    SearchPath::iterator ip = std::find(the_set.begin(), the_set.end(), val);
    if (ip==the_set.end())
    {
        the_set.add_path(val);
#if defined(HAS_PYTHON) && defined(HAS_PYDAQPP)
        if (type==pythonpath)
        {
            PythonLoader::getInstance().add_path(val);
        }
#endif   
    }
}

void PathConfig::add_path(PathType type, const SearchPath &P)
{
    paths[type].add_path(P);

}

void PathConfig::set_path(PathType type, const SearchPath &P)
{
    paths[type] = P;
}

void PathConfig::save_config()
   throw (ConfigException)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    node = cfg->get_paths_node();
    if (node==0)
    {
        ConfigParser *cfg = ConfigParser::theConfig();
        node=cfg->create_child(cfg->root(),"paths",std::string());
    }
    save_block(libpath, "libpaths");
    save_block(gladepath, "gladepaths");
    save_block(configpath, "configpaths");
    save_block(pythonpath, "pythonpaths");
}
void PathConfig::save_block(PathType type, const std::string &xpath)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    int inode, nnode;
    std::vector<xmlNodePtr> vnode;
    nnode = cfg->get_node("//paths/"+xpath, vnode);
    for (inode=0; inode<nnode; inode++)
    {
        xmlNodePtr lp = vnode[inode];
        cfg->remove_child(lp);
    }
    xmlNodePtr lp = cfg->create_child<std::string>(node, xpath, std::string());
    
    SearchPath::iterator ip;
    for (ip=paths[type].begin(); ip!=paths[type].end();ip++)
    {
        if ( *ip != ".")
            cfg->create_child<std::string>(lp, "path", *ip);
    }
}


void PathConfig::cpy(const PathConfig &s)
{
    paths.clear();
    
    
    paths[libpath]    = s.paths.find(libpath)->second;
    paths[gladepath] = s.paths.find(gladepath)->second;
    paths[configpath] = s.paths.find(configpath)->second;
}

std::string PathConfig::find_library(const std::string &lib)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    PathConfig *cpath = dynamic_cast<PathConfig *>(cfg->get_handler("paths"));
    SearchPath libpaths(cpath->get_path(libpath));
    char *ld_path=getenv("LD_LIBRARY_PATH");   
    if (ld_path)
        libpaths.add_path(std::string(ld_path));

    libpaths.add_path(LIB_DIR);
 
    return libpaths.find_file(lib);
}

std::string PathConfig::find_glade_file(const std::string &file)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    PathConfig *cpath = dynamic_cast<PathConfig *>(cfg->get_handler("paths"));
    SearchPath gpaths(cpath->get_path(gladepath));
    gpaths.add_path(std::string(DATA_DIR)+std::string("/glade"));
    return gpaths.find_file(file);
}

SearchPath PathConfig::get_path(PathType type)
{
    return paths[type];
}
