#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <dlfcn.h>
#include <daq/Rmanager.h>
#include "Config.h"
#include "config_utils.h"
#if defined(HAS_PYTHON) && defined(HAS_PYDAQPP)
#include <python/PythonLoader.h>
#include <python/pymodule_parser.h>
#endif
static int  __enable__()
{
    return -1;
}
static void __dummy_void__()
{}


PluginParser::LibMap PluginParser::libmap;

PluginParser::PluginParser() :
    trigger_enable(__enable__), trigger_disable(__dummy_void__),
    trigger_reset(__dummy_void__), trigger_start(__dummy_void__),
    trigger_stop(__dummy_void__), data_logger(0)
{
}

PluginParser::~PluginParser()
{
}

void PluginParser::reset_plugins()
{
    trigger_enable = __enable__;
    trigger_disable = __dummy_void__;
    trigger_reset = __dummy_void__;
    trigger_start = __dummy_void__;
    trigger_stop = __dummy_void__;
    data_logger = 0;
    run_manager_factory.clear();
}

void PluginParser::parse_node( xmlNodePtr _node) throw( ConfigException )
{
    std::string name;
    xmlNodePtr child;
    node = _node;
 
    child = node->xmlChildrenNode;

    for (; child; child = child->next)
    {
        if (child->type != XML_ELEMENT_NODE)
            continue;

        name = (char *)child->name;
        if (name=="plugin")
        {
            parse_plugin(child);
        }
        else if (name=="interrupt")
        {
            parse_interrupt(child);
        }
        else if (name=="run_managers")
        {
            parse_run_manager(child);
        }
        else if (name=="data_logger")
        {
            parse_datalogger(child);
        }
        else
        {
            std::cout << "Wrong element " << name << " in plugins" << std::endl;
        }
    }
}

void PluginParser::parse_datalogger(xmlNodePtr node) throw( ConfigException )
{
    ConfigParser *cfg = ConfigParser::theConfig();
    std::string lib, symbol;
    try
    {
        lib = cfg->get_value<std::string>(node, "lib");
    }
    catch ( ConfigException &e )
    {
        std::cout << "ConfigParser: data_logger " 
                  << " does not have the 'lib' element"
                  << std::endl;
        return;
    }

    try
    {
        symbol = cfg->get_value<std::string>(node, "hook");
    }
    catch ( ConfigException &e )
    {
        std::cout << "ConfigParser: data_logger "
                  << " does not have the 'hook' element"
                  << std::endl;
        return;
    }

    Hook hook = load_symbol(symbol, lib);
    if (hook)
    {
        data_logger = (LoggerCreator)hook;
    }
}

void PluginParser::parse_plugin(xmlNodePtr node)
    throw ( ConfigException )
{
    ConfigParser *cfg = ConfigParser::theConfig();
    std::string lib, symbol, lang;
    std::string name = cfg->get_attribute<std::string>(node, "name");
    std::string module = cfg->get_attribute<std::string>(node, "module");

    try
    {
        lib = cfg->get_value<std::string>(node, "lib");
    }
    catch ( ConfigException &e )
    {
        std::cout << "ConfigParser: plugin " << name
                  << " does not have the 'lib' element"
                  << std::endl;
        return;
    }

    try
    {
        symbol = cfg->get_value<std::string>(node, "hook");
    }
    catch ( ConfigException &e )
    {
        std::cout << "ConfigParser: plugin " << name
                  << " does not have the 'hook' element"
                  << std::endl;
        return;
    }
    
    try 
    {
        lang = cfg->get_attribute<std::string>(node,"lang");
        lang = downcase(lang);
    }
    catch ( ConfigException &e)
    {
        lang = "c++";
    }

    if ( lang=="c++")
    {
        Hook hook = load_symbol(symbol, lib);
        if (hook)
        {
            ModuleParser *m = hook();
            if (m)
                cfg->register_handler(module, m);
        }
    }
    else
    {
#if defined(HAS_PYTHON) && defined(HAS_PYDAQPP)
        ModuleParser *m = PythonLoader::getInstance().create_module_parser(name, lib, symbol);
        if (m)
        {
            cfg->register_handler(module, m);
        }
        else 
        {
            std::cout << "Could not create the module parser for " << name << "::" << module << std::endl;
        }
#else
        std::cout << "Python modules not allowed" << std::endl;
#endif
    }
    
}

void PluginParser::parse_interrupt(xmlNodePtr nint)throw ( ConfigException )
{
    ConfigParser *cfg = ConfigParser::theConfig();
    std::string lib, symbol;

    lib = cfg->get_value<std::string>(nint, "lib");
    try
    {
        symbol = cfg->get_value<std::string>(nint, "enable");
        Hook en = load_symbol(symbol, lib);
        if (en)
            trigger_enable = (Hook::IntFunc)en;
    }
    catch ( ConfigException &e )
    {}

    try
    {
        symbol = cfg->get_value<std::string>(nint, "disable");
        Hook dis = load_symbol(symbol, lib);
        if (dis)
            trigger_disable = (Hook::VoidFunc)dis;
    }
    catch ( ConfigException &e )
    {}

    try
    {
        symbol = cfg->get_value<std::string>(nint, "reset");
        Hook res = load_symbol(symbol, lib);
        if (res)
            trigger_reset = (Hook::VoidFunc)res;
    }
    catch ( ConfigException &e )
    {}

    try
    {
        symbol = cfg->get_value<std::string>(nint, "start");
        Hook start = load_symbol(symbol, lib);
        if (start)
            trigger_start = (Hook::VoidFunc)start;
    }
    catch ( ConfigException &e )
    {}

    try
    {
        symbol = cfg->get_value<std::string>(nint, "stop");
        Hook stopit = load_symbol(symbol, lib);
        if (stopit)
            trigger_stop = (Hook::VoidFunc)stopit;
    }
    catch ( ConfigException &e )
    {}
}

void PluginParser::parse_run_manager(xmlNodePtr node_inp) throw ( ConfigException )
{
    ConfigParser *cfg = ConfigParser::theConfig();
    std::string lib, symbol;
    // There should only be one. Loop on the children
    xmlNode *node = node_inp->xmlChildrenNode;
    for (;node;node = node->next)
    {
        if ( node->type != XML_ELEMENT_NODE )
            continue;

        std::string name = (char *)node->name;
        lib = cfg->get_value<std::string>(node, "lib");
        symbol = cfg->get_value<std::string>(node, "hook");

        Hook func = load_symbol(symbol, lib);
        if (!func)
        {
            std::cout << "*** Problems loading hook for RunManager " << name << std::endl;
            continue;
        }
        run_manager_factory[name] = (RunManagerCreator)func;
        if (name == "data")
        {
            run_manager_factory["DataRunManager"] = (RunManagerCreator)func;
        }
        else if (name == "int_trg")
        {
            run_manager_factory["IntTrgRunManager"] = (RunManagerCreator)func;
        }
        else if (name == "pedestal")
        {
            run_manager_factory["PedestalRunManager"] = (RunManagerCreator)func;
        }
        else if (name == "pulse")
        {
            run_manager_factory["PulseRunManager"] = (RunManagerCreator)func;
        }
        else if (name == "scan")
        {
            run_manager_factory["ScanRunManager"] = (RunManagerCreator)func;
        }
        else
        {
            std::cout << "*** RunManager of type: " << name << " cannot be handled ***"
            << std::endl;
        }
    }

}

DAQpp::FileIOManager *PluginParser::create_logger(bool cmp, int sz)
{
    if (data_logger)
    {
        DAQpp::FileIOManager *D = (*data_logger)(cmp, sz);
        return D;
    }
    return 0;
}

DAQpp::RunManager * PluginParser::create_run_manager(const std::string &type,
                                                     const std::string &name)
{
    RunManagerFactory::const_iterator ip = run_manager_factory.find(type);
    if (ip == run_manager_factory.end())
    {
        // Not there, return the default
        return Rmanager::new_manager(type, name);
    }
    else
    {
        return (*ip->second)(type, name);
    }
}


void *PluginParser::load_library(const std::string &inp_lib)
    throw (PluginException)
{
    std::string lib = inp_lib;
    void *handle = 0;

    // No lib given, load the main program
    if (lib.empty())
        lib = "__main__";

    std::cout<<"Loading "<<lib<<std::endl;
 // Check if the library is already loaded and if not,
    // load it.
    LibMap::iterator ip = libmap.find(lib);
    if (ip == libmap.end())
    {
        if (lib == "__main__")
            handle = dlopen(0, RTLD_LAZY | RTLD_GLOBAL);
        else
        {
            // Loop on all the paths in the search path
            std::string path = PathConfig::find_library(lib);
            if (path.empty())
            {
                // It is, may be in the LD_LIBRARY_PATH
                handle = dlopen(lib.c_str(), RTLD_LAZY | RTLD_GLOBAL);
            }
            else
                handle = dlopen(path.c_str(),RTLD_LAZY | RTLD_GLOBAL);
        }
        if (!handle)
        {
            std::cout << "Could not load library " << lib << std::endl
                      << "..." << dlerror()
                      << std::endl;
            return 0;
        }
        libmap[lib] = handle;
    }
    else
    {
        handle = ip->second;
    }    
    
    return handle;
}

Hook PluginParser::load_symbol(const std::string &symbol, const std::string &inp_lib)
    throw (PluginException)
{
  std::cout<<"Loading "<<inp_lib<<std::endl;
    void *handle = load_library(inp_lib);
    if (!handle)
        return Hook(0);
    
    // We have the library, now find the symbol
    void *symb = dlsym(handle, symbol.c_str());
    if (!symb)
    {
        std::cout << "Could not find symbol " << symbol << " in library " << inp_lib << std::endl;
        return Hook(0);
    }
    else
        return Hook(symb);    
}

void PluginParser::set_interrups(const std::string &lib, 
                           const std::string &enable,
                           const std::string &disable,
                           const std::string &start,
                           const std::string &stop,
                           const std::string reset)
{
    // find the interrupts node
    ConfigParser *cfg = ConfigParser::theConfig();
    xmlNodePtr nn,child, plg = cfg->get_plugins_node();
    for (nn=0,child = plg->children;child; child=child->next)
    {
        if (!strcmp((char *)child->name,"interrupt"))
        {
            nn = child;
            break;
        }
    }
    if (!nn)
    {
        nn=cfg->create_child(plg,"interrupt","");
    }
    if (lib.empty())
    {
        cfg->remove_child(nn,"lib");
    }
    else
    {
        cfg->check_value<std::string>(nn,"lib",lib);
    }
    if (enable.empty())
    {
        cfg->remove_child(nn,"enable");
    }
    else
    {
        cfg->check_value<std::string>(nn,"enable",enable);
    }
    
    if (disable.empty())
    {
        cfg->remove_child(nn,"disable");
    }
    else
    {
        cfg->check_value<std::string>(nn,"disable",disable);
    }
    
    if (start.empty())
    {
        cfg->remove_child(nn, "start");
    }
    else
    {
        cfg->check_value<std::string>(nn,"start",start);
    }
    
    if ( stop.empty())
    {
        cfg->remove_child(nn,"stop");
    }
    else
    {
        cfg->check_value<std::string>(nn,"stop",stop);
    }
    
    if ( reset.empty() )
    {
        cfg->remove_child(nn,"reset");
    }
    else
    {
        cfg->check_value<std::string>(nn,"reset",reset);
    }
}
        
void PluginParser::set_data_logger(const std::string &lib, const std::string &hook)
{
    // find the datalogger node
    ConfigParser *cfg = ConfigParser::theConfig();
    xmlNodePtr nn,child, plg = cfg->get_plugins_node();
    for (nn=0,child = plg->children;child; child=child->next)
    {
        if (!strcmp((char *)child->name,"data_logger"))
        {
            nn = child;
            break;
        }
    }
    if (!nn)
    {
        nn=cfg->create_child(plg,"data_logger","");
    }
    cfg->check_value<std::string>(nn,"lib", lib);
    cfg->check_value<std::string>(nn,"hook", hook);
}

void PluginParser::set_runmanager(const std::string &type, const std::string &lib, const std::string &hook)
{
    // find the run_manager node
    ConfigParser *cfg = ConfigParser::theConfig();
    xmlNodePtr nn,child, plg = cfg->get_plugins_node();
    for (nn=0,child = plg->children;child; child=child->next)
    {
        if (!strcmp((char *)child->name,"run_managers"))
        {
            nn = child;
            break;
        }
    }
    if (!nn)
    {
        nn=cfg->create_child(plg,"run_managers","");
    }
    
    // Now find the run manager type
    plg = nn;
    for (nn=0, child=plg->children; child; child=child->next)
    {
        if (!strcmp((char *)child->name,type.c_str()))
        {
            nn = child;
            break;
        }
    }
    if (!nn)
    {
        nn=cfg->create_child(plg,type.c_str(),"");
    }
    cfg->check_value<std::string>(nn,"lib",lib);
    cfg->check_value<std::string>(nn,"hook", hook);
}

void PluginParser::set_plugin(const std::string &name, 
                              const std::string &module, 
                              const std::string &lib, 
                              const std::string &hook)
{
    
}


void PluginParser::LibMap::reset_libs()
{
    // unload the libraries
    iterator ip;
    for (ip = begin();ip != end();ip++)
    {
        dlclose( ip->second );
    }
}


Hook::~Hook()
{
    
}
