#include "ScanConfig.h"

void ScanConfig::parse_node(xmlNodePtr _n) 
    throw(ConfigException)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    node = _n;
    xmlNodePtr sc = node->xmlChildrenNode;
    nmap.clear();
    try
    {
        nevts = cfg->get_attribute<int>(node, "nevts");
    }
    catch ( ConfigException &e)
    {
        nevts=100;
    }
    try
    {
        do_burst = cfg->get_attribute<bool>(node, "burst");
    }
    catch ( ConfigException &e )
    {
        do_burst = false;
    }
    try
    {
        _delay = cfg->get_attribute<double>(node, "delay");
    }
    catch ( ConfigException &e )
    {
        _delay = 0.5;
    }
    try
    {
        _rate = cfg->get_attribute<double>(node, "rate");
    }
    catch ( ConfigException &e )
    {
        _rate = 100.;
    }
    for (; sc; sc = sc->next)
    {
        if (sc->type != XML_ELEMENT_NODE )
            continue;

        std::string name = cfg->get_attribute<std::string>(sc, "name");
        nmap[name] = sc;
        scpoint.add_point(name, parse_scan(sc));
    }
}

ScanPoint ScanConfig::parse_scan(xmlNodePtr _n)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    std::string nm, key;
    double from, to, step;
    std::string sc_name = cfg->get_attribute<std::string>(_n, "name");
    ScanPoint scp(sc_name);
    xmlNodePtr var = _n->xmlChildrenNode;
    for (; var; var = var->next)
    {
        if (xmlStrcmp(var->name, (const xmlChar *) "type"))
            continue;

        nm = cfg->get_attribute<std::string>(var, "name");
        from = cfg->get_attribute<double>(var, "from");
        to = cfg->get_attribute<double>(var, "to");
        step = cfg->get_attribute<double>(var, "step");

        key = sc_name + "/"+ nm;
        nmap[key] = var;

        ScanVar::ScanType tp;
        if (nm == "pulse")
        {
            tp = ScanVar::pulse;
        }
        else if (nm == "channel")
        {
            tp = ScanVar::channel;
        }
        else if (nm == "threshold")
        {
            tp = ScanVar::threshold;
        }
        else if (nm == "trim")
        {
            tp = ScanVar::trim;
        }
        else if (nm == "xposition")
        {
            tp = ScanVar::xposition;
        }
        else if (nm == "yposition")
        {
            tp = ScanVar::yposition;
        }
        else if (nm == "zposition")
        {
            tp = ScanVar::zposition;
        }
        else if (nm == "angle1")
        {
            tp = ScanVar::angle1;
        }
        else if (nm == "angle2")
        {
            tp = ScanVar::angle2;
        }
        else if (nm == "angle3")
        {
            tp = ScanVar::angle3;
        }
        else if (nm == "user1")
        {
            tp = ScanVar::user1;
        }
        else if (nm == "user2")
        {
            tp = ScanVar::user2;
        }
        else if (nm == "user3")
        {
            tp = ScanVar::user3;
        }
        else if (nm == "user4")
        {
            tp = ScanVar::user4;
        }
	else if (nm=="thrs_trim")
	{
	   tp = ScanVar::thrs_trim;
	}
	else if (nm=="cal_dac")
	{
	   tp = ScanVar::cal_dac;
	}
        ScanVar var(tp, from, to, step, 0.);
        scp.add_var(var );
    }
    return scp;
}
void ScanConfig::save_config() throw(ConfigException)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    node = cfg->get_scans_node();
    if (node == 0)
        return;

    cfg->check_attribute<int>(node, "nevts", nevts);
    cfg->check_attribute<bool>(node, "burst", do_burst);
    cfg->check_attribute<double>(node, "delay", _delay);
    cfg->check_attribute<double>(node, "rate", _rate);

    NodeMap new_scans;
    std::map<std::string, ScanPoint>::iterator ip;
    for (ip = scpoint.begin(); ip != scpoint.end(); ip++)
    {
        std::cout << "Scan: "<< ip->first<< std::endl;
        NodeMap::iterator inm = nmap.find(ip->first);
        if (inm == nmap.end() )
        {
            std::cout << "Scan "<< ip->first<< " is new ["
                    << ip->second.get_name() << "]"<< std::endl;
            xmlNodePtr scan = xmlNewChild(node, NULL, (xmlChar *)"scan", NULL);
            cfg->set_attribute<std::string>(scan, "name", ip->first);
            new_scans[ip->first] = scan;
            new_scan(scan, ip->second, new_scans);
        }
        save_scan(ip->second);
    }
    nmap.insert(new_scans.begin(), new_scans.end());
}

void ScanConfig::new_scan(xmlNodePtr scan, ScanPoint &scp, NodeMap &new_scans)
{
    ConfigParser *cfg = ConfigParser::theConfig();

    std::vector<ScanVar>::iterator ip;
    for (ip = scp.begin(); ip != scp.end(); ip++)
    {
        ScanVar &var = *ip;
        std::string nm;
        switch (var.type())
        {
        case ScanVar::pulse:
            nm = "pulse";
            break;
        case ScanVar::channel:
            nm = "channel";
            break;
        case ScanVar::threshold:
            nm = "threshold";
            break;
        case ScanVar::trim:
            nm = "trim";
            break;
        case ScanVar::xposition:
            nm = "xposition";
            break;
        case ScanVar::yposition:
            nm = "yposition";
            break;
        case ScanVar::zposition:
            nm = "zposition";
            break;
        case ScanVar::angle1:
            nm = "angle1";
            break;
        case ScanVar::angle2:
            nm = "angle2";
            break;
        case ScanVar::angle3:
            nm = "angle3";
            break;
        case ScanVar::user1:
            nm = "user1";
            break;
        case ScanVar::user2:
            nm = "user2";
            break;
        case ScanVar::user3:
            nm = "user3";
            break;
        case ScanVar::user4:
            nm = "user4";
            break;
	   case ScanVar::thrs_trim:
	      nm="thrs_trim";
	      break;
	   case ScanVar::cal_dac:
	      nm="cal_dac";
	      break;
        }
        xmlNodePtr n = xmlNewChild(scan, NULL, (xmlChar *)"type", NULL);
        std::string name = scp.get_name() + "/"+ nm;
        double tov = var.first() + var.n_points() * var.step();

        new_scans[name] = n;
        cfg->set_attribute<std::string>(n, "name", nm);
        cfg->set_attribute<double>(n, "from", var.first());
        cfg->set_attribute<double>(n, "to", tov);
        cfg->set_attribute<double>(n, "step", var.step());
    }

}

void ScanConfig::save_scan(ScanPoint &scp)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    std::vector<ScanVar>::iterator ip;
    for (ip = scp.begin(); ip != scp.end(); ip++)
    {
        ScanVar &var = *ip;
        std::string nm;
        switch (var.type())
        {
        case ScanVar::pulse:
            nm = "pulse";
            break;
        case ScanVar::channel:
            nm = "channel";
            break;
        case ScanVar::threshold:
            nm = "threshold";
            break;
        case ScanVar::trim:
            nm = "trim";
            break;
        case ScanVar::xposition:
            nm = "xposition";
            break;
        case ScanVar::yposition:
            nm = "yposition";
            break;
        case ScanVar::zposition:
            nm = "zposition";
            break;
        case ScanVar::angle1:
            nm = "angle1";
            break;
        case ScanVar::angle2:
            nm = "angle2";
            break;
        case ScanVar::angle3:
            nm = "angle3";
            break;
        case ScanVar::user1:
            nm = "user1";
            break;
        case ScanVar::user2:
            nm = "user2";
            break;
        case ScanVar::user3:
            nm = "user3";
            break;
        case ScanVar::user4:
            nm = "user4";
            break;
	   case ScanVar::thrs_trim:
	      nm = "thrs_trim";
	      break;
	   case ScanVar::cal_dac:
	      nm = "cal_dac";
	      break;
        }
        std::string name = scp.get_name() + "/"+ nm;
        double tov = var.first() + var.n_points() * var.step();

        xmlNodePtr n = nmap[name];
        cfg->check_attribute<std::string>(n, "name", nm);
        cfg->check_attribute<double>(n, "from", var.first());
        cfg->check_attribute<double>(n, "to", tov);
        cfg->check_attribute<double>(n, "step", var.step());
    }
}

std::ostream &operator<<(std::ostream &os, const ScanConfig &p)
{

    os << "Number of events per scan point: "<< p.nevts<< std::endl;
    std::map<std::string, ScanPoint>::const_iterator ip;
    for (ip = p.scpoint.begin(); ip != p.scpoint.end(); ip++)
    {
        os << "Scan "<< ip->second.get_name()<< " Variables:"<< std::endl;
        os << ip->second<< std::endl;
    }
    return os;
}
