/* -*- mode: c++ -*- */
#ifndef __ScanConfig_h__
#define __ScanConfig_h__

#include <map>
#include <config/ConfigParser.h>
#include <daq/ScanPoint.h>

class ScanConfig;
std::ostream &operator<<(std::ostream &os, const ScanConfig &c);
class ScanConfig: public SectionParser
{
private:
    int  nevts; // number of events per scan point
    bool do_burst; // true if pulses are sent in bursts, fasle otherwise
    bool _time; // if true interpret nevts as time in seconds
    double _delay;
    double _rate;
    ScanList scpoint;

public:
    ScanConfig() : nevts(100), do_burst(true), _time(false)
    {}
    ~ScanConfig()
    {}

    int get_nevts() const
    {
        return nevts;
    }
    void set_nevts(int x)
    {
        nevts = x;
    }

    bool burst() const
    {
        return do_burst;
    }
    void burst(bool x)
    {
        do_burst = x;
    }

    double delay() const
    {
        return _delay;
    }
    void delay(double x)
    {
        _delay = x;
    }

    bool is_time() const
    {
        return _time;
    }
    void is_time(bool x)
    {
        _time = x;
    }

    double rate() const
    {
        return _rate;
    }
    void rate(double x)
    {
        _rate = x;
    }

    friend std::ostream &operator<<(std::ostream &os, const ScanConfig &c);

    void parse_node(xmlNodePtr )
       throw (ConfigException);
    ScanPoint parse_scan(xmlNodePtr );
    void save_config() throw (ConfigException);
    void save_scan(ScanPoint & );
    void new_scan(xmlNodePtr node, ScanPoint &scp, NodeMap &mnew);

    ScanPoint *get_scan(const std::string &name)
    {
        ScanList::iterator ip = scpoint.find(name);
        if (ip == scpoint.end())
            return 0;
        else
            return &ip->second;
    }
    void add_scan(const std::string &name, const ScanPoint &p)
    {
        scpoint.add_point(name, p);
    }
    void remove_scan(const std::string &name)
    {
        scpoint.erase(name);
    }
    int size()
    {
        return scpoint.size();
    }
    ScanList::iterator end()
    {
        return scpoint.end();
    }
    ScanList::iterator begin()
    {
        return scpoint.begin();
    }
    ScanList::const_iterator end() const
    {
        return scpoint.end();
    }
    ScanList::const_iterator begin() const
    {
        return scpoint.begin();
    }
    const ScanList &get_scan_list() const
    {
        return scpoint;
    }
    void set_scan_list(const ScanList &s)
    {
        scpoint = s;
    }
};

#endif
