#include <iostream>
#include <sstream>
#include <map>
#include <iterator>
#include <cstdlib>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <limits.h>
#include "SearchPath.h"
#include "config_utils.h"

//#define __DEBUG__
#if defined(__DEBUG__)
#define WHOAMI(str) std::cout << "*** SearchPath::" << __FUNCTION__ << ":" << (str) << ":" << std::endl;
#else
#define WHOAMI(str)
#endif

SearchPath::SearchPath(const std::string &p)
{
    WHOAMI("Constructor");
    // Add the <null> and the current directory in the path list
    add_path(p);
    
    SearchPath::iterator ip;
    bool add_null=true, add_cwd=true;
    char buf[PATH_MAX];
    std::string cwd = getcwd(buf, PATH_MAX);
    for (ip=begin(); ip!=end(); ip++)
    {
        if ( ip->empty() )
            add_null = false;
        
        if ( *ip == "." || *ip==cwd)
            add_cwd = false;
    }
    if (add_cwd)
        prepend_path(".");
    
    if (add_null)
        prepend_path("");
    
    purge();
}

void SearchPath::cpy(const SearchPath &p)
{
    WHOAMI("");
    clear();
    insert(begin(), p.begin(), p.end());
}

SearchPath::SearchPath(const SearchPath &P)
{
    WHOAMI("copy constructor");
    cpy(P);
}
SearchPath &SearchPath::operator=(const SearchPath &P)
{
    WHOAMI("assignment operator");
    if (&P!=this)
        cpy(P);


    return *this;
}


SearchPath::~SearchPath()
{
    WHOAMI("destuctor");
}

bool SearchPath::has_path(const std::string &path) const
{
    return (_paths.find(path)!=_paths.end());
}

void SearchPath::add_path(const std::string &p)
{
    WHOAMI("");
#ifdef __DEBUG__
    std::cout << "Before adding " << p << std::endl
              << *this << std::endl;
#endif
    char buf[512];
    std::istringstream path(p);
    while (path)
    {
        path.getline(buf, sizeof(buf), ':');
        if (*buf)
        {
            std::string path = trim_str(buf);
            if ( _paths.insert(std::pair<std::string, int>(path,0)).second)
            {
                push_back( trim_str(buf) );
            }
        }
    }

#ifdef __DEBUG__
    std::cout << "After adding " << p << std::endl
              << *this << std::endl;
#endif
}
void SearchPath::add_path(const SearchPath &p)
{
    WHOAMI("");
#ifdef __DEBUG__
    std::cout << "Before adding (\n" << p << "\n)" <<std::endl
              << *this << std::endl;
#endif
    const_iterator ip;
    for (ip=p.begin(); ip!=p.end();ip++)
    {
        add_path(*ip);
    }

#ifdef __DEBUG__
    std::cout << "After adding (\n" << p << "\n)" <<std::endl
              << *this << std::endl;
#endif
}

void SearchPath::prepend_path(const std::string &p)
{
    WHOAMI("");
#ifdef __DEBUG__
    std::cout << "Before prepending " << p << std::endl
              << *this << std::endl;
#endif
    std::vector<std::string> p0;
    char buf[512];
    std::istringstream path(p);
    while (path)
    {
        path.getline(buf, sizeof(buf), ':');
        if (*buf)
        {
            std::string path = trim_str(buf);
            _paths[path] = 0;
            p0.push_back( path );
        }
    }
    
    insert(begin(), p0.begin(), p0.end());
#ifdef __DEBUG__
    std::cout << "After prepending " << p << std::endl
              << *this << std::endl;
#endif
}
void SearchPath::prepend_path(const SearchPath &p)
{
    WHOAMI("");
#ifdef __DEBUG__
    std::cout << "Before prepending (\n" << p << "\n)" <<std::endl
              << *this << std::endl;
#endif
    
    const_iterator ip;
    for (ip=p.begin(); ip!=p.end();++ip)
        _paths[*ip] = 0;
    
    insert(begin(), p.begin(), p.end());
    
#ifdef __DEBUG__
    std::cout << "After prepending (\n" << p << "\n)" <<std::endl
              << *this << std::endl;
#endif
}

void SearchPath::purge()
{
    WHOAMI("");
    std::map<std::string, SearchPath::iterator> index;
    std::map<std::string, SearchPath::iterator>::iterator jp;
    std::vector<SearchPath::iterator> vremove;
    std::vector<SearchPath::iterator>::iterator jrm;

#ifdef __DEBUG__
    std::cout << "Before purging:" << std::endl
              << *this << std::endl
              << "---------------------------"
              << std::endl;
#endif
    SearchPath::iterator ip;
    for (ip=begin();ip!=end();ip++)
    {
        if (index.empty())
        {
            index[*ip] = ip;
            continue;
        }
        
        jp = index.find( *ip );
        if (jp!=index.end())
        {
            vremove.push_back(ip);
        }
        else
            index[ *ip ] = ip;
    }
    
    // remove repeated instances
    for ( jrm=vremove.begin(); jrm!=vremove.end(); jrm++)
        erase( *jrm );

#ifdef __DEBUG__
    std::cout << "After purging:" << std::endl
              << *this << std::endl
              << "---------------------------"
              << std::endl;
#endif
}

std::ostream &operator<<(std::ostream &os, const SearchPath &P)
{
    std::copy(P.begin(),P.end(),std::ostream_iterator<std::string>(os,"\n"));
    return os;
}

bool SearchPath::is_file(const std::string &file)
{
    struct stat buf;
    if (stat(file.c_str(), &buf) != 0)
        return false;
    
    if (!S_ISREG(buf.st_mode))
        return false;
    
    return true;
}


std::string SearchPath::find_file(const std::string &fnam)
{
    if ( is_file(fnam) )
        return fnam;
    
    iterator ip;
    std::string rc;
    for (ip=begin(); ip!=end(); ip++)
    {
        std::string abspath = *ip + std::string("/") + fnam;
        if (is_file(abspath))
        {
            rc = abspath;
            break;
        }
    }
    return rc;
}
