#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <DAQ++/DAQmanager.h>
#include <daq/PedestalRunManager.h>
#include <daq/PulseRunManager.h>
#include <daq/DataRunManager.h>
#include <daq/Rmanager.h>
#include <daq/GenModule.h>
#include <daq/DAQcontroller.h>
#include <gui/MainWindow.h>
#include <gui/RunMenu.h>
#include <config/Config.h>


#if defined(HAS_DAQcorba)
#include <daq/CORBAServer.h>
#include <daq/DummyModule.h>
#else
class MyServer
{
    public:
        MyServer() {}
        ~MyServer() {}
};
#endif


DAQcontroller DAQcontroller::theInstance;

DAQcontroller::DAQcontroller()
{

}

DAQcontroller::~DAQcontroller()
{

}

Rmanager *DAQcontroller::get_run_manager()
{
    DAQpp::DAQmanager *daq_manager
            = DAQpp::DAQmanager::theDAQmanager();
    return (Rmanager *)daq_manager->find_runmanager("main");
}

void DAQcontroller::set_main_window(MainWindow *w)
{
    wnd = w;
    if (!wnd)
        return;

    DAQpp::DAQmanager *daq_manager
        = DAQpp::DAQmanager::theDAQmanager();
#if defined(HAS_DAQcorba)
    // Set the Mainwindow pointer in the dummy module
    DummyModule *dummy = dynamic_cast<DummyModule *>(
                             daq_manager->find_module("VMEDAQconfig"));

    if ( dummy )
        dummy->main_window( wnd );
#endif
    DAQpp::ModulesMap::const_iterator mm;
    std::vector<std::string> ids;
    for (mm = daq_manager->ModuleMap().begin(); mm != daq_manager->ModuleMap().end(); ++mm)
    {
        if (dynamic_cast<GenModule *>(mm->second))
            ids.push_back(mm->second->get_id() );
    }
    if (wnd)
    {
        // The main RunMenu
        RunMenu::find("main")->update( RunMenu::translate( get_run_manager()->GetStatus().get_status()) );
        wnd->set_module_list(ids);
    }
}

void DAQcontroller::setup_daq(int _argc, char **_argv)
{
    int iarg;

    DAQpp::DAQmanager *daq_manager
    = DAQpp::DAQmanager::theDAQmanager();

    // Create the main RunManager
    ConfigParser *cfg = ConfigParser::theConfig();
    GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));
    Rmanager *run_manager = Rmanager::create_run_manager("data","main");
    DAQpp::ModulesMap::const_iterator mm;
    for (mm = daq_manager->ModuleMap().begin(); mm != daq_manager->ModuleMap().end(); ++mm)
    {
        run_manager->addModule(mm->second);

        GenModule *md = dynamic_cast<GenModule *>(mm->second);
        if (md)
        {
            md->get_analysis()->DoPedestalUpdate(gnrl->get_ped_update() );
        }
    }

    run_manager->set_max_events(0);

    // start the DAQ loop
    DAQpp::DAQmanager::theDAQmanager()->StartRun(DAQpp::DAQmanager::Loop);

//    if (_argc==0 && wnd)
//        set_main_window(wnd);

#if defined(HAS_DAQcorba)
    if (_argc>0)
    {
        //---------------------------------------------------------------------
        // DAQcorba stuff
        //---------------------------------------------------------------------
        ior_loc = std::string( getenv("HOME") ) + "/WWW/vmedaq.ior";
        for (iarg=0;iarg<_argc;iarg++)
        {
            if ( !strcmp(_argv[iarg],"ior") )
                ior_loc = _argv[iarg];
        }
        corba_server = new MyServer();
        setup_corba(_argc, _argv, *corba_server, ior_loc);
    }
#endif

}

void DAQcontroller::reset_daq()
{
    std::cout << "Reseting DAQ..." << std::endl;
    const DAQpp::ModulesMap &mmap =
        DAQpp::DAQmanager::theDAQmanager()->ModuleMap();

    // First the RunManagers
    const DAQpp::RunManagerMap &rmap =
        DAQpp::DAQmanager::theDAQmanager()->RunMmap();

    DAQpp::RunManagerMap::const_iterator ir;
    std::vector<std::string> ids;
    for (ir=rmap.begin();ir!=rmap.end();++ir)
    {
        ids.push_back( ir->second->get_id() );
    }

    std::vector<std::string>::iterator jp;
    for (jp=ids.begin();jp!=ids.end();jp++)
    {
        // Free the modules attached to this run manager
        DAQpp::RunManager *rm = DAQpp::DAQmanager::theDAQmanager()->find_runmanager(*jp);
        std::cout << "Clean up RunManager " << rm->get_id() << std::endl;
        std::cout << "...Number of modules: " << rm->countModules() << std::endl;
        //rm->for_each_module( free_modules(rm) );
        delete rm;
    }

    DAQpp::ModulesMap::const_iterator im;
    ids.clear();
    for (im=mmap.begin();im!=mmap.end();++im)
    {
        if (dynamic_cast<GenModule *>(im->second))
            ids.push_back(im->second->get_id());
    }
    for (jp=ids.begin();jp!=ids.end();jp++)
    {
        DAQpp::Module *md = DAQpp::DAQmanager::theDAQmanager()->find_module(*jp);
        delete md;
    }
    if (wnd)
        wnd->clear_modules();

    DAQpp::DAQmanager::theDAQmanager()->StopRun();
}

void DAQcontroller::stop_daq()
{
    std::cout << "Shuting down the DAQ" << std::endl;
    reset_daq();
#if defined(HAS_DAQcorba)
    std::cout << "Shuting down the CORBA server " << std::endl;
    corba_server->shutdown();
    unlink(ior_loc.c_str());
#endif
}
