#include <ctime>
#include <unistd.h>
#include "DataRunManager.h"
#include "GenModule.h"
#include "../data/DataFormat.h"
#include <config/Config.h>


DataRunManager::DataRunManager(DAQpp::DAQid id) : 
    Rmanager(id), tmped(0.)
{
    run_type = Data;
}

DataRunManager::~DataRunManager()
{
    std::cout << "...destroying DataRunManager " << get_id() << std::endl;
}

int  DataRunManager::writeData(int size, const char *data)
{
    unsigned int *dt =
        const_cast<unsigned int *>((const unsigned int *)data);

    dt[0] |= DataRecord;
    dt[1]  = (dt[1] & 0xff000000) | (get_event_number() % 0xffffff);

    return Rmanager::writeData(size, data);
}

void DataRunManager::EndOfEvent()
{
    ConfigParser *cfg = ConfigParser::theConfig();
    GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));
    
    // Call the general one
    Rmanager::EndOfEvent();

    // write pedestals periodically
    if (GetStatus().time() >= tmped)
    {
        if ( get_logger() )
        {
            writePedestals(obuf);
            logger->write(obuf.str().c_str(), obuf.str().size());
            obuf.str("");
        }
        tmped = GetStatus().time() + gnrl->get_ped_period();
    }
}


void DataRunManager::PrepareForRun()
{
    ConfigParser *cfg = ConfigParser::theConfig();
    GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));
    tmped = gnrl->get_ped_period();

    Rmanager::PrepareForRun();
}


