/* -*- mode: c++ -*- */
#ifndef __DataRunManager_h__
#define __DataRunManager_h__

#include <daq/Rmanager.h>

/** \ingroup DAQ
 * 
 * RunManager for normal data taking.
 * 
 * The only methods that it overrides are PrepareForRund, which will
 * tell the modules this is a Data run, writeData to set the data 
 * type in the data header and EndOfEvent, that will check if the time
 * has come to dump the pedestals into the data file. 
 *   
 */
class DataRunManager : public Rmanager
{
    private:
        /// time interval to write pedestals
        double tmped;
    public:
        // Constructor.
        DataRunManager(DAQpp::DAQid id);
        
        /// Destructor.
        ~DataRunManager();

        /**
         * Prepares the modules to run. It just tells them the 
         * run type
         */
        void PrepareForRun();
        
        /**
         * Writes the data.
         * It just intercepts the data block and sets the data type
         * to DataRecord. Then it delegates to Rmanager::writeData
         */
        int writeData(int size, const char *data);
        
        /**
         * Called at the end of each event. This method will dump the
         * pedestals into the data file every tmped seconds.
         */
        void EndOfEvent();
};

#endif

