#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <DAQ++/DAQmanager.h>
#include "DummyModule.h"
#include <gui/MainWindow.h>
#include <gui/RunMenu.h>
#include <daq/Rmanager.h>
#include <config/ConfigParser.h>
#include <config/GeneralConfig.h>
#include <config/IOConfig.h>
#include <config/PathConfig.h>

#if defined(HAS_DAQcorba)
#include <DAQcorba/Server.h>
#include <DAQcorba/DAQ-utils.h>
#include <DAQcorba/MsgManager.h>
#endif

DummyModule::DummyModule(DAQpp::DAQid const id, MainWindow *ptr)
        : Module(id), mw(ptr), the_spy(this)
{
    add_par("IO/run_number", new DAQpp::Par< int >(0));
    add_par("IO/compression", new DAQpp::Par< bool >(true));
    add_par("IO/prefix", new DAQpp::Par< std::string >(""));
    add_par("DAQ/trigger_type", new DAQpp::Par< std::string >("") );
    add_par("DAQ/pedestal_run", new DAQpp::Par< int >(0) );
    add_par("DAQ/log_data", new DAQpp::Par< bool >(false) );
    add_par("DAQ/interrupt", new DAQpp::Par< std::string >(""));
    add_par("DAQ/maxevents", new DAQpp::Par< int >(-1));
    add_par("DAQ/Start", new DAQpp::Par< int >(0) );
    add_par("DAQ/Stop", new DAQpp::Par< int >(0) );
    add_par("sync", new DAQpp::Par< int >(0) );
    sync();
}

void DummyModule::sync()
{
    DAQpp::DAQmanager *daq_manager
    = DAQpp::DAQmanager::theDAQmanager();
    ConfigParser *cfg = ConfigParser::theConfig();
    IOConfig *io = dynamic_cast<IOConfig *>(cfg->get_handler("IO"));
    GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));
    Rmanager *rm = dynamic_cast<Rmanager *>(daq_manager->find_runmanager("main"));

    set_par<int>("IO/run_number", io->get_run_number());
    set_par<bool>("IO/compression", io->get_compression());
    set_par<std::string>("IO/prefix", io->get_prefix());
    set_par<int>("DAQ/pedestal_run", gnrl->get_ped_nevts());
    if ( mw )
    {
        if ( main_window()->btn_active("swTriggerType") )
        {
            set_par<std::string>("DAQ/trigger_type", "external");
        }
        else
        {
            set_par<std::string>("DAQ/trigger_type", "internal");
        }
        if ( main_window()->btn_active("swInterrupt") )
        {
            set_par<std::string>("DAQ/interrupt", "poll");
        }
        else
        {
            set_par<std::string>("DAQ/interrupt", "interrupt");
        }
    }
    if (rm)
    {
        set_par<int>("DAQ/maxevents", rm->get_max_events());
    }
}


/**
 * Called when the trigger type property has changed
 */
void DummyModule::Spy::change_trigger_type(const std::string &type)
{
    if ( !md->main_window() )
        return;

    if ( type == "external" )
    {
        md->main_window()->activate_button("swTriggerType", true);
    }
    else if ( type == "internal" )
    {
        md->main_window()->activate_button("swTriggerInternal", true);
    }
}

/**
 * Called when the interrupt type property has changed
 */
void DummyModule::Spy::change_interrupt(const std::string &type)
{
    if ( !md->main_window() )
        return;

    if ( type == "poll" )
    {
        md->main_window()->activate_button("swInterrupt", true);
    }
    else if ( type == "interrupt" )
    {
        md->main_window()->activate_button("swInterruptInt", true);
    }
}

/**
 * Called when a pedestal run is required
 */
void DummyModule::Spy::do_pedestal(int nevt)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));

    if (nevt <= 0)
        return;

    gnrl->set_ped_nevts( nevt );
    md->main_window()->click_button("btnPedestal");
}
void DummyModule::Spy::update(Observable *o, Argument *arg)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    IOConfig *io = dynamic_cast<IOConfig *>(cfg->get_handler("IO"));
    DAQpp::Parameter *par = dynamic_cast<DAQpp::Parameter *>(o);
    std::string par_name = par->get_name();

    DAQpp::DAQmanager *daq_manager
    = DAQpp::DAQmanager::theDAQmanager();
    Rmanager *rm = dynamic_cast<Rmanager *>(daq_manager->find_runmanager("main"));

    if (par_name == "IO/run_number")
    {
        Arg<int> *v = static_cast< Arg<int> * > (arg);
        if (io)
            io->set_run_number( v->get_value() );
    }
    else if ( par_name == "IO/compression" )
    {
        Arg<bool> *v = static_cast< Arg<bool> * > (arg);
        if (io)
            io->set_compression( v->get_value() );
    }
    else if ( par_name == "IO/prefix" )
    {
        Arg<std::string> *v = static_cast< Arg<std::string> * > (arg);
        if (io)
            io->set_prefix( v->get_value() );
    }
    else if ( par_name == "DAQ/trigger_type" )
    {
        Arg<std::string> *v = static_cast< Arg<std::string> * > (arg);
        change_trigger_type( v->get_value() );
    }
    else if ( par_name == "DAQ/interrupt" )
    {
        Arg<std::string> *v = static_cast< Arg<std::string> * > (arg);
        change_interrupt( v->get_value() );
    }
    else if ( par_name == "DAQ/pedestal_run" )
    {
        Arg<int> *v = static_cast< Arg<int> * > (arg);
        do_pedestal( v->get_value() );
    }
    else if ( par_name == "DAQ/Start" )
    {
        Arg<int> *v = static_cast< Arg<int> * > (arg);
        RunMenu *menu = RunMenu::find("main");
        if (menu)
        {
            std::cout << "Starting a run" << std::endl;
            rm->set_max_events( v->get_value() );
            menu->send_command( run );
        }
    }
    else if ( par_name == "DAQ/Stop" )
    {
        RunMenu *rm = RunMenu::find("main");
        if (rm)
        {
            std::cout << "Stopping the run" << std::endl;
            rm->send_command( stop );
        }
    }
    else if ( par_name == "DAQ/log_data" )
    {
        Arg<bool> *v = static_cast< Arg<bool> * > (arg);
        md->main_window()->activate_button("swLogData", v->get_value());
    }
    else if ( par_name == "maxevents")
    {
        Arg<int> *v = static_cast< Arg<int> * > (arg);
        md->main_window()->int2entry("txtNoOfEvents", v->get_value());
        rm->set_max_events( v->get_value() );
    }
    else if ( par_name == "sync" )
    {
        md->sync();
    }
}

