#include <unistd.h>
#include <DAQ++/DAQmanager.h>
#include "IntTrgRunManager.h"
#include "GenModule.h"
#include "../data/Analysis.h"
#include "../data/DataFormat.h"

IntTrgRunManager::IntTrgRunManager(DAQpp::DAQid id) : 
    Rmanager(id)
{
    run_type = IntTrg;
    
    /** No monitor since we call Analysis::Fill in writeData
     */
    set_monitor(0);
}

IntTrgRunManager::~IntTrgRunManager()
{
    std::cout << "...destroying IntTrgRunManager " << get_id() << std::endl;
}

void IntTrgRunManager::Trigger()
{
    ObserverList::iterator ip;
    Observable &O = *(*this)();
    for (ip=O.begin();ip!=O.end();ip++)
    {
        GenModule *m = dynamic_cast<GenModule *>( *ip );
        if ( m )
        {
            m->Reset();
            m->Trigger();
        }
    }
}

int  IntTrgRunManager::writeData(int size, const char *dt)
{
    int rc = 0;
    // get the module id
    GenModule *module;
    unsigned int *data = reinterpret_cast<unsigned int *>(
                              const_cast<char *>(dt)
                          );
    DAQpp::DAQid id = (unsigned long)((data[2] & 0xfff00000) >> 20);
    data[1] =  (data[1] & 0xff000000) | (get_event_number() % 0xffffff);
    DAQpp::Module *mm = DAQpp::DAQmanager::theDAQmanager()->find_module(id);
    if ( mm )
    {
        module = dynamic_cast<GenModule *>(mm);
        if ( module )
        {
            module->get_analysis()->Fill(const_cast<unsigned int *>(data));
            if ( module->get_analysis()->max_pulse() > 3.5 )
            {
                data[0] |= DataRecord;
                data[2] |= module->get_data_type();
                obuf.write(dt, size);
                rc = 1;
            }
        }
    }
    return rc;
}




