#include <unistd.h>
#include <DAQ++/DAQmanager.h>
#include "PedestalRunManager.h"
#include "GenModule.h"
#include "../data/Analysis.h"


PedestalRunManager::PedestalRunManager(DAQpp::DAQid id) : 
    IntTrgRunManager(id)
{
    run_type = Pedestal;

    /** No monitor since we call Analysis::Fill in writeData
     */
    set_monitor(0);
    
    // No data logging
    stop_logging();
    

}

PedestalRunManager::~PedestalRunManager()
{
    std::cout << "...destroying PedestalRunManager " << get_id() << std::endl;
}



int PedestalRunManager::writeData(int size, const char *dt)
{
    int rc = 0;
    
    /**
     * We call here the Analysis::Fill method of the corresponding
     * data block.
     * 
     */
    // get the module id
    GenModule *module;
    unsigned int *data = reinterpret_cast<unsigned int *>(
                              const_cast<char *>(dt)
                          );
    DAQpp::DAQid id = (unsigned long)((data[2] & 0xfff00000) >> 20);
    data[1] =  (data[1] & 0xff000000) | (get_event_number() % 0xffffff);
    DAQpp::Module *mm = DAQpp::DAQmanager::theDAQmanager()->find_module(id);
    if ( mm )
    {
        module = dynamic_cast<GenModule *>(mm);
        if ( module )
        {
            module->get_analysis()->Fill(const_cast<unsigned int *>(data));
            rc = 1;
        }
    }
    return rc;
}

void PedestalRunManager::PrepareForRun()
{
    IntTrgRunManager::PrepareForRun();

    ObserverList::iterator ip;
    Observable &O = *(*this)();
    for (ip=O.begin();ip!=O.end();ip++)
    {
        GenModule *m = dynamic_cast<GenModule *>( *ip );
        std::cout << "Module "
                  << std::hex << m << std::dec
                  << std::endl;
        if ( m )
        {
            m->reset_monitor_data();
            m->get_analysis()->reset_pedestals();

            // Make sure that Analysis will update the pedestals
            bool val = m->get_analysis()->DoingPedestalUpdate();
            try
            {
                m->set_par("__update_pedestals__", val);
            }
            catch (DAQpp::ParameterNonExistent &ex)
            {
                m->add_par("__update_pedestals__", new DAQpp::Par<bool>(val));
            }
            m->get_analysis()->DoPedestalUpdate(true);
        }        
    }
}

void PedestalRunManager::EndOfRun()
{
    ObserverList::iterator ip;
    Observable &O = *(*this)();
    for (ip=O.begin();ip!=O.end();ip++)
    {
        GenModule *m = dynamic_cast<GenModule *>( *ip );
        if ( m )
        {
            std::cout << "Updating pedestals for " << m->get_id() << std::endl;
            m->update_pedestals();
            m->get_analysis()->DoPedestalUpdate( m->get_par_value<bool>("__update_pedestals__"));
        }
    }
}
