#include <unistd.h>
#include "PulseRunManager.h"
#include "GenModule.h"


PulseRunManager::PulseRunManager(DAQpp::DAQid id, PulseGenerator *pg)
        : Rmanager(id), _do_burst(false)
        , pulse_generator(pg)
{
    run_type = Pulse;
    if (!pg)
        pg = new PulseGenerator();
        
    set_pulse_generator(pg);
}

PulseRunManager::~PulseRunManager()
{
    std::cout << "...destroying PulseRunManager " << get_id() << std::endl;
    delete pulse_generator;
}

void
PulseRunManager::set_pulse_generator( PulseGenerator *pg )
{
    if (pulse_generator )
    {
        delete pulse_generator;
    }
    pulse_generator = pg;
    pulse_generator->Reset();
}

void PulseRunManager::set_pulse(double x)
{
    _pulse = x;
    pulse_generator->set_pulse( get_pulse() );
}

void PulseRunManager::setup_pulse(double x, int nevts)
{
    if (nevts <= 0)
        nevts = 1000;

    int    ncycles     = _do_burst ? nevts : 1;
    double pulse_period = 1. / _rate;
    double pulse_width  = 0.5*pulse_period;
    double Wburst      = ncycles * pulse_period;

    set_pulse(x);
    pulse_generator->disable_output();
    if (_do_burst) 
    {
        pulse_generator->define_burst(_delay, _rate, ncycles, Wburst);
    }
    else
    {
        pulse_generator->define_pulse(pulse_period, pulse_width, _delay);    
    }
    pulse_generator->activate_burst(_do_burst);
    pulse_generator->enable_output();
}


void PulseRunManager::EndOfRun()
{
	pulse_generator->disable_output();
	Rmanager::EndOfRun();
}

void PulseRunManager::send_pulse()
{
        pulse_generator->trigger();
}

void PulseRunManager::set_output(bool x)
{
    if (x)
    {
        pulse_generator->enable_output();
    }
    else 
    {
        pulse_generator->disable_output();   
    }
}

