#ifndef __PulseRunManager_h__
#define __PulseRunManager_h__

#include <daq/Rmanager.h>
#include <daq/PulseGenerator.h>

/**
 * \brief class PulseRunManager.
 * \ingroup DAQ
 * 
 * This class handles a data acquisition mode in which
 * the signal is generated by an external pulse generator.
 * 
 */
class PulseRunManager : public Rmanager
{
    private:
        /// the channel of the pulse generator
        int _channel;

        /// the pulse amplitude
        double _pulse;

        /// the delay of the pulse
        double _delay;

        /// the rate of the pulse
        double _rate;

        /// tells if the pulse generator produces single pulses
        /// or bursts of equal pulses
        bool _do_burst;

        /// a pointer to a pulse generator
        PulseGenerator *pulse_generator;

    public:
        /// Constructor
        PulseRunManager(DAQpp::DAQid id, PulseGenerator *pg=0);

        /// Destructor
        ~PulseRunManager();

        /// Sets the pointer to the pulse generator
        void set_pulse_generator(PulseGenerator *pg);

        void EndOfRun();

        void setup_pulse(double ampl, int nevts);

        void set_pulse(double x);
        double get_pulse() const
        {
            return _pulse;
        }
        void send_pulse();

        void burst(bool x)
        {
            _do_burst = x;
        }
        bool burst() const
        {
            return _do_burst;
        }

        double delay() const
        {
            return _delay;
        }
        void delay(double x)
        {
            _delay = x;
        }

        double rate() const
        {
            return _rate;
        }
        void rate(double x)
        {
            _rate = x;
        }

        void set_channel(int x)
        {
            _channel = x;
        }
        int get_channel() const
        {
            return _channel;
        }

        void set_output(bool x);
};

#endif

