#include "ScanPoint.h"
static std::string __names[] = { "Channel", "Pulse", "Threshold", "Trim",
                                 "xposition", "yposition", "zposition",
                                 "angle1", "angle2", "angle3", "user1",
                                 "user2", "user3", "user4", "thrs_trim", "cal_dac" };

static std::string *__first = __names;
static std::string *__last = __first + sizeof(__names) / sizeof(std::string);
std::vector<std::string> ScanVar::names = std::vector<std::string>(__first, __last);


ScanVar::ScanVar(ScanVar::ScanType tp,
                 double frst, double last, double stp, double dflt)
        : _type(tp), _first(frst), _step(stp),
        _default(dflt), _value(dflt)
{
    adjust(frst, last, stp);
}

void ScanVar::cpy(const ScanVar &s)
{
    _type      = s._type;
    _npoints = s._npoints;
    _first       = s._first;
    _step       = s._step;
    _default = s._default;
    _value    = s._value;
}
void ScanVar::adjust(double first, double last, double step)
{
    if (last < first)
    {
        double tmp = last;
        last = first;
        first = tmp;
    }
    _step = step;
    _npoints = (int)((last - first) / _step) + 1;
    _first = first;
}

std::ostream &ScanVar::dump(std::ostream &os)
{
    os << int(_type) 
        << " " << first() << " " << last()
        << " " << step() << " " << get_default()
        << " " << value()
        << std::endl;
    return os;
}
std::istream &ScanVar::load(std::istream &is)
{
    double type, first, last, step;
    is >> type >> first >> last >> step >> _default >> _value >> std::ws;
    _type = (ScanType)type;
    adjust(first, last, step);
    return is;
}


//////////////////////////////////////////////////////////////////////////
// FillList
//
//   Author: Carlos Lacasta
//   Purpose: Prepares the list of ScanPoints from the list of
//            scan variables
//
//////////////////////////////////////////////////////////////////////////
int ScanPoint::FillList(std::vector<ScanVar>::iterator from,
                        std::vector<ScanVar>::iterator to,
                        ScanPoint &sc,
                        std::vector<ScanPoint> &lst)
{
    if ( from == to )
        return 1;

    ScanPoint sc0(sc);
    std::vector<ScanVar>::iterator ip = sc0.find_type(from->type());

    int i, iok;
    for (i = 0;i < from->n_points();i++)
    {
        ip->set_value( ip->first() + i * ip->step() );
        iok = FillList(from + 1, to, sc0, lst);
        if ( iok )
            lst.push_back( sc0 );
    }
    return 0;
}

std::ostream &ScanPoint::dump(std::ostream &os)
{
    std::vector<ScanVar>::iterator ip;
    os << vvar.size() << " " << get_name() << std::endl;
    for (ip = begin();ip != end();++ip)
        ip->dump(os);
    return os;
}
std::istream &ScanPoint::load(std::istream &is)
{
    std::string line;
    int ivar, nvar;
    is >> std::ws;
    if (is.eof() || !is)
        return is;

    is >> nvar >> line >> std::ws;
    if (is.eof())
    {
        std::cout << "Wrong syntax in ScanPoint input stream" << std::endl;
        return is;
    }

    vvar.clear();
    set_name(line);
    for (ivar = 0;ivar < nvar;ivar++)
    {
        ScanVar v((ScanVar::ScanType)0);
        v.load(is);
        vvar.push_back(v);
    }

    return is;
}

int ScanPoint::npoints() const
{
    std::vector<ScanVar>::const_iterator ip;
    int n = 1;
    for (ip = vvar.begin();ip != vvar.end();++ip)
        n *= ip->n_points();

    return n;
}

void ScanList::cpy(const ScanList &s)
{
    ScanList::const_iterator ip;
    for (ip = s.begin();ip != s.end();++ip)
        (*this)[ip->first] = ip->second;
}
// I/O
std::ostream &ScanList::dump(std::ostream &os)
{
    ScanList::iterator ip;
    for (ip = begin();ip != end();++ip)
    {
        ip->second.dump(os);
    }
    return os;
}
std::istream &ScanList::load(std::istream &is)
{
    do
    {
        ScanPoint sp;
        sp.load(is);
        if (sp.get_name().empty())
            break;
        (*this)[sp.get_name()] = sp;
    }
    while (is);
    return is;
}
