#include <iostream>
#include <gmodule.h>
#include "VMEmanager.h"

using namespace std;

class DummyVME : public VMEmanager
{
    public:
        DummyVME()
        {}

        bool connected() const
        {
            return false;
        }
};


VMEmanager *__dummy_factory__()
{
    static DummyVME mngr;
    return &mngr;
}

VMEmanager::VME_Factory  VMEmanager::__factory__ = __dummy_factory__;

void VMEmanager::set_default_factory()
{
    __factory__ = __dummy_factory__;
}

void
VMEmanager::set_factory(const std::string &routine, const std::string lib)
{
    VME_Factory   tmp;
    GModule*      module;
    const char*   libname = lib.empty() ? 0 : lib.c_str();

    module = g_module_open( libname, G_MODULE_BIND_LAZY);
    if (!module && libname)
    {
        std::string new_path = std::string(LIB_DIR) + "/vmedaq/" + lib;
        module = g_module_open( new_path.c_str(), G_MODULE_BIND_LAZY);
    }
    if (!module)
    {
        std::cout << "VMEmanager::set_factory: could not open "
                  << lib
                  << std::endl << '\t' << g_module_error()
                  << std::endl;
        return;
    }
    if (!g_module_symbol(module, routine.c_str(), (gpointer *)&tmp))
    {
        std::cout << "VMEmanager::set_factory: could not find "
                  << routine << " in " << lib
                  << std::endl << '\t' << g_module_error()
                  << std::endl;
        g_module_close(module);
        return;
    }

    // We want to find the destructor...
    g_module_make_resident( module );

    __factory__ = tmp;
}

int
VMEmanager::stack_execute(std::vector<VMEcommand> &stack, std::vector<unsigned int> &data, int ntry)
{
  bool debug=0;
	
	
  if (stack.empty()) return 0;
  int size = data.size();
  unsigned int dat;
  std::vector<VMEcommand>::iterator it=stack.begin();
  string mode("write");

  for (;it!=stack.end();++it) 
     for (int i=ntry-1;i>-1;i--){	  
    switch ((*it).type()) {
      case VMEcommand::read8 :
	mode=string("read");
        dat = (unsigned int)read8(it->add());
        if (!i) 
		data.push_back(dat);
        break;
      case VMEcommand::write8 :
	dat = (unsigned int) it->data();
        write8(it->add(),(unsigned char)it->data());
        break;
      case VMEcommand::read16 :
	mode=string("read");
        dat = (unsigned int)read16(it->add());
        if (!i) 
		data.push_back(dat);
        break;
      case VMEcommand::write16 :
	dat = (unsigned int) it->data();
        write16((*it).add(),(unsigned short)it->data());
        break;
      case VMEcommand::read32 :
	mode=string("read");
        dat = (unsigned int)read32(it->add());
        if (!i)
		data.push_back(dat);
        break;
      case VMEcommand::write32 :
	dat = (unsigned int) it->data();
        write32(it->add(),(unsigned long)it->data());
        break;
      case VMEcommand::readA32D32 :
	mode=string("read");
        dat = (unsigned int)readA32D32(it->add());
        if (!i)
	        data.push_back(dat);
        break;
      case VMEcommand::writeA32D32 :
	dat = (unsigned int) it->data();
        writeA32D32(it->add(),(unsigned long)it->data());
        break;
      default :
        std::cout << "VMEmanager::stack_execute: Error\n"
                  << "  unrecognized command type: " << it->type() << "\n"
                  << "  aborting stack..." << std::endl;
        return -1;
    }
    if (debug)
      cout<<mode<<" address="<<hex<<it->add()<<dec<<"\tdata="<<hex<<dat
    	<<"\tstatus="<<hex<<get_rc()<<dec<<endl;
  }

  return (data.size() - size);
}
