#include <stdlib.h>
#include "random.h"

/*@(#)***************************************************************** 
 * Random Numbers
 *
 *   Created: Thu Aug 31 16:33:17 2000        Author: Carlos Lacasta
 *   Purpose: Set of routines to generate random numbers
 *
 **********************************************************************/

double randflat(double a,double b) {
   double x = ((double)rand())/((double)RAND_MAX);
   return a+x*(b-a);
}
double randuniform() {
   double x = ((double)rand())/((double)RAND_MAX);
   return x;
}
double randgauss(double mean,double stdev) {
   static int haveCachedNormal = 0;
   static double  cachedNormal = 0.;
   static double  pMean=0.,pStdDev=0.;

   if ( mean!=pMean || pStdDev!=stdev ) {
      haveCachedNormal = 0;
      pMean = mean;
      pStdDev=stdev;
   }
   if (haveCachedNormal == 1) {
      haveCachedNormal = 0;
      return(cachedNormal * pStdDev + pMean );
   } else {
      for(;;) {
	 double u1 = randuniform();
	 double u2 = randuniform();
	 double v1 = 2 * u1 - 1;
	 double v2 = 2 * u2 - 1;
	 double w = (v1 * v1) + (v2 * v2);
      
//
//	We actually generate two IID normal distribution variables.
//	We cache the one & return the other.
// 
	 if (w <= 1) {
	    double y = sqrt( (-2 * log(w)) / w);
	    double x1 = v1 * y;
	    double x2 = v2 * y;
	 
	    haveCachedNormal = 1;
	    cachedNormal = x2;
	    return(x1 * pStdDev + pMean);
	 }
      }
   }

}

void randgauss2d(double m1, double s1, double m2, double s2, double corr, double *x, double *y)
{
    double f1,f2;
    double v1,v2;
    f1 = sqrt((1.+corr)/2.);
    f2 = sqrt((1.-corr)/2.);

    v1 = randgauss( 0.,1.);
    v2 = randgauss( 0.,1.);

    *x = m1 + s1*(f1*v1+f2*v2);
    *y = m2 + s2*(f1*v1-f2*v2);
}

