#include <cstring>
#include "HstData.h"

std::ostream &operator<<(std::ostream &os, const HstData &h)
{
    int nval = 0;
    os.write((char *)&h._size, 5*sizeof(int));
    if ( h.get_map() )
    {
        nval = h.get_map()->nx();
        os.write((char *)&nval, sizeof(int));
        nval = h.get_map()->ny();
        os.write((char *)&nval, sizeof(int));
    }
    else
    {
        os.write((char *)&nval, sizeof(int));
        os.write((char *)&nval, sizeof(int));
    }
    os.write((char *)&h.rmax, 2*sizeof(double));
    os.write((char *)h.ptr, h._size*sizeof(double));
    return os;
}
std::istream &operator>>(std::istream &is, HstData &h)
{
    int szold = h._size;
    int nval;
    is.read((char *)&h._size, 5*sizeof(int));
    is.read((char *)&nval, sizeof(int));
    is.read((char *)&nval, sizeof(int));
    is.read((char *)&h.rmax, 2*sizeof(double));

    if ( h.ptr && h._size > szold)
    {
        delete [] h.ptr;
        h.ptr = new double[h._size];
        h.init_ptr();
    }

    is.read((char *)h.ptr, h._size*sizeof(double));
    return is;
}


HstData::HstData(int n1, int n2, int nb, double mn, double mx)
        : _size(0), _bfsiz(0), nchip(n1), nchan(n2), nbins(nb), rmax(mx), rmin(mn), ptr(0), map(0)
{
    ntot = nchip * nchan;
    set_bins(nbins, rmin, rmax);
}

void HstData::set_bins(int nb, double vmin, double vmax)
{
    nbins = nb;
    if ( rmin < rmax)
    {
        rmin = vmin;
        rmax = vmax;
    }
    else
    {
        rmin = vmax;
        rmax = vmin;
    }
    bwidth = ((double)nbins) / (rmax - rmin);
    init_ptr();
}


void HstData::init_ptr()
{
    double *dmy;
    int sz;
    int sz2d;
    
    if (map)
    {
        sz2d = map->nx()*map->ny();
        if (sz2d<ntot)
            sz2d = ntot;
    }
    else
        sz2d = ntot;

    sz =
        ntot + // pedestal
        ntot + // noise
        ntot + // hitmap
        2 * sz2d + // both 2D hitmaps
        ntot + // output
        ntot + // signal process
        (nchip + 1) * nbins +     // signal 1D hst
        (ntot + nchan) * nbins +  // signal 2D hst
        nchip +                   // common mode
        nchip                     // average noise
        ;

    if ( sz > _bfsiz )
    {
        if (ptr)
            delete [] ptr;
        _bfsiz = sz;
        ptr = new double[_bfsiz];
    }
    _size = sz;

    dmy      = ptr;
    ped      = dmy;
    dmy += ntot;
    noise    = dmy;
    dmy += ntot;
    hitmap   = dmy;
    dmy += ntot;
    h2dmap   = dmy;
    dmy += sz2d;
    h2dmap_c = dmy;
    dmy += sz2d;
    out      = dmy;
    dmy += ntot;
    chan     = dmy;
    dmy += ntot;
    hsig     = dmy;
    dmy += (nchip + 1) * nbins;
    hsig2    = dmy;
    dmy += (ntot + nchan) * nbins;
    cmmd     = dmy;
    dmy += nchip;
    avnoise  = dmy;

    Init();
}

void HstData::Init()
{
    int i;
    for (i = 0;i < _size;i++)
        ptr[i] = 0.;
}

HstData &HstData::operator=(const HstData &d)
{
    nchip = d.nchip;
    nchan = d.nchan;
    ntot = nchip * nchan;
    set_bins(d.nbins, d.rmin, d.rmax);
    memcpy(ptr, d.ptr, _size*sizeof(double));
    return *this;
}


HstData::~HstData()
{
    delete [] ptr;
    if (map)
        delete map;
}

void HstData::set_map(const PadMap &m)
{
    if (map)
        delete map;

    map = new PadMap(m);
    init_ptr();
}
