#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <algorithm>
#include <iterator>
#include <cstring>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <cerrno>
#include <gtk/gtk.h>
#include <DAQ++/DAQmanager.h>
#include <gtkmessagebox.h>
#include "RunMenu.h"
#include "module_gui.h"
#include "MainWindow.h"
#include "ScanDialog.h"
#include "Preferences.h"
#include "PluginSettings.h"
#include "gtkutils.h"
#include <daq/Rmanager.h>
#include <daq/GenModule.h>
#include <daq/DataRunManager.h>
#include <daq/ScanRunManager.h>
#include <daq/PedestalRunManager.h>
#include <daq/IntTrgRunManager.h>
#include <daq/DataLogger.h>
#include <data/Analysis.h>
#include <daq/DAQcontroller.h>

#include <gtk-hst/gtkhistogram.h>
#include <gtk-hst/tracer.h>
#include <gtk-hst/gtkgauge.h>


#include <daq/DAQinterrupt.h>
#include <config/Config.h>

#include "Pedestal.h"

#define UPDATE_TMOUT 1000

std::string get_proc_mem();
std::string check_output_files(const std::string &patt, int &ignore);

void activate_group(GtkWidget *w, gboolean sensitive)
{
    g_return_if_fail( GTK_IS_RADIO_BUTTON (w) );
    GSList *slist = GTK_RADIO_BUTTON(w)->group;
    for (; slist; slist = slist->next)
    {
        GtkRadioButton *btn = GTK_RADIO_BUTTON(slist->data);
        gtk_widget_set_sensitive( GTK_WIDGET(btn), sensitive );
    }
}

class StateCondition : public DAQpp::Condition, public Observer
{
private:
    DAQpp::RunState::StatusType _state, _newstate;
public:
    StateCondition( DAQpp::RunState::StatusType state, DAQpp::RunState &S )
            : _state(state), _newstate(DAQpp::RunState::last_state)
    {
        _state = state;
        S.addObserver( *this );
    }
    void set_state(DAQpp::RunState::StatusType state)
    {
        _state = state;
    }
    void update(Observable *o, Argument *arg)
    {
        DAQpp::RunState *state = dynamic_cast<DAQpp::RunState *>(o);
        _newstate = state->get_status();
        notify_all();
    }
    bool condition()
    {
        return _state == _newstate;
    }
};


template <>
void StChanged<MainWindow>::update_state(DAQpp::RunState::StatusType st)
{

    switch ( st )
    {
    case DAQpp::RunState::NotDefined:

        break;
    case DAQpp::RunState::NotReady:

        break;
    case DAQpp::RunState::GettingReady:
        break;

    case DAQpp::RunState::Ready:
        obj->set_active_widgets(true);
        break;
    case DAQpp::RunState::Paused:

        break;
    case DAQpp::RunState::Starting:
        {
            DAQpp::RunManager *runm = DAQpp::DAQmanager::theDAQmanager()->find_runmanager("main");
            if (runm)
            {
                Rmanager *R = (Rmanager *)runm;
                if (!R->get_logger())
                {
                    gtk_label_set_text(GTK_LABEL(obj->get_widget("lblDataFile")),"Not Logging");
                }
            }
            obj->set_active_widgets(false);
            break;
        }
    case DAQpp::RunState::Running:
        // block some widgets

        break;
    case DAQpp::RunState::TimeOut:

        break;
    default:
        break;
    }
    RunMenu::find("main")->update(st);
};


int quitting_gtk(gpointer ptr)
{
    std::cout << "Bye !!!" << std::endl;
    DAQpp::DAQmanager::theDAQmanager()->StopRun();
    return 0;
}

struct _load_file_data
{
    char *file;
    MainWindow *main_window;
};
typedef struct _load_file_data load_file_data;

int load_configuration_file(load_file_data *data)
{
    const char *xmlFile = data->file;
    std::cout << "You selected file " << xmlFile << std::endl;

    // Reset the ConfigParser. This will also reset the DAQ.
    ConfigParser *cfg = ConfigParser::theConfig();
    cfg->reset();

    // Now load the new config file
    if (!cfg->parse(xmlFile))
    {
        std::cout << "LoadFile: Problems parsing " << xmlFile << std::endl;
        cfg->reset();
    }
    else
    {
        DAQcontroller::instance()->setup_daq(0,0);
        DAQcontroller::instance()->set_main_window(data->main_window);
    }
    data->main_window->start_timer();
    g_free(data->file);
    g_free(data);
    return 0;
}

MainWindow::MainWindow() :
    do_pedestal_update(true),
    doScan(false), nScan(100), state(this),
    update_tmout(-1), trg_type("data"), nrun(0)
{

    // Get a pointer to the configuration manager
    ConfigParser *cfg = ConfigParser::theConfig();

    // Get current run number
    SectionParser *sp = cfg->get_handler("IO");
    if ( sp )
    {
        IOConfig *io = dynamic_cast<IOConfig *>(sp);
        nrun = io->get_run_number();
    }

    // Init. the scan variables
    sp = cfg->get_handler("scans");
    if (sp)
    {
        ScanConfig *scf = dynamic_cast<ScanConfig *>(sp);
        if ( scf->size() > 0 )
            scpoint = scf->begin()->second;

        nScan = scf->get_nevts();

    }
    gtk_quit_add(1, quitting_gtk, 0);
}

MainWindow::~MainWindow()
{}

void MainWindow::auto_connect()
{
    SET_CONNECT( onQuit );
    SET_CONNECT( onLoadConfig );
    SET_CONNECT( onSaveConfig );
    SET_CONNECT( onSaveAsConfig );
    SET_CONNECT( onPreferences );
    SET_CONNECT( onLoadPedestals);
    SET_CONNECT( onSavePedestals);
    SET_CONNECT( onAbout );
    SET_CONNECT( onManual);
    SET_CONNECT( onMonitor );


    SET_CONNECT( onTrgType );
    SET_CONNECT( onInterrupt );
    SET_CONNECT( onScan );
    SET_CONNECT( onPedestal );
    SET_CONNECT( onLogData );
    SET_CONNECT( on_plugins_activate );

}
void MainWindow::set_handlers()
{
    handlers["create_memory_monitor"] = (GladeHandler)create_memory_monitor;
    handlers["create_gauge_eff"] = (GladeHandler)create_gauge_eff;
    handlers["create_gauge_rate"] = (GladeHandler)create_gauge_rate;
}

void MainWindow::user_init()
{
    GtkWidget *tmp;

    //---------------------------------------------------------------------
    // Monitor switch icons
    //---------------------------------------------------------------------
    tmp = get_widget("toggleMonitor");
    g_object_set_data( G_OBJECT(tmp), "monitor_yes", create_pixmap(tmp, "monitor.png"));
    g_object_set_data( G_OBJECT(tmp), "monitor_no", create_pixmap(tmp, "no_monitor.png"));



    //---------------------------------------------------------------------
    // Add the RunMenu
    // We are assuming that the main RunManager is called "main"
    // in the server
    //---------------------------------------------------------------------
    RunMenu::new_menu( "main", get_widget("btnRunControl"));
    DAQpp::RunManager *rm =
        DAQpp::DAQmanager::theDAQmanager()->find_runmanager("main");

    //---------------------------------------------------------------------
    // Now we add the StChanged<MainWindow> element as an observer of
    // the main RunManager so that the Window will be notified whenever
    // the RUnManager changes its state.
    //---------------------------------------------------------------------
    if ( rm )
      rm->get_status().addObserver( state );

    //---------------------------------------------------------------------
    // Define the updates
    //---------------------------------------------------------------------
    addUpdate( new UpdateLabelTime( get_widget("lblRunDuration")));
    addUpdate( new UpdateLabelNtrig( get_widget("lblNtrigger")));
    addUpdate( new UpdateGaugeRate( get_widget("gauge_rate")));
    addUpdate( new UpdateGaugeEff( get_widget("gauge_eff") ));
    addUpdate( new UpdateThroughPut( get_widget("memory-monitor") ));

    //---------------------------------------------------------------------
    // Add the time out routine to update the labels
    //---------------------------------------------------------------------
    start_timer();

    // Add the modules
    // g_idle_add( (GSourceFunc)do_add_modules, this );
}

void MainWindow::start_timer()
{
    update_tmout = g_timeout_add(UPDATE_TMOUT, onUpdate, this);
}

void MainWindow::stop_timer()
{
    if (update_tmout>=0)
    {
        gtk_timeout_remove(update_tmout);
        update_tmout=-1;
    }
}

void MainWindow::set_module_list(const std::vector<std::string> &ids)
{
    module_list = ids;
    add_modules();
}

gboolean MainWindow::do_clear_modules(MainWindow *mw)
{
    GtkTable *module_table =  GTK_TABLE(mw->get_widget("module-table"));
    mw->module_ptr.clear();
    if (module_table)
    {
        gtk_container_foreach(GTK_CONTAINER(module_table),
                              (GtkCallback) gtk_widget_hide,
                              NULL);
        gtk_container_foreach(GTK_CONTAINER(module_table),
                              (GtkCallback) gtk_widget_destroy,
                              NULL);
    }
    return FALSE;
}

void MainWindow::add_modules()
{
    g_idle_add((GSourceFunc)do_add_modules, this);
}

void MainWindow::clear_modules()
{
    g_idle_add((GSourceFunc)do_clear_modules, this);
}

gboolean MainWindow::do_add_modules(MainWindow *mw)
{
    mw->add_modules_true();
    return FALSE;
}

void MainWindow::add_modules_true()
{
    GtkTable *module_table =  GTK_TABLE(get_widget("module-table"));
    do_clear_modules(this);

    // Prepare the list of MODULEs in the scroll window
    module_gui *dpx;
    int i, j, ndsp, nrow, icol, ncol = 3;

    if ( module_list.empty() )
        return;

    module_ptr.clear();

    ndsp = module_list.size();
    nrow = ndsp / ncol;
    if (nrow * ncol < ndsp)
        nrow++;
    gtk_table_resize(module_table, nrow, ncol);
    GtkRequisition dsp_size = { -1, -1};
    for (i = j = 0; i < ndsp; i++)
    {
        dpx = new module_gui(module_list[i]);
        dpx->init(get_file_name(), "dsp-frame");
        module_ptr.push_back(dpx);
        if (i && !(i % ncol))
            j++;
        icol = i % ncol;
        gtk_table_attach(module_table, dpx->get_main_window(),
                         icol, icol + 1, j, j + 1,
                         (GtkAttachOptions) 0, (GtkAttachOptions) 0, 0, 0);
        if (dsp_size.width<0)
        {
            gtk_widget_size_request(dpx->get_main_window(), &dsp_size);
        }
    }
    gtk_widget_set_size_request(get_widget("viewport-module-table"), ncol*(2+dsp_size.width), int((2+dsp_size.height)+0.5));
    gtk_widget_queue_resize(get_widget("despex_list"));
    gtk_widget_queue_resize(get_main_window());
}

/**
 * Sets a number of widget sensitive/insensitive
 */
void MainWindow::set_active_widgets(bool sens)
{
    gtk_widget_set_sensitive(get_widget("open_config"), sens);
    gtk_widget_set_sensitive(get_widget("txtNoOfEvents"), sens);

    activate_group(get_widget("swTriggerType"), sens);
    activate_group(get_widget("swInterrupt"), sens);
    //gtk_widget_set_sensitive(get_widget("toggleMonitor"), sens);

    gtk_widget_set_sensitive(get_widget("swLogData"), sens);

    gtk_widget_set_sensitive(get_widget("btnScan"), sens);
    gtk_widget_set_sensitive(get_widget("btnPedestal"), sens);

    std::vector<module_gui *>::iterator it;
    for (it = module_ptr.begin();it != module_ptr.end();++it)
        gtk_widget_set_sensitive( (*it)->get_widget("swLocal"), sens);

}

/**
 * It is called every UPDATE_TMOUT milliseconds and will update the
 * different widgets.
 */
int MainWindow::onUpdate(void *ptr)
{
    MainWindow *wnd = reinterpret_cast<MainWindow *>(ptr);
    if ( wnd == 0 )
        return 1;

    int i, j, nRm, nMd;
    Update *rml, *mdl;
    // Updates the state of all the RunManager and Module objects
    Update_all(nRm, &rml, nMd, &mdl);

    static std::string id("main");
    for (i = 0;i < nRm;i++)
    {
        if ( rml[i].id == id )
        {
            wnd->updateAll( rml[i] );
            break;
        }
    }

    std::vector<module_gui *>::iterator it;
    for (it = wnd->module_ptr.begin();it != wnd->module_ptr.end();++it)
    {
        for (j = 0;j < nMd;j++)
        {
            if ( (*it)->id() == mdl[j].id )
            {
                (*it)->update( mdl[j] );
                break;
            }
        }
    }

    delete [] rml;
    delete [] mdl;



    //---------------------------------------------------------------------
    // Get RunManager
    //---------------------------------------------------------------------
    Rmanager *rm =
        dynamic_cast<Rmanager *>(
            DAQpp::DAQmanager::theDAQmanager()->find_runmanager("main")
        );


    // Data logging
    if ( rm && rm->get_logger() )
    {
        std::string fnam = rm->get_logger()->get_file_name();
        gtk_label_set_text(GTK_LABEL(wnd->get_widget("lblDataFile")), fnam.c_str());
        gtk_progress_set_value(GTK_PROGRESS(wnd->get_widget("progressbar")),
                               100.* rm->get_logger()->relative_size());
    }
    else
    {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wnd->get_widget("swLogData")), FALSE);
    }

    //---------------------------------------------------------------------
    // Check kind of run
    //---------------------------------------------------------------------
    ScanRunManager *sc = dynamic_cast<ScanRunManager *>(rm);
    GtkWidget *lbl = wnd->get_widget("lblRunMode");
    GtkWidget *tmp = wnd->get_widget("lblScanInfo");
    if ( sc )
    {
        gtk_label_set_text(GTK_LABEL(lbl), "Scan");
        if ( sc->GetStatus().isRunning() )
        {
            const ScanPoint &point = sc->get_current();
            std::ostringstream ostr;
            std::vector<ScanVar>::const_iterator ip;
            ostr << point.get_name() << ":> ";
            for (ip = point.begin();ip != point.end();++ip)
            {
                ostr << ScanVar::get_name(ip->type()) << ": "
                << std::setiosflags(std::ios::fixed )
                << std::setw(6) << std::setprecision(2)
                << ip->value() << " ";
            }

            gtk_label_set_text(GTK_LABEL(tmp), ostr.str().c_str());
        }
    }
    else
    {
        IntTrgRunManager *trm = dynamic_cast<IntTrgRunManager *>(rm);
        if ( trm )
        {
            gtk_label_set_text(GTK_LABEL(lbl), "IntTrg");
            gtk_label_set_text(GTK_LABEL(tmp), "<none>");
        }
        else
        {
            DataRunManager *drm = dynamic_cast<DataRunManager *>(rm);
            if ( drm )
            {
                gtk_label_set_text(GTK_LABEL(lbl), "Data");
                gtk_label_set_text(GTK_LABEL(tmp), "<none>");
            }
        }
    }

    // Process memory
    std::string omem = get_proc_mem();
    if (!omem.empty())
    {
        GtkWidget *wl = wnd->get_widget("lblMemory");
        gtk_label_set_text(GTK_LABEL(wl), omem.c_str());
    }

    //---------------------------------------------------------------------
    // Progress
    //---------------------------------------------------------------------
    //    if ( rm ) {
    //       DataLogger *lg = (DataLogger *)(rm->get_logger());
    //       if ( lg ) {
    //    tmp = glade_xml_get_widget(wnd->xml(), "prgFifo");
    //    gtk_progress_bar_update(GTK_PROGRESS_BAR(tmp),lg->get_fifo_free());
    //       }
    //    }
    return 1;
}

void MainWindow::process_text(const std::string &s, GtkWidget *w)
{
    if ( s == "txtNoOfEvents" )
    {
        int mxevts = entry2int(s, -1);
        DAQpp::RunManager *rm =
            DAQpp::DAQmanager::theDAQmanager()->find_runmanager("main");

        if ( rm )
            rm->set_max_events(mxevts);
    }
}



//---------------------------------------------------------------------
// Configuration
//---------------------------------------------------------------------
void MainWindow::onLoadConfig(GtkWidget *, MainWindow *mw)
{
    mw->stop_timer();
    char *cwd = new char[1024];
    std::string std = std::string( getenv("HOME") ) + "/.vmedaq";
    getcwd(cwd, 1024);
    if ( chdir(std.c_str() ) )
    {
        std::ostringstream ostr;
        GtkMessagebox *mb;
        ostr << "$HOME/.vmdaq does not exist" << std::endl
             << "This is the \"standard\" place to " << std::endl
             << "put configuration files"
             ;
        mb = GTK_MESSAGEBOX(gtk_messagebox_new("vmedaq", ostr.str().c_str(),
                                               MSG_BOX_CRITICAL,
                                               MSG_BOX_OK));
        gtk_messagebox_exec(mb);
    }

    const char *filters[] =
        {"*.xml", 0
        };
    std::string ofile = ::get_file_name("Load config", TRUE, filters);
    if (!ofile.empty())
    {
//        const char *xmlFile = ofile.c_str();
//        std::cout << "You selected file " << xmlFile << std::endl;
//
//        // Reset the ConfigParser. This will also reset the DAQ.
//        ConfigParser *cfg = ConfigParser::theConfig();
//        cfg->reset();
//
//        // Now load the new config file
//        std::vector<std::string> ids;
//        if (!cfg->parse(xmlFile))
//        {
//            std::cout << "LoadFile: Problems parsing " << xmlFile << std::endl;
//            cfg->reset();
//        }
//        else
//        {
//            DAQcontroller::instance()->setup_daq(0,0);
//            DAQcontroller::instance()->set_main_window(mw);
//        }
        load_file_data *data  = g_new0(load_file_data, 1);
        data->file = g_strdup( ofile.c_str() );
        data->main_window = mw;
        g_idle_add( (GSourceFunc)load_configuration_file, (gpointer)data);
    }
    chdir(cwd);
    delete [] cwd;
//    mw->start_timer();
    std::cout << "Done with load config..." << std::endl;
}

void MainWindow::onQuit(MainWindow *mw)
{
    //   delete mw;
    gtk_main_quit();
}

void MainWindow::onSaveConfig(GtkWidget *, MainWindow *mw)
{
    char *cwd = new char[1024];
    std::string std = std::string( getenv("HOME") ) + "/.vmedaq";
    getcwd(cwd, 1024);
    if ( chdir(std.c_str() ) )
    {
        std::ostringstream ostr;
        GtkMessagebox *mb;
        ostr << "$HOME/.vmdaq does not exist" << std::endl
        << "This is the \"standard\" place to " << std::endl
        << "put configuration files"
        ;
        mb = GTK_MESSAGEBOX(gtk_messagebox_new("vmedaq", ostr.str().c_str(),
                                               MSG_BOX_CRITICAL,
                                               MSG_BOX_OK));
        gtk_messagebox_exec(mb);
    }

    const char *filters[] =
        {"*.xml", 0
        };
    std::string ofile = ::get_file_name("Save config", FALSE, filters);
    if (!ofile.empty())
    {
        ConfigParser::theConfig()->save_config( ofile.c_str() );
    }
    chdir(cwd);
    delete [] cwd;
}
void MainWindow::onSaveAsConfig(GtkWidget *w, MainWindow *mw)
{
    mw->onSaveConfig(w, mw);
}

void MainWindow::onPreferences(GtkWidget *w, MainWindow *mw)
{
    Preferences *wpref = new Preferences;
    wpref->init(mw->get_file_name(), "Preferences");
}

void MainWindow::onLoadPedestals(GtkWidget *w, MainWindow *mw)
{
    PedestalGui *P = new PedestalGui(true, "Load Pedestals");
    P->init(mw->get_file_name(), "Pedestals");
}

void MainWindow::onSavePedestals(GtkWidget *w, MainWindow *mw)
{
    PedestalGui *P = new PedestalGui(false, "Save Pedestals");
    P->init(mw->get_file_name(), "Pedestals");
}

void MainWindow::onAbout(GtkWidget *w, MainWindow *mw)
{
    std::cout << "Not yet implemented" << std::endl;
}

void MainWindow::onManual(GtkWidget *w, MainWindow *mw)
{
    gtk_help_display(NULL);
}


void MainWindow::onScan(GtkWidget *w, MainWindow *mw)
{
    SectionParser *sp = ConfigParser::theConfig()->get_handler("scans");
    ScanConfig *scf = dynamic_cast<ScanConfig *>(sp);
    ScanDialog *sc = new ScanDialog( mw->nScan, scf->get_scan_list() );

    sc->scan( mw->doScan );
    if ( !mw->scpoint.get_name().empty() )
        sc->set_scan_point( mw->scpoint.get_name() );
    sc->delay ( scf->delay() );
    sc->burst ( scf->burst() );
    sc->is_time( scf->is_time() );
    sc->rate ( scf->rate() );
    sc->init( mw->get_file_name(), "ScanDialog");
    int rc = gtk_dialog_run(GTK_DIALOG(sc->get_main_window()));
    if (rc == GTK_RESPONSE_OK)
    {
        ScanList sl;
        sc->copy_scan_point( mw->scpoint );
        sc->copy_scan_list(sl);
        scf->set_scan_list(sl);
        mw->doScan = sc->scan();
        mw->nScan = sc->get_nevts();

        scf->burst( sc->burst() );
        scf->is_time( sc->is_time() );
        scf->delay( sc->delay() );
        scf->rate ( sc->rate() );

        // Create the RunManager
        if ( mw->doScan )
        {
            std::string old_type = mw->trg_type;
            mw->trg_type = "scan";
            ScanRunManager *rm = dynamic_cast<ScanRunManager *>(mw->CreateRunManager());
            mw->trg_type = old_type;

            // Set the attributes
            rm->set_scan_list(mw->nScan, mw->scpoint());
            rm->delay( scf->delay() );
            rm->burst( scf->burst() );
            rm->rate( scf->rate() );
            rm->is_time( scf->is_time() );
        }
        else
        {
        	// go back to the previous run manager
        	mw->CreateRunManager();
        }
    }
    delete sc;
}


void MainWindow::on_plugins_activate(GtkWidget *w, MainWindow *mw)
{
    PluginSettings *ps = new PluginSettings();
    ps->init( mw->get_file_name(), "plugin_dialog");
    int rc = gtk_dialog_run(GTK_DIALOG(ps->get_main_window()));
    if (rc == GTK_RESPONSE_OK)
    {

    }
    delete ps;
}

//std::vector<DAQpp::Module *> depleteRM(DAQpp::RunManager *rm)
//{
//    std::vector<DAQpp::Module *> out;
//    std::set<Observer *>::iterator ip;
//
//    // fill the vector of modules first
//    for (ip = (*rm)()->Observers().begin();ip != (*rm)()->Observers().end();++ip)
//    {
//        DAQpp::Module *md = dynamic_cast<DAQpp::Module *>(*ip);
//        if ( md )
//            out.push_back(md);
//    }
//
//    std::vector<DAQpp::Module *>::iterator im;
//    for (im = out.begin();im != out.end();im++)
//        (*im)->UnRegisterDAQ();
//
//    return out;
//}

void MainWindow::onInterrupt(GtkWidget *w, MainWindow *mw)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    PluginParser *plugin = dynamic_cast<PluginParser *>(cfg->get_handler("plugins"));

    bool do_int = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))
                  ? false : true;


    std::cout << "Interrupts enabled: " << do_int << std::endl;
    if ( do_int )
    {
        int rc = plugin->enable_interrupts();
        if (rc)
        {
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), FALSE);
            do_int = false;
        }
    }
    else
        plugin->disable_interrupts();

    mw->check_for_interrupts(&do_int);
}

void MainWindow::onTrgType(GtkWidget *w, MainWindow *mw)
{
    bool real = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))
                ? true : false;

    mw->trg_type = real ? "data" : "int_trg" ;
    mw->CreateRunManager();
}


void MainWindow::onLogData(GtkWidget *w, MainWindow *mw)
{
    bool log = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))
               ? true : false;

    std::ostringstream ostr;
    std::string msg;
    GtkWidget *tmp =  mw->get_widget("lblDataFile");
    // find the run manager
    Rmanager *rm =
        dynamic_cast<Rmanager *>(
            DAQpp::DAQmanager::theDAQmanager()->find_runmanager("main")
        );

    if ( !log )
    {
        ostr << "Not logging";
        msg = ostr.str();
        if ( rm )
            rm->stop_logging();
    }
    else
    {
        // build the file name
        ConfigParser *cfg = ConfigParser::theConfig();
        SectionParser *sp = cfg->get_handler("IO");
        IOConfig *io = dynamic_cast<IOConfig *>(sp);

        mw->nrun = io->get_run_number();
        ostr << io->get_output_path() << "/"
        << io->get_prefix()
        << std::setw(6) << std::setfill('0') << mw->nrun
        << ".data"
        ;

        int ignore;
        std::string msg = check_output_files(ostr.str(), ignore);
        if (!msg.empty())
        {
            int btns = MSG_BOX_OK;
            if (!ignore)
                btns |= MSG_BOX_CANCEL;

            GtkMessagebox *mb =
                GTK_MESSAGEBOX(gtk_messagebox_new("vmedaq", msg.c_str(),
                                                  MSG_BOX_CRITICAL,
                                                  btns));
            if ( gtk_messagebox_exec(mb) != MSG_BOX_OK || ignore)
                return;
        }

        if ( rm )
        {
            io->set_run_number( ++mw->nrun );
            std::cout << "*** Adding a data logger to " << rm->get_id() << std::endl;
            rm->log_data(ostr.str().c_str(),
                         io->get_compression(),
                         io->get_max_size()
                        );
            msg = rm->get_logger()->get_file_name();
        }
    }
    gtk_label_set_text(GTK_LABEL(tmp), msg.c_str());
}

struct module_save_config : public std::unary_function<Observer *, void>
{
    module_save_config()
    {}
    void operator() (Observer *o)
    {
        GenModule *m = dynamic_cast<GenModule *>(o);
        if ( m )
        {
            ConfigParser *cfg = ConfigParser::theConfig();
            ModuleParser *mp = dynamic_cast<ModuleParser *>(cfg->get_id_handler(m->get_id()));
            mp->save_config_id(m->get_id());
        }
    }
};

struct module_restore_config : public std::unary_function<Observer *, void>
{
    module_restore_config()
    {}
    void operator() (Observer *o)
    {
        GenModule *m = dynamic_cast<GenModule *>(o);
        if ( m )
        {
            ConfigParser *cfg = ConfigParser::theConfig();
            ModuleParser *mp = dynamic_cast<ModuleParser *>(cfg->get_id_handler(m->get_id()));
            mp->parse_node_id(m->get_id());
        }
    }
};


typedef void *(*fthread)(void *);
void MainWindow::do_pedestal(MainWindow *mw)
{
    ///
    ConfigParser *cfg = ConfigParser::theConfig();
    GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));
    int ped_min_sample = gnrl->get_ped_min_sample();

    // check if we where running in interrupt mode
    GtkWidget *sw = mw->get_widget("swInterrupt");
    bool do_int = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sw))
                  ? false : true;

    std::cout << "Pedestals: trigger type "
              << (do_int ? "Interrupt" : "Poll")
              << std::endl;

    if (do_int)
    {
        std::cout << "...found that we are running with interrupt trigger\n"
                  << "...changing to Poll trigger. It will be resotred at the end."
                  << std::endl;
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sw), TRUE);
        gtk_widget_queue_draw(sw);
    }

    // Get the current main RunManager
    Rmanager *rm = dynamic_cast<Rmanager *>(DAQpp::DAQmanager::theDAQmanager()->find_runmanager("main"));
    if (!rm)
    {
        std::cout << "\n\n*************************\n"
                  << "* Non existing main RunManager. "
                  << "This should not happen.\n"
                  << "**************************\n\n\n"
                  << std::endl;
        delete mw;
        return;
    }

    // Save the current configuration of the modules
    rm->for_each_module( module_save_config() );

    // Create the Pedestal RunManager
    rm = Rmanager::create_run_manager("pedestal","main");
    rm->get_status().addObserver( mw->state );
    rm->set_max_events(gnrl->get_ped_nevts());
    gnrl->set_ped_min_sample( gnrl->get_ped_nevts() );

    // Start the run
    std::cout << "Starting Pedestal Run" << std::endl;
    rm->set_monitor(0);
    DAQpp::DAQmanager::theDAQmanager()->startRunManager(rm);
    if (!rm->isActive() )
        rm->Activate();

    // Wait until end of run
//    StateCondition wait_state(DAQpp::RunState::Stopped, rm->GetStatus());
//    wait_state.wait();
//    wait_state.set_state( DAQpp::RunState::Ready );
//    wait_state.wait();
//    rm->GetStatus().deleteObserver(wait_state);

    do
    {
        struct timespec qw =
            {
                0, 500000000
            };
        nanosleep(&qw, NULL);
    }
    while ( DAQpp::DAQmanager::theDAQmanager()->countActiveRunManager() > 0 );

    std::cout << "...done pedestal run" << std::endl;

    // restore previous state
    gnrl->set_ped_min_sample( ped_min_sample );
    if (do_int)
    {
        std::cout << "...Restoring Interrupt trigger" << std::endl;
        GtkWidget *sw = mw->get_widget("swInterruptInt");
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sw), TRUE);
        gtk_widget_queue_draw(sw);
    }
    rm->for_each_module( module_restore_config() );

    // This is where we create the new RunManager
    mw->CreateRunManager();
}

void MainWindow::onPedestal(GtkWidget *w, MainWindow *mw)
{
    // start the thread
    pthread_t t;
    pthread_create(&t, 0, (fthread)do_pedestal, mw);
}


void MainWindow::onMonitor(GtkWidget *w, MainWindow *mw)
{
    GtkWidget *img;
    bool do_monitor = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));

    // find the run manager
    Rmanager *rm =
        dynamic_cast<Rmanager *>(
            DAQpp::DAQmanager::theDAQmanager()->find_runmanager("main")
        );
    if (do_monitor)
    {
        ConfigParser *cfg = ConfigParser::theConfig();
        GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));
        img = GTK_WIDGET(g_object_get_data(G_OBJECT(w), "monitor_yes"));
        if (!rm->get_monitor()) {
            rm->set_monitor( new MyMonitor(rm, gnrl->get_monitor_buffer()) );
            rm->get_monitor()->start_monitor();
        }
    }
    else
    {
        img = GTK_WIDGET(g_object_get_data(G_OBJECT(w), "monitor_no"));
        if (rm->get_monitor()) {
            std::cout << "Stopping current monitor" << std::endl;
            rm->get_monitor()->stop_monitor();
            std::cout << "...done" << std::endl;
            rm->set_monitor(0);
        }
    }
    change_button_pixmap(w, img);
}

GtkWidget *
MainWindow::create_gauge_rate(MainWindow *mw, char *, char *, char *, int , int)
{
    GtkWidget *w = gtk_gauge_new(0., 10000.);
    gtk_gauge_set_color(GTK_GAUGE(w), "green");
    gtk_gauge_set_title(GTK_GAUGE(w), "Rate (Hz)");
    return w;
}
GtkWidget *
MainWindow::create_gauge_eff(MainWindow *mw, char *, char *, char *, int , int)
{
    GtkWidget * w = gtk_gauge_new(0., 100.);
    gtk_gauge_set_title(GTK_GAUGE(w), "Efficiency");
    return w;
}
GtkWidget *
MainWindow::create_memory_monitor(MainWindow *mw, char *, char *, char *, int , int)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));
    HstTracer *tracer = hst_tracer_new(50);
    GtkWidget *w = gtk_hst1d_new( HST_FUNCTION(tracer) );
    gtk_hst1d_draw_x_axis(GTK_HST1D(w), HST_AXIS_NONE);
    gtk_hst1d_draw_y_axis(GTK_HST1D(w), HST_AXIS_MIDDLE);

//    gtk_hst1d_set_colors(GTK_HST1D(w),
//                         "foreground", "firebrick",
//                         "background", "black",
//                         "axis", "darkseagreen", NULL);

    gtk_hst1d_set_colors(GTK_HST1D(w),
                         "background", gnrl->tracers().bg().c_str(),
                         "foreground", gnrl->tracers().fg().c_str(),
                         "axis", gnrl->tracers().axis().c_str(),
                         NULL
                         );
    gtk_hst1d_sync_range(GTK_HST1D(w), TRUE);
    gtk_hst1d_monitor_start(GTK_HST1D(w), 1000);
    g_object_set_data(G_OBJECT(w), "hst", tracer);
    return w;
}

Rmanager *MainWindow::CreateRunManager()
{
    Rmanager *rm;

    std::cout << "Creating a " << trg_type << " RunManager" << std::endl;
    rm = Rmanager::create_run_manager(trg_type,"main");
    std::cout << "...done" << std::endl;

    std::cout << "Active RMs: "
    << DAQpp::DAQmanager::theDAQmanager()->countActiveRunManager()
    << std::endl;
    std::cout << "is  active ? " << (rm->isActive() ? "yes" : "no") << std::endl;

    // Register to the new RunState
    std::cout << "Registering to the new RunState" << std::endl;
    rm->get_status().addObserver( state );


    onMonitor(get_widget("toggleMonitor"), this);


    check_for_interrupts();
    return rm;
}

void MainWindow::check_for_interrupts(bool *z)
{
    bool do_int;
    if (z)
        do_int = *z;
    else
    {
        do_int = gtk_toggle_button_get_active(
                     GTK_TOGGLE_BUTTON(get_widget("swInterrupt")))
                 ? false : true;
    }
    DAQpp::RunManager *rm =
        DAQpp::DAQmanager::theDAQmanager()->find_runmanager("main");

    if (!rm)
        return;

    try
    {
        rm->set_par<bool>("doInterrupts", do_int);
    }
    catch (DAQpp::ParameterNonExistent &ex)
    {
        rm->add_par("doInterrupts", new DAQpp::Par<bool>(do_int));
    }
}

std::string check_output_files(const std::string &fnam, int &ignore)
{
    char line[1024];
    std::string dir_name, base_name, extension;
    std::string::size_type e = fnam.find_last_of('.');
    if ( e == fnam.npos )
    {
        base_name = fnam;
        extension = "data";
    }
    else
    {
        base_name = fnam.substr(0, e);
        extension = fnam.substr(e + 1, fnam.npos);
    }
    e = fnam.find_last_of('/');
    if (e == fnam.npos)
    {
        dir_name = '.';
    }
    else
    {
        dir_name = fnam.substr(0, e);
    }
    if (access(dir_name.c_str(), W_OK))
    {
        ignore = 1;
        std::ostringstream ostr;
        ostr << "You cannot write in " << dir_name
        << std::endl;
        return ostr.str();
    }

    std::ostringstream ostr;
    ostr << "ls " << base_name << "_*." << extension << " 2>/dev/null";

    FILE *pf = popen(ostr.str().c_str(), "r");
    int n_f_ok = 0;
    int n_w_ok = 0;
    int n_fifo = 0;
    ignore = 0;
    while ( fgets(line, sizeof(line), pf) )
    {
        struct stat bstat;
        line[strlen(line)-1] = 0;
        n_f_ok++;
        //       if (!access(line,W_OK))
        //   n_w_ok++;
        if ( !stat(line, &bstat) )
        {
            if ( bstat.st_mode & S_IWUSR )
            {
                if (S_ISFIFO(bstat.st_mode))
                {
                    n_fifo ++;
                }
                n_w_ok++;
            }
        }
    }
    pclose(pf);

    if (n_f_ok == 0 || n_w_ok == n_fifo)
        return std::string();

    if (n_w_ok != n_f_ok)
        ignore = 1;

    std::ostringstream omsg;
    omsg << "This file series exists already.";
    if (ignore)
    {
        omsg << "You are not allowed to overwrite some of those files."
        << std::endl
        << "Choose another prefix, run number of folder."
        ;
    }
    else
    {
        omsg << "Do you want to overwrite ?";
    }
    return omsg.str();
}


static char sbuf[4096];
static int page_k_shift=0;

int read_proc_file()
{
    std::ostringstream ostr;
    ostr << "/proc/" << getpid() << "/statm" << std::ends;
    std::string proc_file = ostr.str();

    int fd = open(proc_file.c_str(), O_RDONLY);
    if (fd < 0 && !proc_file.empty())
    {
        std::cout << "\n\nCould not open " << proc_file << std::endl << std::endl;
        return -1;
    }
    int iok, r = 0;
    *sbuf=0;
    do
    {
        iok = read(fd, sbuf, 4096);
        if (iok>0)
            r+=iok;
    } while (iok<0 && errno==EINTR);
    close(fd);
    return r;
}
std::string get_proc_mem()
{
    std::string out;
    if (page_k_shift==0)
    {
       int j;
       for (j = getpagesize(); j > 1024; j >>= 1)
          page_k_shift++;
    }
    int nr = read_proc_file();
    if (nr)
    {
        unsigned long size;
        sscanf(sbuf, "%ld", &size);
        size <<= page_k_shift;
        std::ostringstream oval;
        if (size>1048576)
        {
            oval << std::setw(7) << std::setprecision(3) << size/1048576. << " Gb";
        }
        else if (size>1024)
        {
            oval << std::setw(7) << std::setprecision(3) << size/1024. << " Mb";
        }
        else
            oval << std::setw(7) << std::setprecision(3) << size << " kb";

        out = oval.str();
    }
    return out;
}
