#include <gtk/gtkmain.h>
#include <iostream>
#include <DAQ++/DAQmanager.h>
#include "MsgManager.h"
#include "RunMenu.h"
#include "ParamSlider.h"

MsgManager::MsgManager()
{
    DAQpp::DAQmanager::theDAQmanager()->addObserver( *this );
}

/**
 * This routine receives the notifications of DAQmanager
 */
void MsgManager::update(Observable *o, Argument *arg)
{
    Arg< DAQpp::DAQmanager::newObjArg> &msg =
        *static_cast< Arg< DAQpp::DAQmanager::newObjArg> * > (arg);
        DAQpp::Module *module;
        DAQpp::RunManager *run_manager;

        switch (msg().first)
        {
            case DAQpp::DAQmanager::nwModule:
                module = reinterpret_cast<DAQpp::Module *>(msg().second);
                std::cout << "MsgManager: adding module " << module->get_id() << std::endl;
                break;
            case DAQpp::DAQmanager::rmModule:
                module = reinterpret_cast<DAQpp::Module *>(msg().second);
                std::cout << "MsgManager: removing module " << module->get_id() << std::endl;
                break;
            case DAQpp::DAQmanager::nwRunManager:
                run_manager = reinterpret_cast<DAQpp::RunManager *>(msg().second);
                std::cout << "MsgManager: adding run manager " << run_manager->get_id() << std::endl;
                run_manager->get_status().addObserver( theMsgManager()->state );
                break;
            case DAQpp::DAQmanager::rmRunManager:
                run_manager = reinterpret_cast<DAQpp::RunManager *>(msg().second);
                std::cout << "MsgManager: removing run manager " << run_manager->get_id() << std::endl;
                break;
            default:
                break;
        }
}

/**
 * We are notified of any change of state of a RunManager through
 * this method that transmits the corresponding new state to the
 * RunMenu associated to the "changing" RunManager
 * 
 */
void MsgManager::StateChanged::update(Observable *o, Argument *arg)
{
    Arg<DAQpp::RunState::StatusType> *st = static_cast< Arg<DAQpp::RunState::StatusType> * > (arg);
    DAQpp::RunState *state = dynamic_cast<DAQpp::RunState *>(o);
    if ( state == 0 )
            return;

    // Update all the corresponding RunMenu
    RunMenu *rm = RunMenu::find(  state->get_name() );
    if ( rm )
        rm->update( *st );
};

void MsgManager::ParameterChanged::update(Observable *o, Argument *arg)
{
    Arg<int> *v = static_cast< Arg<int> * > (arg);
    DAQpp::Parameter *par = dynamic_cast<DAQpp::Parameter *>(o);

    if ( par == 0 )
    {
        std::cout << "I got a Request of parameter change from a non-parameter"
                  << std::endl;
        return;
    }

    // Update the ParamSliders
    std::string id = par->get_id().str() + "/" + par->get_name();
    std::cout << "Parameter " << id << " changes to " << (*v)() << std::endl;
    ParamSlider *sld = ParamSlider::find( id );
    if ( sld )
        sld->set_value_silent( (*v)() );
};

void MsgManager::ObjectAddedRemoved::update(Observable *o, Argument *arg)
{
    // DO nothing for now

};
