#include <iostream>
#include <DAQ++/DAQmanager.h>
#include <DAQ++/Module.h>
#include "ParamSlider.h"

ParamSlider::SliderList ParamSlider::vlist;

void ParamSlider::init()
{
    cb_id = g_signal_connect(adj,
                             "value_changed",
                             G_CALLBACK (change_value),
                             this);
}

void ParamSlider::delete_all()
{
    SliderList::iterator ip;
    for (ip = vlist.begin();ip != vlist.end();++ip)
        delete ip->second;

    vlist.clear();
}

ParamSlider *ParamSlider::find( const std::string &pid)
{
    SliderList::iterator ip = vlist.find(pid);
    if ( ip == vlist.end() )
        return 0;
    else
        return ip->second;
}

void ParamSlider::remove(const std::string &pid)
{
    SliderList::iterator ip = vlist.find(pid);
    if ( ip == vlist.end() )
        return;

    delete ip->second;
    vlist.erase( ip );
}


ParamSlider *ParamSlider::new_slider( const DAQpp::DAQid &id,
                                      const std::string &p,
                                      GtkAdjustment *o)
{
    ParamSlider* ps = new ParamSlider( id, p, o);
    std::string pid = id() + "/" + p;
    vlist[ pid ] = ps;
    return ps;
}


void ParamSlider::change_value(GtkAdjustment *w, ParamSlider *ptr)
{
    if ( ptr == 0 )
        return;

    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    DAQpp::Module *module;

    try
    {
        module = daqm->find_module(ptr->get_id());
    }
    catch ( ... )
    {
        return;
    }
    if ( module == 0 )
        return;

    try
    {
        set_par_value(module->get_par(ptr->get_par_name().c_str()),
                      (int)ptr->get_value(), true);
    }
    catch (DAQpp::Exception &ex )
    {
        std::cout << "Bad parameter type matching: " << ex << std::endl;
    }
    catch ( ... )
    {
        return;
    }
}
