#include <iostream>
#include <cstring>
#include <config/ConfigParser.h>
#include <config/GeneralConfig.h>
#include <config/config_utils.h>
#include <DAQ++/DAQmanager.h>
#include <daq/VMEmanager.h>
#include <daq/GenModule.h>
#include <daq/DAQcontroller.h>
#include "PluginSettings.h"
#include "PluginHook.h"
#include "gtkutils.h"
#include "gtkmessagebox.h"

enum ModuleHeader
{
    MD_ELEMENT,
    MD_ID,
    MD_ADDRESS,
    MD_NCOL
};

enum PluginHeader
{
    PL_NAME=0,
    PL_ELEMENT,
    PL_HOOK,
    PL_LIB,
    PL_NCOL
};

enum RMHeader
{
    RM_TYPE,
    RM_HOOK,
    RM_LIB,
    RM_NCOL
};

enum DriverHeader
{
    DRV_TYPE,
    DRV_HOOK,
    DRV_LIB,
    DRV_NCOL
};

static void on_cell_edited(GtkCellRendererText *renderer,
                           gchar               *path,
                           gchar               *new_text,
                           gpointer             user_data)
{
    GObject *O = G_OBJECT(g_object_get_data(G_OBJECT(renderer),"tree"));
    if (!O)
        return;

    long col = (long)user_data;
    GtkTreeView *tree = GTK_TREE_VIEW(O);
    GtkTreeModel *model = gtk_tree_view_get_model(tree);
    GtkTreeIter iter;

    if ( gtk_tree_model_get_iter_from_string(model, &iter, path) )
    {
        gtk_tree_store_set(GTK_TREE_STORE(model),&iter, col, new_text, -1);
    }
}


PluginSettings::PluginSettings():
    plugin_store(0), rm_store(0), md_store(0), drv_store(0)
{
}

PluginSettings::~PluginSettings()
{
    if (plugin_store)
        g_object_unref(plugin_store);

    if (rm_store)
        g_object_unref(rm_store);

    if (md_store)
        g_object_unref(md_store);

    if (drv_store)
        g_object_unref(drv_store);
}

void PluginSettings::auto_connect()
{
    SET_CONNECT(on_help);
    SET_CONNECT(on_find_hook);
    SET_CONNECT(on_btnAdd);
    SET_CONNECT(on_btnDelete);
    SET_CONNECT(on_btnUp);
    SET_CONNECT(on_btnDown);
    SET_CONNECT(on_btnEdit);
    SET_CONNECT(on_btnHook);
    SET_CONNECT(on_btnOK);
    SET_CONNECT(on_btnBrowseInt);
    SET_CONNECT(on_btnEnable);
    SET_CONNECT(on_btnDisable);
    SET_CONNECT(on_btnStart);
    SET_CONNECT(on_btnStop);
    SET_CONNECT(on_btnReset);
    SET_CONNECT(on_runmanager_add);
    SET_CONNECT(on_runmanager_remove);
    SET_CONNECT(on_runmanager_hook);
    SET_CONNECT(on_data_logger);
    SET_CONNECT(on_module_add);
    SET_CONNECT(on_module_remove);
    SET_CONNECT(on_driver_add);
    SET_CONNECT(on_driver_remove);
    SET_CONNECT(on_driver_hook);
}

void PluginSettings::user_init()
{
    GtkTreeView *tree = GTK_TREE_VIEW(get_widget("treePlugins"));
    GtkTreeIter iter;
    GtkCellRenderer *txt;

    plugin_store = gtk_tree_store_new(PL_NCOL,
                               G_TYPE_STRING,
                               G_TYPE_STRING,
                               G_TYPE_STRING,
                               G_TYPE_STRING);
    gtk_tree_view_set_model(tree, GTK_TREE_MODEL(plugin_store));
    txt = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                    "Name",
                                                    txt,
                                                    "text", PL_NAME,
                                                    NULL);
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                    "Element",
                                                    gtk_cell_renderer_text_new(),
                                                    "text", PL_ELEMENT,
                                                    NULL);
    txt = gtk_cell_renderer_text_new();
    g_object_set(G_OBJECT(txt),
                 "mode",GTK_CELL_RENDERER_MODE_EDITABLE,
                 "editable", TRUE,
                 NULL);
    g_signal_connect(G_OBJECT(txt),"edited", G_CALLBACK(on_cell_edited),(gpointer)PL_HOOK);
    g_object_set_data(G_OBJECT(txt),"tree",tree);
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                "Hook",
                                                txt,
                                                "text", PL_HOOK,
                                                NULL);

    txt = gtk_cell_renderer_text_new();
    g_object_set(G_OBJECT(txt),
                 "mode",GTK_CELL_RENDERER_MODE_EDITABLE,
                 "editable", TRUE,
                 NULL);
    g_signal_connect(G_OBJECT(txt),"edited", G_CALLBACK(on_cell_edited),(gpointer)PL_LIB);
    g_object_set_data(G_OBJECT(txt),"tree",tree);
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                "Lib",
                                                txt,
                                                "text", PL_LIB,
                                                NULL);


    tree = GTK_TREE_VIEW(get_widget("treeRunManager"));
    rm_store = gtk_tree_store_new(RM_NCOL, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
    gtk_tree_view_set_model(tree, GTK_TREE_MODEL(rm_store));

    txt = gtk_cell_renderer_text_new();
    g_object_set(G_OBJECT(txt),"xalign", 0.0, NULL);
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                "Type",
                                                txt,
                                                "text", RM_TYPE,
                                                NULL);
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                "Hook",
                                                txt,
                                                "text", RM_HOOK,
                                                NULL);
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                "Library",
                                                txt,
                                                "text", RM_LIB,
                                                NULL);



    ConfigParser *cfg = ConfigParser::theConfig();
    xmlNodePtr plg, plugins = cfg->get_plugins_node();
    if (!plugins)
    {
        std::cout << "PluginSettins::user_init: something went wrong" << std::endl;
    }

    std::string name, module, hook, lib;
    for (plg=plugins->children;plg;plg=plg->next)
    {
        if ( !strcmp((char *)plg->name, "plugin") )
        {
            try
            {
                name = cfg->get_attribute<std::string>(plg,"name");
            }
            catch (ConfigException &e) {}
            try
            {
                module = cfg->get_attribute<std::string>(plg,"module");
            }
            catch (ConfigException &e) {}
            try
            {
                hook = cfg->get_value<std::string>(plg,"hook");
            }
            catch (ConfigException &e) {}
            try
            {
                lib = cfg->get_value<std::string>(plg, "lib");
            }
            catch (ConfigException &e) {}

            gtk_tree_store_append(plugin_store,&iter,0);

            gtk_tree_store_set(plugin_store,&iter,
                               PL_NAME, name.c_str(),
                               PL_ELEMENT, module.c_str(),
                               PL_HOOK,hook.c_str() ,
                               PL_LIB, lib.c_str(),
                               -1
            );
        }
        else if (!strcmp((char *)plg->name, "interrupt"))
        {
            try
            {
                lib = cfg->get_value<std::string>(plg, "lib");
                set_text("entryIntLibrary", lib);
            }
            catch (ConfigException &e) {}
            try
            {
                hook = cfg->get_value<std::string>(plg, "enable");
                set_text("entryIntEnable", hook);
            }
            catch (ConfigException &e) {}
            try
            {
                hook = cfg->get_value<std::string>(plg, "disable");
                set_text("entryIntDisable", hook);
            }
            catch (ConfigException &e) {}
            try
            {
                hook = cfg->get_value<std::string>(plg, "start");
                set_text("entryIntStart", hook);
            }
            catch (ConfigException &e) {}
            try
            {
                hook = cfg->get_value<std::string>(plg, "stop");
                set_text("entryIntStop", hook);
            }
            catch (ConfigException &e) {}
            try
            {
                hook = cfg->get_value<std::string>(plg, "reset");
                set_text("entryIntReset", hook);
            }
            catch (ConfigException &e) {}

        }
        else if (!strcmp((char *)plg->name,"run_managers"))
        {
            xmlNodePtr child = plg->children;
            for (;child; child = child->next)
            {
                hook.clear();
                lib.clear();
                try
                {
                    hook = cfg->get_value<std::string>(child, "hook");
                }
                catch (ConfigException &e) {}
                try
                {
                    lib = cfg->get_value<std::string>(child, "lib");
                }
                catch (ConfigException &e) {}

                gtk_tree_store_append(rm_store,&iter,0);
                gtk_tree_store_set(rm_store,&iter,
                                   RM_TYPE, (char *)child->name,
                                   RM_HOOK, hook.c_str(),
                                   RM_LIB, lib.c_str(),
                                   -1
                    );
            }
        }
        else if ( !strcmp((char *)plg->name, "data_logger"))
        {
            hook.clear();
            lib.clear();
            try
            {
                hook = cfg->get_value<std::string>(plg, "hook");
            }
            catch (ConfigException &e) {}
            try
            {
                lib = cfg->get_value<std::string>(plg, "lib");
            }
            catch (ConfigException &e) {}
            set_text("entryDlib",lib);
            set_text("entryDhook", hook);
        }
    }


    /** Modules
     *
     */
    tree = GTK_TREE_VIEW(get_widget("treeModules"));
    md_store = gtk_tree_store_new(MD_NCOL,
                                  G_TYPE_STRING,
                                  G_TYPE_STRING,
                                  G_TYPE_STRING);
    gtk_tree_view_set_model(tree, GTK_TREE_MODEL(md_store));
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                "Element",
                                                gtk_cell_renderer_text_new(),
                                                "text", MD_ELEMENT,
                                                NULL);
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                "ID",
                                                gtk_cell_renderer_text_new(),
                                                "text", MD_ID,
                                                NULL);
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                "Address",
                                                gtk_cell_renderer_text_new(),
                                                "text", MD_ADDRESS,
                                                NULL);

    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    DAQpp::ModulesMap::const_iterator it;
    for (it=daqm->ModuleMap().begin();it!=daqm->ModuleMap().end();it++)
    {
        if (!dynamic_cast<GenModule *>(it->second))
            continue;

        DAQpp::Module *md = it->second;
        std::string element;
        std::ostringstream oaddr;
        try
        {
            element = md->get_par_value<std::string>("__section_name__");
        }
        catch (DAQpp::ParameterNonExistent &e) {}

        GenModule *gm = GenModule::find_module(md->get_id());
        if (gm)
        {
            oaddr << "0x" << std::hex << gm->get_address();
            gtk_tree_store_append(md_store,&iter,0);
            gtk_tree_store_set(md_store,&iter,
                               MD_ELEMENT, element.c_str(),
                               MD_ID, (const char *)md->get_id(),
                               MD_ADDRESS, oaddr.str().c_str(),
                               -1
            );
        }
        else
        {
            std::cout << "**** Should not happen !!! **** " << __FILE__ << "::" << __FUNCTION__ << std::endl;
        }
    }

    /*
     * Drivers
     */
    tree = GTK_TREE_VIEW(get_widget("treeDrivers"));
    drv_store = gtk_tree_store_new(DRV_NCOL,
                                  G_TYPE_STRING,
                                  G_TYPE_STRING,
                                  G_TYPE_STRING);
    gtk_tree_view_set_model(tree, GTK_TREE_MODEL(drv_store));
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                "Driver",
                                                gtk_cell_renderer_text_new(),
                                                "text", DRV_TYPE,
                                                NULL);
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                "Hook",
                                                gtk_cell_renderer_text_new(),
                                                "text", DRV_HOOK,
                                                NULL);
    gtk_tree_view_insert_column_with_attributes(tree,-1,
                                                "Library",
                                                gtk_cell_renderer_text_new(),
                                                "text", DRV_LIB,
                                                NULL);

    GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));

    gtk_tree_store_append(drv_store,&iter,0);
    gtk_tree_store_set(drv_store,&iter,
                       DRV_TYPE, "VME",
                       DRV_HOOK, gnrl->get_vme_factory().c_str(),
                       DRV_LIB, gnrl->get_vme_lib().c_str(),
                       -1
    );

}

void PluginSettings::on_btnOK(GtkWidget *, PluginSettings *ps)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    std::string sval, lib, hook;
    GtkTreeIter iter;
    GtkTreeModel *model;
    gboolean valid;
    xmlNodePtr node, child, plugins = cfg->get_plugins_node();


    /*
     * Stop the run temporarily
     */
    DAQcontroller::instance()->stop_daq();
    /*
     * Setup plugins
     */
    std::map<std::string, std::string>::iterator im;
    for (im=ps->deleted_plugins.begin(); im!=ps->deleted_plugins.end();im++)
    {
        cfg->remove_plugin(im->first);
    }
    ps->deleted_plugins.clear();


    model = GTK_TREE_MODEL(ps->plugin_store);
    valid = gtk_tree_model_get_iter_first(model, &iter);
    while (valid)
    {
        gchar *gname, *gelement, *ghook, *glib;
        gtk_tree_model_get(model, &iter,
                           PL_NAME, &gname,
                           PL_ELEMENT, &gelement,
                           PL_HOOK, &ghook,
                           PL_LIB, &glib,
                           -1);
        xmlNodePtr node = cfg->find_plugin(gname);
        if (!node)
        {
            node = cfg->create_child(plugins,"plugin","");
            cfg->set_attribute<std::string>(node,"name", gname);
            cfg->set_attribute<std::string>(node,"module", gelement);
        }
        cfg->check_value<std::string>(node,"hook",ghook);
        cfg->check_value<std::string>(node,"lib",glib);
        g_free(gname);
        g_free(gelement);
        g_free(ghook);
        g_free(glib);
        valid = gtk_tree_model_iter_next(model, &iter);
    }

    /*
     *  Setup data logger
     */
    lib=ps->get_text("entryDlib");
    hook=ps->get_text("entryDhook");
    if (!hook.empty())
    {
        xmlNodePtr data_logger = cfg->find_element(plugins, "data_logger");
        if (!data_logger)
            data_logger = cfg->create_child(plugins,"data_logger","");

        cfg->check_value<std::string>(data_logger, "lib", lib);
        cfg->check_value<std::string>(data_logger,"hook", hook);
    }
    /*
     * Setup run managers
     */
    xmlNodePtr val, runm = cfg->find_element(plugins, "run_managers");

    model = GTK_TREE_MODEL(ps->rm_store);
    valid = gtk_tree_model_get_iter_first(model, &iter);
    while (valid)
    {
        gchar *gstype, *gslib, *gshook;
        gtk_tree_model_get(model, &iter,
                           RM_TYPE, &gstype,
                           RM_LIB, &gslib,
                           RM_HOOK, &gshook,
                           -1);
        if (!runm)
            runm = cfg->create_child(plugins, "run_managers","");

        val = cfg->find_element(runm,gstype);
        if (!val)
            val =cfg->create_child(runm, gstype,"");

        cfg->check_value<std::string>(val,"lib",gslib);
        cfg->check_value<std::string>(val,"hook", gshook);
        valid = gtk_tree_model_iter_next(model, &iter);
        g_free(gstype);
        g_free(gslib);
        g_free(gshook);
    }

    /*
     * Setup Interrupts
     */
    xmlNodePtr intr = cfg->find_element(plugins,"interrupt");
    if (!intr)
    {
        intr = cfg->create_child(plugins,"interrupt","");
    }

    sval = ps->get_text("entryIntLibrary");
    if (!sval.empty())
    {
        cfg->check_value<std::string>(intr,"lib", sval);
    }

    sval = ps->get_text("entryIntEnable");
    if (!sval.empty())
    {
        cfg->check_value<std::string>(intr, "enable", sval);
    }

    sval = ps->get_text("entryIntDisable");
    if (!sval.empty())
    {
        cfg->check_value<std::string>(intr, "disable", sval);
    }

    sval = ps->get_text("entryIntStart");
    if (!sval.empty())
    {
        cfg->check_value<std::string>(intr, "start", sval);
    }

    sval = ps->get_text("entryIntStop");
    if (!sval.empty())
    {
        cfg->check_value<std::string>(intr, "stop", sval);
    }
    sval = ps->get_text("entryIntReset");
    if (!sval.empty())
    {
        cfg->check_value<std::string>(intr, "reset", sval);
    }


    /*
     * Modules
     */
    /*
     * We first remove the deleted the modules
     */
    for (im=ps->deleted_modules.begin(); im!=ps->deleted_modules.end();im++)
    {
        cfg->remove_module(im->second, im->first);
    }
    ps->deleted_modules.clear();
    /*
     * Now we add the new ones
     */
    xmlNodePtr modules = cfg->get_module_node();
    model = GTK_TREE_MODEL(ps->md_store);
    valid = gtk_tree_model_get_iter_first(model, &iter);
    while (valid)
    {
        gchar *gelement, *gid, *gaddr;
        gtk_tree_model_get(model, &iter,
                           MD_ELEMENT, &gelement,
                           MD_ID, &gid,
                           MD_ADDRESS, &gaddr,
                           -1);
        val = cfg->find_module(gelement, gid);
        if (!val)
        {
            val = cfg->create_child(modules, gelement,"");
            cfg->set_attribute(val,"name", gid);
            cfg->set_attribute(val,"address",gaddr);
        }
        valid = gtk_tree_model_iter_next(model, &iter);
        g_free(gelement);
        g_free(gid);
        g_free(gaddr);
    }

    cfg->save_config( "/tmp/___vmedaq___.xml" );
    cfg->reset();
    if (cfg->parse("/tmp/___vmedaq___.xml"))
    {
        DAQcontroller::instance()->setup_daq(0,0);
    }
    unlink("/tmp/___vmedaq___.xml");

    /*
     *  Now we loop again on the existing modules in order to set
     * their default values in the configuration.
     */
    model = GTK_TREE_MODEL(ps->md_store);
    valid = gtk_tree_model_get_iter_first(model, &iter);
    while (valid)
    {
        gchar *gelement, *gid;
        gtk_tree_model_get(model, &iter,
                           MD_ELEMENT, &gelement,
                           MD_ID, &gid,
                           -1);
        node = cfg->find_module(gelement, gid);
        if (node)
        {
            SectionParser *parser = cfg->get_handler(gelement);
            if (parser)
            {
                parser->save_config_node(node);
            }
        }
        g_free(gelement);
        g_free(gid);
        valid = gtk_tree_model_iter_next(model, &iter);
    }

    /**
     * The drivers
     */
    model = GTK_TREE_MODEL(ps->drv_store);
    valid = gtk_tree_model_get_iter_first(model, &iter);
    while (valid)
    {
        gchar *gtype, *ghook, *glib;
        gtk_tree_model_get(model, &iter,
                           DRV_TYPE, &gtype,
                           DRV_HOOK, &ghook,
                           DRV_LIB, &glib,
                           -1);

        std::string drv_type = downcase(gtype);
        std::string drv_hook = trim_str(ghook);
        std::string drv_lib  = trim_str(glib);
        if (drv_type=="vme")
        {
            if ( drv_hook.empty() && drv_lib.empty() )
            {
                VMEmanager::VME()->set_default_factory();
            }
            else
            {
                VMEmanager::VME()->set_factory(drv_hook, drv_lib);
            }
        }

        g_free(gtype);
        g_free(ghook);
        g_free(glib);
        valid = gtk_tree_model_iter_next(model, &iter);
    }

}

void PluginSettings::on_btnAdd(GtkWidget *, PluginSettings *ps)
{
    GtkTreeIter iter;
    std::vector< std::string > values;
    int nc = get_a_string_list("Plugin name",
                               "Plugin name:Plugin element",
                               values);
    if ( nc!=2 )
    {
        GtkWidget *msg = gtk_messagebox_new("vmedaq",
                                            "I need both the name\nand the element",
                                            MSG_BOX_CRITICAL,
                                            MSG_BOX_OK);
        gtk_messagebox_exec(GTK_MESSAGEBOX(msg));
        return;
    }
    if ( values[0].empty() || values[1].empty() )
    {
        GtkWidget *msg = gtk_messagebox_new("vmedaq",
                                            "I need both the name\nand the element",
                                            MSG_BOX_CRITICAL,
                                            MSG_BOX_OK);
        gtk_messagebox_exec(GTK_MESSAGEBOX(msg));
        return;
    }
    gtk_tree_store_append(ps->plugin_store,&iter,0);
    gtk_tree_store_set(ps->plugin_store,&iter,
                       PL_NAME, values[0].c_str(),
                       PL_ELEMENT, values[1].c_str(),
                       PL_HOOK,"" ,
                       PL_LIB, "",
                       -1
                );


}
void PluginSettings::on_btnDelete(GtkWidget *, PluginSettings *ps)
{
    GtkTreeModel *model;
    GtkTreeIter iter;
    GtkTreeView *list = GTK_TREE_VIEW( ps->get_widget("treePlugins") );
    GtkTreeSelection  *selection = gtk_tree_view_get_selection(list);
    if ( gtk_tree_selection_get_selected (selection, &model, &iter))
    {
        gchar *gname,*gelem;
        gtk_tree_model_get(model, &iter,
                           PL_NAME, &gname,
                           PL_ELEMENT, &gelem,
                           -1);
        ps->deleted_plugins[gname]=gelem;
        g_free(gname);
        g_free(gelem);
        gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
    }
}
void PluginSettings::on_btnEdit(GtkWidget *, PluginSettings *)
{
    GtkWidget *msg = gtk_messagebox_new("vmedaq","Not yet implemented",MSG_BOX_INFORMATION,MSG_BOX_OK);
    gtk_messagebox_exec(GTK_MESSAGEBOX(msg));
}
void PluginSettings::on_btnUp(GtkWidget *, PluginSettings *ps)
{
    ps->move_row(TRUE);
}
void PluginSettings::on_btnDown(GtkWidget *, PluginSettings *ps)
{
    ps->move_row(FALSE);
}
void PluginSettings::on_btnHook(GtkWidget *, PluginSettings *ps)
{
    GtkTreeView *cl = GTK_TREE_VIEW(ps->get_widget("treePlugins"));
    GtkTreeModel *model;
    GtkTreeIter iter;

    GtkTreeSelection  *selection = gtk_tree_view_get_selection(cl);
    if ( gtk_tree_selection_get_selected (selection, &model, &iter))
    {
        gchar *libname;
        gtk_tree_model_get(model, &iter,PL_LIB, &libname,-1);
        PluginHook *ph = new PluginHook(libname);
        ph->init( ps->get_file_name(), "hook_dialog");
        g_free(libname);
        int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
        if (rc == GTK_RESPONSE_OK)
        {
            gtk_tree_store_set(GTK_TREE_STORE(model),&iter,
                               PL_LIB, ph->lib().c_str(),
                               PL_HOOK, ph->hook().c_str(),
                               -1
                               );
        }
        delete ph;
    }
}

void PluginSettings::on_runmanager_add(GtkWidget *, PluginSettings *ps)
{
    const char *rtypes[] = {"data","int_trg","pulse", "pedestal", "scan", 0};
    std::vector< std::string > values;
    int nc = get_a_string_list("RunManager plugin",
                               "Type - any of data, int_trg, pulse, pedestal or scan:Hook:Library",
                               values);

    std::string run_type = downcase(values[0]);
    bool found=false;
    int  ityp;
    for (ityp=0;rtypes[ityp];ityp++)
    {
        if ( !strcmp(rtypes[ityp], run_type.c_str()))
        {
            found = true;
            break;
        }
    }
    if (!found)
    {
        GtkWidget *msg = gtk_messagebox_new("vmedaq",
                                            "A RunManager type must be any of:\n"
                                            "   data\n   int_trg\n   pulse\n   pedestal\n   scan\n" ,
                                            MSG_BOX_CRITICAL,
                                            MSG_BOX_OK);
        gtk_messagebox_exec(GTK_MESSAGEBOX(msg));
        return;
    }
    GtkTreeIter iter;
    gtk_tree_store_append(ps->rm_store,&iter,0);
    gtk_tree_store_set(ps->rm_store,&iter,
                       RM_TYPE, run_type.c_str(),
                       RM_HOOK, values[1].c_str(),
                       RM_LIB, values[2].c_str(),
                       -1
                    );
}
void PluginSettings::on_runmanager_remove(GtkWidget *, PluginSettings *ps)
{
    GtkTreeModel *model;
    GtkTreeIter iter;
    GtkTreeView *list = GTK_TREE_VIEW( ps->get_widget("treeRunManager") );
    GtkTreeSelection  *selection = gtk_tree_view_get_selection(list);
    if ( gtk_tree_selection_get_selected (selection, &model, &iter))
    {
        gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
    }
}

void PluginSettings::on_runmanager_hook(GtkWidget *, PluginSettings *ps)
{
    GtkTreeView *cl = GTK_TREE_VIEW(ps->get_widget("treeRunManager"));
    GtkTreeModel *model;
    GtkTreeIter iter;

    GtkTreeSelection  *selection = gtk_tree_view_get_selection(cl);
    if ( gtk_tree_selection_get_selected (selection, &model, &iter))
    {
        gchar *libname;
        gtk_tree_model_get(model, &iter,RM_LIB, &libname,-1);
        PluginHook *ph = new PluginHook(libname);
        ph->init( ps->get_file_name(), "hook_dialog");
        g_free(libname);
        int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
        if (rc == GTK_RESPONSE_OK)
        {
            gtk_tree_store_set(GTK_TREE_STORE(model),&iter,
                               RM_LIB, ph->lib().c_str(),
                               RM_HOOK, ph->hook().c_str(),
                               -1
            );
        }
        delete ph;
    }
}

void PluginSettings::on_data_logger(GtkWidget *w, PluginSettings *ps)
{
    std::string lib = ps->get_text("entryDlib");
    PluginHook *ph = new PluginHook(lib);
    ph->init( ps->get_file_name(), "hook_dialog");
    int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
    if (rc == GTK_RESPONSE_OK)
    {
        ps->set_text("entryDlib",ph->lib());
        ps->set_text("entryDhook", ph->hook());
    }
    delete ph;
}

void PluginSettings::on_find_hook(GtkWidget *, PluginSettings *ps)
{
    PluginHook *ph = new PluginHook();
    ph->init( ps->get_file_name(), "hook_dialog");
    int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
    if (rc == GTK_RESPONSE_OK)
    {
        ps->set_text("lblLib", ph->lib());
        ps->set_text("lblHook", ph->hook());
    }
    delete ph;
}

void PluginSettings::on_help(GtkWidget *, PluginSettings *P)
{

}

void
PluginSettings::move_row(gboolean up)
{
    GtkTreeView *cl = GTK_TREE_VIEW(get_widget("treePlugins"));
    GtkTreeModel *model;
    GtkTreeIter iter, iter1;

    GtkTreeSelection  *selection = gtk_tree_view_get_selection(cl);
    if ( gtk_tree_selection_get_selected (selection, &model, &iter))
    {
        GtkTreePath *path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);

        if ( up )
            gtk_tree_path_prev (path);
        else
            gtk_tree_path_next (path);

        if ( gtk_tree_model_get_iter (model, &iter1, path) )
        {
            gtk_tree_store_swap( GTK_TREE_STORE(model), &iter, &iter1);
        }
    }
}



void PluginSettings::on_btnBrowseInt(GtkWidget *, PluginSettings *ps)
{
    const char *filters[] = {"*.so", 0};
    std::string ofile = ::get_file_name("Find shared library", TRUE, filters);
    if (!ofile.empty())
    {
        ps->set_text("entryIntLibrary", ofile);
    }
}
void PluginSettings::on_btnEnable(GtkWidget *, PluginSettings *ps)
{
    PluginHook *ph = new PluginHook(ps->get_text("entryIntLibrary"));
    ph->init( ps->get_file_name(), "hook_dialog");
    int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
    if (rc == GTK_RESPONSE_OK)
    {
        ps->set_text("entryIntEnable", ph->hook());
    }
    delete ph;
}
void PluginSettings::on_btnDisable(GtkWidget *, PluginSettings *ps)
{
    PluginHook *ph = new PluginHook(ps->get_text("entryIntLibrary"));
    ph->init( ps->get_file_name(), "hook_dialog");
    int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
    if (rc == GTK_RESPONSE_OK)
    {
        ps->set_text("entryIntDisable", ph->hook());
    }
    delete ph;
}
void PluginSettings::on_btnStart(GtkWidget *, PluginSettings *ps)
{
    PluginHook *ph = new PluginHook(ps->get_text("entryIntLibrary"));
    ph->init( ps->get_file_name(), "hook_dialog");
    int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
    if (rc == GTK_RESPONSE_OK)
    {
        ps->set_text("entryIntStart", ph->hook());
    }
    delete ph;
}
void PluginSettings::on_btnStop(GtkWidget *, PluginSettings *ps)
{
    PluginHook *ph = new PluginHook(ps->get_text("entryIntLibrary"));
    ph->init( ps->get_file_name(), "hook_dialog");
    int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
    if (rc == GTK_RESPONSE_OK)
    {
        ps->set_text("entryIntStop", ph->hook());
    }
    delete ph;

}
void PluginSettings::on_btnReset(GtkWidget *, PluginSettings *ps)
{
    PluginHook *ph = new PluginHook(ps->get_text("entryIntLibrary"));
    ph->init( ps->get_file_name(), "hook_dialog");
    int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
    if (rc == GTK_RESPONSE_OK)
    {
        ps->set_text("entryIntReset", ph->hook());
    }
    delete ph;
}

void PluginSettings::on_module_add(GtkWidget *, PluginSettings *h)
{
    int nvar;
    std::vector< std::string> values;
    nvar = get_a_string_list("New module","Element:ID:Address", values);
    if (nvar==3)
    {
        GtkTreeIter iter;
        gtk_tree_store_append(h->md_store,&iter,0);
        gtk_tree_store_set(h->md_store,&iter,
                           MD_ELEMENT, values[0].c_str(),
                           MD_ID, values[1].c_str(),
                           MD_ADDRESS, values[2].c_str(),
                           -1
                );
    }
}
void PluginSettings::on_module_remove(GtkWidget *, PluginSettings *h)
{
    GtkTreeModel *model;
    GtkTreeIter iter;
    GtkTreeView *list = GTK_TREE_VIEW( h->get_widget("treeModules") );
    GtkTreeSelection  *selection = gtk_tree_view_get_selection(list);
    if ( gtk_tree_selection_get_selected (selection, &model, &iter))
    {
        gchar *module_id, *element;
        gtk_tree_model_get(model, &iter,
                           MD_ID, &module_id,
                           MD_ELEMENT, &element,
                           -1);
        h->deleted_modules[module_id]=element;
        g_free(module_id);
        g_free(element);
        gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
    }

}

void PluginSettings::on_driver_add(GtkWidget *, PluginSettings *ps)
{
    std::string driver = get_a_string("Driver type","Type the driver identifier");
    GtkTreeIter iter;
    gtk_tree_store_append(ps->drv_store,&iter,0);
    gtk_tree_store_set(ps->drv_store,&iter,
                       DRV_TYPE, driver.c_str(),
                       DRV_HOOK, "",
                       DRV_LIB, "",
                       -1
    );
}
void PluginSettings::on_driver_remove(GtkWidget *, PluginSettings *h)
{
    GtkTreeModel *model;
    GtkTreeIter iter;
    GtkTreeView *list = GTK_TREE_VIEW( h->get_widget("treeDrivers") );
    GtkTreeSelection  *selection = gtk_tree_view_get_selection(list);
    if ( gtk_tree_selection_get_selected (selection, &model, &iter))
    {
        gtk_tree_store_set(GTK_TREE_STORE(model),&iter,
                           DRV_HOOK,"",
                           DRV_LIB,"",
                           -1);
    }
}
void PluginSettings::on_driver_hook(GtkWidget *, PluginSettings *ps)
{
    GtkTreeView *cl = GTK_TREE_VIEW(ps->get_widget("treeDrivers"));
    GtkTreeModel *model;
    GtkTreeIter iter;

    GtkTreeSelection  *selection = gtk_tree_view_get_selection(cl);
    if ( gtk_tree_selection_get_selected (selection, &model, &iter))
    {
        gchar *libname;
        gtk_tree_model_get(model, &iter,DRV_LIB, &libname,-1);
        PluginHook *ph = new PluginHook(libname);
        ph->init( ps->get_file_name(), "hook_dialog");
        g_free(libname);
        int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
        if (rc == GTK_RESPONSE_OK)
        {
            std::pair<std::string, std::string> jp = split_path(ph->lib());
            std::cout << "lib folder: " << jp.first << " lib name " << jp.second << std::endl;
            gtk_tree_store_set(GTK_TREE_STORE(model),&iter,
                               DRV_LIB, ph->lib().c_str(),
                               DRV_HOOK, ph->hook().c_str(),
                               -1
            );
        }
        delete ph;
    }
}
