
#include <stdio.h>
#include <gtk/gtksignal.h>
#include <gtk/gtk.h>
#include "SpinSlider.h"

enum {
    CHANGED,
    LAST_SIGNAL
};

static GtkTooltips *tooltips = 0;

static void spinslider_class_init (SpinSliderClass * klass);
static void spinslider_init (SpinSlider * spsl);
static void spinslider_value_changed (GtkWidget * widget, SpinSlider * spsl);
static void spinslider_size_allocate (GtkWidget * widget, GtkAllocation * allocation);
static void spinslider_size_request (GtkWidget *, GtkRequisition *);
//static void spinslider_send_configure (SpinSlider * gg);

static guint spinslider_signals[LAST_SIGNAL] = { 0 };

GType spinslider_get_type ()
{
    static guint spsl_type = 0;

    if (!spsl_type) {
	static const GTypeInfo spsl_info = {
	    sizeof (SpinSliderClass),
	    NULL,           /* base_init */
	    NULL,           /* base_finalize */
	    (GClassInitFunc)spinslider_class_init,
	    NULL,           /* class_finalize */
	    NULL,           /* class_data */
	    sizeof (SpinSlider),
	    0,              /* n_preallocs */
	    (GInstanceInitFunc)spinslider_init
	};

	spsl_type =
	    g_type_register_static (GTK_TYPE_FIXED,
				    "SpinSlider",
				    &spsl_info,
				    0);
    }

    return spsl_type;
}

static void spinslider_class_init (SpinSliderClass * class)
{
    GtkObjectClass *object_class;
    GtkWidgetClass *widget_class;
    object_class = (GtkObjectClass *) class;
    widget_class = (GtkWidgetClass *) class;
    spinslider_signals[CHANGED] =
	g_signal_new ("value_changed",
		      G_TYPE_FROM_CLASS (class),
		      GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
		      GTK_SIGNAL_OFFSET (SpinSliderClass, spinslider),
		      NULL, NULL,
		      gtk_marshal_NONE__NONE,G_TYPE_NONE, 0);
	
    widget_class->size_allocate = spinslider_size_allocate;
    widget_class->size_request = spinslider_size_request;

    class->spinslider = NULL;
}

static void spinslider_init (SpinSlider * spsl)
{
    if (tooltips == 0)
	tooltips = gtk_tooltips_new ();
    
    spsl->adj = (GtkAdjustment *) gtk_adjustment_new (0., 0., 0., 0., 0., 0.);
    spsl->mScale = (GtkHScale *) gtk_hscale_new (spsl->adj);
    spsl->mSpin = (GtkSpinButton *) gtk_spin_button_new (spsl->adj, 0, 1);
    gtk_scale_set_draw_value (GTK_SCALE (spsl->mScale), 0);
    gtk_tooltips_set_tip (tooltips,
			  GTK_WIDGET(spsl->mScale),
			  "Move the tab to change the value on the spin", 0);
    gtk_tooltips_set_tip (tooltips,
			  GTK_WIDGET(spsl->mSpin),
			  "Type a value and the scale tab will move accordingly", 0);
    gtk_fixed_put (GTK_FIXED (spsl), (GtkWidget *) spsl->mSpin, 0, 0);
    gtk_fixed_put (GTK_FIXED (spsl), (GtkWidget *) spsl->mScale, 45, 0);
    gtk_signal_connect (GTK_OBJECT (spsl->adj), "value_changed", GTK_SIGNAL_FUNC (spinslider_value_changed), spsl);

    gtk_spin_button_set_snap_to_ticks (spsl->mSpin, TRUE);
    gtk_spin_button_set_numeric (spsl->mSpin, TRUE);
    spsl->ratio = 0.2;
}

GtkWidget *gtk_spinslider_new (gfloat v, gfloat lw, gfloat up, gfloat stp, gfloat pg, gfloat pg_siz)
{

    SpinSlider *spsl = gtk_type_new (spinslider_get_type ());
    spsl->adj->lower = lw;
    spsl->adj->upper = up;
    spsl->adj->step_increment = stp;
    spsl->adj->page_increment = pg;
    spsl->adj->page_size = pg_siz;
    spsl->ratio = 0.2;
    gtk_adjustment_set_value (spsl->adj, v);
    return GTK_WIDGET (spsl);
}

void gtk_spinslider_set_ratio (SpinSlider * spsl, float x)
{
    spsl->ratio = x;
}

gfloat gtk_spinslider_get_value (SpinSlider * spsl)
{
//  g_return_if_fail (spsl != NULL);
//  g_return_if_fail (IS_SPINSLIDER (spsl));
    return spsl->adj->value;
}

void gtk_spinslider_set_digits (SpinSlider * spsl, int nd)
{
    gtk_spin_button_set_digits (spsl->mSpin, nd);
}

void gtk_spinslider_set_value (SpinSlider * w, gfloat v)
{
    gtk_adjustment_set_value (w->adj, v);
}

void gtk_spinslider_reset (SpinSlider * spsl, gfloat lw, gfloat up, gfloat stp, gfloat pg)
{
    spsl->adj->lower = lw;
    spsl->adj->upper = up;
    spsl->adj->step_increment = stp;
    spsl->adj->page_increment = pg;
    gtk_adjustment_changed (GTK_ADJUSTMENT (spsl->adj));
//   gtk_adjustment_clamp_page (spsl->adj,lw,up);
}

void spinslider_value_changed (GtkWidget * widget, SpinSlider * spsl)
{
    gtk_signal_emit (GTK_OBJECT (spsl), spinslider_signals[CHANGED]);
}


void spinslider_size_request (GtkWidget * w, GtkRequisition * requisition)
{
    GtkWidget *child;
    GtkRequisition child_req;
    SpinSlider *spsl;
    if (w == NULL)
	return;
    if (!IS_SPINSLIDER (w))
	return;
    if (requisition == NULL)
	return;

    spsl = SPINSLIDER (w);
    requisition->width = 0;
    requisition->height = 0;

    child = (GtkWidget *) spsl->mSpin;
    gtk_widget_size_request (child, &child_req);
    requisition->height = MAX (child_req.height, requisition->height);
    requisition->width += child_req.width;

    child = (GtkWidget *) spsl->mScale;
    gtk_widget_size_request (child, &child_req);
    requisition->height = MAX (child_req.height, requisition->height);
    requisition->width += child_req.width;

}
void spinslider_size_allocate (GtkWidget * widget, GtkAllocation * allocation)
{
    GtkAllocation aszspin, aszscale;
    SpinSlider *spsl = (SpinSlider *) widget;
    if (widget == NULL)
	return;
    if (!IS_SPINSLIDER (widget))
	return;
    if (allocation == NULL)
	return;

    widget->allocation = *allocation;
    aszspin.x = 0;
    aszspin.y = 0;
    aszspin.width = spsl->ratio * allocation->width;
    aszspin.height = allocation->height;
    aszscale.x = aszspin.x + aszspin.width;
    aszscale.y = 0;
    aszscale.width = allocation->width - aszspin.width;
    aszscale.height = allocation->height;
    gtk_widget_size_allocate ((GtkWidget *) spsl->mSpin, &aszspin);
    gtk_widget_size_allocate ((GtkWidget *) spsl->mScale, &aszscale);

    if (GTK_WIDGET_REALIZED (widget)) {
//	gdk_window_move_resize (widget->window, allocation->x, allocation->y, allocation->width, allocation->height);

//	spinslider_send_configure (SPINSLIDER (widget));
    }

}


/*
static void spinslider_send_configure (SpinSlider * gg)
{
    GtkWidget *widget;
    GdkEventConfigure event;

    widget = GTK_WIDGET (gg);

    event.type = GDK_CONFIGURE;
    event.window = widget->window;
    event.send_event = TRUE;
    event.x = widget->allocation.x;
    event.y = widget->allocation.y;
    event.width = widget->allocation.width;
    event.height = widget->allocation.height;

    gtk_widget_event (widget, (GdkEvent *) & event);
}
*/
