#include <iostream>
#include <sstream>
#include <iomanip>
#include <cstring>
#include "DAQ++/DAQmanager.h"
#include "WidgetUpdate.h"

#include <gtk-hst/histogram.h>
#include <gtk-hst/gtkgauge.h>
#include <gtk-hst/gtkhistogram.h>

void RunManager_update(const std::string &id, Update &up)
{
    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    if ( daqm == 0 )
        return;

    DAQpp::RunManager *runm = daqm->find_runmanager(id.c_str());
    if ( !runm )
    {
        memset(&up, 0, sizeof(Update));
        return;
    }
    DAQpp::RunState &st = runm->get_status();
    up.id = runm->get_id().str();
    up.ntrig = st.n_trigger();
    up.nwriten = st.n_written();
    up.eff = st.eff();
    up.rate = st.rate();
    up.time = st.time();
    up.throughput = st.get_throughput();

}
void Module_update(const std::string &id, Update &up)
{
    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    if ( daqm == 0 )
        return;

    DAQpp::Module *dpx = daqm->find_module(id);
    if (!dpx)
    {
        memset(&up, 0, sizeof(Update));
        return;
    }

    DAQpp::RunState &st = dpx->get_status();
    up.id = dpx->get_id().str();
    up.ntrig = st.n_trigger();
    up.nwriten = st.n_written();
    up.eff = st.eff();
    up.rate = st.rate();
    up.time = st.time();
    up.throughput = st.get_throughput();

}

static void get_update(std::vector<Update> &rml, std::vector<Update> &mdl)
{
    int index;
    rml.clear();
    mdl.clear();
    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    if ( daqm == 0 )
        return;
    // Run managers
    DAQpp::RunManagerMap runm = daqm->RunMmap();
    DAQpp::RunManagerMap::iterator it;
    for ( index = 0, it = runm.begin();it != runm.end(); ++it, index++ )
    {
        DAQpp::RunState &st   = it->second->get_status();
        Update up;
        up.id         = it->second->get_id().str();
        up.ntrig      = st.n_trigger();
        up.nwriten    = st.n_written();
        up.eff        = st.eff();
        up.rate       = st.rate();
        up.time       = st.time();
        up.throughput = st.get_throughput();
        rml.push_back(up);
    }

    // Modules
    DAQpp::ModulesMap mmap = daqm->ModuleMap();
    DAQpp::ModulesMap::iterator itm;
    for ( index = 0, itm = mmap.begin();itm != mmap.end(); ++itm, index++ )
    {

        DAQpp::RunState &st   = itm->second->get_status();
        Update up;
        up.id         = itm->second->get_id().str();
        up.ntrig      = st.n_trigger();
        up.nwriten    = st.n_written();
        up.eff        = st.eff();
        up.rate       = st.rate();
        up.time       = st.time();
        up .throughput = st.get_throughput();
        mdl.push_back( up );
    }
}

void Update_all(int &nRm, Update **rml, int &nMd, Update **mdl)
{
    nRm = nMd = 0;
    *rml = * mdl = 0;

    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    if ( daqm == 0 )
        return;


    std::vector<Update> vrml, vmdl;
    get_update(vrml, vmdl);

    int i;
    nRm = vrml.size();
    *rml = new Update[nRm];
    for (i = 0;i < nRm;i++)
    {
        (*rml)[i].id = vrml[i].id;
        (*rml)[i].ntrig = vrml[i].ntrig;
        (*rml)[i].nwriten = vrml[i].nwriten;
        (*rml)[i].eff = vrml[i].eff;
        (*rml)[i].rate = vrml[i].rate;
        (*rml)[i].time = vrml[i].time;
        (*rml)[i].throughput = vrml[i].throughput;
    }

    nMd = vmdl.size();
    *mdl = new Update[nMd];
    for (i = 0;i < nMd;i++)
    {
        (*mdl)[i].id = vmdl[i].id;
        (*mdl)[i].ntrig = vmdl[i].ntrig;
        (*mdl)[i].nwriten = vmdl[i].nwriten;
        (*mdl)[i].eff = vmdl[i].eff;
        (*mdl)[i].rate = vmdl[i].rate;
        (*mdl)[i].time = vmdl[i].time;
        (*mdl)[i].throughput = vmdl[i].throughput;
    }
}



//---------------------------------------------------------------------
// Class Widget
//---------------------------------------------------------------------
Widget::~Widget()
{
    if (w && GTK_IS_WIDGET(w) )
    {
        GList *lst = (GList *)g_object_get_data(G_OBJECT(w), "WidgetPtr");
        if (!lst)
        {
            std::cout << "Widget::~Widget() : the widget does not have the list"
            << std::endl
            << "\tThis should not have happened!!!"
            << std::endl;
        }
        else
        {
            // remove ourselves from the list
            if (!disconnecting)
            {
                lst = g_list_remove(lst, this);
                g_object_set_data(G_OBJECT(w), "WidgetPtr", lst);
            }
        }
        g_object_unref(G_OBJECT(w));
    }
}

void Widget::set_widget(GtkWidget *o)
{
    if (o == 0)
    {
        w = 0;
        return;
    }

    g_return_if_fail( GTK_IS_WIDGET(o) );

    // Get the list of Widget owning this GtkWidget
    GList *lst = (GList *)g_object_get_data(G_OBJECT(o), "WidgetPtr");
    if ( !lst )
    {
        // Create a new onw
        lst = g_list_append(lst, this);
        g_object_set_data(G_OBJECT(o), "WidgetPtr", lst);
        gtk_signal_connect(GTK_OBJECT(o), "destroy",
                           GTK_SIGNAL_FUNC(disconnect),
                           NULL);
    }
    else
    {
        // Check if we are already there and if not, add ourselves to the list
        GList *tmp = g_list_find(lst, this);
        if (!tmp)
        {
            lst = g_list_append(lst, this);
            g_object_set_data(G_OBJECT(o), "WidgetPtr", lst);
        }
    }
    w = o;
    g_object_ref(w);
}

void Widget::disconnect(GtkWidget *wdgt, gpointer ptr)
{
    GList *lst = (GList *)g_object_get_data(G_OBJECT(wdgt), "WidgetPtr");
    if ( !lst )
        return;

    for (;lst;)
    {
        Widget *wu = reinterpret_cast<Widget *>( lst->data );
        if (wu)
        {
            wu->disconnecting = true;
            delete wu;
        }

        lst = g_list_next(lst);
    }
    g_list_free(lst);
}

//---------------------------------------------------------------------
// Updates
//---------------------------------------------------------------------
int UpdateLabelNtrig::update(Update & ptr)
{
    std::ostringstream ostr;
    ostr << ptr.ntrig << std::ends;
    gtk_label_set_text(GTK_LABEL(wdgt()), ostr.str().c_str());
    return 0;
}

int UpdateLabelTime::update(Update & ptr)
{
    std::ostringstream ostr;
    int tt, seconds, minutes, hours, days;

    tt = (int)ptr.time;
    seconds = tt % 60;
    minutes = (tt / 60) % 60;
    hours   = ( tt / 3600 ) % 3600;
    days    = tt / 86400;

    ostr << days << " "
    << std::setfill(' ') << std::setw(2) << hours << ":"
    << std::setfill('0') << std::setw(2) << minutes << ":"
    << std::setfill('0') << std::setw(2) << seconds
    << std::ends;

    gtk_label_set_text(GTK_LABEL(wdgt()), ostr.str().c_str());

    return 0;
}

int UpdateGaugeEff::update( Update & ptr)
{
    gtk_gauge_set_value(GTK_GAUGE(wdgt()), ptr.eff);
    return 0;
}

int UpdateGaugeRate::update( Update & ptr)
{
    gtk_gauge_set_value(GTK_GAUGE(wdgt()), ptr.rate);
    return 0;
}

int UpdateLabelRate::update( Update & ptr)
{
    std::ostringstream ostr;
    ostr << std::setiosflags(std::ios::fixed )
        << std::setw(6) << std::setprecision(1) << ptr.rate
        << std::resetiosflags ( std::ios::fixed )
            << std::ends;
    gtk_label_set_text(GTK_LABEL(wdgt()), ostr.str().c_str());
    return 0;
}

int UpdateThroughPut::update( Update &ptr )
{
    GtkHst1D *hst;
    double dT, dX;
    HstHistogram *tracer;
    if ( ! GTK_IS_HST1D(wdgt()) )
        return -1;

    hst = GTK_HST1D(wdgt());
    if ( ! HST_IS_HISTOGRAM( hst->func ) )
        return -1;

    dX = (ptr.throughput - thrp) / 1024.;
    dT = (ptr.time == tm0 ? ptr.time : ptr.time - tm0);
    tracer = HST_HISTOGRAM(hst->func);
    hst_histogram_fill(tracer, dX / dT, 1.);

    tm0 = ptr.time;
    thrp = ptr.throughput;
    return 0;
}
