#include <gtk/gtksignal.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkhbbox.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtkmain.h>
#include <gtk/gtkpixmap.h>
#include "gtkmessagebox.h"

static GtkDialog *parent_class = NULL;
static void messagebox_class_init (GtkMessageboxClass * msg);
static void messagebox_init (GtkMessagebox * msg);
static void messagebox_destroy (GtkObject *);
static void messagebox_button (GtkWidget *, gpointer);
static void messagebox_done (GtkMessagebox *, GtkMessageboxButton);

/* XPM */
static const char *none_xpm[] = {
/* w95mbx02 pixmap
 * width height ncolors chars_per_pixel */
    "32 32 6 1",
/* colors */
    " 	s none	m white	c none",
    ".	s s_darkgray	m black	c #808080",
    "X	s s_gray	m white	c #c0c0c0",
    "o	s s_white	m white	c white",
    "O	s s_blue	m white	c #0000FF",
    "+	s s_black	m black	c black",
     /**/ "           ........             ",
    "        ...XooooooX...          ",
    "      ..XooooooooooooX..        ",
    "     .XooooooooooooooooX.       ",
    "    .oooooooo.OO.Xooooooo+      ",
    "   .oooooooXOOOOOOXooooooo+     ",
    "  .oooooooXOXooOOOOXooooooo+    ",
    " .XoooooooOOooooOOOOoooooooX+   ",
    " .ooooooooOOOOooOOOOoooooooo+.  ",
    ".XooooooooOOOOoXOOOOooooooooX+. ",
    ".oooooooooXOOXoOOOOoooooooooo+. ",
    ".oooooooooooooXOOOooooooooooo+..",
    ".oooooooooooooOOOoooooooooooo+..",
    ".oooooooooooooOOXoooooooooooo+..",
    ".oooooooooooooOOooooooooooooo+..",
    ".XooooooooooooooooooooooooooX+..",
    " .oooooooooooXOOXooooooooooo+...",
    " .XooooooooooOOOOooooooooooX+...",
    "  .ooooooooooOOOOoooooooooo+... ",
    "   +oooooooooXOOXooooooooo+.... ",
    "    +oooooooooooooooooooo+....  ",
    "     +XooooooooooooooooX+....   ",
    "      ++XooooooooooooX++....    ",
    "       .+++XooooooX+++.....     ",
    "        ...+++Xooo+.......      ",
    "          ....+ooo+.....        ",
    "             .+ooo+..           ",
    "               +oo+..           ",
    "                +o+..           ",
    "                 ++..           ",
    "                  ...           ",
    "                   ..           "
};

/* XPM */
static const char *information_xpm[] = {
/* w95mbx04 pixmap
 * width height ncolors chars_per_pixel */
    "32 32 6 1 ",
/* colors */
    "0 c #000000   m black  s s_black",
    "a c #808080   m black  s s_darkgray",
    "A c #c0c0c0   m white  s s_gray",
    "B c #0000FF   m white  s s_blue",
    "W c #FFFFFF   m white  s s_white",
    "  c none      m white  s none",
    "           aaaaaaaa             ",
    "        aaaAWWWWWWAaaa          " "      aaAWWWWWWWWWWWWAaa        ",
    "     aAWWWWWWWWWWWWWWWWAa       ",
    "    aWWWWWWWABBBBAWWWWWWW0      ",
    "   aWWWWWWWWBBBBBBWWWWWWWW0     ",
    "  aWWWWWWWWWBBBBBBWWWWWWWWW0    ",
    " aAWWWWWWWWWABBBBAWWWWWWWWWA0   ",
    " aWWWWWWWWWWWWWWWWWWWWWWWWWW0a  ",
    "aAWWWWWWWWWWWWWWWWWWWWWWWWWWA0a ",
    "aWWWWWWWWWWBBBBBBBWWWWWWWWWWW0a ",
    "aWWWWWWWWWWWWBBBBBWWWWWWWWWWW0aa",
    "aWWWWWWWWWWWWBBBBBWWWWWWWWWWW0aa",
    "aWWWWWWWWWWWWBBBBBWWWWWWWWWWW0aa",
    "aWWWWWWWWWWWWBBBBBWWWWWWWWWWW0aa",
    "aAWWWWWWWWWWWBBBBBWWWWWWWWWWA0aa",
    " aWWWWWWWWWWWBBBBBWWWWWWWWWW0aaa",
    " aAWWWWWWWWWWBBBBBWWWWWWWWWA0aaa",
    "  aWWWWWWWWBBBBBBBBBWWWWWWW0aaa ",
    "   0WWWWWWWWWWWWWWWWWWWWWW0aaaa ",
    "    0WWWWWWWWWWWWWWWWWWWW0aaaa  ",
    "     0AWWWWWWWWWWWWWWWWA0aaaa   ",
    "      00AWWWWWWWWWWWWA00aaaa    ",
    "       a000AWWWWWWA000aaaaa     ",
    "        aaa000AWWW0aaaaaaa      ",
    "          aaaa0WWW0aaaaa        ",
    "             a0WWW0aa           ",
    "               0WW0aa           ",
    "                0W0aa           ",
    "                 00aa           ",
    "                  aaa           ",
    "                   aa           ",
};

/* XPM */
static const char *critical_xpm[] = {
/* w95mbx01 pixmap
 * width height ncolors chars_per_pixel */
    "32 32 6 1 ",
/* colors */
    "0 c #000000   m black  s s_black",
    "r c #800000   m black  s s_darkred",
    "a c #808080   m black  s s_darkgray",
    "R c #FF0000   m white  s s_red",
    "W c #FFFFFF   m white  s s_white",
    "  c none      m white  s none",
    "           rrrrrrrr             ",
    "        rrrRRRRRRRRrrr          " "       rRRRRRRRRRRRRRRr         ",
    "     rrRRRRRRRRRRRRRRRRrr       ",
    "    rRRRRRRRRRRRRRRRRRRRRr      ",
    "   rRRRRRRRRRRRRRRRRRRRRRRr     ",
    "   rRRRRRRRRRRRRRRRRRRRRRRra    ",
    "  rRRRRRRWRRRRRRRRRRWRRRRRRra   ",
    " rRRRRRRWWWRRRRRRRRWWWRRRRRRr   ",
    " rRRRRRWWWWWRRRRRRWWWWWRRRRRra  ",
    " rRRRRRRWWWWWRRRRWWWWWRRRRRRraa ",
    "rRRRRRRRRWWWWWRRWWWWWRRRRRRRRra ",
    "rRRRRRRRRRWWWWWWWWWWRRRRRRRRRra ",
    "rRRRRRRRRRRWWWWWWWWRRRRRRRRRRraa",
    "rRRRRRRRRRRRWWWWWWRRRRRRRRRRRraa",
    "rRRRRRRRRRRRWWWWWWRRRRRRRRRRRraa",
    "rRRRRRRRRRRWWWWWWWWRRRRRRRRRRraa",
    "rRRRRRRRRRWWWWWWWWWWRRRRRRRRRraa",
    "rRRRRRRRRWWWWWRRWWWWWRRRRRRRRraa",
    " rRRRRRRWWWWWRRRRWWWWWRRRRRRraaa",
    " rRRRRRWWWWWRRRRRRWWWWWRRRRRraaa",
    " rRRRRRRWWWRRRRRRRRWWWRRRRRRraa ",
    "  rRRRRRRWRRRRRRRRRRWRRRRRRraaa ",
    "   rRRRRRRRRRRRRRRRRRRRRRRraaaa ",
    "   rRRRRRRRRRRRRRRRRRRRRRRraaa  ",
    "    rRRRRRRRRRRRRRRRRRRRRraaa   ",
    "     rrRRRRRRRRRRRRRRRRrraaaa   ",
    "      arRRRRRRRRRRRRRRraaaaa    ",
    "       arrrRRRRRRRRrrraaaaa     ",
    "         aarrrrrrrraaaaaa       ",
    "          aaaaaaaaaaaaaa        ",
    "             aaaaaaaa           ",
};

/* XPM */
static const char *warning_xpm[] = {
/* w95mbx03 pixmap
 * width height ncolors chars_per_pixel */
    "32 32 6 1 ",
/* colors */
    "0 c #000000   m black  s s_black",
    "y c #808000   m black  s s_darkyellow",
    "a c #808080   m black  s s_darkgray",
    "A c #c0c0c0   m white  s s_gray",
    "Y c #FFFF00   m white  s s_yellow",
    "  c none      m white  s none",
    "             yyy                ",
    "            yYYA0               " "           yYYYYA0a             ",
    "           yYYYYY0aa            ",
    "          yYYYYYYA0aa           ",
    "          yYYYYYYY0aa           ",
    "         yYYYYYYYYA0aa          ",
    "         yYYYYYYYYY0aa          ",
    "        yYYYYYYYYYYA0aa         ",
    "        yYYYA000AYYY0aa         ",
    "       yYYYY00000YYYA0aa        ",
    "       yYYYY00000YYYY0aa        ",
    "      yYYYYY00000YYYYA0aa       ",
    "      yYYYYY00000YYYYY0aa       ",
    "     yYYYYYY00000YYYYYA0aa      ",
    "     yYYYYYYy000yYYYYYY0aa      ",
    "    yYYYYYYYA000AYYYYYYA0aa     ",
    "    yYYYYYYYY000YYYYYYYY0aa     ",
    "   yYYYYYYYYYy0yYYYYYYYYA0aa    ",
    "   yYYYYYYYYYA0AYYYYYYYYY0aa    ",
    "  yYYYYYYYYYYY0YYYYYYYYYYA0aa   ",
    "  yYYYYYYYYYYYYYYYYYYYYYYY0aa   ",
    " yYYYYYYYYYYYA00AYYYYYYYYYA0aa  ",
    " yYYYYYYYYYYY0000YYYYYYYYYY0aa  ",
    "yYYYYYYYYYYYY0000YYYYYYYYYYA0aa ",
    "yYYYYYYYYYYYYA00AYYYYYYYYYYY0aa ",
    "yYYYYYYYYYYYYYYYYYYYYYYYYYYY0aaa",
    "yYYYYYYYYYYYYYYYYYYYYYYYYYYA0aaa",
    " yYYYYYYYYYYYYYYYYYYYYYYYYA0aaaa",
    "  y000000000000000000000000aaaaa",
    "    aaaaaaaaaaaaaaaaaaaaaaaaaaa ",
    "     aaaaaaaaaaaaaaaaaaaaaaaaa  ",
};

static char *button_text[] = {
    "OK",
    "Cancel",
    "Yes",
    "No",
    "Abort",
    "Retry",
    "Ignore",
    0
};


GType gtk_messagebox_get_type ()
{
    static guint msg_type = 0;

    if (!msg_type) {
	static const GTypeInfo  msg_info = {
	    sizeof (GtkMessageboxClass),
	    NULL,           /* base_init */
	    NULL,           /* base_finalize */
	    (GClassInitFunc)messagebox_class_init,
	    NULL,           /* class_finalize */
	    NULL,           /* class_data */
	    sizeof (GtkMessagebox),
	    0,              /* n_preallocs */
	    (GInstanceInitFunc)messagebox_init
	};

	msg_type =
	    g_type_register_static (GTK_TYPE_DIALOG,
				    "GtkMessagebox",
				    &msg_info,0);
    }

    return msg_type;
}

void messagebox_class_init (GtkMessageboxClass * klass)
{
    GtkObjectClass *obj_class;
    GtkDialogClass *dlg_class;

    obj_class = (GtkObjectClass *) klass;
    dlg_class = (GtkDialogClass *) klass;
    parent_class = gtk_type_class (GTK_TYPE_DIALOG);

    obj_class->destroy = messagebox_destroy;
}

void messagebox_init (GtkMessagebox * mb)
{
    mb->app_text = NULL;
    mb->msg_text = NULL;
    mb->icon_type = MSG_BOX_NO_ICON;
    mb->button_list = MSG_BOX_OK;
    mb->rc = MSG_BOX_NONE;
}

void messagebox_destroy (GtkObject * obj)
{
    GtkMessagebox *mb = GTK_MESSAGEBOX (obj);
    g_free (mb->app_text);
    g_free (mb->msg_text);
    mb->app_text = NULL;
    mb->msg_text = NULL;

    /* chain up */
    if (GTK_OBJECT_CLASS (parent_class)->destroy)
	(*GTK_OBJECT_CLASS (parent_class)->destroy) (obj);
}

static GtkWidget *add_button (GtkMessagebox * mb, GtkWidget * bb, const char *lbl)
{
    GtkWidget *btn;
    btn = gtk_button_new_with_label (lbl);
    gtk_widget_show (btn);
    gtk_object_set_data (GTK_OBJECT (btn), "mbox", mb);
    gtk_signal_connect (GTK_OBJECT (btn), "clicked", GTK_SIGNAL_FUNC (messagebox_button), (gpointer) mb->rc);
    gtk_container_add (GTK_CONTAINER (bb), btn);
    GTK_WIDGET_SET_FLAGS (btn, GTK_CAN_DEFAULT);
    return btn;
}

GtkWidget *gtk_messagebox_new (const char *app, const char *msg, GtkMessageboxIcon i_type, unsigned long b_list)
{
    GtkMessagebox *mbox = gtk_type_new ( GTK_TYPE_MESSAGEBOX );
    GtkDialog *dlg = GTK_DIALOG (mbox);
    GtkWidget *hbox;
    GtkWidget *vbox;
    GtkWidget *pixmap;
    GdkColormap *colormap;
    GdkPixmap *gdkpixmap;
    GdkBitmap *mask;

    GtkWidget *lblApplication;
    GtkWidget *lblMessage;
    GtkWidget *hbuttonbox;
    GtkWidget *btn;
    const char **pxdata;

    mbox->app_text = g_strdup (app);
    mbox->msg_text = g_strdup (msg);
    mbox->icon_type = i_type;
    mbox->button_list = b_list;

    gtk_container_set_border_width (GTK_CONTAINER (mbox), 10);
    gtk_window_set_title (GTK_WINDOW (mbox), mbox->app_text);
    gtk_window_set_modal (GTK_WINDOW (mbox), TRUE);
    gtk_window_set_policy (GTK_WINDOW (mbox), FALSE, FALSE, TRUE);


    gtk_widget_show (dlg->vbox);
    gtk_box_set_spacing (GTK_BOX (dlg->vbox), 15);
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_widget_show (hbox);
    gtk_box_pack_start (GTK_BOX (dlg->vbox), hbox, TRUE, TRUE, 0);

    switch (mbox->icon_type) {
    case MSG_BOX_INFORMATION:
	pxdata = information_xpm;
	break;
    case MSG_BOX_WARNING:
	pxdata = warning_xpm;
	break;
    case MSG_BOX_CRITICAL:
	pxdata = critical_xpm;
	break;
    default:
	pxdata = none_xpm;
	break;
    }
    colormap = gtk_widget_get_colormap (GTK_WIDGET (mbox));
    gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &mask, NULL, (gchar **) pxdata);
    if (gdkpixmap == NULL)
	g_warning ("Couldn't create pixmap\n");
    else {
	pixmap = gtk_pixmap_new (gdkpixmap, mask);
	gtk_widget_show (pixmap);
	gtk_box_pack_start (GTK_BOX (hbox), pixmap, FALSE, TRUE, 0);
	gtk_misc_set_alignment (GTK_MISC (pixmap), 0.1, 0.25);
	gdk_pixmap_unref (gdkpixmap);
	gdk_bitmap_unref (mask);
    }

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox);
    gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);

    lblApplication = gtk_label_new ("");
    gtk_widget_show (lblApplication);
    gtk_box_pack_start (GTK_BOX (vbox), lblApplication, FALSE, FALSE, 0);

    lblMessage = gtk_label_new (mbox->msg_text);
    gtk_widget_show (lblMessage);
    gtk_box_pack_start (GTK_BOX (vbox), lblMessage, TRUE, FALSE, 0);
    gtk_label_set_justify (GTK_LABEL (lblMessage), GTK_JUSTIFY_LEFT);
    gtk_label_set_line_wrap (GTK_LABEL (lblMessage), TRUE);

    gtk_container_set_border_width (GTK_CONTAINER (dlg->action_area), 0);
    gtk_widget_show (dlg->action_area);

    hbuttonbox = gtk_hbutton_box_new ();
    gtk_widget_show (hbuttonbox);
    gtk_box_pack_start (GTK_BOX (dlg->action_area), hbuttonbox, TRUE, TRUE, 0);


    if (mbox->button_list & MSG_BOX_OK) {
	mbox->rc = MSG_BOX_OK;
	btn = add_button (mbox, hbuttonbox, button_text[0]);
    }
    if (mbox->button_list & MSG_BOX_CANCEL) {
	mbox->rc = MSG_BOX_CANCEL;
	btn = add_button (mbox, hbuttonbox, button_text[1]);
    }
    if (mbox->button_list & MSG_BOX_YES) {
	mbox->rc = MSG_BOX_YES;
	btn = add_button (mbox, hbuttonbox, button_text[2]);
    }
    if (mbox->button_list & MSG_BOX_NO) {
	mbox->rc = MSG_BOX_NO;
	btn = add_button (mbox, hbuttonbox, button_text[3]);
    }
    if (mbox->button_list & MSG_BOX_ABORT) {
	mbox->rc = MSG_BOX_ABORT;
	btn = add_button (mbox, hbuttonbox, button_text[4]);
    }
    if (mbox->button_list & MSG_BOX_RETRY) {
	mbox->rc = MSG_BOX_RETRY;
	btn = add_button (mbox, hbuttonbox, button_text[5]);
    }
    if (mbox->button_list & MSG_BOX_IGNORE) {
	mbox->rc = MSG_BOX_IGNORE;
	btn = add_button (mbox, hbuttonbox, button_text[6]);
    }
    mbox->rc = MSG_BOX_NONE;
    return GTK_WIDGET (mbox);
}

void messagebox_button (GtkWidget * w, gpointer ptr)
{
    GtkMessageboxButton btype = (GtkMessageboxButton) ptr;
    GtkMessagebox *mb = GTK_MESSAGEBOX (gtk_object_get_data (GTK_OBJECT (w), "mbox"));
    messagebox_done (mb, btype);
}


void messagebox_done (GtkMessagebox * mb, GtkMessageboxButton btype)
{
    gtk_widget_hide (GTK_WIDGET (mb));
    mb->rc = btype;
    gtk_main_quit ();
}

GtkMessageboxButton gtk_messagebox_exec (GtkMessagebox * mb)
{
    GtkMessageboxButton rc;
    mb->rc = MSG_BOX_NONE;
    gtk_widget_show (GTK_WIDGET (mb));
    gtk_main ();
    rc = mb->rc;
    gtk_widget_destroy (GTK_WIDGET (mb));
    return rc;
}

GtkMessageboxButton gtk_messagebox_run (GtkMessagebox * mb)
{
    GtkMessageboxButton rc;
    
    mb->rc = MSG_BOX_NONE;
    gtk_dialog_run( GTK_DIALOG(mb) );
    rc = mb->rc;
    gtk_widget_destroy (GTK_WIDGET (mb));
    return rc;
}
