#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <iostream>
#include <cstdio>
#include <cstring>
#include <vector>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <gtk/gtk.h>
#include <gui/gtkutils.h>
#include <gui/GladeApp.h>
#include <config/ConfigParser.h>
#include <config/config_utils.h>



//=====================================================================
// A missing routine in gtk.
// Returns the current item in an option menu
//=====================================================================
gint gtk_option_menu_get_history(GtkOptionMenu * option_menu)
{
    GtkWidget *active_widget;

    g_return_val_if_fail(GTK_IS_OPTION_MENU(option_menu), -1);

    active_widget = gtk_menu_get_active(GTK_MENU(option_menu->menu));

    if (active_widget)
        return g_list_index(GTK_MENU_SHELL(option_menu->menu)->children,
                            active_widget);
    else
        return -1;
}
gint gtk_menu_get_history(GtkMenu * menu)
{
    GtkWidget *active_widget;

    g_return_val_if_fail(GTK_IS_MENU(menu), -1);

    active_widget = gtk_menu_get_active(menu);

    if (active_widget)
        return g_list_index(GTK_MENU_SHELL(menu)->children, active_widget);
    else
        return -1;
}


/* This is an internally used function to create pixmaps. */
GtkWidget*
create_pixmap                          (GtkWidget       *widget,
                                        const gchar     *filename)
{
    GtkWidget *pixmap;
    std::string fnam = filename;
    std::string path = GladeApp::find_file( fnam );
    
    if (path.empty())
    {
        std::cout << "Could not find " << filename << std::endl;
        return 0;
    }

    pixmap = gtk_image_new_from_file (path.c_str());
    g_object_ref( pixmap );
    return pixmap;
}
void change_button_pixmap(GtkWidget *btn, GtkWidget *img)
{
    g_return_if_fail( GTK_IS_BUTTON(btn) );
    if ( img == NULL )
        return;

    GtkContainer *bin = GTK_CONTAINER(btn);
    GList *children;
    GtkWidget *child;
    bool done = false;
    while (!done)
    {
        children = gtk_container_get_children(bin);
        for ( ; children; children = children->next )
        {
            child = GTK_WIDGET(children->data);
            if ( GTK_IS_CONTAINER(child) )
            {
                bin = GTK_CONTAINER(child);
                break;
            }
            else if ( GTK_IS_IMAGE(child) )
            {
                gtk_container_remove (GTK_CONTAINER(bin), child);
                gtk_container_add(GTK_CONTAINER(bin), img);
                done = true;
                break;
            }
        }
    }
    gtk_widget_show_all(btn);
}

std::string get_file_name(const char *title, gboolean for_open, const char **filters )
{
    std::string out;
    GtkWidget *fsel;
    
#if  GTK_CHECK_VERSION(2,4,0)
    int i;
    GtkFileFilter *filter = 0;

    if ( filters )
    {
        int nfilter = 0;
        filter = gtk_file_filter_new ();
        for (i = 0;filters[i];i++)
        {
            gtk_file_filter_set_name(filter, filters[i]);
            gtk_file_filter_add_pattern (filter, filters[i]);
            nfilter++;
        }
        if (!nfilter)
        {
            g_object_unref( G_OBJECT(filter) );
            filter = 0;
        }
    }

    GtkFileChooserAction action;
    if (for_open)
    {
        action = GTK_FILE_CHOOSER_ACTION_OPEN;
    }
    else
    {
        action = GTK_FILE_CHOOSER_ACTION_SAVE;
    }

    fsel = gtk_file_chooser_dialog_new(title, NULL,
                                       action,
                                       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                       GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                       NULL);
    if (filter)
        gtk_file_chooser_add_filter( GTK_FILE_CHOOSER (fsel), filter);

#define fsel_get_file(fsel)    gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fsel))
#define the_good_answer  GTK_RESPONSE_ACCEPT
#else
    fsel = gtk_file_selection_new (title);
#define fsel_get_file(fsel)    gtk_file_selection_get_filename (GTK_FILE_SELECTION (fsel))
#define the_good_answer  GTK_RESPONSE_OK
#endif

    if ( gtk_dialog_run (GTK_DIALOG (fsel)) == the_good_answer)
        out = fsel_get_file(fsel);

    gtk_widget_destroy(fsel);
    return out;
}


void
gtk_help_display(const gchar *link_id)
{
#if HAS_SCROLLKEEPER
    static gboolean yelp_works = TRUE;
#endif

    gchar *child_argv[3];



#if HAS_SCROLLKEEPER
    if (yelp_works)
    {
        child_argv[0] = "yelp";

        if (link_id)
        {
            child_argv[1] = g_strconcat(("ghelp://" DATA_DIR
                                         "/help/vmedaq.xml?"),
                                        link_id, NULL);
        }
        else
            child_argv[1] = "ghelp://" DATA_DIR "/help/vmedaq.xml";

        child_argv[2] = NULL;
        if (!g_spawn_async(NULL, child_argv, NULL, G_SPAWN_SEARCH_PATH,
                           NULL, NULL, NULL, NULL))
            yelp_works = FALSE;


        printf("%s\n", child_argv[1]);
        if (link_id)
            g_free(child_argv[1]);
    }

    else
    {
#endif

        /* FIXME: Make fallback browser configurable. */
        if (strlen(BROWSER)>1)
        {
            child_argv[0] = g_strdup(BROWSER);
        }
        else
            return;

        if (link_id)
        {
            child_argv[1] = g_strconcat(DATA_DIR "/help/vmedaq.html#",
                                        link_id, NULL);
        }
        else
            child_argv[1] = DATA_DIR "/help/vmedaq.html";

        child_argv[2] = NULL;

        /* FIXME: If not works, pop up configuration dialog. */
        g_spawn_async(NULL, child_argv, NULL, G_SPAWN_SEARCH_PATH,
                      NULL, NULL, NULL, NULL);

        g_free(child_argv[0]);

        if (link_id)
            g_free(child_argv[1]);

#if HAS_SCROLLKEEPER
    }
#endif
}


std::string get_a_string(const std::string &title, const std::string &prompt)
{
    std::string out;
    GtkWidget *dialog = gtk_dialog_new_with_buttons (
                            title.c_str(),
                            (GtkWindow *)0,
                            (GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
                            GTK_STOCK_OK,
                            GTK_RESPONSE_ACCEPT,
                            GTK_STOCK_CANCEL,
                            GTK_RESPONSE_REJECT,
                            NULL);

    GtkWidget *vbox = gtk_vbox_new( FALSE, 2 );
    gtk_container_set_border_width( GTK_CONTAINER(vbox), 5 );
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox, FALSE, FALSE, 0);

    GtkWidget *label = gtk_label_new(prompt.c_str());
    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

    GtkWidget *answ = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(vbox), answ, FALSE, FALSE, 0);

    gtk_button_box_set_layout( GTK_BUTTON_BOX(GTK_DIALOG(dialog)->action_area),
                               GTK_BUTTONBOX_END);

    gtk_widget_show_all( dialog );

    if ( gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT )
    {
        out = gtk_entry_get_text(GTK_ENTRY(answ));

    }
    gtk_widget_hide(dialog);
    gtk_widget_destroy(dialog);

    return out;
}

int get_a_string_list(const std::string &title, const std::string &prompt, std::vector<std::string> &values)
{
    std::vector<std::string> msg;
    stringtok< std::vector<std::string> >(msg, prompt,":;");
    std::vector<std::string>::iterator ip;

    values.clear();
    GtkWidget *dialog = gtk_dialog_new_with_buttons (
                                                     title.c_str(),
                                                     (GtkWindow *)0,
                                                     (GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
                                                     GTK_STOCK_OK,
                                                     GTK_RESPONSE_ACCEPT,
                                                     GTK_STOCK_CANCEL,
                                                     GTK_RESPONSE_REJECT,
                                                     NULL);

    GtkWidget *vbox = gtk_vbox_new( FALSE, 2 );
    gtk_container_set_border_width( GTK_CONTAINER(vbox), 5 );
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox, FALSE, FALSE, 0);

    std::vector< GtkWidget *> entries;
    for (ip=msg.begin();ip!=msg.end();ip++)
    {
        GtkWidget *label = gtk_label_new((*ip).c_str());
        gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
        gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

        GtkWidget *answ = gtk_entry_new();
        gtk_box_pack_start(GTK_BOX(vbox), answ, FALSE, FALSE, 0);
        entries.push_back(answ);
    }
    gtk_button_box_set_layout( GTK_BUTTON_BOX(GTK_DIALOG(dialog)->action_area),
                               GTK_BUTTONBOX_END);

    gtk_widget_show_all( dialog );

    if ( gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT )
    {
        std::vector< GtkWidget *>::iterator iw;
        for (iw=entries.begin(); iw!=entries.end();iw++)
        {
            values.push_back( gtk_entry_get_text(GTK_ENTRY( *iw )));
        }

    }
    gtk_widget_hide(dialog);
    gtk_widget_destroy(dialog);
    return values.size();
}