#include <iostream>
#include <iomanip>
#include <sstream>
#include <string>
#include <cstring>
#include <DAQ++/DAQmanager.h>
#include <daq/DataRunManager.h>
#include <daq/GenModule.h>
#include "gtkutils.h"
#include "module_gui.h"
#include "ParamSlider.h"
#include "RunMenu.h"

#include <config/ConfigParser.h>
#include <config/ModuleParser.h>
#include <gui/MonitorWindow.h>
//////////////////////////////////////////////////////////////////////////
// module_gui
//
//   Created: Mon May 27 11:40:26 2002        Author: Carlos Lacasta
//   Purpose:
//
//////////////////////////////////////////////////////////////////////////
module_gui::module_gui(const std::string &id)
        : _monitor(0), ignoreMonBtn(false), _control(0), _config(0), _dac(0)
{

//---------------------------------------------------------------------
// Get the pointer to the module
//---------------------------------------------------------------------
    DAQpp::Module *dm = DAQpp::DAQmanager::theDAQmanager()->find_module(id);
    if (dm==0) {
        std::cout << "module_gui: DAQmanager could not find module [" << id << "]" << std::endl;   
    }
    module = dynamic_cast<GenModule *>(dm);
    if (!module) {
        std::cout << "module_gui: module id=" << id << " is not a GenModule" << std::endl;
        return;
    }
}

module_gui::~module_gui()
{}

void
module_gui::auto_connect()
{
    SET_CONNECT( on_btnConfig );
    SET_CONNECT( on_swLocal );
    SET_CONNECT( on_swMonitor );
}

void
module_gui::user_init()
{
    GtkWidget *w, *lbl;
    if (module==0)
    {
        lbl = get_widget("lblModuleID");
        gtk_label_set_markup(GTK_LABEL(lbl),"<b>Unknown</b>");   
        return;
    }

    // Module id label
    std::ostringstream mid;
    lbl = get_widget("lblModuleID");
    mid << "<b>" << module->get_id() << "</b>";
    gtk_label_set_markup(GTK_LABEL(lbl), mid.str().c_str());

//---------------------------------------------------------------------
// add updates
//---------------------------------------------------------------------
    addUpdate( new UpdateLabelNtrig( get_widget("lblNtrig")) );
    addUpdate( new UpdateLabelRate( get_widget("lblRate")) );

//---------------------------------------------------------------------
// Subscribe to the state
//---------------------------------------------------------------------
    // Local Button
    w = get_widget("swLocal");
    g_object_set_data(G_OBJECT(w), "global", create_pixmap(w, "status_lock.png"));
    g_object_set_data(G_OBJECT(w), "local", create_pixmap(w, "status_unlocked.png"));
    if ( module->is_local() )
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
    else
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), FALSE);


    gtk_widget_unparent(get_main_window());
}

int module_gui::update(Update &up)
{
//    Update up;
//    Module_update(_id,up);
    updateAll( up );

    return 1;
}

void
module_gui:: on_btnConfig(GtkWidget * w, module_gui * ptr)
{
    if ( ptr->_config )
        return;

    if (!ptr->module)
        return;
        
    ModuleParser *mp = dynamic_cast<ModuleParser *>(
                           ConfigParser::theConfig()->get_id_handler(ptr->module->get_id()));

    ptr->_config = mp->config(ptr->module->get_id());
    if (ptr->_config)
        g_signal_connect(G_OBJECT(ptr->_config), "destroy",
                         G_CALLBACK(onConfig_destroy),
                         ptr);

}

void list_w_properties(GtkWidget *w)
{
    GObjectClass *wclass;
    const gchar *object_class_name;
    guint arg;
    gchar flags[16], *pos;
    GParamSpec **properties;
    guint n_properties;
    const gchar *name, *nick, *blurb, *dot;
    GType object_type = G_OBJECT_TYPE(G_OBJECT(w)); 
    
    wclass = G_OBJECT_CLASS( g_type_class_peek (object_type) );
    if (!wclass)
        return;

    properties = g_object_class_list_properties (wclass, &n_properties);
    object_class_name = g_type_name (object_type);
    
    std::cout << "Widget: " << gtk_widget_get_name(w) << " class: " << object_class_name << std::endl;
    for (arg=0;arg<n_properties;arg++)
    {
        GParamSpec *spec = properties[arg];
        
        name = g_param_spec_get_name(spec);
        nick = g_param_spec_get_nick (spec);
        blurb = g_param_spec_get_blurb (spec);
        
        
        dot = "";
        if (blurb) {
            int str_len = strlen (blurb);
            if (str_len > 0  && blurb[str_len - 1] != '.')
                dot = ".";
        }
        std::cout << "name: " << name << " [nick: " << nick << " blurb: " << blurb << ']' << std::endl;
    }
}

void module_gui:: on_swMonitor(GtkWidget * w, module_gui * ptr)
{

    if ( ptr->ignoreMonBtn )
    {
        ptr->ignoreMonBtn = false;
        return;
    }
    
    if (!ptr->module)
        return;
    
    GtkToggleButton *btn = GTK_TOGGLE_BUTTON(ptr->get_widget("swMonitor"));
    bool zbtn = gtk_toggle_button_get_active(btn);


    if ( zbtn && ptr->_monitor == 0 )
    {
        ModuleParser *mp = dynamic_cast<ModuleParser *>(
                               ConfigParser::theConfig()->get_id_handler(ptr->module->get_id()));

        if (mp == 0)
        {
            std::cout << "Module " << ptr->module->get_id() << " does not seem to exist" << std::endl;
            return;
        }

        MonitorWindow *mw = mp->monitor(ptr->module->get_id(), ptr->get_file_name());
        //list_w_properties( mw->widget());
        if (!mw || mw->widget() == 0)
        {
            if (zbtn)
                gtk_toggle_button_set_active(btn, false);
            else
                gtk_toggle_button_set_active(btn, true);

            std::cout << "It seems the module does not define a monitor window widget" << std::endl;
            return;
        }

        gtk_signal_connect(GTK_OBJECT(mw->widget()),
                           "destroy",
                           GTK_SIGNAL_FUNC(onMonitor_destroy),
                           ptr);

        // add updates for the Monitor
        std::vector<WidgetUpdate *>::const_iterator ip;
        for (ip = mw->updates().begin();ip != mw->updates().end();ip++)
        {
            ptr->addUpdate( *ip );
            ptr->monitorUpdates.insert( *ip );
        }

        // Local or Global mode ?
        GtkWidget *w = ptr->get_widget("swLocal");
        if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
            mw->inLocal();
        else
            mw->inGlobal();

        ptr->_monitor = mw;
    }
    if ( !zbtn && ptr->_monitor )
    {
        delete ptr->_monitor;
        ptr->_monitor = 0;
        ptr->ignoreMonBtn = false;
    }
    return;
}

void module_gui::on_swLocal(GtkWidget * wdgt, module_gui * ptr)
{
    DAQpp::DAQmanager *mngr = DAQpp::DAQmanager::theDAQmanager();
    GtkWidget *img;
    if (!ptr->module)
        return;

    GtkWidget *w = ptr->get_widget("swLocal");
    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))
    {
        ptr->module->in_local();
        img = GTK_WIDGET(g_object_get_data(G_OBJECT(wdgt), "local"));
        change_button_pixmap(wdgt, img);
        // Create a new RunManager for this module
        // Default is DataRunManager
        std::ostringstream rmID;
        rmID << "rm." << ptr->module->get_id();
        DAQpp::RunManager *main = mngr->find_runmanager("main");
        DAQpp::RunManager *rm = new DataRunManager( rmID.str() );
        main->deleteModule(ptr->module);
        rm->addModule(ptr->module);

        if ( ptr->_monitor )
            ptr->_monitor->inLocal();
    }
    else
    {
        // Add the module back in the main RunManager
        img = GTK_WIDGET(g_object_get_data(G_OBJECT(wdgt), "global"));
        change_button_pixmap(wdgt, img);
        std::ostringstream rmID;
        rmID << "rm." << ptr->module->get_id();
        DAQpp::RunManager *rm = mngr->find_runmanager(rmID.str());
        if ( rm )
        {
            rm->deleteModule(ptr->module);
            delete rm;
        }

        DAQpp::RunManager *main = mngr->find_runmanager("main");
        ptr->module->in_global( (*main)() );
        main->addModule(ptr->module);


        if ( ptr->_monitor )
            ptr->_monitor->inGlobal();
    }
}

void  module_gui::onConfig_destroy(GtkWidget * w, module_gui * ptr)
{
    ptr->_config = 0;
}
void  module_gui::onMonitor_destroy(GtkWidget * w, module_gui * ptr)
{
    WidgetHolder::updateList::iterator it;
    for (it = ptr->monitorUpdates.begin();it != ptr->monitorUpdates.end();++it)
        ptr->deleteUpdate( *it );

    ptr->monitorUpdates.clear();

    ptr->ignoreMonBtn = true;
    ptr->_monitor = 0;

    GtkToggleButton *btn
    = GTK_TOGGLE_BUTTON(ptr->get_widget( "swMonitor"));
    gtk_toggle_button_set_active(btn, FALSE);
}

std::string module_gui::id() const
{
    return module->get_id();
}
