
#include <gtk/gtksignal.h>
#include <gtk/gtkobject.h>
#include <gtk/gtkmain.h>
#include <gtk/gtkmenuitem.h>
#include "runmenu.h"

enum {
    COMMAND,
    LAST_SIGNAL
};

static void runmenu_class_init (RunmenuClass * klass);
static void runmenu_init (Runmenu * runm);
static guint runmenu_signals[LAST_SIGNAL] = { 0 };

static void on_getready (GtkWidget *, void *);
static void on_start (GtkWidget *, void *);
static void on_stop (GtkWidget *, void *);
static void on_pause (GtkWidget *, void *);
static void on_continue (GtkWidget *, void *);
static void on_reset (GtkWidget *, void *);
static int stop_it (gpointer);



GType runmenu_get_type ()
{
    static guint runm_type = 0;

    if (!runm_type) {
	static const GTypeInfo runm_info = {
	    sizeof (RunmenuClass),
	    NULL,           /* base_init */
	    NULL,           /* base_finalize */
	    (GClassInitFunc)runmenu_class_init,
	    NULL,           /* class_finalize */
	    NULL,           /* class_data */
	    sizeof (Runmenu),
	    0,              /* n_preallocs */
	    (GInstanceInitFunc)runmenu_init
	};

	runm_type =
	    g_type_register_static (GTK_TYPE_MENU,
				    "Runmenu",
				    &runm_info,0);
    }

    return runm_type;
}

static void runmenu_class_init (RunmenuClass * class)
{
    
    GtkObjectClass *object_class;
    GtkWidgetClass *widget_class;
    object_class = (GtkObjectClass *) class;
    widget_class = (GtkWidgetClass *) class;
    runmenu_signals[COMMAND] =
	g_signal_new ("command",
		      G_TYPE_FROM_CLASS (class),
		      GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
		      GTK_SIGNAL_OFFSET (RunmenuClass, command),
		      NULL, NULL,
		      gtk_marshal_NONE__UINT, GTK_TYPE_NONE, 1, GTK_TYPE_UINT);
    
    class->command = NULL;
}

static void runmenu_init (Runmenu * runm)
{
    GtkWidget *separator, *quit;
    /* 
       menuitem = gtk_menu_item_new(); gtk_menu_item_set_submenu
       (GTK_MENU_ITEM (menuitem), GTK_WIDGET(runm)); */

    runm->tmout = 0;
    runm->have_to_stop = 0;
    runm->igetready = (GtkWidget *) gtk_menu_item_new_with_label ("Get Ready");
    runm->istartrun = (GtkWidget *) gtk_menu_item_new_with_label ("Start");
    runm->istoprun = (GtkWidget *) gtk_menu_item_new_with_label ("Stop");
    runm->ipauserun = (GtkWidget *) gtk_menu_item_new_with_label ("Pause");
    runm->icontinuerun = (GtkWidget *) gtk_menu_item_new_with_label ("Continue");
    runm->ireset = (GtkWidget *) gtk_menu_item_new_with_label ("Reset");
    separator = (GtkWidget *) gtk_menu_item_new ();
    quit = (GtkWidget *) gtk_menu_item_new_with_label ("Quit");
    gtk_container_add (GTK_CONTAINER (runm), runm->igetready);
    gtk_container_add (GTK_CONTAINER (runm), runm->istartrun);
    gtk_container_add (GTK_CONTAINER (runm), runm->istoprun);
    gtk_container_add (GTK_CONTAINER (runm), runm->ipauserun);
    gtk_container_add (GTK_CONTAINER (runm), runm->icontinuerun);
    gtk_container_add (GTK_CONTAINER (runm), separator);
    gtk_container_add (GTK_CONTAINER (runm), runm->ireset);
    gtk_container_add (GTK_CONTAINER (runm), quit);

    gtk_widget_show (runm->igetready);
    gtk_widget_show (runm->istartrun);
    gtk_widget_show (runm->istoprun);
    gtk_widget_show (runm->ipauserun);
    gtk_widget_show (runm->icontinuerun);
    gtk_widget_show (runm->ireset);
    gtk_widget_show (separator);
    gtk_widget_show (quit);

    gtk_signal_connect (GTK_OBJECT (runm->igetready), "activate", GTK_SIGNAL_FUNC (on_getready), runm);
    gtk_signal_connect (GTK_OBJECT (runm->istartrun), "activate", GTK_SIGNAL_FUNC (on_start), runm);
    gtk_signal_connect (GTK_OBJECT (runm->istoprun), "activate", GTK_SIGNAL_FUNC (on_stop), runm);
    gtk_signal_connect (GTK_OBJECT (runm->ipauserun), "activate", GTK_SIGNAL_FUNC (on_pause), runm);
    gtk_signal_connect (GTK_OBJECT (runm->icontinuerun), "activate", GTK_SIGNAL_FUNC (on_continue), runm);
    gtk_signal_connect (GTK_OBJECT (runm->ireset), "activate", GTK_SIGNAL_FUNC (on_reset), runm);
    gtk_signal_connect (GTK_OBJECT (quit), "activate", GTK_SIGNAL_FUNC (gtk_main_quit), runm);

    gtk_widget_set_sensitive (runm->igetready, TRUE);
    gtk_widget_set_sensitive (runm->istartrun, FALSE);
    gtk_widget_set_sensitive (runm->istoprun, FALSE);
    gtk_widget_set_sensitive (runm->ipauserun, FALSE);
    gtk_widget_set_sensitive (runm->ireset, TRUE);
    gtk_widget_set_sensitive (runm->icontinuerun, FALSE);
}

GtkWidget *gtk_runmenu_new ()
{
    Runmenu *runm = gtk_type_new (runmenu_get_type ());
    return GTK_WIDGET (runm);
}

gint stop_it (gpointer ptr)
{
    Runmenu *runm = (Runmenu *) ptr;
    if (runm->have_to_stop) {
	runm->have_to_stop = 0;
	on_stop (NULL, ptr);
	return 0;
    }
    return 1;
}

void on_getready (GtkWidget * w, void *ptr)
{
    gtk_widget_set_sensitive (RUNMENU (ptr)->igetready, TRUE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->istartrun, TRUE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->istoprun, FALSE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->ipauserun, FALSE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->icontinuerun, FALSE);
    gtk_signal_emit (GTK_OBJECT (ptr), runmenu_signals[COMMAND], gettingready);
    gtk_signal_emit (GTK_OBJECT (ptr), runmenu_signals[COMMAND], getready);
}
void on_start (GtkWidget * w, void *ptr)
{
    RUNMENU (ptr)->tmout = gtk_timeout_add (100, stop_it, ptr);
    RUNMENU (ptr)->have_to_stop = 0;
    gtk_widget_set_sensitive (RUNMENU (ptr)->igetready, FALSE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->istartrun, FALSE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->istoprun, TRUE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->ipauserun, TRUE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->icontinuerun, FALSE);
    gtk_signal_emit (GTK_OBJECT (ptr), runmenu_signals[COMMAND], run);
}
void on_stop (GtkWidget * w, void *ptr)
{
    if ( RUNMENU (ptr)->tmout!=0 ) {
	gtk_timeout_remove (RUNMENU (ptr)->tmout);
	RUNMENU (ptr)->tmout = 0;
    }
    RUNMENU (ptr)->have_to_stop = 0;
    gtk_widget_set_sensitive (RUNMENU (ptr)->igetready, TRUE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->istartrun, TRUE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->istoprun, FALSE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->ipauserun, FALSE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->icontinuerun, FALSE);
    gtk_signal_emit (GTK_OBJECT (ptr), runmenu_signals[COMMAND], stop);
}
void on_pause (GtkWidget * w, void *ptr)
{
    gtk_widget_set_sensitive (RUNMENU (ptr)->igetready, FALSE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->istartrun, FALSE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->istoprun, TRUE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->ipauserun, FALSE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->icontinuerun, TRUE);
    gtk_signal_emit (GTK_OBJECT (ptr), runmenu_signals[COMMAND], pauserun);
}
void on_continue (GtkWidget * w, void *ptr)
{
    gtk_widget_set_sensitive (RUNMENU (ptr)->igetready, FALSE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->istartrun, FALSE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->istoprun, TRUE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->ipauserun, TRUE);
    gtk_widget_set_sensitive (RUNMENU (ptr)->icontinuerun, FALSE);
    gtk_signal_emit (GTK_OBJECT (ptr), runmenu_signals[COMMAND], resume);
}

void on_reset (GtkWidget * w, void *ptr)
{
    gtk_signal_emit (GTK_OBJECT (ptr), runmenu_signals[COMMAND], send_reset);
}

void gtk_runmenu_stop (Runmenu * run)
{
    g_return_if_fail (run != NULL);
    g_return_if_fail (IS_RUNMENU (run));
    run->have_to_stop = 1;
}

void gtk_runmenu_emit(Runmenu *rm, RunStates cmmd) {
    switch (cmmd) {
    case getready:
	on_getready(NULL, rm);
	break;
    case run:
	on_start(NULL, rm);
	break;
    case stop:
	on_stop(NULL, rm);
	break;
    case pauserun:
	on_pause(NULL, rm);
	break;
    case resume:
	on_continue(NULL, rm);
	break;
    case send_reset:
	on_reset(NULL, rm);
	break;
    default:
	break;
    }
}

void gtk_runmenu_set_notready(Runmenu *runm) {
    gtk_widget_set_sensitive (runm->igetready, TRUE);
    gtk_widget_set_sensitive (runm->istartrun, FALSE);
    gtk_widget_set_sensitive (runm->istoprun, FALSE);
    gtk_widget_set_sensitive (runm->ipauserun, FALSE);
    gtk_widget_set_sensitive (runm->ireset, TRUE);
    gtk_widget_set_sensitive (runm->icontinuerun, FALSE);
    gtk_widget_queue_draw(GTK_WIDGET(runm));
}
void gtk_runmenu_set_ready(Runmenu *mn) {
    gtk_widget_set_sensitive (mn->igetready, TRUE);
    gtk_widget_set_sensitive (mn->istartrun, TRUE);
    gtk_widget_set_sensitive (mn->istoprun, FALSE);
    gtk_widget_set_sensitive (mn->ipauserun, FALSE);
    gtk_widget_set_sensitive (mn->icontinuerun, FALSE);
    gtk_widget_queue_draw(GTK_WIDGET(mn));
}
void gtk_runmenu_set_running(Runmenu *mn) {
    gtk_widget_set_sensitive (mn->igetready, FALSE);
    gtk_widget_set_sensitive (mn->istartrun, FALSE);
    gtk_widget_set_sensitive (mn->istoprun, TRUE);
    gtk_widget_set_sensitive (mn->ipauserun, TRUE);
    gtk_widget_set_sensitive (mn->icontinuerun, FALSE);
    gtk_widget_queue_draw(GTK_WIDGET(mn));
    
}
void gtk_runmenu_set_paused(Runmenu *mn) {
    gtk_widget_set_sensitive (mn->igetready, FALSE);
    gtk_widget_set_sensitive (mn->istartrun, FALSE);
    gtk_widget_set_sensitive (mn->istoprun, TRUE);
    gtk_widget_set_sensitive (mn->ipauserun, FALSE);
    gtk_widget_set_sensitive (mn->icontinuerun, TRUE);
    gtk_widget_queue_draw(GTK_WIDGET(mn));

}

