#include <iostream>
#include <daq/GenModule.h>
#include <config/ConfigParser.h>
#include <config/ModuleParser.h>

#include "MyScanRunManager.h"

MyScanRunManager::MyScanRunManager(DAQpp::DAQid id) :
    ScanRunManager(id)
{
}

static int __dummy_cnt__ = 0;
void MyScanRunManager::init_run()
{
    __dummy_cnt__ = 0;
    std::cout << __PRETTY_FUNCTION__ << ':' << __LINE__ << std::endl;
}

void MyScanRunManager::new_point()
{
    std::cout << __PRETTY_FUNCTION__ << ':' << __LINE__ << std::endl;
}

void MyScanRunManager::new_trigger()
{
    if (__dummy_cnt__++ < 3)
        std::cout << __PRETTY_FUNCTION__ << ':' << __LINE__ << std::endl;
}


/**
 * This is the dummy module
 */
class MyDummyModule : public GenModule
{
    public:
        MyDummyModule(const DAQpp::DAQid &id, unsigned long add);
        ~MyDummyModule()
        {
        }

        // Here we are notified of any change in the parameters
        void set_scan_point(ScanPoint &);

        // The DAQ++ stuff which we do not need
        bool hasData()
        {
            return false;
        }
        char *getData(int &size)
        {
            size=0;
            return 0;
        }
        int PrepareForRun()
        {
            return 0;
        }
        int Start(int)
        {
            return 0;
        }
        int Stop(int)
        {
            return 0;
        }
        int Pause()
        {
            return 0;
        }
        int Continue()
        {
            return 0;
        }
        int Reset()
        {
            return 0;
        }
        int GetReady()
        {
            return 0;
        }
        int Trigger()
        {
            return 0;
        }
        void flush()
        {
        }

        unsigned long get_data_type() const
        {
            return 0xe0000;
        }
};

MyDummyModule::MyDummyModule(const DAQpp::DAQid &id, unsigned long add) :
    GenModule(id, add)
{
    // reset pedestals and noise to sensible values
    fill_n(back_inserter(get_par_value< std::vector<double> >("pedestal") ), 1, 0.);
    fill_n(back_inserter(get_par_value< std::vector<double> >("noise") ), 1, 0.);

    this->ana = new Analysis(1,1);
}

void MyDummyModule::set_scan_point(ScanPoint &P)
{
    __dummy_cnt__ = 0;
    std::cout << __PRETTY_FUNCTION__ << ':' << __LINE__ << std::endl;
}

/**
 * This is another dummy object that only creates the dummy module
 */
class MyDummyModuleConf : public ModuleParser
{
    public:
        MyDummyModuleConf()
        {
        }
        ~MyDummyModuleConf()
        {
        }

        void parse_node(xmlNodePtr node) throw (ConfigException);
        void save_config() throw (ConfigException)
        {
            save_common();
        }
        ;

};

void MyDummyModuleConf::parse_node(xmlNodePtr _node) throw (ConfigException)
{
    unsigned long address;
    std::string saddress, name;

    // The configuration manager handler
    ConfigParser *cfg = ConfigParser::theConfig();

    // store the node pointer
    node = _node;
    name = node_name();

    // get the module id
    id = cfg->get_attribute<std::string>(node, "name");

    // get the address
    saddress = cfg->get_attribute<std::string>(node, "address");

    std::istringstream os(saddress);
    os >> std::hex >> address;

    new MyDummyModule(id, address);

    // Now parse the things which are common to all the modules, that
    // is the monitor and padmap elements
    parse_common("//"+name, id);
}

/*
 * The hooks
 */
extern "C"
{
    MyScanRunManager *create_new_manager(const std::string &type, const std::string &name)
    {
        return new MyScanRunManager(name);
    }

    MyDummyModuleConf *create_dummy_module()
    {
        return new MyDummyModuleConf();
    }
}

