#include <iostream>
#include <iomanip>
#include <config/ConfigParser.h>
#include <config/ModuleParser.h>
#include <config/GeneralConfig.h>

#include "test_analysis.h"

void test_analysis::init_run()
{
    std::cout << "** test_analysis::init_run\n" << std::endl;
}
void test_analysis::end_run()
{
    std::cout << "\n**test_analysis::end_run" << std::endl;
}

void test_analysis::Analyze()
{
    std::cout << "\rAnalyze -> "
              << std::setw(10)
              << trigger
              << std::flush;
}

void test_analysis::decode(unsigned int *data)
{
    // Get some importand handlers of the configuration
    ConfigParser *cfg = ConfigParser::theConfig();
    GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));

    // In order to make a fast analysis of the data, we should provide
    // the pedestal calculation routines the list of channels to be
    // considered.
    std::vector<int> list_of_channels;

    // We assume the Analysis::Fill has made all the checks on
    // the data integrity
    int size = data[2] & 0xfff;
    int i, j;

    for (i = 3, j = 0; j < size; j++, i++)
    {
        Event()[j] = data[i];

        // We assume all the channels are good
        list_of_channels.push_back(j);

        // Note: we assumed here that each word corresponds to one
        // channel and that all the channels are valid. One may have the
        // scenario in which the date is perversly encoded and we only
        // find, at the end of the process a list of channels with
        // signal. In that situation, we would call
        //
        // ana->add_hit(j)
        // ana->Event()[j] = ana->Chan()[j] = signal
        //
        // because we know that channel j has signal...
    }

    // If we are not interested in pedestals and in finding the
    // channels with signal, ignore this...
    bool find_pulse = true; // or false
    if (gnrl->get_ped_method() == GeneralConfig::normal)
    {
        // pedestals are computed in the usual way and, eventualy
        // updated. Analysis will find the channels with signal by
        // applying a simple cut on S/N
        update_pedestals(list_of_channels, find_pulse);
    }
    else
    {
        // pedestals are computed by an adaptative algorithm. One has to
        // specified if Analyis has to find the channels with signal
        update_pedestals_fast(list_of_channels, find_pulse);
    }
}
