#/usr/bin/env python
""" This is an example of a module implemented in Python
"""
import daqpp
import pyvmedaq
import libxml2


import array
import random
import random

import gobject
import gtk
import gtkhst

import GladeApp

class MonitorWindow(object):
    """ The monitor window
    """
    def __init__(self, ana=None):
        self.ana = ana
        self.create_window()
        
    
    def on_close(self, widget, *args):
        self.monitor.hide()
        self.monitor.destroy()
        
    def create_histogram(self,widget_name,str1, str2, int1, int2):
        if not self.ana:
            return gtk.Label("No analysis found")
        
        hst = None
        if widget_name=="hstSignal":
            hst = gtkhst.Histogram(self.ana.nbins,self.ana.range_min,self.ana.range_max,self.ana.signal )
        elif widget_name=="hEvent":
            hst = gtkhst.Histogram(self.ana.ntot,0,self.ana.ntot,self.ana.event )
        elif widget_name=="hSignal":
            hst = gtkhst.Histogram(self.ana.ntot,0,self.ana.ntot,self.ana.channels )
        elif widget_name=="hPed":
            hst = gtkhst.Histogram(self.ana.ntot,0,self.ana.ntot,self.ana.ped )
        elif widget_name=="hNoise":
            hst = gtkhst.Histogram(self.ana.ntot,0,self.ana.ntot,self.ana.noise )
            
        widget = gtkhst.Hst1D(hst)
        widget.monitor_start(250)
        return widget
    
    def create_window(self):
        """ Creates the monitor window
        """
        wnd = gtk.Dialog()
        wnd.set_title("Monitor")
        
        vbox = wnd.vbox
        
        notebook = gtk.Notebook()
        vbox.pack_start(notebook, True, True)
        
        vboxEvent = gtk.VBox(False, 5)
        notebook.add(vboxEvent)
        vboxEvent.set_border_width(3)
        
        hstSignal = self.create_histogram("hstSignal", "hstSignal", "", 0, 0)
        vboxEvent.pack_start(hstSignal, True, True, 0)
        
        hEvent = self.create_histogram("hEvent", "hEvent", "", 0, 0)
        vboxEvent.pack_start(hEvent, True, True, 0)
        
        hSignal = self.create_histogram("hSignal", "hSignal", "", 0, 0)
        vboxEvent.pack_start(hSignal, True, True, 0)
        
        notebook.set_tab_label(notebook.get_nth_page(0), gtk.Label("Data"))
        
        vboxPedestal = gtk.VBox(False, 5)
        notebook.add(vboxPedestal)
        vboxPedestal.set_border_width(3)
        
        hPed = self.create_histogram("hPed", "hPed", "", 0, 0)
        vboxPedestal.pack_start(hPed, True, True, 0)
        
        hNoise = self.create_histogram("hNoise", "hNoise", "", 0, 0)
        vboxPedestal.pack_start(hNoise, True, True, 0)
        
        notebook.set_tab_label(notebook.get_nth_page(1), gtk.Label("Pedestals"))
        
        action_area = wnd.action_area
        action_area.set_layout(gtk.BUTTONBOX_END)
        
        close_button = gtk.Button("close", stock=gtk.STOCK_CLOSE)
        wnd.add_action_widget(close_button, gtk.RESPONSE_CLOSE)
        pyvmedaq.connect(close_button, "clicked", self.on_close)
        
        self.monitor = wnd
        self.hstSignal = hstSignal
        self.hEvent = hEvent
        self.hSignal = hSignal
        self.hPed = hPed
        self.hNoise = hNoise
        
        wnd.show_all()
        
class MyAnalysis(pyvmedaq.Analysis):
    """ The analysis class
    """
    def __init__(self, nchip, nchan):
        super(MyAnalysis, self).__init__(nchip, nchan)
        
        
    def data_valid(self,data):
        """ Checks if the data has the correct structure
        """
        dsz = data[0] & 0xfff;
        sz  = data[2] & 0xfff;

        # Check data integrity
        if dsz != data[-1] |  dsz - sz != 4:
            return False;

        return True
        
    def decode(self, data):
        """ Decodes the data
        """
        out = self.event
        val = data[3:-1]
        if val.nitems != 128:
            print 'decode:',data[1], 'array size is',val.nitems,' instead of ',128,'array type %c' % val.type
            
        for i,v in enumerate(val):
            try:
                out[i] = v
            except IndexError:
                break
            
        self.compute_pedestals(range(0,self.ntot))
        
        

class MyModule(pyvmedaq.GenModule):
    """ This is a very simple module that defines a global buffer
        for storing the data.
    """
    def __init__(self,id, add, size=128):
        super(MyModule,self).__init__(id,add)
        self.size = size
        self.set_analysis( MyAnalysis(1, size) )
        self.data_buffer = daqpp.Buffer("L",[0L for x in range(0,size+4)])
        self.cntr = 0

    def PrepareForRun(self):
        print self.get_id(),"\t+-> PrepareForRun"
        return 1

    def Start(self,mode):
        print self.get_id(),"\t+-> Start",mode
        return 1

    def Stop(self,mode):
        print self.get_id(),"\t+-> Stop",mode
        return 1

    def Trigger(self):
        return 1

    def hasData(self):
        return True

    def get_data_type(self):
        """ This is the data type of the data block
        """
        return 0xd000

    def getData(self):
        self.increase_evt_counter()
        size = self.size + 4
        data = daqpp.Buffer("I",size)
        if size!=132 or data.nitems!=132:
            print '***** getData: wrong size:', data.nitems
        data[0] = (pyvmedaq.DataHeader | pyvmedaq.DataRecord | size)
        data[size-1] = size
        data[1] = self.evt_counter
        data[2] = int(str(self.get_id())) <<20
        for i in range(0,self.size):
            ival = int(random.gauss(100.,5.))
            data[3+i] = ival
            
        ichan = int(random.uniform(0,self.size))
        if random.random()>0.75:
            val = random.gauss(100.,10.)
        else:
            val = random.gauss(200.,5.)
            
        if val>0.:
            data[3+ichan] += int(val)
            
        if data.nitems != size:
            print 'getData: array size is',data.nitems,' instead of ',size
            
        return data;


ModuleList = []

class module_parser(pyvmedaq.ModuleParser):
    """ This is the module parser. It should create the module
        when parsing the corresponding section of the configuration
        file
    """
    def __init__(self):
        """ Whe create here the instance of the parser
        """
        super(module_parser, self).__init__()
        self.node = None
        self.mw = None
        
    def parse_node(self, _node):
        """ We should parse the config file element here.
            It creates the module as well
        """
        self.node = libxml2.xmlNode(_node)
        id = self.node.prop("name")
        self.id = id
        addr = self.node.prop("address")
        md = MyModule(id,int(addr,16))
        #ModuleList.append(md)
        
    def widget_destroy(self, wdgt):
        print 'module_parser.py: destroying widget',wdgt.get_name()
        
    def monitor(self,id, file):
        module = daqpp.find_module(id)
        self.mw = MonitorWindow(module.analysis)
        return self.mw.monitor
        
def create_test_module(id):
    print "Creating module %s" % id
    return module_parser()

def main():
    md = MyModule(512)
    print md.get_id()
    
    
if __name__=="__main__":
    main()