#include <sstream>
#include <DAQ++/DAQmanager.h>
#include "ADCmoduleGuiConfig.h"


ADCmoduleGuiConfig::ADCmoduleGuiConfig(const DAQpp::DAQid &id, ADCmodule::ADCThreshold tt, const std::vector<int> &T)
    : _id(id), thrs(T), thrs_type(tt)
{
}

ADCmoduleGuiConfig::~ADCmoduleGuiConfig()
{
}

void ADCmoduleGuiConfig::user_init()
{
    std::ostringstream ostr;
    std::string title = gtk_window_get_title(GTK_WINDOW(get_main_window()));
    ostr << title << " / Module "<< _id();
    gtk_window_set_title(GTK_WINDOW(get_main_window()), ostr.str().c_str());

    
    // Threshold type
    switch ( thrs_type)
    {
    case ADCmodule::Low:
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("btnStepLow")), TRUE);
        break;
    case ADCmodule::High:
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("btnStepHigh")), TRUE);
        break;
    case ADCmodule::Disable:
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("btnDisableThrs")), TRUE);
        break;
    default:
        std::cout << "ADCmoduleGuiCOnfig: wrong threshold type: " << thrs_type << std::endl;
    }
    
    
    // Now the heavy part... Resize the tables with the masks and threshold
    int i, k, ic, ir;
    int nchan = 32;
    int ncol = 8;
    int nrow = nchan / ncol + (nchan % ncol ? 1 : 0);
    GtkWidget *tmp;
    char line[10];
    GtkTable *tbl[2];
    tbl[0] = GTK_TABLE(get_widget("tableThresholds"));
    tbl[1] = GTK_TABLE(get_widget("tableMask"));

    // Clear and resize the tables
    for (k = 0; k < 2; k++)
    {
        gtk_widget_hide( GTK_WIDGET(tbl[k]));
        gtk_container_foreach(GTK_CONTAINER(tbl[k]), (GtkCallback) gtk_widget_destroy, NULL);
        gtk_table_resize(tbl[k], nrow, ncol + 1);
        for (i = 0, ir = -1; i < nchan; i++, ic++)
        {
            if (!(i % ncol))
            {
                ir++;
                ic = 0;
                // add the label
                sprintf(line, "%d", i);
                tmp = gtk_label_new(line);
                gtk_table_attach(tbl[k], tmp, ic, ic + 1, ir, ir + 1, (GtkAttachOptions) 0, (GtkAttachOptions) 0, 0, 0);
                ic++;
            }
            if (k)
            {
                // create the check button
                tmp = gtk_check_button_new();
                //     gtk_widget_show(tmp);
                gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(tmp), FALSE);
                gtk_widget_set_size_request(tmp, 20, 20);
                gtk_table_attach(tbl[k], tmp, ic, ic + 1, ir, ir + 1, (GtkAttachOptions) (GTK_EXPAND), (GtkAttachOptions) (0), 0, 0);
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), get_mask(i));
                gtk_signal_connect(GTK_OBJECT(tmp), "toggled",GTK_SIGNAL_FUNC(onBtnToggled), this);

            }
            else
            {
                tmp = gtk_entry_new();
                sprintf(line, "%d", get_thres(i));
                gtk_entry_set_text(GTK_ENTRY(tmp), line);
                gtk_widget_set_size_request(tmp, 20, 20);
                gtk_table_attach(tbl[k], tmp, ic, ic + 1, ir, ir + 1, (GtkAttachOptions) (GTK_EXPAND |GTK_FILL),
                                 (GtkAttachOptions) (0), 0, 0);
                gtk_signal_connect(GTK_OBJECT(tmp), "changed",GTK_SIGNAL_FUNC(onTxtChanged), this);

            }
            gtk_object_set_data(GTK_OBJECT(tmp), "channel", (gpointer) i);
            gtk_object_set_data(GTK_OBJECT(tmp), "type", (gpointer) k);
        }
    }
    for (k = 0; k < 2; k++)
        gtk_widget_show_all( GTK_WIDGET(tbl[k]));

}

void ADCmoduleGuiConfig::auto_connect()
{
    SET_CONNECT( onOK );
    SET_CONNECT( onApply );
    SET_CONNECT( onCancel );
    SET_CONNECT( on_thrs_type);
    /*
    connect["btnStepLow"] = (GCallback)on_thrs_type;
    connect["btnStepHigh"] = (GCallback)on_thrs_type;
    connect["btnDisableThrs"] = (GCallback)on_thrs_type;
    */   
}
void ADCmoduleGuiConfig::process_text(const std::string &s, GtkWidget *w)
{
        
}

void ADCmoduleGuiConfig::update()
{
    
}

void ADCmoduleGuiConfig::set_values()
{
    ADCmodule *m = dynamic_cast<ADCmodule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(_id));
    if ( !m )
        return;

    for (int i=0;i<32;i++)
    {
        m->set_threshold( i, get_thres(i));
        m->set_mask(i, get_mask(i));
    }
    m->set_thrs_type(thrs_type);
}


void ADCmoduleGuiConfig::onOK(GtkWidget *w, ADCmoduleGuiConfig *d)
{
    if (d)
    {
        onApply(w, d);
        onCancel(w, d);
    }
}
void ADCmoduleGuiConfig::onApply(GtkWidget *w, ADCmoduleGuiConfig *d)
{
    d->set_values();
}
void ADCmoduleGuiConfig::onCancel(GtkWidget *w, ADCmoduleGuiConfig *d)
{
    delete d;
}

void ADCmoduleGuiConfig::onTxtChanged(GtkWidget *w, ADCmoduleGuiConfig *d)
{
    int ival;
    long ichan = (long) gtk_object_get_data(GTK_OBJECT(w), "channel");
    const char *cval = gtk_entry_get_text(GTK_ENTRY(w));

    char *endp;
    ival = strtol(cval, &endp, 0);
    if (endp != cval )
    {
        ival &= 0xff;
        d->set_thesthold(ichan,ival);
    }
}

void ADCmoduleGuiConfig::onBtnToggled(GtkWidget *w, ADCmoduleGuiConfig *d)
{
    long ichan = (long) gtk_object_get_data(GTK_OBJECT(w), "channel");
    int lval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
 
    d->set_mask(ichan, lval);
    
}

void ADCmoduleGuiConfig::on_thrs_type(GtkWidget *wdgt, ADCmoduleGuiConfig *d)
{
    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wdgt)))
    {
        std::string wnam = gtk_widget_get_name(wdgt);
        std::cout << "threshold type " << wnam << std::endl;
        if ( wnam == "btnStepLow")
        {
            d->thrs_type = ADCmodule::Low;
        }
        else if ( wnam == "btnStepHigh")
        {
            d->thrs_type = ADCmodule::High;
        }
        else
            d->thrs_type = ADCmodule::Disable;
    }
}
