/* -*- mode: c++ -*- */
#ifndef ADCMODULEGUICONFIG_H_
#define ADCMODULEGUICONFIG_H_

#include <string>
#include <vector>
#include <DAQ++/DAQid.h>
#include <gui/GladeApp.h>
#include <gui/WidgetUpdate.h>
#include <ADCmodule.h>

class ADCmoduleGuiConfig : public GladeApp
{
private:
    DAQpp::DAQid _id;
    std::vector<int> thrs;
    ADCmodule::ADCThreshold thrs_type;

    
    static void onOK(GtkWidget *w, ADCmoduleGuiConfig *d);
    static void onApply(GtkWidget *w, ADCmoduleGuiConfig *d);
    static void onCancel(GtkWidget *w, ADCmoduleGuiConfig *d);
    static void onBtnToggled(GtkWidget *, ADCmoduleGuiConfig *);
    static void onTxtChanged(GtkWidget *, ADCmoduleGuiConfig *);
    static void on_thrs_type(GtkWidget *, ADCmoduleGuiConfig *);
    
    int get_thres(int x) const
    {
        return thrs[x] & 0xff;
    }
    void set_thesthold(int chan, int val)
    {
        int xval = (val & 0xff);
        thrs[chan] = (thrs[chan]&0x100) | xval;
    }
    void set_mask(int chan, bool val)
    {
        if (val)
            thrs[chan] |= 0x100;
        else
            thrs[chan] &= 0xff;
    }
    bool get_mask(int x) const
    {
        return (thrs[x]&0x100)!=0;
    }
    
    
public:
	ADCmoduleGuiConfig(const DAQpp::DAQid &id, ADCmodule::ADCThreshold tt, const std::vector<int> &T);
	virtual ~ADCmoduleGuiConfig();
	
	void update();
	void set_values();

	// GladeApp methods
	void user_init();
	void auto_connect();
	void process_text(const std::string &s, GtkWidget *w);
};

#endif /*ADCMODULEGUICONFIG_H_*/
