#include <config/Config.h>
#include "ADCmodule_analysis.h"

ADCmodule_analysis::ADCmodule_analysis(int n_chip, int n_chan) :
    Analysis(n_chip, n_chan)
{
}

ADCmodule_analysis::~ADCmodule_analysis()
{
}

void ADCmodule_analysis::Analyze()
{
    int i, i_max;
    double xs, xs_max;
    double *chan = Chan();
    double *noise = Noise();

    if (hits.empty() )
        return;

    sn_max = 0.;

    // Look for the highest pulse
    xs_max = -1.e10;
    i_max = -1;
    std::vector<int> isig;
    for (i = hits.top(); 1;)
    {
        if (noise[i] > 0.25) {
            xs = chan[i] / noise[i];
            isig.push_back(i);
        }

        hits.pop();
        if (hits.empty() )
            break;

        i = hits.top();
    }

    // If found, fill histograms
    if (isig.empty())
        return;

    m_fill.acquire();
   
    double channel_sum = 0; 
    std::vector<int>::iterator ip;
    for (ip=isig.begin();ip!=isig.end();++ip)
    {
        i_max = *ip;
        HitMap()[i_max]++;
        int ib = get_bin(chan[i_max]);
        if (ib >= 0&& ib < get_nbins() )
        {
            Signal(i_max / Nchan() )[ib]++;
            //Signal(-1)[ib]++;
            channel_sum += chan[i_max];
            Signal2D(i_max)[ib]++;
            if (get_map())
            {
                int ix, iy, ip;
                get_map()->pad(i_max, ix, iy);
                ip = ix + get_map()->nx() * iy;
                H2Map()[ ip ]++;
                if (chan[i_max] >= emin && chan[i_max] <= emax )
                    H2MapC()[ ip ]++;
            }
        }
    }
    int ib = get_bin(channel_sum);
    if ( ib >= 0 && ib < get_nbins() ) Signal(-1)[ib]++;
    else if ( ib < 0 ) Signal(-1)[0]++;
    else Signal(-1)[get_nbins()-1]++;
    m_fill.release();
}

void ADCmodule_analysis::decode(unsigned int *data)
{
    double *out = Event();
    double *chan = Chan();
    int i, ip, ibuffer, nbuffer, add, val;
    ConfigParser *cfg = ConfigParser::theConfig();
    GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));

    DoCommonMode(false);

    // Clear output array
    for (i = 0; i < Ntot(); i++)
        chan[i] = out[i] = 0.;

    std::vector<int> chan_list;
    chan_list.reserve(Ntot());

    nbuffer = data[3];
    ip=4;
    for (ibuffer=0; ibuffer<nbuffer; ibuffer++)
    {
        int header = data[ip]>>16;
        if (header!=0xacdc)
        {
            std::cout << "Wrong event format"<< std::endl;
            return;
        }
        int nchan = data[ip++] & 0xffff;
        for (i=0; i<nchan; i++, ip++)
        {
            add = (data[ip] >> 16) & 0x1F;
            val = data[ip] & 0xFFF;
            if (val==0) val = (data[ip] >> 20) & 0xFFF;
            out[add] = val/4;
            chan_list.push_back(add);
        }
        // skip the event counter
        ip++;
    }

    if (gnrl->get_ped_method() == GeneralConfig::normal)
    {
        update_pedestals(chan_list, true);
    }
    else
    {
        update_pedestals_fast(chan_list, true);
    }
}
