#ifndef __AgilentPulseGenerator_h__
#define __AgilentPulseGenerator_h__

#include <string>
#include <daq/PulseGenerator.h>

/**
 * \brief class AgilentPulseGenerator
 * \ingroup DAQ++
 * 
 * This class provides an interface for a pulse generator.
 * 
 */
class AgilentPulseGenerator : public PulseGenerator
{
public:
    enum Impedance {Low, High};
private:
    int dev;

    int status;
    int instr;

    bool check_status();
public:
    /// Constructor
    AgilentPulseGenerator(int id);

    /// Destructor
    virtual ~AgilentPulseGenerator();
    int SetID(int gid);

    // Sends a GPIB command
    bool SendCmd(char*, int, const char*);

    // Resets
    void Reset();

    // Check initialization
    bool operator!() const
    {
        return dev < 0;
    }

    /*      +----------+              +------ . . . .
     *      |          |              |
     *      |          |              |
     *   ---+          +--------------+
     *     <>         <>
     *     Rise time  Fall time
     *
     *      <-- width ->
     *      <------------Period  ----->
     */
    void define_pulse(double period, double width, double raise);
    // Configure output
    void ConfigureOutput(double imp);
    // Set Output amplitude
    void set_pulse(double v);
    // Set the trigger frequency
    void set_frequency(double x);

    // Activate Burst
    void activate_burst(bool z);
    /**
     * Defines the burst
     * @param delay: delay of burst w.r.t the external trigger
     * @param freq: frequency
     * @param npulse: number of pulses in the burst
     * @param period: period
     */
    void define_burst(double delay, double freq, int npulse, double period);
    // Sets trigger modePulseGenerator::A
    void trigger();
    void set_trigger_type(TriggerType v);
    void set_delay(double x);
    
    // enables and disables the output
    void set_output(bool z);
    void enable_output() { set_output(true); }
    void disable_output() { set_output(false); }
};

#endif
