#include <config/Config.h>
#include "BGOmodule_analysis.h"
#include <list>
#include <map>
#include "BGOmap.h"
#include <fstream>

using namespace std;

BGOmodule_analysis::BGOmodule_analysis(int n_chip, int n_chan) :
  Analysis(n_chip, n_chan),nBad(0)
{
  //perform usual analysis on 32 inputs
  ana=new Analysis(1,32);
  ana->DoCommonMode(false);
  ana->DoPedestalUpdate(true);

  //this is more of a holder for proper event display, so don't do any fancy computing
  DoCommonMode(false);
  DoPedestalUpdate(false);
  for (int i=0;i<n_chip;i++) mapList.push_back(new BGOmap);


}

BGOmodule_analysis::~BGOmodule_analysis()
{
  delete ana;
}

int getModuleOffset(int v785id){
   switch(v785id){
	case 300: return 0;
	case 301: return 2;
	case 302: return 1;
}
  std::cout<<"Unkonwn module "<<v785id<<std::endl;
  return 0;
}

void BGOmodule_analysis::Analyze() {

  double *sig = ana->Chan();
  //double *noise = ana->Noise();
  
  int ioff=getModuleOffset(id);

  //id is known so we can load the gains
  if (!gain.size()){
    gain.resize(Nchip()*Nchan());
    ostringstream os;
    os<<mesh_path<<"/gains.out";
    //string gainFile="/home/gamma/Andrej/root_scripts/gains.out";
    ifstream inf(os.str().c_str());
    if (!inf){
      cout<<"Could not open file "<<os.str()<<endl;
      for (vector<double>::iterator it=gain.begin();it!=gain.end();it++) (*it)=1;
    }
    while (inf){
      int off=256*ioff;
      int i;
      double a;
      inf>>i>>a;
      if (!inf) break;
      if (i-off>-1 && i-off<(Nchan()*Nchip())) gain[i-off]=a;
    }
    inf.close();
  }
  
  int iBGOmodule;
  //temporary hitList
  list<int> hitList=findHits(iBGOmodule);
  if (hitList.size()<1)
    return;
  
  //do mutex magic
  m_fill.acquire();
  
  for (int i=0;i<Ntot();i++) Chan()[i]=0;

  for(list<int>::iterator it=hitList.begin();it!=hitList.end();it++){
    //if ((*it)==1) continue;//skip hits from 1st bgomodule in each bundle; they were found out to be bad
    int iModule=(*it)+8*ioff;
    
    //get the map
    BGOmap *bMap=mapList[(*it)];
    if (!bMap->is_set()){
      ostringstream os;
      //os<<"/home/gamma/Andrej/root_scripts/Chan"<<iModule<<"_mesh.txt";
      os<<mesh_path<<"/Chan"<<iModule<<"_mesh.txt";
      bMap->parseBoundaries(os.str().c_str());
      if (!bMap->is_set()){
	//cout<<"Troubles setting pmt module="<<iModule<<endl;
	//continue;
      }
    }
    //get position
    double s=0;
    int i0=4*(*it);
    double m_sig[4];
    for (int i=0;i<4;i++) {
      m_sig[i]=sig[i0+i];
      if(m_sig[i]<0) m_sig[i]=0;
      s+=m_sig[i];
    }

    BGOmap::Point p;
    p.x=(m_sig[2]+m_sig[1]-m_sig[3]-m_sig[0])/s;
    p.y=(m_sig[3]+m_sig[2]-m_sig[1]-m_sig[0])/s;
    //cout<<"Hit "<<(*it)<<" ->["<<i0<<"]; ("<<m_sig[0]<<' '<<m_sig[1]<<' '<<m_sig[2]<<' '<<m_sig[3]<<")->"<<s<<endl;
    //cout<<'('<<p.x<<','<<p.y<<")->";
    p.x=64*(p.x+1);
    p.y=64*(p.y+1);
    //cout<<'('<<p.x<<','<<p.y<<')'<<endl;

    //find crystal ID from boundary map
    int cID=bMap->is_set() ? bMap->getCrystalID(p) : -1;
    if (cID<0) {
      //cout<<"nBad="<<nBad++<<" iEvt="<<iEvt<<endl;
      //continue;
    }
    //there is a hit right there
    
    int hitCrystal=-1;
    if (cID>-1){
      hitCrystal=(*it)*Nchan()+cID;
      hits.push(hitCrystal);
      s/=gain[hitCrystal];
      HitMap()[hitCrystal]++;
      Chan()[hitCrystal]=s;
    }
    int ib = get_bin(s);

    
    if (ib >= 0 && ib < get_nbins() ) {
      
      Signal((*it))[ib]++;
      Signal(-1)[ib]++;
      
      if (cID<0) continue;
      //skip the rest for bad signals
      Signal2D(hitCrystal)[ib]++;
      if (get_map()){
	int ix, iy, ip;
	get_map()->pad(hitCrystal, ix, iy);
	ip = ix + get_map()->nx() * iy;
	H2Map()[ ip ]++;
	if (s >= emin && s <= emax )
	  H2MapC()[ ip ]++;
      }
    }
  }
  m_fill.release();
}

void BGOmodule_analysis::decode(unsigned int *data)
{
  double *out = ana->Event();
  double *chan = ana->Chan();
  int i, ip, ibuffer, nbuffer, add, val;
  ConfigParser *cfg = ConfigParser::theConfig();
  GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));
  
  while (hits.size()) hits.pop();

  //get the id
  id=static_cast<int>((data[2]>>20)&0xfff);
  iEvt=static_cast<int>(data[1]);
  //std::cout<<"id="<<id<<std::endl;
  // Clear output array
  for (i = 0; i < Ntot(); i++)
    chan[i] = out[i] = 0.;

  std::vector<int> chan_list;
  chan_list.reserve(Ntot());
  
  nbuffer = data[3];
  ip=4;
    for (ibuffer=0; ibuffer<nbuffer; ibuffer++)
      {
        int header = data[ip]>>16;
        if (header!=0xacdc)
        {
            std::cout << "Wrong event format"<< std::endl;
            return;
        }
        int nchan = data[ip++] & 0xffff;
	//cout<<"nchan="<<nchan<<endl;
        for (i=0; i<nchan; i++, ip++)
        {
            add = (data[ip] >> 16) & 0x1F;
            val = data[ip] & 0xFFF;
            if (val==0) val = (data[ip] >> 20) & 0xFFF;
            out[add] = val/4;
	    //cout<<"add="<<add<<" val="<<val<<" out="<<out[add]<<endl;
            chan_list.push_back(add);
        }
        // skip the event counter
        ip++;
    }

    if (gnrl->get_ped_method() == GeneralConfig::normal)
    {
        ana->update_pedestals(chan_list, true);
    }
    else
    {
        ana->update_pedestals_fast(chan_list, true);
    }

    
    //the ana->Chan() should now be filled with pedestal subtracted data while out or ana->Event() should have raw data

    for (int i=0;i<Ntot();i++){
      Event()[i]=ana->Event()[i/8];
      //Chan()[i]=ana->Chan()[i/8];
      Noise()[i]=ana->Noise()[i/8];
      Ped()[i]=ana->Ped()[i/8];

    }
    


}

list<int> BGOmodule_analysis::findHits(int& iBlock){
  //iBlock holds the first channel of the quartet with the hits

  //cout<<"findHits"<<endl;
  list<int> hitBlockList;
  

  list<int> hitCandidates;
  double max=0;
  int iMax=0;
  double *sig=ana->Chan(0);
  double *noise=ana->Noise(0);
  for (int i=0;i<ana->Ntot();i++){
    //int ich = static_cast<int>( (*it) -> channel() );
    if ( sig [i] > 10 * noise [ i ] ){

      //cout<<"Found signal in ch="<<ich<<" sig= "<< (*it)->signal() 
	//  <<" pedestal=" << pedestal [ ich ] << " noise="<< noise [ ich ] <<endl;
      hitCandidates.push_back(i);
    }
    if ( sig[i] > max ) {
      max=sig[i];
      iMax=i;
    }
  }

  //cout<<"Max signal in "<<(*itMax)->channel()<<" sig="<<(*itMax)->signal()<<endl;
  
  if (hitCandidates.size()<1) return hitBlockList;
  

  //first guess is just the channel with the maximum signal
  iBlock=iMax/4;

  //a more refined guess is based on the fact that more than one pmt
  //in the block are hit; so for each hit channel find number of
  //channels that were hit in its block

  std::map<int,int> hitBlocks;
  
  for (list<int>::iterator it=hitCandidates.begin();it!=hitCandidates.end();it++){
    int tBlock= (*it) / 4;
    if (hitBlocks.find(tBlock)==hitBlocks.end()){
      //cout<<"New cluster ch= "<<(*it)->channel()<<" sig= "<< (*it)->signal() 
      //  <<" pedestal=" << pedestal [ (*it)->channel() ] << " noise="<< noise [ (*it)->channel() ] <<endl;
      hitBlocks[tBlock]=0;
    }
    hitBlocks[tBlock]++;
  }
  
  //good block has at least two PMTs with a signal
  list< std::map<int,int>::iterator > goodBlocks;

  for (std::map<int,int>::iterator it=hitBlocks.begin(); it!=hitBlocks.end(); it++)
    if (it->second>1) goodBlocks.push_back(it);
	
  
  //if there are no good blocks, the block with the max signal will do
  if (goodBlocks.size()<1) goodBlocks.push_back( hitBlocks.find(iBlock) );

  //make sure iBlock is a good block 
  //goodBlock has at least one entry which is either iBlock or another good block
  if (goodBlocks.size()>0){
    list < std::map<int, int> :: iterator >:: iterator it;
    for (it=goodBlocks.begin(); it!=goodBlocks.end(); it++)
      //found it; we're done
      if ((*it)->first==iBlock) break;
    
    if (it==goodBlocks.end()){
      //if it isn't - make iBlock the block with the maximum signal from the good blocks
      double max=-1e30;
      for (it=goodBlocks.begin(); it!=goodBlocks.end(); it++){
	int tBlock=(*it)->first;
	//find maximum of this block
	double maxBlock=0;
	for (list<int>::iterator ic=hitCandidates.begin(); ic!=hitCandidates.end(); ic++)
	  if ( (*ic) / 4 == tBlock && sig[(*ic)] > maxBlock ) maxBlock = sig[(*ic)];
	
	//bigger than global max -> take it
	if (maxBlock > max ){
	  max=maxBlock;
	  iBlock=(*it)->first;
	}
      }
    }
  }
	
  for (list<std::map<int,int>::iterator>::iterator it=goodBlocks.begin();it!=goodBlocks.end();it++)
    hitBlockList.push_back((*it)->first);
  return hitBlockList;
  //return static_cast<int>(goodBlocks.size());

}
