#include <iostream>
#include <sstream>
#include <vector>
#include <DAQ++/DAQmanager.h>
#include <config/ConfigParser.h>
#include <config/ModuleParser.h>
#include <config/GeneralConfig.h>
#include <data/Analysis.h>
#include <CSprint.h>

class ParseCS : public ModuleParser
{
public:
    ParseCS()
    {}
    ~ParseCS()
    {
        std::cout << "deleting ParseCS" << std::endl;
    }

    void       parse_node_id(const std::string &)
        throw(ConfigException);
    void       parse_node(xmlNodePtr )
        throw(ConfigException);

    void       save_config() throw (ConfigException);
    void       save_config_id(const std::string &) throw (ConfigException);
    void       save_config_node(xmlNodePtr) throw (ConfigException);

    _GtkWidget *config(const std::string &)
    {
        return 0;
    }

    MonitorWindow *monitor(const std::string &, const std::string &glade_file)
    {
        return 0;
    }
};

//=====================================================================
// Parsing stuff
//=====================================================================

void ParseCS::parse_node_id(const std::string &name)
    throw(ConfigException)
{
    std::ostringstream ostr;
    ConfigParser *cfg = ConfigParser::theConfig();
    DAQpp::Module *dm = DAQpp::DAQmanager::theDAQmanager()->find_module(name);

    if (!dm)
        return;

    if ( !dynamic_cast<CSPRINT *>(dm) )
    {
        std::cout << "ParseCS: module " << name
                  << " is not of type CSPRINT "
                  << std::endl;
        return;
    }

    int nmod;
    std::vector<xmlNodePtr> vmod;
    ostr << "//CSPRINT[@name=\"" << name << "\"]";
    nmod = cfg->get_node(ostr.str(), vmod);
    if (nmod)
        parse_node(vmod[0]);

}

void ParseCS::parse_node(xmlNodePtr _n)
    throw(ConfigException)
{
    ConfigParser *cfg = ConfigParser::theConfig();

    // we quit if the node is 0
    if (_n == 0) return;

    node = _n;
    id = cfg->get_attribute<std::string>(node, "name");

    // Check if the module already exist
    CSPRINT *m = 0;
    DAQpp::Module *dm;

    if ( (dm = DAQpp::DAQmanager::theDAQmanager()->find_module(id)) )
        m = dynamic_cast<CSPRINT *>(dm);


    // Now we start parsing
    std::ostringstream ostr;
    std::vector<xmlNodePtr> vthrs;
    int nthr;

    ostr << "//CSPRINT[@name=\"" << id << "\"]/device";
    nthr = cfg->get_node(ostr.str(), vthrs);
    if (nthr > 1)
    {
        std::cout << "ParseCS: more than one device set given to module "
                  << id << std::endl
                  << "Only the first one will be considered"
                  << std::endl;
    }
    std::string device = cfg->get_value<std::string>(vthrs[0]);


    if (!m) // Create the module
        m = new CSPRINT(id, device.c_str());

}



void ParseCS::save_config()
    throw (ConfigException)
{
    std::vector<xmlNodePtr> vmod;
    std::vector<xmlNodePtr>::iterator ip;
    ConfigParser *cfg = ConfigParser::theConfig();

    cfg->get_node("//CSPRINT", vmod);
    for (ip = vmod.begin();ip != vmod.end();ip++)
        save_config_node( *ip );
}

void ParseCS::save_config_id(const std::string &name)
    throw (ConfigException)
{
    std::ostringstream ostr;
    ConfigParser *cfg = ConfigParser::theConfig();
    DAQpp::Module *dm = DAQpp::DAQmanager::theDAQmanager()->find_module(name);

    if (!dm)
        return;

    if ( !dynamic_cast<CSPRINT *>(dm) )
    {
        std::cout << "ParseCS: module " << name
                  << " is not of type ADCodule "
                  << std::endl;
        return;
    }

    int nmod;
    std::vector<xmlNodePtr> vmod;
    ostr << "//CSPRINT[@name=\"" << name << "\"]";
    nmod = cfg->get_node(ostr.str(), vmod);
    if (nmod)
        save_config_node(vmod[0]);

}


void ParseCS::save_config_node(xmlNodePtr _n)
    throw (ConfigException)
{
    ConfigParser *cfg = ConfigParser::theConfig();

    node = _n;
    id = cfg->get_attribute<std::string>(node, "name");
    CSPRINT *m = 0;
    DAQpp::Module *dm;

    // find the module
    if ( !(dm = DAQpp::DAQmanager::theDAQmanager()->find_module(id)) )
        return;

    if ( !(m = dynamic_cast<CSPRINT *>(dm)) )
        return;

    save_common();
}


//=====================================================================
// Trigger interrupt stuff
//=====================================================================
#include <sys/poll.h>
#include <set>

struct dev_list
{
    int nfd;
    struct pollfd *ufds;
};

/*
  This is the trigger handler that DAQmanager needs.
  It should return true when a trigger is available and
  false otherwise. As it is implemented it will block
  until there is data
*/
class InterruptTrigger : public DAQpp::DAQTrigger
{
private:
    dev_list *dl;
public:
    InterruptTrigger(): dl(0)
    {}
    ~InterruptTrigger()
    {
        disableInterrupts();
    }
    void enableInterrupts();
    void disableInterrupts();
    bool operator()();
};
bool InterruptTrigger::operator()()
{
    if ( dl==0 )
        return false;

    int rc = poll(dl->ufds, dl->nfd, -1);
    if ( rc > 0 )
    {
        int i, nready = 0;
        for (i = 0;i < dl->nfd;i++)
        {
            if ( dl->ufds[i].revents & (POLLNVAL | POLLERR | POLLHUP) )
                continue;
            if ( dl->ufds[i].revents & (POLLIN | POLLPRI) )
                nready++;
        }
        return nready > 0;
    }
    return false;
}

void InterruptTrigger::enableInterrupts()
{
    std::cout << "Enabling interrupts" << std::endl;

    // Look for any CSprint module and make a list
    // with all the devices to check
    ConfigParser *cfg = ConfigParser::theConfig();
    int imod, nmod;
    std::vector<xmlNodePtr> vmod;
    nmod = cfg->get_node("//CSPRINT", vmod);
    if (!nmod)
        return;

    std::set<std::string> devs;
    for (imod = 0;imod < nmod;imod++)
    {
        xmlNode *node = vmod[imod];
        std::string d = cfg->get_value<std::string>(node, "device");
        devs.insert(d);
    }

    // Fill the structure needed to poll on those devices
    std::set<std::string>::iterator ip;
    dl = new dev_list;
    dl->nfd = devs.size();
    dl->ufds = new struct pollfd[dl->nfd];
    for (imod = 0, ip = devs.begin();ip != devs.end();++ip, imod++)
    {
        std::cout << *ip << std::endl;
        dl->ufds[imod].fd = open((*ip).c_str(), O_RDONLY);
        dl->ufds[imod].events = POLLIN | POLLPRI;
    }
}

void InterruptTrigger::disableInterrupts()
{
    if (dl)
    {
        delete [] dl->ufds;
        delete dl;
    }
    dl = 0;
}

static InterruptTrigger trigger;

extern "C"
{
    ModuleParser *create_CSPRINT()
    {
        ParseCS *gp = new ParseCS();
        return gp;
    }

    /*
      This is the routine called to enable trigger
      interrupts. If things went smooth it should return
      0, otherwise -1.
    */
    int enable_interrupts()
    {
        trigger.enableInterrupts();

        // Make DAQmanager aware of this
        DAQpp::DAQmanager::theDAQmanager()->set_trigger(&trigger);

        return 0;
    }

    /*
      This is the routine called to disable trigger interrupts
    */
    void disable_interrupts()
    {
        std::cout << "Disabling interrupts" << std::endl;
        DAQpp::DAQmanager::theDAQmanager()->set_trigger(0);
        trigger.disableInterrupts();
    }
}
