#include <iostream>
#include <sstream>
#include <iterator>
#include <vector>
#include <DAQ++/DAQmanager.h>
#include <config/ConfigParser.h>
#include <config/ModuleParser.h>
#include <config/GeneralConfig.h>
#include <data/Analysis.h>
#include "FBTDCmodule.h"

class ParseFBTDC : public ModuleParser
{
public:
    ParseFBTDC()
    {}
    ~ParseFBTDC()
    {
        std::cout << "deleting ParseFBTDC" << std::endl;
    }

    void       parse_node_id(const std::string &)
        throw(ConfigException);
    void       parse_node(xmlNodePtr )
        throw(ConfigException);

    void       save_config() throw (ConfigException);
    void       save_config_id(const std::string &) throw(ConfigException);
    void       save_config(xmlNodePtr);

    _GtkWidget *config(const std::string &)
    {
        return 0;
    }
    MonitorWindow *monitor(const std::string &,const std::string &glade_file)
    {
        return 0;
    }
};


//=====================================================================
// Parsing stuff
//=====================================================================

void ParseFBTDC::parse_node_id(const std::string &name)
    throw(ConfigException)
{
    std::ostringstream ostr;
    ConfigParser *cfg = ConfigParser::theConfig();
    DAQpp::Module *dm = DAQpp::DAQmanager::theDAQmanager()->find_module(name);

    if (!dm)
        return;

    if ( !dynamic_cast<FBTDCmodule *>(dm) )
    {
        std::cout << "ParseFBTDC: module " << name
                  << " is not of type FBTDCmodule "
                  << std::endl;
        return;
    }

    int nmod;
    std::vector<xmlNodePtr> vmod;
    ostr << "//FBTDC[@name=\"" << name << "\"]";
    nmod = cfg->get_node(ostr.str(), vmod);
    if (nmod)
        parse_node(vmod[0]);

}

void ParseFBTDC::parse_node(xmlNodePtr _n)
    throw(ConfigException)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    int address;
    std::string saddress;
    // we quit if the node is 0
    if (_n == 0) return;

    node = _n;
    id = cfg->get_attribute<std::string>(node, "name");
    saddress = cfg->get_attribute<std::string>(node, "address");

    std::istringstream os(saddress);
    os >> address;                      // slot number (decimal)

    // Check if the module already exist
    FBTDCmodule *m = 0;
    DAQpp::Module *dm;

    if ( (dm = DAQpp::DAQmanager::theDAQmanager()->find_module(id)) )
        m = dynamic_cast<FBTDCmodule *>(dm);

    if (!m) // Create the module
        m = new FBTDCmodule(id, address);

    // should not happen unless we run out of memory
    if (m == 0)
        return;

    // Now we start parsing
    std::ostringstream ostr;
    std::vector<xmlNodePtr> vthrs;
    int nthr;

    ostr << "//FBTDC[@name=\"" << id << "\"]/FBTDCthrs";
    nthr = cfg->get_node(ostr.str(), vthrs);
    if (nthr > 1)
    {
        std::cout << "FBTDCParse: more than one threshold set given to module "
                  << id << std::endl
                  << "Only the first one will be considered"
                  << std::endl;
    }
    xmlNodePtr nd = vthrs[0];
    std::vector<int> v;
    cfg->get_array<int>(nd, v);
    m->set_threshold( v );

    // range
    parse_common("//FBTDC", id);
}



void ParseFBTDC::save_config()
    throw (ConfigException)
{
    std::vector<xmlNodePtr> vmod;
    std::vector<xmlNodePtr>::iterator ip;
    ConfigParser *cfg = ConfigParser::theConfig();

    cfg->get_node("//FBTDC", vmod);
    for (ip = vmod.begin();ip != vmod.end();ip++)
        save_config( *ip );
}

void ParseFBTDC::save_config_id(const std::string &name)
    throw (ConfigException)
{
    std::ostringstream ostr;
    ConfigParser *cfg = ConfigParser::theConfig();
    DAQpp::Module *dm = DAQpp::DAQmanager::theDAQmanager()->find_module(name);

    if (!dm)
        return;

    if ( !dynamic_cast<FBTDCmodule *>(dm) )
    {
        std::cout << "ParseFBTDC: module " << name
                  << " is not of type FBTDCodule "
                  << std::endl;
        return;
    }

    int nmod;
    std::vector<xmlNodePtr> vmod;
    ostr << "//FBTDC[@name=\"" << name.c_str() << "\"]";
    nmod = cfg->get_node(ostr.str(), vmod);
    if (nmod)
        save_config(vmod[0]);

}


void ParseFBTDC::save_config(xmlNodePtr _n)
{
    ConfigParser *cfg = ConfigParser::theConfig();

    node = _n;
    id = cfg->get_attribute<std::string>(node, "name");
    FBTDCmodule *m = 0;
    DAQpp::Module *dm;

    // find the module
    if ( !(dm = DAQpp::DAQmanager::theDAQmanager()->find_module(id)) )
        return;

    if ( !(m = dynamic_cast<FBTDCmodule *>(dm)) )
        return;

    std::ostringstream othrs;
    std::vector<int> thrs = m->get_threshold();
    std::copy(thrs.begin(), thrs.end(), std::ostream_iterator<int>(othrs, " "));
    cfg->set_value<std::string>(node, "FBTDCthrs", othrs.str());

    save_common();
}


extern "C"
{
    ModuleParser *create_FBTDCmodule()
    {
        ParseFBTDC *gp = new ParseFBTDC();
        return gp;
    }
}

