#include <string>
#include <sstream>
#include <vector>
#include <cstring>

#include <gtk-hst/histogram.h>
#include <gtk-hst/histogram2d.h>
#include <gtk-hst/tracer.h>

#include <gtk-hst/gtkhistogram.h>
#include <gtk-hst/gtkhistogram2d.h>
#include <gtk-hst/gtkgauge.h>
#include <gtk-hst/histogram.h>
#include <gtk-hst/histogram2d.h>
#include <gtk-hst/tracer.h>

#include <data/Analysis.h>
#include <daq/GenModule.h>
#include <gui/gtkutils.h>
#include <gui/RunMenu.h>
#include <config/ConfigParser.h>
#include <config/GeneralConfig.h>

#include <DAQ++/DAQmanager.h>
#include "FPGACaenV1495_gui.h"

static int hst_update_freq = 1000;

#define IS_THIS_HST(w,item)  (!strncmp(gtk_widget_get_name(GTK_WIDGET(w)),item,strlen(item)))
struct _GtkHst1DEvent
{
        guint key;
        guint button;
        double x, y;
        double x0, y0;
};

//FPGACaenV1495_monitor __def_monitor__("__def_monitor__","",0);
void gui_force_load()
{
    
}

//=====================================================================
// constructor
//=====================================================================
FPGACaenV1495_monitor::FPGACaenV1495_monitor(const DAQpp::DAQid &id)
  : FPGACaenV1495_gui(id)
  , tmout(0)
  , tmout_stats(0)
  , g_chip(-1)
  , _ana(0)
{
    GenModule *md = dynamic_cast<GenModule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(_id));
    if (md) _ana = md->get_analysis();

    // find out if HstHistogram has the stats property
    GParamSpec *spec;
    GObjectClass *klass = (GObjectClass *)g_type_class_ref(HST_TYPE_HISTOGRAM);
    spec = g_object_class_find_property(klass, "stats");
    has_stats_prop = (spec != 0);
}

FPGACaenV1495_monitor::~FPGACaenV1495_monitor()
{
    if (tmout) gtk_timeout_remove(tmout);
    if (tmout_stats) gtk_timeout_remove(tmout_stats);

    while (updating) ;

    RunMenu::delete_menu(_id);
    stop_histograms();
    hlist.clear();
}

void
FPGACaenV1495_monitor::inLocal()
{
    GtkWidget *tmp = get_widget("runcontrol_button");
    if (tmp) gtk_widget_set_sensitive(tmp, TRUE);

    activate_group(get_widget("swDSPTrgType"), TRUE);
}

void
FPGACaenV1495_monitor::inGlobal()
{
    GtkWidget *tmp = get_widget("runcontrol_button");
    if (tmp) gtk_widget_set_sensitive(tmp, FALSE);

    activate_group(get_widget("swDSPTrgType"), FALSE);
}

void
FPGACaenV1495_monitor::user_init()
{
    // Set the window title
    std::ostringstream ostr;
    ostr << _id() << " - Data Monitor";
    gtk_window_set_title(GTK_WINDOW(get_main_window()), ostr.str().c_str());

    // Prepare the list of coincidences
    if (_ana)
    {
        int i;
        GtkWidget *item, *menu, *tmp = get_widget("coincidence_optionmenu");
        // put the real list of coincidences
        gtk_option_menu_remove_menu(GTK_OPTION_MENU(tmp));
        menu = gtk_menu_new();
        for (i = 0; i <= _ana->Nchip(); i++)
        {
            std::ostringstream ostr;
            if (i)
                ostr << i - 1;
            else
                ostr << "All";
            item = gtk_menu_item_new_with_label(ostr.str().c_str());
            gtk_widget_show(item);
            gtk_menu_append(GTK_MENU(menu), item);
        }
        gtk_option_menu_set_menu(GTK_OPTION_MENU(tmp), menu);
        g_signal_connect(G_OBJECT(menu), "selection_done", G_CALLBACK(on_NewChip), this);
        // Set current chip
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 0);
    }

    // Show all the widgets
    gtk_widget_show_all(get_main_window());

    //---------------------------------------------------------------------
    // Get run menu button
    //---------------------------------------------------------------------
    RunMenu::new_menu(std::string("rm." + _id.str()), get_widget("runcontrol_button") );

    tmout = gtk_timeout_add(500, on_Update, this);
    tmout_stats = gtk_timeout_add(2 * hst_update_freq, on_UpdateStats, this);

    on_NewChip(NULL, this);
    on_Update( this);
}

void
FPGACaenV1495_monitor::auto_connect()
{
    SET_CONNECT( on_quit_button_clicked );
    SET_CONNECT( on_refresh_button_clicked );
    SET_CONNECT( on_help_button_clicked );
}

void
FPGACaenV1495_monitor::set_handlers()
{
    handlers["create_hst_signal"] = (GladeHandler)create_hst_signal;
    handlers["create_hitmap"] = (GladeHandler)create_hitmap;
    handlers["create_event_display" ] = (GladeHandler)create_event_display;
}

void
FPGACaenV1495_monitor::on_refresh_button_clicked(GtkWidget *w, FPGACaenV1495_monitor *m)
{
    DAQpp::Module *dpx;
    dpx = DAQpp::DAQmanager::theDAQmanager()->find_module(m->_id);
    if (!dpx) return;

    dpx->reset_monitor_data();
    return;
}

void
FPGACaenV1495_monitor::on_quit_button_clicked(GtkWidget *w, FPGACaenV1495_monitor *m)
{
    m->updating = false;
    delete m;
}

void
FPGACaenV1495_monitor::on_help_button_clicked(GtkWidget *w, FPGACaenV1495_monitor *)
{
    gtk_help_display("monitoring-chapter");
}

void
FPGACaenV1495_monitor::on_NewChip(GtkWidget *w, FPGACaenV1495_monitor *m)
{
    HstList::iterator ip;
    GtkWidget *opt = m->get_widget("coincidence_optionmenu");
    int g_chip = gtk_option_menu_get_history(GTK_OPTION_MENU(opt));
    g_chip --;
    m->g_chip = g_chip;

    // Set the pointers to the corresponding histograms
    if (m->hlist.empty()) return;

    int nbins = m->_ana->get_nbins();
    double rmin = m->_ana->get_range_min();
    double rmax = m->_ana->get_range_max();

    for (ip = m->hlist.begin(); ip != m->hlist.end(); ++ip)
    {
        GtkWidget *w = *ip;
        if (IS_THIS_HST(w, "Signal1D"))
        {
            g_object_set( G_OBJECT(GTK_HST1D(w)->func) ,
                          "nx", nbins, "xmin", rmin, "xmax", rmax,
                          "ptr", m->_ana->Signal(g_chip),
                          NULL);
        }
    }
}

int
FPGACaenV1495_monitor::on_Update(void *p)
{
    FPGACaenV1495_monitor *mon = reinterpret_cast<FPGACaenV1495_monitor *>(p);

    mon->updating = true;

    // perform any necessary updates

    mon->updating = false;
    return 1;
}

int
FPGACaenV1495_monitor::on_UpdateStats(void *p)
{
    FPGACaenV1495_monitor *mon = reinterpret_cast<FPGACaenV1495_monitor *>(p);
    mon->update_histograms();
    return true;
}

void FPGACaenV1495_monitor::stop_histograms()
{
    HstList::iterator it;
    for (it = hlist.begin(); it != hlist.end(); ++it)
        gtk_hst1d_monitor_stop( GTK_HST1D(*it));
}

void FPGACaenV1495_monitor::update_histograms()
{
    HstList::iterator it;
    if (!has_stats_prop)
        return;

    for (it = hlist.begin(); it != hlist.end(); ++it)
    {
        if ( GTK_IS_HST2D( *it ))
            continue;

        if ( !HST_IS_HISTOGRAM( GTK_HST1D(*it)->func ))
            continue;

        HstHistogram *hst= HST_HISTOGRAM( GTK_HST1D(*it)->func );
        // Aply this trick to update the histogram stats
        g_object_set(G_OBJECT(hst), "stats", 0, NULL);
    }
}

GtkWidget * FPGACaenV1495_monitor::create_hst_signal(FPGACaenV1495_monitor *M, char *name, char *, char *, int, int)
{
    return M->create_2D_frame("Signal", "Signal (ADC)", NULL, "Signal (ADC)", "Channel");
}

GtkWidget * FPGACaenV1495_monitor::create_hitmap(FPGACaenV1495_monitor *M, char *name, char *, char *, int, int)
{
    return M->create_2D_frame("Map", "Channel", "Hit map", "Pad X", "Pad Y");
}

GtkWidget * FPGACaenV1495_monitor::create_event_display(FPGACaenV1495_monitor *M, char *name, char *, char *, int,
                                          int)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));
    
    GtkHst1D *evt;
    HstHistogram *hst = hst_histogram_new(M->_ana->Ntot(), 0, M->_ana->Ntot(), 0);
    g_object_set(G_OBJECT(hst), "ptr", M->_ana->Event(), NULL);
    GtkWidget *w = gtk_hst1d_new( HST_FUNCTION(hst));
    evt = GTK_HST1D(w);
    gtk_hst1d_draw_x_axis(evt, HST_AXIS_NONE);
    gtk_hst1d_draw_y_axis(evt, HST_AXIS_MIDDLE);
    g_object_set( G_OBJECT(evt),
    "hymin", std::min(-50., M->_ana->get_range_min()) ,
    "hymax", M->_ana->get_range_max(),
    NULL);
    gtk_hst1d_sync_range(GTK_HST1D(w), TRUE);
    gtk_hst1d_set_colors(evt, 
                         "foreground", gnrl->tracers().fg().c_str(), //"firebrick", 
                         "background", gnrl->tracers().bg().c_str(), //"black", 
                         "axis", gnrl->tracers().axis().c_str(), //"darkseagreen", 
                         NULL);
    gtk_hst1d_monitor_start(evt, hst_update_freq / 2);
    g_object_set_data(G_OBJECT(w), "hst", hst);
    M->hlist.insert(w);
    return w;
}

GtkWidget*
FPGACaenV1495_monitor::create_2D_frame(const std::string &prefix, char *xtit1,
                                       char *ytit1, char *xtit2, char *ytit2)
{
    GtkWidget *w;
    GtkWidget *bx1d, *bx2d;
    HstHistogram *h1d=0;
    HstHistogram2D *h2d=0;
    GtkHst1D *wh1d;
    GtkHst2D *wh2d=0;
    std::string nam1D = prefix + "1D";
    std::string nam2D = prefix + "2D";

    /* 1D histogram */
    if (prefix == "Signal")
    {
        h1d = hst_histogram_new(_ana->get_nbins(), _ana->get_range_min(),
                                _ana->get_range_max(), 0);
        g_object_set(G_OBJECT(h1d), "ptr", _ana->Signal(-1), NULL);

        h2d = hst_histogram2d_new(_ana->get_nbins(), _ana->get_range_min(),
                                  _ana->get_range_max(), _ana->Ntot(), 0., _ana->Ntot(),
                                   0);
        g_object_set(G_OBJECT(h2d), "ptr", _ana->Signal2D(), NULL);
    }
    else if (prefix == "Map")
    {
        h1d = hst_histogram_new(_ana->Ntot(), 0., _ana->Ntot(), 0);
        g_object_set(G_OBJECT(h1d), "ptr", _ana->HitMap(), NULL);
        if (_ana->get_map() )
        {
            const PadMap *pm = _ana->get_map();
            h2d = hst_histogram2d_new(pm->nx(), 0., pm->nx(), pm->ny(), 0., pm->ny(), 0);
            g_object_set(G_OBJECT(h2d), "ptr", _ana->H2MapC(), NULL);
        }
        else
        {
            h2d = hst_histogram2d_new( 1, 0., 1., 1, 0., 1., 0);
            g_object_set(G_OBJECT(h2d), "ptr", _ana->H2Map(), NULL);
        }
    }
    else
        h1d = hst_histogram_new(10, 0., 10., 0);

    wh1d = GTK_HST1D(gtk_hst1d_new( HST_FUNCTION(h1d) ));
    g_object_set_data( G_OBJECT(wh1d), "hst", h1d);
    if (xtit1)
        gtk_hst1d_set_xtitle(wh1d, xtit1);
    if (ytit1)
        gtk_hst1d_set_ytitle(wh1d, ytit1);
    gtk_hst1d_monitor_start(wh1d, hst_update_freq);
    gtk_hst1d_set_colors(wh1d, "background", "white", NULL);
    gtk_hst1d_sync_range(wh1d, TRUE);
    gtk_widget_set_name(GTK_WIDGET(wh1d), nam1D.c_str());
    bx1d = gtk_hbox_new(FALSE, 0);
    gtk_widget_set_name(bx1d, "Box1D");
    g_object_set_data(G_OBJECT(bx1d), "hst", wh1d);
    gtk_box_pack_start(GTK_BOX (bx1d), GTK_WIDGET(wh1d), TRUE, TRUE, 0);

    /* 2D histogram */
    if (h2d)
    {
        wh2d = GTK_HST2D(gtk_hst2d_new( h2d ));
        g_object_set_data( G_OBJECT(wh2d), "hst", h2d);
        gtk_widget_set_name(GTK_WIDGET(wh2d), nam2D.c_str());
        if (xtit2)
            gtk_hst1d_set_xtitle(GTK_HST1D(wh2d), xtit2);
        if (ytit2)
            gtk_hst1d_set_ytitle(GTK_HST1D(wh2d), ytit2);
        gtk_hst1d_set_colors(GTK_HST1D(wh2d), "background", "white", NULL);
        gtk_hst1d_sync_range(GTK_HST1D(wh2d), TRUE);
        bx2d = gtk_hbox_new(FALSE, 0);
        gtk_widget_set_name(bx2d, "Box2D");
        g_object_set_data(G_OBJECT(bx2d), "hst", wh2d);
        gtk_box_pack_start(GTK_BOX (bx2d), GTK_WIDGET(wh2d), TRUE, TRUE, 0);
    }

    w = gtk_frame_new(NULL);
    gtk_container_add(GTK_CONTAINER (w), GTK_WIDGET(bx1d));
    gtk_widget_show_all(w);
    g_object_set_data(G_OBJECT(w), "Hst1D", wh1d);
    g_object_set_data(G_OBJECT(w), "Hst2D", wh2d);
    g_object_set_data(G_OBJECT(w), "Box1D", bx1d);
    g_object_set_data(G_OBJECT(w), "Box2D", bx2d);
    g_object_set_data(G_OBJECT(w), "hidden", bx2d);
    hlist.insert(GTK_WIDGET(wh1d));
    hlist.insert(GTK_WIDGET(wh2d));
    return w;
}

void
FPGACaenV1495_monitor::resize_hst1d(GtkWidget *w, int nx, double xmin, double xmax,
                                    gpointer ptr, double hmin, double hmax)
{
    GtkHst1D *hst;
    g_return_if_fail( w != 0 );
    g_return_if_fail( GTK_IS_HST1D(w) );
    hst = GTK_HST1D(w);
    g_object_set( G_OBJECT(hst->func),
                  "nx", nx, "xmin", xmin, "xmax", xmax,
                  "ptr", ptr,
                  NULL);
    if (hmin != hmax)
        g_object_set( G_OBJECT(hst),
        "hymax", hmax,
        "hymin", hmin,
        NULL);
    hlist.insert(w);
}

void
FPGACaenV1495_monitor::resize_hst2d(GtkWidget *w, int nx, double xmin, double xmax, int ny,
                            double ymin, double ymax, gpointer ptr)
{
    GtkHst2D *hst;
    g_return_if_fail( w != 0 );
    g_return_if_fail( GTK_IS_HST2D(w) );

    hst = GTK_HST2D(w);

    g_object_set( G_OBJECT(GTK_HST1D(hst)->func),
                  "nx", nx, "xmin", xmin, "xmax", xmax,
                  "ny", ny, "ymin", ymin, "ymax", ymax,
                  "ptr", ptr,
                  NULL);

    hlist.insert(GTK_WIDGET(hst));
}

// Begin config gui implementation
void
FPGACaenV1495_config::user_init()
{
    std::ostringstream ostr;
    ostr << _id << " - Module configuration";
    gtk_window_set_title(GTK_WINDOW(get_main_window()), ostr.str().c_str());

    update();
}

void
FPGACaenV1495_config::update()
{
    DAQpp::Module *dm = DAQpp::DAQmanager::theDAQmanager()->find_module(_id);
    if (!dm)
        return;

    FPGACaenV1495 *m = 0;
    if ( !(m = dynamic_cast<FPGACaenV1495 *>(dm)))
    {
        std::cout << "FPGACaenV1495_config: module "<< _id << " is not of type FPGACaenV1495 "
                << std::endl;
        return;
    }

    unsigned int gatewidth = m->get_par_value<unsigned int>("gatewidth");
    unsigned int glevel    = m->get_par_value<unsigned int>("glevel");
    unsigned int amask     = m->get_par_value<unsigned int>("amask");
    unsigned int bmask     = m->get_par_value<unsigned int>("bmask");
    unsigned int cmask     = m->get_par_value<unsigned int>("cmask");

    gtk_spin_button_set_value(reinterpret_cast<GtkSpinButton*>(get_widget("gatewidth_spinbutton")),gatewidth);
    if ( glevel==0 )
      gtk_toggle_button_set_active(reinterpret_cast<GtkToggleButton*>(get_widget("glevel_radiobutton0")),true);
    else
      gtk_toggle_button_set_active(reinterpret_cast<GtkToggleButton*>(get_widget("glevel_radiobutton1")),true);
    for (int i=0;i<32;i++) {
      std::ostringstream oss;
      oss << "amask_togglebutton" << i;
      if ( amask&(1<<i) )
        gtk_toggle_button_set_active(reinterpret_cast<GtkToggleButton*>(get_widget(oss.str())),false);
      else
        gtk_toggle_button_set_active(reinterpret_cast<GtkToggleButton*>(get_widget(oss.str())),true);
      oss.str("");
      oss << "bmask_togglebutton" << i;
      if ( bmask&(1<<i) )
        gtk_toggle_button_set_active(reinterpret_cast<GtkToggleButton*>(get_widget(oss.str())),false);
      else
        gtk_toggle_button_set_active(reinterpret_cast<GtkToggleButton*>(get_widget(oss.str())),true);
      oss.str("");
      oss << "cmask_togglebutton" << i;
      if ( cmask&(1<<i) )
        gtk_toggle_button_set_active(reinterpret_cast<GtkToggleButton*>(get_widget(oss.str())),false);
      else
        gtk_toggle_button_set_active(reinterpret_cast<GtkToggleButton*>(get_widget(oss.str())),true);
    }
}

void
FPGACaenV1495_config::on_cancel_button_clicked(GtkWidget *w, FPGACaenV1495_config *m)
{
    delete m;
}

void
FPGACaenV1495_config::on_apply_button_clicked(GtkWidget *w, FPGACaenV1495_config *m)
{
    m->apply();
}

void
FPGACaenV1495_config::on_ok_button_clicked(GtkWidget *w, FPGACaenV1495_config *m)
{
    m->apply();
    delete m;
}

void
FPGACaenV1495_config::apply()
{
    DAQpp::Module *dm = DAQpp::DAQmanager::theDAQmanager()->find_module(_id);
    if (!dm)
        return;

    FPGACaenV1495 *m = 0;
    if ( !(m = dynamic_cast<FPGACaenV1495 *>(dm)))
    {
        std::cout << "FPGACaenV1495_config: module "<< _id << " is not of type FPGACaenV1495 "
                << std::endl;
        return;
    }

    GtkSpinButton *spin = reinterpret_cast<GtkSpinButton*>(get_widget("gatewidth_spinbutton"));
    unsigned int gatewidth = (unsigned int)gtk_spin_button_get_value(spin);

    GtkToggleButton *toggle = reinterpret_cast<GtkToggleButton*>(get_widget("glevel_radiobutton1"));
    unsigned int glevel = 0;
    if ( gtk_toggle_button_get_active(toggle) ) glevel = 1;

    unsigned int amask=0,bmask=0,cmask=0;
    for (int i=0;i<32;i++) {
      std::ostringstream oss;
      oss << "amask_togglebutton" << i;
      toggle = reinterpret_cast<GtkToggleButton*>(get_widget(oss.str()));
      if ( !gtk_toggle_button_get_active(toggle) ) amask |= (1<<i);
      oss.str("");
      oss << "bmask_togglebutton" << i;
      toggle = reinterpret_cast<GtkToggleButton*>(get_widget(oss.str()));
      if ( !gtk_toggle_button_get_active(toggle) ) bmask |= (1<<i);
      oss.str("");
      oss << "cmask_togglebutton" << i;
      toggle = reinterpret_cast<GtkToggleButton*>(get_widget(oss.str()));
      if ( !gtk_toggle_button_get_active(toggle) ) cmask |= (1<<i);
    }

    m->set_par<unsigned int>("gatewidth",gatewidth);
    m->set_par<unsigned int>("glevel", glevel);
    m->set_par<unsigned int>("amask", amask);
    m->set_par<unsigned int>("bmask", bmask);
    m->set_par<unsigned int>("cmask", cmask);
}
