#include "ReconModule.h"
#include "ReconModule_analysis.h"
#include "data/DataFormat.h"
#include "DAQ++/DAQmanager.h"

#include "FPGACaenV1495.h"

reconModule::reconModule(const DAQpp::DAQid &id, unsigned long address)
  : GenModule(id, address)
{
    // reset pedestals and noise to sensible values
  //fill_n(back_inserter( get_par_value< std::vector<double> > ("pedestal") ), nchan*nchip, 0.);
  //fill_n(back_inserter( get_par_value< std::vector<double> > ("noise") ), nchan*nchip, 0.);


  // Using DAQ++ parameters can be useful, especially if you want to
  // access them the DAQcorba
  //add_par("efficiency", new DAQpp::Par<double>(0.75));
  //add_par("mean", new DAQpp::Par<double>(100.));
  //add_par("width", new DAQpp::Par<double>(10.));



  // now we should create the Analysis for the monitoring
  // never forget to do this !!!
  std::cout<<"Creating reconModule "<<id<<" "<<std::hex<<address<<std::dec<<std::endl;
  ana = new reconModule_analysis(4,251);
  //ana->Resize(4, 251);
  

}

int reconModule::Start(int ){
  ana->init_run();
  return 0;
}


char *reconModule::getData(int &size){
  //reconModule is a knowledgeble module -> it knows what each module represents
  //it should also contain positions of individual modules (plenty to parse!)

  unsigned int int_id=get_id();
  size=4;
  unsigned int *data = reinterpret_cast<unsigned int *>(allocate(sizeof(unsigned int) * size));
  data[0]=DataHeader | DataRecord | size;
  data[1]=get_evt_counter();
  data[2]=((int_id & 0xfff) << 20) | get_data_type();
  data[3]=size;
  //std::cout<<"reconModule::getData:: data[2]="<<std::hex<<data[2]<<std::dec<<std::endl;
  //unsigned int uint_id=(data[2] & 0xfff00000) >> 20;
  //DAQpp::DAQid id(uint_id);
  //DAQpp::Module *mm = DAQpp::DAQmanager::theDAQmanager()->find_module(id);
  //std::cout<<"Found module "<<mm<<std::endl;
  size *=sizeof(unsigned int);
  return reinterpret_cast<char *>(data);
}

void reconModule::set_scan_point(ScanPoint &scp){
  
  std::cout<<"Setting scan point"<<std::endl;
  
  // Set variables to the values in the current point
  // For all the modules
  Reset();
  std::vector<ScanVar>::iterator ip;
  
  if ( true )
    {
      std::cout << "\t+-> reconModule::set_scan_point [" << get_id() << "]"
		<< std::endl;
    }
  
  ip = scp.find_type( ScanVar::angle1 );
  if (ip==scp.end()) return;

  (dynamic_cast<reconModule_analysis *>(ana))->set_angle(ip->value());
}

