#include "ReconModule.h"

#include <config/ConfigParser.h>
#include <config/ModuleParser.h>
#include <config/GeneralConfig.h>
#include <DAQ++/DAQmanager.h>
#include <gui/MonitorWindow.h>
#include <gui/DefaultMonitorGUI.h>
//#include "moduleGuiConfig.h"

class reconModuleConf : public ModuleParser{
public:
  reconModuleConf(){;}
  void parse_node(xmlNodePtr node) throw (ConfigException);
  void save_config() throw(ConfigException);
  MonitorWindow *monitor(const std::string &id, const std::string &glade_file);

};


extern "C"
{
  reconModuleConf *create_reconModuleConf()
  {
    reconModuleConf *cnf = new reconModuleConf();
    return cnf;
  }
}

void reconModuleConf::parse_node(xmlNodePtr _node) throw (ConfigException)
{
  unsigned long address;
  std::string saddress;
  
  // The configuration manager handler
  ConfigParser *cfg = ConfigParser::theConfig();
  
  // the node name
  name = node_name();
  
  // store the node pointer
  node = _node;
  
  // get the module id
  id = cfg->get_attribute<std::string>(node, "name");

  // get the address
  saddress = cfg->get_attribute<std::string>(node, "address");
  
  std::istringstream os(saddress);
  os >> std::hex >> address;
  
  // Check if the module is already there
  DAQpp::Module *dm;
  reconModule *module = 0;
  
  if ( (dm = DAQpp::DAQmanager::theDAQmanager()->find_module(id)))
    module = dynamic_cast<reconModule *>(dm);
  
  if (!module)
    {
        // The module is not there... create
      module = new reconModule(id, address);
    }
  
  // Now parse the things which are common to all the modules, that
  // is the monitor and padmap elements
  parse_common("//"+name, id);
}
  
void reconModuleConf::save_config() throw(ConfigException)
{
  // The configuration manager handler
  ConfigParser *cfg = ConfigParser::theConfig();
  
  // Save common stuff
  save_common();
}


//GUI


class ReconMonitorWindow : public MonitorWindow
{
    private:
        DefaultMonitorGUI *_mon;
    public:
        ReconMonitorWindow(const std::string &name, const std::string &glade_file,
                         reconModule *m)
        {
            _mon = new DefaultMonitorGUI( name,
                    glade_file,
                    m->get_analysis());
            _mon->init(glade_file.c_str(), "dsp-monitor");
            _widget = _mon->get_main_window();
        }
        ~ReconMonitorWindow()
        {
            delete _mon;
        }

        DefaultMonitorGUI *mon()
        {
            return _mon;
        }
};

MonitorWindow *reconModuleConf::monitor(const std::string &name, const std::string &glade_file)
{
    DAQpp::Module *dm = DAQpp::DAQmanager::theDAQmanager()->find_module(name);
    if (!dm)
        return 0;

    reconModule *m = 0;
    if ( !(m = dynamic_cast<reconModule *>(dm)))
    {
        std::cout << "reconModuleConf: module "<< name<< " is not of type reconModule "
                << std::endl;
        return 0;
    }

    ReconMonitorWindow *mw = new ReconMonitorWindow(name, glade_file, m);

    mw->add_update(new UpdateLabelNtrig(mw->mon()->get_widget("lblDSPnEvts")));
    if (mw->mon()->get_widget("gauge_chip_rate") )
      mw->add_update(new UpdateGaugeRate(mw->mon()->get_widget("gauge_chip_rate") ));
    
    if (mw->mon()->get_widget("gauge_chip_eff") )
      mw->add_update(new UpdateGaugeEff( mw->mon()->get_widget("gauge_chip_eff")));
    
    return mw;

}


static void __attribute__ ((constructor)) __init__lib__()
{
  // init threads
  std::cout << "### Loading reconModule..." << std::endl;
}

static void __attribute__ ((destructor)) __close__lib__()
{
  std::cout << "### Unloading reconModule..." << std::endl;
}



