/* -*- mode: c++ -*- */
#ifndef __SIS3300moduleAna_h__
#define __SIS3300moduleAna_h__

#include <vector>
#include <data/Analysis.h>
#include <data/DataFormat.h>
#include "WaveForm.h"

/**
 * This is the class that performs the analysis of the data sampled by
 * the Monitor process. This process, if enabled, will call
 * Analysis::Fill, which in turn will call
 *    Analysis::data_valid(unsigned int *data) to validate the data
 *                                              chunk
 * and, if the data is valid,
 *    Analysis::Analyze();
 *
 * Analyze expects the data already decoded and stored somehow in the
 * class. The data decoding is performed by ModuleHandler::decode and
 * that could be a good place to store the data internally in the
 * Analysis class.
 *
 * Both data_valid and Analyze are virtual methods so you can derive
 * from Analysis and overwrite those methods.
 *
 **/

class SIS3300moduleAna : public Analysis
{
    private:
        double *pulse_data;

        double *w1, *w2;

        double cut;

        std::vector<WaveForm*> vWaveForm;

        int ibad, process;
        bool simulate;

        void arm_WaveForms();
        void disarm_WaveForms();

        int unwrapData(const unsigned int *unwrap_data, int npmts, int nsamples,
                       u_int32_t ring_pointer, double *pulse_data);
        double integrate(int ipmt, int low, int high, int nsamples,
                         double *pulse_pmt_shape);

        void process_event();
        double calculate_ttime();
        void simulate_event();

    public:
        SIS3300moduleAna(int n_chip, int n_chan) :
            Analysis(n_chip, n_chan)
        {
        }
        bool data_valid(unsigned int *data) const;
        void decode(unsigned int *data);
        void Analyze();
        void init_run();
        void end_run();
};

#endif
