#include <iostream>
#include <iomanip>
#include <sstream>
#include <cstdio>
#include <DAQ++/DAQmanager.h>
#include <gui/gtkutils.h>
#include <gui/ParamSlider.h>
#include "SIS3300module.h"
#include "SIS3300moduleGuiConf.h"
#include <data/Analysis.h>
#include <config/ConfigParser.h>
#include <config/ModuleParser.h>


//=====================================================================
// Constructor
//=====================================================================
SIS3300moduleGuiConf::SIS3300moduleGuiConf(
    const DAQpp::DAQid &id)
        : _id(id), df(0), _monitor(0)
{
//---------------------------------------------------------------------
// Get the pointer to the module
//---------------------------------------------------------------------
    DAQpp::Module *dpx;
    dpx = DAQpp::DAQmanager::theDAQmanager()->find_module(_id);
    if ( dpx == 0 )
        return;

    module = dynamic_cast<SIS3300module *>(dpx);
    if (!module)
        return;
}

void
SIS3300moduleGuiConf::auto_connect()
{
    SET_CONNECT( onOK );
    SET_CONNECT( onApply );
    SET_CONNECT( onCancel );
    SET_CONNECT( on_checkALL );
    SET_CONNECT( on_checkEditThresholds );
}

void
SIS3300moduleGuiConf::user_init()
{
    update();

    if ( module->get_slave() )
    {
        gtk_widget_set_sensitive(get_widget("spinnInputs"), false);
        gtk_widget_set_sensitive(get_widget("optnSamples"), false);
        gtk_widget_set_sensitive(get_widget("spindelay"), false);
    }
    /*
    //---------------------------------------------------------------------
    // Readout mode
    //---------------------------------------------------------------------
       if (module) {
          double2entry("txtWped",module->get_analysis()->get_ped_weight(),"%6.4f");
          double2entry("txtWnoise",module->get_analysis()->get_noise_weight(),"%6.4f");
       }
    //---------------------------------------------------------------------
    // Sliders
    //---------------------------------------------------------------------
       if ( module ) {
          GtkSpinButton *btn  = GTK_SPIN_BUTTON( get_widget("spinThreshold") );
          GtkRange      *scale= GTK_RANGE( get_widget("scaleThreshold") );
          GtkAdjustment *adj  = gtk_range_get_adjustment(scale);
          gtk_spin_button_set_adjustment( btn, adj );
          ParamSlider::new_slider(_id,"delay",adj)->set_value_silent(module->get_delay());
       }
    */
}

SIS3300moduleGuiConf::~SIS3300moduleGuiConf()
{ }

void
SIS3300moduleGuiConf::onOK(GtkWidget *w, SIS3300moduleGuiConf *d)
{
    if (d)
    {
        onApply(w, d);
        onCancel(w, d);
    }
}

void
SIS3300moduleGuiConf::onApply(GtkWidget *w, SIS3300moduleGuiConf *d)
{
    d->set_values();
    d->update();
}

void
SIS3300moduleGuiConf::onCancel(GtkWidget *w, SIS3300moduleGuiConf *d)
{
    delete d;
}

void
SIS3300moduleGuiConf::update()
{
    int checknSamples = module->get_nSamples();
    GtkWidget *tmp = get_widget("optnSamples");

    std::cout << "nSamples  " << checknSamples << std::endl;

    switch (checknSamples)
    {
    case 0x80:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 0);
        break;
    case 0x100:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 1);
        break;
    case 0x200:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 2);
        break;
    case 0x400:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 3);
        break;
    case 0x800:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 4);
        break;
    case 0x1000:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 5);
        break;
    case 0x4000:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 6);
        break;
    case 0x20000:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 7);
        break;
    default:
        std::cout << "nSamples = " << checknSamples << " is not a valid page size" << std::endl;
    }


    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinnInputs")),
                              module->get_nInputs());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spindelay")),
                              module->get_delay());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold1")),
                              module->get_threshold1());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold2")),
                              module->get_threshold2());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold3")),
                              module->get_threshold3());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold4")),
                              module->get_threshold4());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold5")),
                              module->get_threshold5());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold6")),
                              module->get_threshold6());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold7")),
                              module->get_threshold7());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold8")),
                              module->get_threshold8());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinmean")),
                              module->get_mean());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinwidth")),
                              module->get_width());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinefficiency")),
                              module->get_efficiency());
}

void
SIS3300moduleGuiConf::set_values()
{
    int val, checknSamples = module->get_nSamples();
    GtkWidget *tmp = get_widget( "optnSamples");
    int select = gtk_option_menu_get_history(GTK_OPTION_MENU(tmp));

    std::cout << "old nSamples value = " << checknSamples << std::endl;

    switch (select)
    {
    case 0:
        val = 0x80;
        break;
    case 1:
        val = 0x100;
        break;
    case 2:
        val = 0x200;
        break;
    case 3:
        val = 0x400;
        break;
    case 4:
        val = 0x800;
        break;
    case 5:
        val = 0x1000;
        break;
    case 6:
        val = 0x4000;
        break;
    case 7:
        val = 0x20000;
        break;
    default:
        val = 0;
        std::cout << "menu selection = " << select << std::endl;
        std::cout << "menu selection invalid" << std::endl;
    }
    if (val > 0)
    {
        module->set_nSamples(val);
        std::cout << "new nSamples value =   " << module->get_nSamples() << std::endl;
    }

    std::cout << "old nInputs value =    " << module->get_nInputs() << std::endl;
    module->set_nInputs(entry2int("spinnInputs"));
    std::cout << "new nInputs value =    " << module->get_nInputs() << std::endl;

    std::cout << "old delay value =      " << module->get_delay() << std::endl;
    module->set_delay(entry2int("spindelay"));
    std::cout << "new delay value =      " << module->get_delay() << std::endl;

    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkALL"))) )
    {
        std::cout << "old threshold1 value = " << module->get_threshold1() << std::endl;
        module->set_threshold1(entry2int("spinALL"));
        std::cout << "new threshold1 value = " << module->get_threshold1() << std::endl;

        std::cout << "old threshold2 value = " << module->get_threshold2() << std::endl;
        module->set_threshold2(entry2int("spinALL"));
        std::cout << "new threshold2 value = " << module->get_threshold2() << std::endl;

        std::cout << "old threshold3 value = " << module->get_threshold3() << std::endl;
        module->set_threshold3(entry2int("spinALL"));
        std::cout << "new threshold3 value = " << module->get_threshold3() << std::endl;

        std::cout << "old threshold4 value = " << module->get_threshold4() << std::endl;
        module->set_threshold4(entry2int("spinALL"));
        std::cout << "new threshold4 value = " << module->get_threshold4() << std::endl;

        std::cout << "old threshold5 value = " << module->get_threshold5() << std::endl;
        module->set_threshold5(entry2int("spinALL"));
        std::cout << "new threshold5 value = " << module->get_threshold5() << std::endl;

        std::cout << "old threshold6 value = " << module->get_threshold6() << std::endl;
        module->set_threshold6(entry2int("spinALL"));
        std::cout << "new threshold6 value = " << module->get_threshold6() << std::endl;

        std::cout << "old threshold7 value = " << module->get_threshold7() << std::endl;
        module->set_threshold7(entry2int("spinALL"));
        std::cout << "new threshold7 value = " << module->get_threshold7() << std::endl;

        std::cout << "old threshold8 value = " << module->get_threshold8() << std::endl;
        module->set_threshold8(entry2int("spinALL"));
        std::cout << "new threshold8 value = " << module->get_threshold8() << std::endl;
    }
    else
    {
        std::cout << "old threshold1 value = " << module->get_threshold1() << std::endl;
        module->set_threshold1(entry2int("spinthreshold1"));
        std::cout << "new threshold1 value = " << module->get_threshold1() << std::endl;

        std::cout << "old threshold2 value = " << module->get_threshold2() << std::endl;
        module->set_threshold2(entry2int("spinthreshold2"));
        std::cout << "new threshold2 value = " << module->get_threshold2() << std::endl;

        std::cout << "old threshold3 value = " << module->get_threshold3() << std::endl;
        module->set_threshold3(entry2int("spinthreshold3"));
        std::cout << "new threshold3 value = " << module->get_threshold3() << std::endl;

        std::cout << "old threshold4 value = " << module->get_threshold4() << std::endl;
        module->set_threshold4(entry2int("spinthreshold4"));
        std::cout << "new threshold4 value = " << module->get_threshold4() << std::endl;

        std::cout << "old threshold5 value = " << module->get_threshold5() << std::endl;
        module->set_threshold5(entry2int("spinthreshold5"));
        std::cout << "new threshold5 value = " << module->get_threshold5() << std::endl;

        std::cout << "old threshold6 value = " << module->get_threshold6() << std::endl;
        module->set_threshold6(entry2int("spinthreshold6"));
        std::cout << "new threshold6 value = " << module->get_threshold6() << std::endl;

        std::cout << "old threshold7 value = " << module->get_threshold7() << std::endl;
        module->set_threshold7(entry2int("spinthreshold7"));
        std::cout << "new threshold7 value = " << module->get_threshold7() << std::endl;

        std::cout << "old threshold8 value = " << module->get_threshold8() << std::endl;
        module->set_threshold8(entry2int("spinthreshold8"));
        std::cout << "new threshold8 value = " << module->get_threshold8() << std::endl;
    }

    std::cout << "old mean value =       " << module->get_mean() << std::endl;
    module->set_mean(entry2double("spinmean"));
    std::cout << "new mean value =       " << module->get_mean() << std::endl;

    std::cout << "old width value =      " << module->get_width() << std::endl;
    module->set_width(entry2double("spinwidth"));
    std::cout << "new width value =      " << module->get_width() << std::endl;

    std::cout << "old efficiency value = " << module->get_efficiency() << std::endl;
    module->set_efficiency(entry2double("spinefficiency"));
    std::cout << "new efficiency value = " << module->get_efficiency() << std::endl;
}

void
SIS3300moduleGuiConf::toggleThresholds ()
{
    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkALL"))) )
    {
        gtk_widget_set_sensitive(get_widget("spinALL"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold1"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold2"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold3"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold4"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold5"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold6"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold7"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold8"), false);
    }
    else
    {
        gtk_widget_set_sensitive(get_widget("spinALL"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold1"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold2"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold3"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold4"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold5"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold6"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold7"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold8"), true);
    }
}

void
SIS3300moduleGuiConf::toggleEditThresholds ()
{
    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkEditThresholds"))) )
    {
        gtk_widget_set_sensitive(get_widget("checkALL"), true);
        toggleThresholds();
    }
    else
    {
        gtk_widget_set_sensitive(get_widget("checkALL"), false);
        gtk_widget_set_sensitive(get_widget("spinALL"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold1"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold2"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold3"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold4"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold5"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold6"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold7"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold8"), false);
    }
}

void
SIS3300moduleGuiConf::on_checkALL(GtkWidget *w, SIS3300moduleGuiConf *d)
{
    d->toggleThresholds();
}

void
SIS3300moduleGuiConf::on_checkEditThresholds(GtkWidget *w, SIS3300moduleGuiConf *d)
{
    d->toggleEditThresholds();
}
