#ifndef __SIS3300moduleGuiMon_h__
#define __SIS3300moduleGuiMon_h__
#include <iostream>
#include <iomanip>
#include <sstream>
#include <set>

#include <DAQ++/DAQid.h>
#include <data/Analysis.h>
#include <gui/WidgetUpdate.h>
#include <gui/GladeApp.h>


//////////////////////////////////////////////////////////////////////////
// class SIS3300moduleGuiMon
//
//   Created: Fri Aug 04 14:00:45 2006        Author: Eric Cochran
//   Purpose:
//
//   Modified from:
// class Monitor
//
//   Created: Mon May 27 17:42:33 2002        Author: Carlos Lacasta
//   Purpose:
//
//////////////////////////////////////////////////////////////////////////

class SIS3300moduleGuiMon : public GladeApp
{
private:
    DAQpp::DAQid _id;
    int       tmout;
    int       g_chip;
    Analysis *_ana;
    bool      updating;

    typedef std::set<GtkWidget *> HstList;
    HstList hlist;

    static void on_Quit(GtkWidget *w, SIS3300moduleGuiMon *m);
    static void on_Reset(GtkWidget *w, SIS3300moduleGuiMon *m);
    static void on_ResetPed(GtkWidget *w, SIS3300moduleGuiMon *m);
    static void on_Help(GtkWidget *w, SIS3300moduleGuiMon *m);
    static void on_NewChip(GtkWidget *, SIS3300moduleGuiMon *m);
    static void on_ChangeView(GtkWidget *, SIS3300moduleGuiMon *m);
    static void on_Change2Noise(GtkWidget *, SIS3300moduleGuiMon *m);
    static void on_Btn2DSignal(GtkToggleButton *, SIS3300moduleGuiMon *m);
    static void on_Btn2DMap(GtkWidget *, SIS3300moduleGuiMon *m);
    static void on_BtnApplyCut(GtkWidget *, SIS3300moduleGuiMon *m);
    static void on_BtnResetCut(GtkWidget *, SIS3300moduleGuiMon *m);
    static gboolean on_WindowSelDown(GtkWidget *, GdkEventButton *, SIS3300moduleGuiMon *);
    static gboolean on_WindowSelUp(GtkWidget *, GdkEventButton *, SIS3300moduleGuiMon *);
    static int  on_Update(void *);

    static GtkWidget *create_hst_pedestals(SIS3300moduleGuiMon *, char *, char *, char *, int , int);
    static GtkWidget *create_hst_noise(SIS3300moduleGuiMon *, char *, char *, char *, int , int);
    static GtkWidget *create_hst_signal(SIS3300moduleGuiMon *, char *, char *, char *, int , int);
    static GtkWidget *create_event_display(SIS3300moduleGuiMon *, char *, char *, char *, int , int);
    static GtkWidget *create_hitmap(SIS3300moduleGuiMon *, char *, char *, char *, int , int);
    static GtkWidget *create_tracer_cmmd(SIS3300moduleGuiMon *, char *, char *, char *, int , int);

    static GtkWidget *create_gauge_rate(SIS3300moduleGuiMon *, char *, char *, char *, int , int);
    static GtkWidget *create_gauge_eff(SIS3300moduleGuiMon *, char *, char *, char *, int , int);

    GtkWidget *create_2D_frame( const std::string &prefix,
                                char *xtit1, char *ytit1,
                                char *xtit2, char *ytit2 );

    void stop_histograms();
    void update_histograms();
    void reset_histograms();

    void toggle2Dhst(const char *frame, GtkToggleButton *);
    void resize_hst1d(GtkWidget *, int, double, double, gpointer, double hmin = 0., double hmax = 0.);
    void resize_hst2d(GtkWidget *, int, double, double, int, double, double, gpointer);

public:
    SIS3300moduleGuiMon(const DAQpp::DAQid &id);
    ~SIS3300moduleGuiMon();

    void inLocal();
    void inGlobal();

    // GladeApp methods
    void user_init();
    void auto_connect();
    void set_handlers();
};

#endif
