#include <iostream>
#include <iomanip>
#include <sstream>
#include <cstdio>
#include <DAQ++/DAQmanager.h>
#include <gui/gtkutils.h>
#include <gui/ParamSlider.h>
#include "SecondDetector.h"
#include "SIS3300module.h"
#include "SecondDetectorGuiConf.h"
#include <data/Analysis.h>
#include <config/ConfigParser.h>
#include <config/ModuleParser.h>

//=====================================================================
// Constructor
//=====================================================================
SecondDetectorGuiConf::SecondDetectorGuiConf(const DAQpp::DAQid &id) :
    _id(id), df(0), _monitor(0)
{
    //---------------------------------------------------------------------
    // Get the pointer to the module
    //---------------------------------------------------------------------
    DAQpp::Module *dpx;
    dpx = DAQpp::DAQmanager::theDAQmanager()->find_module(_id);
    if (dpx == 0)
        return;

    module = dynamic_cast<SecondDetector *>(dpx);
    if (!module)
        return;
}

void SecondDetectorGuiConf::auto_connect()
{
    SET_CONNECT( onOK );
    SET_CONNECT( onApply );
    SET_CONNECT( onCancel );
    SET_CONNECT( on_ConfigAmp );
    SET_CONNECT( on_checkEditThresholds );
    SET_CONNECT( on_checkAllThresholds );
    SET_CONNECT( on_checkEditAmplifiers );
    SET_CONNECT( on_checkAllAmplifiers );
    SET_CONNECT( on_spinAmplifier );
    SET_CONNECT( on_checkAmpEditDelay );
    SET_CONNECT( on_checkAmpEditGains );
    SET_CONNECT( on_checkAmpAllGains );
}

void SecondDetectorGuiConf::user_init()
{
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("checkAllThresholds")), true);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("checkEditThresholds")), false);
    toggleEditThresholds();

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("checkEditAmplifiers")), false);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("checkAllAmplifiers")), true);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("checkAmpEditDelay")), false);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("checkAmpEditGains")), false);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("checkAmpAllGains")), true);
    toggleEditAmplifiers();

    gtk_spin_button_set_range(GTK_SPIN_BUTTON(get_widget("spinAmplifier")), 0, (module->get_par_value<int>("nAmplifiers") - 1));

    update();

    /*
     //---------------------------------------------------------------------
     // Readout mode
     //---------------------------------------------------------------------
     if (dpx)
     {
     double2entry("txtWped",dpx->get_analysis()->get_ped_weight(),"%6.4f");
     double2entry("txtWnoise",dpx->get_analysis()->get_noise_weight(),"%6.4f");
     }

     //---------------------------------------------------------------------
     // Sliders
     //---------------------------------------------------------------------
     if ( dpx )
     {
     GtkSpinButton *btn  = GTK_SPIN_BUTTON( get_widget("spinThreshold") );
     GtkRange      *scale= GTK_RANGE( get_widget("scaleThreshold") );
     GtkAdjustment *adj  = gtk_range_get_adjustment(scale);
     gtk_spin_button_set_adjustment( btn, adj );
     ParamSlider::new_slider(_id,"delay",adj)->set_value_silent(dpx->get_delay());
     }
     */
}

SecondDetectorGuiConf::~SecondDetectorGuiConf()
{
}

void SecondDetectorGuiConf::onOK(GtkWidget *w, SecondDetectorGuiConf *d)
{
    if (d)
    {
        onApply(w, d);
        onCancel(w, d);
    }
}

void SecondDetectorGuiConf::onApply(GtkWidget *w, SecondDetectorGuiConf *d)
{
    d->set_values();
    d->update();
}

void SecondDetectorGuiConf::onCancel(GtkWidget *w, SecondDetectorGuiConf *d)
{
    delete d;
}

void SecondDetectorGuiConf::on_ConfigAmp(GtkWidget *w, SecondDetectorGuiConf *d)
{
    d->set_values();
    d->update();
    d->ConfigAmp();
}

void SecondDetectorGuiConf::ConfigAmp()
{
    module->ConfigAmplifiers();
}

void SecondDetectorGuiConf::update()
{
    int checknSamples = module->get_nSamples();
    GtkWidget *tmp = get_widget("optnSamples");

    switch (checknSamples)
    {
    case 0x80:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 0);
        break;
    case 0x100:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 1);
        break;
    case 0x200:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 2);
        break;
    case 0x400:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 3);
        break;
    case 0x800:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 4);
        break;
    case 0x1000:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 5);
        break;
    case 0x4000:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 6);
        break;
    case 0x20000:
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 7);
        break;
    default:
        std::cout << "Error::nSamples = "<< checknSamples<< " is not a valid page size"<< std::endl;
    }

    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinnInputs")), module->get_nInputs());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spindelay")), module->get_delay());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold1")), module->get_threshold1());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold2")), module->get_threshold2());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold3")), module->get_threshold3());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold4")), module->get_threshold4());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold5")), module->get_threshold5());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold6")), module->get_threshold6());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold7")), module->get_threshold7());
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinthreshold8")), module->get_threshold8());

    if ( (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkEditAmplifiers")))))
    {
        if ( !(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAllAmplifiers")))))
        {
            getAmpValues();
        }
    }
}

void SecondDetectorGuiConf::set_values()
{
    int val; //, checknSamples = module->get_nSamples();
    GtkWidget *tmp = get_widget("optnSamples");
    int select = gtk_option_menu_get_history(GTK_OPTION_MENU(tmp));

    switch (select)
    {
    case 0:
        val = 0x80;
        break;
    case 1:
        val = 0x100;
        break;
    case 2:
        val = 0x200;
        break;
    case 3:
        val = 0x400;
        break;
    case 4:
        val = 0x800;
        break;
    case 5:
        val = 0x1000;
        break;
    case 6:
        val = 0x4000;
        break;
    case 7:
        val = 0x20000;
        break;
    default:
        val = 0;
        std::cout << "Error::nSamples menu selection = "<< select<< " is invalid"<< std::endl;
    }
    if (val > 0)
    {
        module->set_nSamples(val);
    }

    module->set_nInputs(entry2int("spinnInputs"));
    module->set_delay(entry2int("spindelay"));

    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkEditThresholds"))) )
    {
        setThresholds();
    }

    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkEditAmplifiers"))) )
    {
        setAmpValues();
    }

    setSlaveValues();
}

void SecondDetectorGuiConf::setSlaveValues()
{
    int start_id = module->get_id();
    int iter_id = 0;

    for (int im = 0; im < module->get_nModules(); im++)
    {
        iter_id = start_id + im + 1;
        SIS3300module *m = 0;
        DAQpp::Module *dm;

        std::ostringstream oss(std::ostringstream::out);
        oss << iter_id;

        // find the module
        if ( !(dm = DAQpp::DAQmanager::theDAQmanager()->find_module(oss.str())))
        {
            std::cout << "Error::setSlaveValues(), module "<< iter_id << " not found"<< std::endl;
            return;
        }

        if ( !(m = dynamic_cast<SIS3300module *>(dm)))
        {
            std::cout << "Error::setSlaveValues(), SIS3300module "<< iter_id << " not found"<< std::endl;
            return;
        }

        m->set_nInputs(module->get_nInputs());
        m->set_nSamples(module->get_nSamples());
        m->set_delay(module->get_delay());

        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkEditThresholds"))) )
        {
            m->set_threshold1(module->get_threshold1());
            m->set_threshold2(module->get_threshold2());
            m->set_threshold3(module->get_threshold3());
            m->set_threshold4(module->get_threshold4());
            m->set_threshold5(module->get_threshold5());
            m->set_threshold6(module->get_threshold6());
            m->set_threshold7(module->get_threshold7());
            m->set_threshold8(module->get_threshold8());
        }
    }
}

void SecondDetectorGuiConf::setThresholds()
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAllThresholds"))) )
    {
        std::cout << "old threshold1 value = "<< module->get_threshold1() << std::endl;
        module->set_threshold1(entry2int("spinALL"));
        std::cout << "new threshold1 value = "<< module->get_threshold1() << std::endl;

        std::cout << "old threshold2 value = "<< module->get_threshold2() << std::endl;
        module->set_threshold2(entry2int("spinALL"));
        std::cout << "new threshold2 value = "<< module->get_threshold2() << std::endl;

        std::cout << "old threshold3 value = "<< module->get_threshold3() << std::endl;
        module->set_threshold3(entry2int("spinALL"));
        std::cout << "new threshold3 value = "<< module->get_threshold3() << std::endl;

        std::cout << "old threshold4 value = "<< module->get_threshold4() << std::endl;
        module->set_threshold4(entry2int("spinALL"));
        std::cout << "new threshold4 value = "<< module->get_threshold4() << std::endl;

        std::cout << "old threshold5 value = "<< module->get_threshold5() << std::endl;
        module->set_threshold5(entry2int("spinALL"));
        std::cout << "new threshold5 value = "<< module->get_threshold5() << std::endl;

        std::cout << "old threshold6 value = "<< module->get_threshold6() << std::endl;
        module->set_threshold6(entry2int("spinALL"));
        std::cout << "new threshold6 value = "<< module->get_threshold6() << std::endl;

        std::cout << "old threshold7 value = "<< module->get_threshold7() << std::endl;
        module->set_threshold7(entry2int("spinALL"));
        std::cout << "new threshold7 value = "<< module->get_threshold7() << std::endl;

        std::cout << "old threshold8 value = "<< module->get_threshold8() << std::endl;
        module->set_threshold8(entry2int("spinALL"));
        std::cout << "new threshold8 value = "<< module->get_threshold8() << std::endl;
    }
    else
    {
        std::cout << "old threshold1 value = "<< module->get_threshold1() << std::endl;
        module->set_threshold1(entry2int("spinthreshold1"));
        std::cout << "new threshold1 value = "<< module->get_threshold1() << std::endl;

        std::cout << "old threshold2 value = "<< module->get_threshold2() << std::endl;
        module->set_threshold2(entry2int("spinthreshold2"));
        std::cout << "new threshold2 value = "<< module->get_threshold2() << std::endl;

        std::cout << "old threshold3 value = "<< module->get_threshold3() << std::endl;
        module->set_threshold3(entry2int("spinthreshold3"));
        std::cout << "new threshold3 value = "<< module->get_threshold3() << std::endl;

        std::cout << "old threshold4 value = "<< module->get_threshold4() << std::endl;
        module->set_threshold4(entry2int("spinthreshold4"));
        std::cout << "new threshold4 value = "<< module->get_threshold4() << std::endl;

        std::cout << "old threshold5 value = "<< module->get_threshold5() << std::endl;
        module->set_threshold5(entry2int("spinthreshold5"));
        std::cout << "new threshold5 value = "<< module->get_threshold5() << std::endl;

        std::cout << "old threshold6 value = "<< module->get_threshold6() << std::endl;
        module->set_threshold6(entry2int("spinthreshold6"));
        std::cout << "new threshold6 value = "<< module->get_threshold6() << std::endl;

        std::cout << "old threshold7 value = "<< module->get_threshold7() << std::endl;
        module->set_threshold7(entry2int("spinthreshold7"));
        std::cout << "new threshold7 value = "<< module->get_threshold7() << std::endl;

        std::cout << "old threshold8 value = "<< module->get_threshold8() << std::endl;
        module->set_threshold8(entry2int("spinthreshold8"));
        std::cout << "new threshold8 value = "<< module->get_threshold8() << std::endl;
    }
}

void SecondDetectorGuiConf::toggleThresholds()
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAllThresholds"))) )
    {
        gtk_widget_set_sensitive(get_widget("spinALL"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold1"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold2"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold3"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold4"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold5"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold6"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold7"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold8"), false);
    }
    else
    {
        gtk_widget_set_sensitive(get_widget("spinALL"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold1"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold2"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold3"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold4"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold5"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold6"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold7"), true);
        gtk_widget_set_sensitive(get_widget("spinthreshold8"), true);
    }
}

void SecondDetectorGuiConf::toggleAmpGains()
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAmpAllGains"))) )
    {
        gtk_widget_set_sensitive(get_widget("spinAmpAllGains"), true);
        gtk_widget_set_sensitive(get_widget("spinAmpGain1"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain2"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain3"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain4"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain5"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain6"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain7"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain8"), false);
    }
    else
    {
        gtk_widget_set_sensitive(get_widget("spinAmpAllGains"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain1"), true);
        gtk_widget_set_sensitive(get_widget("spinAmpGain2"), true);
        gtk_widget_set_sensitive(get_widget("spinAmpGain3"), true);
        gtk_widget_set_sensitive(get_widget("spinAmpGain4"), true);
        gtk_widget_set_sensitive(get_widget("spinAmpGain5"), true);
        gtk_widget_set_sensitive(get_widget("spinAmpGain6"), true);
        gtk_widget_set_sensitive(get_widget("spinAmpGain7"), true);
        gtk_widget_set_sensitive(get_widget("spinAmpGain8"), true);
    }
}

void SecondDetectorGuiConf::toggleEditThresholds()
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkEditThresholds"))) )
    {
        gtk_widget_set_sensitive(get_widget("checkAllThresholds"), true);
        toggleThresholds();
    }
    else
    {
        gtk_widget_set_sensitive(get_widget("checkAllThresholds"), false);
        gtk_widget_set_sensitive(get_widget("spinALL"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold1"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold2"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold3"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold4"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold5"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold6"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold7"), false);
        gtk_widget_set_sensitive(get_widget("spinthreshold8"), false);
    }
}

void SecondDetectorGuiConf::toggleEditAmplifiers()
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkEditAmplifiers"))) )
    {
        gtk_widget_set_sensitive(get_widget("checkAllAmplifiers"), true);
        gtk_widget_set_sensitive(get_widget("checkAmpEditDelay"), true);
        gtk_widget_set_sensitive(get_widget("checkAmpEditGains"), true);
        gtk_widget_set_sensitive(get_widget("checkAmpAllGains"), true);
        toggleAmpEditDelay();
        toggleAmpEditGains();
        toggleAllAmplifiers();
    }
    else
    {
        gtk_widget_set_sensitive(get_widget("checkAllAmplifiers"), false);
        gtk_widget_set_sensitive(get_widget("spinAmplifier"), false);
        gtk_widget_set_sensitive(get_widget("checkAmpEditDelay"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpDelay"), false);
        gtk_widget_set_sensitive(get_widget("checkAmpEditGains"), false);
        gtk_widget_set_sensitive(get_widget("checkAmpAllGains"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpAllGains"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain1"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain2"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain3"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain4"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain5"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain6"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain7"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain8"), false);
    }
}

void SecondDetectorGuiConf::toggleAllAmplifiers()
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAllAmplifiers"))) )
    {
        gtk_widget_set_sensitive(get_widget("spinAmplifier"), false);

        std::ostringstream oss, oss_pmt;
        for (int i = 0; i < 8; i++)
        {
            oss.str("");
            oss_pmt.str("");
            oss << "labelAmpGain"<< (i + 1);
            oss_pmt << " gain"<< (i + 1)<< " (pmt*)  ";
            gtk_label_set_text(GTK_LABEL(get_widget(oss.str().c_str())), oss_pmt.str().c_str());
        }
    }
    else
    {
        gtk_widget_set_sensitive(get_widget("spinAmplifier"), true);
        getAmpValues();
    }
}

void SecondDetectorGuiConf::toggleAmpEditDelay()
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAmpEditDelay"))) )
    {
        gtk_widget_set_sensitive(get_widget("spinAmpDelay"), true);
    }
    else
    {
        gtk_widget_set_sensitive(get_widget("spinAmpDelay"), false);
    }
}

void SecondDetectorGuiConf::toggleAmpEditGains()
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAmpEditGains"))) )
    {
        gtk_widget_set_sensitive(get_widget("checkAmpAllGains"), true);
        toggleAmpGains();
    }
    else
    {
        gtk_widget_set_sensitive(get_widget("checkAmpAllGains"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpAllGains"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain1"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain2"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain3"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain4"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain5"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain6"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain7"), false);
        gtk_widget_set_sensitive(get_widget("spinAmpGain8"), false);
    }
}

void SecondDetectorGuiConf::getAmpValues()
{
    int amp, delay;
    amp = entry2int("spinAmplifier");

    std::ostringstream oss, oss_pmt;
    std::vector<int> pmts, gains;

    delay = module->get_AmpDelay(amp);
    module->get_AmpPMTs(amp, pmts);
    module->get_AmpGains(amp, gains);

    gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("spinAmpDelay")), delay);

    int i;
    std::vector<int>::iterator viter;
    for (i = 0, viter = pmts.begin(); viter != pmts.end(); ++viter, i++)
    {
        oss.str("");
        oss_pmt.str("");
        oss << "labelAmpGain"<< (i + 1);
        oss_pmt << " gain"<< (i + 1)<< " (pmt"<< *viter << ")  ";
        gtk_label_set_text(GTK_LABEL(get_widget(oss.str().c_str())), oss_pmt.str().c_str());
    }
    for (i = 0, viter = gains.begin(); viter != gains.end(); ++viter, i++)
    {
        oss.str("");
        oss << "spinAmpGain"<< (i + 1);
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget(oss.str().c_str())), *viter);
    }
}

void SecondDetectorGuiConf::setAmpValues()
{
    int amp, delay;
    std::ostringstream oss;
    std::vector<int> gains;

    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAllAmplifiers"))) )
    {
        for (int j = 0; j < (module->get_par_value<int>("nAmplifiers")); j++)
        {
            amp = j;
            gains.clear();

            if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAmpEditDelay"))) )
            {
                delay = entry2int("spinAmpDelay");
                module->set_AmpDelay(amp, delay);
            }

            int i;
            if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAmpEditGains"))) )
            {
                for (i = 0; i < 8; i++)
                {
                    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAmpAllGains"))) )
                    {
                        gains.push_back(entry2int("spinAmpAllGains"));
                    }
                    else
                    {
                        oss.str("");
                        oss << "spinAmpGain"<< (i + 1);
                        gains.push_back(entry2int(oss.str().c_str()));
                    }
                }
                module->set_AmpGains(amp, gains);
            }
        }
    }
    else
    {
        amp = entry2int("spinAmplifier");
        delay = entry2int("spinAmpDelay");
        module->set_AmpDelay(amp, delay);

        int i;
        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAmpEditGains"))) )
        {
            for (i = 0; i < 8; i++)
            {
                if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(get_widget("checkAmpAllGains"))) )
                {
                    gains.push_back(entry2int("spinAmpAllGains"));
                }
                else
                {
                    oss.str("");
                    oss << "spinAmpGain"<< (i + 1);
                    gains.push_back(entry2int(oss.str().c_str()));
                }
            }
            module->set_AmpGains(amp, gains);
        }
    }
}

void SecondDetectorGuiConf::on_checkEditThresholds(GtkWidget *w, SecondDetectorGuiConf *d)
{
    d->toggleEditThresholds();
}

void SecondDetectorGuiConf::on_checkAllThresholds(GtkWidget *w, SecondDetectorGuiConf *d)
{
    d->toggleThresholds();
}

void SecondDetectorGuiConf::on_checkEditAmplifiers(GtkWidget *w, SecondDetectorGuiConf *d)
{
    d->toggleEditAmplifiers();
}

void SecondDetectorGuiConf::on_checkAllAmplifiers(GtkWidget *w, SecondDetectorGuiConf *d)
{
    d->toggleAllAmplifiers();
}

void SecondDetectorGuiConf::on_spinAmplifier(GtkWidget *w, SecondDetectorGuiConf *d)
{
    d->getAmpValues();
}

void SecondDetectorGuiConf::on_checkAmpEditDelay(GtkWidget *w, SecondDetectorGuiConf *d)
{
    d->toggleAmpEditDelay();
}

void SecondDetectorGuiConf::on_checkAmpEditGains(GtkWidget *w, SecondDetectorGuiConf *d)
{
    d->toggleAmpEditGains();
}

void SecondDetectorGuiConf::on_checkAmpAllGains(GtkWidget *w, SecondDetectorGuiConf *d)
{
    d->toggleAmpGains();
}
