#include <cstring>
#include <gtk-hst/histogram.h>
#include <gtk-hst/histogram2d.h>
#include <gtk-hst/tracer.h>

#include <gtk-hst/gtkhistogram.h>
#include <gtk-hst/gtkhistogram2d.h>
#include <gtk-hst/gtkgauge.h>
#include <gtk-hst/histogram.h>
#include <gtk-hst/histogram2d.h>
#include <gtk-hst/tracer.h>

#include <DAQ++/DAQmanager.h>
#include <data/Analysis.h>
#include <daq/GenModule.h>
#include "SecondDetector.h"
#include "SecondDetectorGuiMon.h"
#include <gui/gtkutils.h>
#include <gui/RunMenu.h>
#include <config/ConfigParser.h>

static int hst_update_freq = 1000;

#define IS_THIS_HST(w,item)  (!strncmp(gtk_widget_get_name(GTK_WIDGET(w)),item,strlen(item)))

struct _GtkHst1DEvent
{
    guint key;
    guint button;
    double x, y;
    double x0, y0;
};

//=====================================================================
// constructor
//=====================================================================
SecondDetectorGuiMon::SecondDetectorGuiMon(const DAQpp::DAQid &id) :
    _id(id), tmout(0), g_chip(-1), _ana(0)
{
    GenModule *md = dynamic_cast<GenModule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(id));
    if (md)
        _ana = md->get_analysis();
}

void SecondDetectorGuiMon::auto_connect()
{
    SET_CONNECT(on_ResetPed );
    SET_CONNECT(on_Reset );
    SET_CONNECT(on_Quit );
    SET_CONNECT(on_Help );
    SET_CONNECT(on_Btn2DSignal );
    SET_CONNECT(on_Btn2DMap );
    SET_CONNECT(on_BtnApplyCut );
    SET_CONNECT(on_BtnResetCut );
}

void SecondDetectorGuiMon::set_handlers()
{
    handlers["create_hst_pedestals"] = (GladeHandler)create_hst_pedestals;
    handlers["create_hst_noise"] = (GladeHandler)create_hst_noise;
    handlers["create_event_display" ] = (GladeHandler)create_event_display;
    handlers["create_hitmap"] = (GladeHandler)create_hitmap;
    handlers["create_tracer_cmmd"] = (GladeHandler)create_tracer_cmmd;
    handlers["create_hst_signal"] = (GladeHandler)create_hst_signal;

    handlers["create_gauge_rate"] = (GladeHandler)create_gauge_rate;
    handlers["create_gauge_eff"] = (GladeHandler)create_gauge_eff;
}

void SecondDetectorGuiMon::user_init()
{
    GtkWidget *tmp;

    // Set the window title
    std::ostringstream ostr;
    ostr << _id() << " - Second Detector Monitor";
    gtk_window_set_title(GTK_WINDOW(get_main_window()), ostr.str().c_str());

    // Prepare the list of chips
    SecondDetector *md = dynamic_cast<SecondDetector *>(DAQpp::DAQmanager::theDAQmanager()->find_module(_id ) );
    if ( !md )
    {
        std::cout << "SecondDetectorGuiMon::user_init  Error: SecondDetector ["<< _id << "] not found"<< std::endl;
        return;
    }

    if (_ana)
    {
        GtkWidget *item, *menu, *tmp = get_widget("optChipSelect");
        // put the real list of chips
        gtk_option_menu_remove_menu(GTK_OPTION_MENU(tmp));
        menu = gtk_menu_new();

        std::vector<int> pmts;

        int i = 0;
        std::vector<int>::iterator viter;
        for (int amp = 0; amp < md->get_nModules(); amp++)
        {
            pmts.clear();
            md->get_AmpPMTs(amp, pmts );
            for (viter = pmts.begin(); viter != pmts.end(); ++viter )
            {
                std::ostringstream ostr;
                if (i)
                    ostr << i - 1<< " (pmt"<< *viter << ")";
                else
                    ostr << "All";
                item = gtk_menu_item_new_with_label(ostr.str().c_str());
                gtk_widget_show(item);
                gtk_menu_append(GTK_MENU(menu), item);

                i++;
            }
        }
        gtk_option_menu_set_menu(GTK_OPTION_MENU(tmp), menu);
        g_signal_connect(G_OBJECT(menu), "selection_done", G_CALLBACK(on_NewChip), this);
        // Set current chip
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 0);
    }

    // Get Common mode tracer
    g_signal_connect(G_OBJECT(gtk_option_menu_get_menu(GTK_OPTION_MENU(get_widget("optMonitorSelect")))), "selection_done",
                     G_CALLBACK(on_Change2Noise), this);
    // Get Signal type selection for the event display
    tmp = gtk_option_menu_get_menu(GTK_OPTION_MENU (get_widget("optViewData")));
    g_signal_connect(G_OBJECT(tmp), "selection_done", G_CALLBACK(on_ChangeView), this);

    // Show all the widgets
    gtk_widget_show_all( get_main_window());

    //---------------------------------------------------------------------
    // Get run menu button
    //---------------------------------------------------------------------
    RunMenu::new_menu(std::string("rm." + _id.str()), get_widget("btnRunControl") );

    tmout = gtk_timeout_add(500, on_Update, this);

    on_NewChip(NULL, this);
    on_Update( this);
}

SecondDetectorGuiMon::~SecondDetectorGuiMon()
{
    if (tmout )
        gtk_timeout_remove(tmout );

    while (updating )
        ;
    RunMenu::delete_menu(_id );

    stop_histograms();
    hlist.clear();
}

void SecondDetectorGuiMon::on_Help(GtkWidget *w, SecondDetectorGuiMon *)
{
    gtk_help_display("monitoring-chapter");
}

void SecondDetectorGuiMon::resize_hst1d(GtkWidget *w, int nx, double xmin, double xmax, gpointer ptr, double hmin, double hmax)
{
    GtkHst1D *hst;
    g_return_if_fail(w != 0);
    g_return_if_fail( GTK_IS_HST1D(w));
    hst = GTK_HST1D(w);
    g_object_set(G_OBJECT(hst->func), "nx", nx, "xmin", xmin, "xmax", xmax, "ptr", ptr, 
    NULL);
    if (hmin != hmax )
        g_object_set(G_OBJECT(hst), "hymax", hmax, "hymin", hmin, 
        NULL);
    hlist.insert(w);
}

void SecondDetectorGuiMon::resize_hst2d(GtkWidget *w, int nx, double xmin, double xmax, int ny, double ymin, double ymax,
                                        gpointer ptr)
{
    GtkHst2D *hst;
    g_return_if_fail(w != 0);
    g_return_if_fail( GTK_IS_HST2D(w));

    hst = GTK_HST2D(w);

    g_object_set(G_OBJECT(GTK_HST1D(hst)->func), "nx", nx, "xmin", xmin, "xmax", xmax, "ny", ny, "ymin", ymin, "ymax", ymax, "ptr", ptr, 
    NULL);

    hlist.insert(GTK_WIDGET(hst));
}

void SecondDetectorGuiMon::inLocal()
{
    GtkWidget *tmp = get_widget("btnRunControl");
    if (tmp )
        gtk_widget_set_sensitive(tmp, TRUE);

    activate_group(get_widget("swDSPTrgType"), TRUE);
}

void SecondDetectorGuiMon::inGlobal()
{
    GtkWidget *tmp = get_widget("btnRunControl");
    if (tmp )
        gtk_widget_set_sensitive(tmp, FALSE);

    activate_group(get_widget("swDSPTrgType"), FALSE);
}

void SecondDetectorGuiMon::stop_histograms()
{
    HstList::iterator it;
    for (it = hlist.begin(); it != hlist.end(); ++it)
        gtk_hst1d_monitor_stop( GTK_HST1D(*it));
}

void SecondDetectorGuiMon::update_histograms()
{
    HstList::iterator it;
    /*
     for (it = hlist.begin(); it!=hlist.end(); ++it)
     if ( GTK_MONITOR(*it)->type == GTK_MONITOR_HST1D )
     gtk_monitor_update_stat ( GTK_MONITOR(*it) );
     */
}

void SecondDetectorGuiMon::on_Quit(GtkWidget *w, SecondDetectorGuiMon *m)
{
    m->updating = false;
    delete m;
}

void SecondDetectorGuiMon::on_Reset(GtkWidget *w, SecondDetectorGuiMon *m)
{
    DAQpp::Module *dpx;
    dpx = DAQpp::DAQmanager::theDAQmanager()->find_module(m->_id);
    if (!dpx)
        return;

    dpx->reset_monitor_data();
    HstList::iterator ip;
    for (ip = m->hlist.begin(); ip != m->hlist.end(); ++ip)
    {
        GtkWidget *w = *ip;
        if (IS_THIS_HST(w, "trCommonMode"))
        {
            hst_histogram_reset(HST_HISTOGRAM( GTK_HST1D(w)->func ));
        }
    }
    return;
}

void SecondDetectorGuiMon::on_ResetPed(GtkWidget *w, SecondDetectorGuiMon *m)
{
    GenModule *dpx;
    dpx = dynamic_cast<GenModule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(m->_id));
    if (!dpx)
        return;

    dpx->get_analysis()->reset_pedestals();
}

void SecondDetectorGuiMon::on_NewChip(GtkWidget *w, SecondDetectorGuiMon *m)
{
    HstList::iterator ip;
    GtkWidget *opt = m->get_widget("optChipSelect");
    int g_chip = gtk_option_menu_get_history(GTK_OPTION_MENU(opt));
    g_chip --;
    m->g_chip = g_chip;
    // Set the pointers to the corresponging histograms
    if (m->hlist.empty())
        return;

    int nbins = m->_ana->get_nbins();
    int nchan = m->_ana->Nchan();
    double rmin = m->_ana->get_range_min();
    double rmax = m->_ana->get_range_max();

    for (ip = m->hlist.begin(); ip != m->hlist.end(); ++ip)
    {
        GtkWidget *w = *ip;
        if (IS_THIS_HST(w, "trCommonMode"))
        {
            hst_histogram_reset(HST_HISTOGRAM( GTK_HST1D(w)->func ));
        }
        else if (IS_THIS_HST(w, "Signal1D"))
        {
            g_object_set(G_OBJECT(GTK_HST1D(w)->func) , "nx", nbins, "xmin", rmin, "xmax", rmax, "ptr", m->_ana->Signal(g_chip), 
            NULL);
        }
        else if (IS_THIS_HST(w, "Signal2D"))
        {
            if (g_chip >= 0&& nchan > 1)
            {
                g_object_set(G_OBJECT(GTK_HST1D(w)->func), "nx", nbins, "xmin", rmin, "xmax", rmax, "ny", nchan, "ymin", double(g_chip*nchan), "ymax", double((g_chip + 1)*nchan), "ptr", m->_ana->Signal2D(g_chip*nchan), 
                NULL);
            }
            else
            {
                g_object_set(G_OBJECT(GTK_HST1D(w)->func), "nx", nbins, "xmin", rmin, "xmax", rmax, "ny", m->_ana->Ntot(), "ymin", 0., "ymax", double(m->_ana->Ntot()), "ptr", m->_ana->Signal2D(), 
                NULL);
            }
        }
    }
    /*
     opt = glade_xml_get_widget(m->_xml,"tglHst");
     if ( opt )
     onToggleHst(GTK_TOGGLE_BUTTON(opt),m);
     */
}

void SecondDetectorGuiMon::on_ChangeView(GtkWidget *w, SecondDetectorGuiMon *m)
{
    GtkHst1D *evtd= GTK_HST1D(m->get_widget("evtDisplay"));
    GtkWidget *active = gtk_menu_get_active(GTK_MENU(w));
    int opt = g_list_index(GTK_MENU_SHELL(w)->children, active);

    if (opt )
        // we want the signal
        g_object_set(G_OBJECT(evtd->func), "ptr", m->_ana->Chan(), NULL);
    else
        // we want the raw data
        g_object_set(G_OBJECT(evtd->func), "ptr", m->_ana->Event(), NULL);
}

void SecondDetectorGuiMon::on_Change2Noise(GtkWidget *w, SecondDetectorGuiMon *m)
{
    GtkWidget *hst = m->get_widget("trCommonMode");
    hst_histogram_reset(HST_HISTOGRAM( GTK_HST1D(hst)->func ));
}

void SecondDetectorGuiMon::toggle2Dhst(const char *frmName, GtkToggleButton *btn)
{
    GtkLabel *lbl= NULL;
    gboolean state = gtk_toggle_button_get_active(btn);
    GtkWidget *frame, *box, *hst;
    void *o;

    if (GTK_IS_LABEL(GTK_BIN(btn)->child) )
        lbl = GTK_LABEL(GTK_BIN(btn)->child);

    /* Get the frame and remove the box from there*/
    frame = get_widget(frmName);
    box = GTK_BIN (frame)->child;
    hst = GTK_WIDGET(g_object_get_data(G_OBJECT(box), "hst") );
    gtk_hst1d_monitor_stop(GTK_HST1D(hst));
    g_object_ref(box);
    g_object_set_data(G_OBJECT(frame), "hidden", box);
    gtk_container_remove(GTK_CONTAINER(frame), box);

    /* Now get the box we should put back in */
    if (state )
    {
        if (lbl)
            gtk_label_set_text(lbl, "2D");
        o = g_object_get_data(G_OBJECT(frame), "Box2D");
        box = GTK_WIDGET(o);
    }
    else
    {
        if (lbl)
            gtk_label_set_text(lbl, "1D");
        o = g_object_get_data(G_OBJECT(frame), "Box1D");
        box = GTK_WIDGET(o);
    }
    gtk_container_add (GTK_CONTAINER (frame), box );
    gtk_widget_show_all(box );

    hst = GTK_WIDGET(g_object_get_data(G_OBJECT(box), "hst") );
    gtk_hst1d_monitor_start( GTK_HST1D(hst), 500);
}

void SecondDetectorGuiMon::on_Btn2DSignal(GtkToggleButton *btn, SecondDetectorGuiMon *m)
{
    m->toggle2Dhst("hstSignal", GTK_TOGGLE_BUTTON(btn));
}

void SecondDetectorGuiMon::on_Btn2DMap(GtkWidget *btn, SecondDetectorGuiMon *m)
{
    m->toggle2Dhst("hstHitMap", GTK_TOGGLE_BUTTON(btn));
}

int SecondDetectorGuiMon::on_Update(void *p)
{
    GtkWidget *tmp;
    SecondDetectorGuiMon *mon = reinterpret_cast<SecondDetectorGuiMon *>(p);

    mon->updating = true;

    // Fill the common mode tracer
    tmp = gtk_option_menu_get_menu(GTK_OPTION_MENU (mon->get_widget("optMonitorSelect")));
    if (tmp )
    {
        GtkMenu *menu = GTK_MENU(tmp);
        if (menu)
        {
            GtkWidget *active = gtk_menu_get_active(GTK_MENU(tmp));
            int opt = g_list_index(GTK_MENU_SHELL(tmp)->children, active);
            int wchip = mon->g_chip >= 0 ? mon->g_chip : 0;
            double val = opt ? mon->_ana->AvNoise(wchip) : mon->_ana->Cmmd(wchip);

            tmp = mon->get_widget("trCommonMode");
            hst_histogram_fill( HST_HISTOGRAM(g_object_get_data(G_OBJECT(tmp), "hst")), val, 1.);
        }
    }

    mon->updating = false;
    return 1;
}

gboolean SecondDetectorGuiMon::on_WindowSelDown(GtkWidget *w, GdkEventButton *ev, SecondDetectorGuiMon *m)
{
    GtkWidget *Emin;
    GtkHst1D *hst;

    if ( !GTK_IS_HST1D(w))
        return FALSE;

    hst = GTK_HST1D(w);
    Emin = m->get_widget("txtEmin");

    if (ev->button == 1)
    {
        std::ostringstream ostr;
        ostr << hst->evt->x;
        gtk_entry_set_text(GTK_ENTRY(Emin), ostr.str().c_str());
    }
    return TRUE;
}

gboolean SecondDetectorGuiMon::on_WindowSelUp(GtkWidget *w, GdkEventButton *ev, SecondDetectorGuiMon *m)
{
    GtkWidget *Emax;
    GtkHst1D *hst;

    if ( !GTK_IS_HST1D(w))
        return FALSE;

    hst = GTK_HST1D(w);
    Emax = m->get_widget("txtEmax");

    if (ev->button == 1)
    {
        std::ostringstream ostr;
        ostr << hst->evt->x;
        gtk_entry_set_text(GTK_ENTRY(Emax), ostr.str().c_str());
    }
    return TRUE;
}

void SecondDetectorGuiMon::on_BtnApplyCut(GtkWidget *w, SecondDetectorGuiMon *m)
{
    double emin, emax;

    emin = m->entry2double("txtEmin", 0.);
    emax = m->entry2double("txtEmax", 99999.);

    m->_ana->SetEcuts(emin, emax);
}

void SecondDetectorGuiMon::on_BtnResetCut(GtkWidget *w, SecondDetectorGuiMon *m)
{
    gtk_entry_set_text(GTK_ENTRY(m->get_widget("txtEmin")), "");
    gtk_entry_set_text(GTK_ENTRY(m->get_widget("txtEmax")), "");
    m->_ana->SetEcuts(0., 0.);
}

GtkWidget *SecondDetectorGuiMon::create_2D_frame(const std::string &prefix, char *xtit1, char *ytit1, char *xtit2, char *ytit2)
{
    GtkWidget *w;
    GtkWidget *bx1d, *bx2d;
    HstHistogram *h1d;
    HstHistogram2D *h2d;
    GtkHst1D *wh1d;
    GtkHst2D *wh2d;
    std::string nam1D = prefix + "1D";
    std::string nam2D = prefix + "2D";

    /* 1D histogram */
    if (prefix == "Signal")
    {
        h1d = hst_histogram_new(_ana->get_nbins(), _ana->get_range_min(), _ana->get_range_max(), 0);
        g_object_set(G_OBJECT(h1d), "ptr", _ana->Signal(-1), NULL);

        h2d = hst_histogram2d_new(_ana->get_nbins(), _ana->get_range_min(), _ana->get_range_max(), _ana->Ntot(), 0., _ana->Ntot(),
                                  0);
        g_object_set(G_OBJECT(h2d), "ptr", _ana->Signal2D(), NULL);
    }
    else if (prefix == "Map")
    {
        h1d = hst_histogram_new(_ana->Ntot(), 0., _ana->Ntot(), 0);
        g_object_set(G_OBJECT(h1d), "ptr", _ana->HitMap(), NULL);
        if (_ana->get_map() )
        {
            const PadMap *pm = _ana->get_map();
            h2d = hst_histogram2d_new(pm->nx(), 0., pm->nx(), pm->ny(), 0., pm->ny(), 0);
            g_object_set(G_OBJECT(h2d), "ptr", _ana->H2MapC(), NULL);
        }
        else
        {
            h2d = hst_histogram2d_new( 1, 0., 1., 1, 0., 1., 0);
            g_object_set(G_OBJECT(h2d), "ptr", _ana->H2Map(), NULL);
        }
    }
    else
        h1d = hst_histogram_new(10, 0., 10., 0);

    wh1d = GTK_HST1D(gtk_hst1d_new( HST_FUNCTION(h1d) ));
    g_object_set_data(G_OBJECT(wh1d), "hst", h1d);
    if (xtit1 )
        gtk_hst1d_set_xtitle(wh1d, xtit1);
    if (ytit1)
        gtk_hst1d_set_ytitle(wh1d, ytit1);
    gtk_hst1d_monitor_start(wh1d, hst_update_freq);
    gtk_hst1d_set_colors(wh1d, "background", "white", NULL);
    gtk_hst1d_sync_range(wh1d, TRUE);
    gtk_widget_set_name(GTK_WIDGET(wh1d), nam1D.c_str());
    bx1d = gtk_hbox_new(FALSE, 0);
    gtk_widget_set_name(bx1d, "Box1D");
    g_object_set_data(G_OBJECT(bx1d), "hst", wh1d);
    gtk_box_pack_start (GTK_BOX (bx1d), GTK_WIDGET(wh1d), TRUE, TRUE, 0);

    /* 2D histogram */
    wh2d = GTK_HST2D(gtk_hst2d_new( h2d ));
    g_object_set_data(G_OBJECT(wh2d), "hst", h2d);
    gtk_widget_set_name(GTK_WIDGET(wh2d), nam2D.c_str());
    if (xtit2 )
        gtk_hst1d_set_xtitle(GTK_HST1D(wh2d), xtit2);
    if (ytit2)
        gtk_hst1d_set_ytitle(GTK_HST1D(wh2d), ytit2);
    gtk_hst1d_set_colors(GTK_HST1D(wh2d), "background", "white", NULL);
    gtk_hst1d_sync_range(GTK_HST1D(wh2d), TRUE);
    bx2d = gtk_hbox_new(FALSE, 0);
    gtk_widget_set_name(bx2d, "Box2D");
    g_object_set_data(G_OBJECT(bx2d), "hst", wh2d);
    gtk_box_pack_start (GTK_BOX (bx2d), GTK_WIDGET(wh2d), TRUE, TRUE, 0);

    if (prefix == "Signal")
    {
        g_signal_connect (GTK_OBJECT (wh1d), "button_press_event", GTK_SIGNAL_FUNC (on_WindowSelDown), this);
        g_signal_connect (GTK_OBJECT (wh1d), "button_release_event", GTK_SIGNAL_FUNC (on_WindowSelUp), this);
        g_signal_connect (GTK_OBJECT (wh2d), "button_press_event", GTK_SIGNAL_FUNC (on_WindowSelDown), this);
        g_signal_connect (GTK_OBJECT (wh2d), "button_release_event", GTK_SIGNAL_FUNC (on_WindowSelUp), this);
    }

    w = gtk_frame_new (NULL);
    gtk_container_add (GTK_CONTAINER (w), GTK_WIDGET(bx1d));
    gtk_widget_show_all(w);
    g_object_set_data(G_OBJECT(w), "Hst1D", wh1d);
    g_object_set_data(G_OBJECT(w), "Hst2D", wh2d);
    g_object_set_data(G_OBJECT(w), "Box1D", bx1d);
    g_object_set_data(G_OBJECT(w), "Box2D", bx2d);
    g_object_set_data(G_OBJECT(w), "hidden", bx2d);
    hlist.insert(GTK_WIDGET(wh1d));
    hlist.insert(GTK_WIDGET(wh2d));
    return w;
}

GtkWidget *SecondDetectorGuiMon::create_hst_pedestals(SecondDetectorGuiMon *M, char *name, char *, char *, int, int)
{
    HstHistogram *hst = hst_histogram_new(M->_ana->Ntot(), 0., M->_ana->Ntot(), 0);
    g_object_set(G_OBJECT(hst), "ptr", M->_ana->Ped(), NULL);
    GtkWidget *w = gtk_hst1d_new( HST_FUNCTION(hst));
    gtk_hst1d_set_ytitle(GTK_HST1D(w), "Pedestals");
    gtk_hst1d_set_xtitle(GTK_HST1D(w), "Channel");
    gtk_hst1d_sync_range(GTK_HST1D(w), TRUE);
    gtk_hst1d_monitor_start(GTK_HST1D(w), hst_update_freq);
    gtk_hst1d_set_colors(GTK_HST1D(w), "background", "white", NULL);
    g_object_set_data(G_OBJECT(w), "hst", hst);
    M->hlist.insert(w);
    return w;
}

GtkWidget *SecondDetectorGuiMon::create_hst_noise(SecondDetectorGuiMon *M, char *name, char *, char *, int, int)
{
    HstHistogram *hst = hst_histogram_new(M->_ana->Ntot(), 0., M->_ana->Ntot(), 0);
    g_object_set(G_OBJECT(hst), "ptr", M->_ana->Noise(), NULL);
    GtkWidget *w = gtk_hst1d_new( HST_FUNCTION(hst));
    gtk_hst1d_set_ytitle(GTK_HST1D(w), "Noise");
    gtk_hst1d_set_xtitle(GTK_HST1D(w), "Channel");
    gtk_hst1d_sync_range(GTK_HST1D(w), TRUE);
    gtk_hst1d_set_colors(GTK_HST1D(w), "background", "white", NULL);
    gtk_hst1d_monitor_start(GTK_HST1D(w), hst_update_freq);
    g_object_set_data(G_OBJECT(w), "hst", hst);
    M->hlist.insert(w);
    return w;
}

GtkWidget *SecondDetectorGuiMon::create_hst_signal(SecondDetectorGuiMon *M, char *name, char *, char *, int, int)
{
    return M->create_2D_frame("Signal", "Signal (ADC)", NULL, "Signal (ADC)", "Channel");
}

GtkWidget *SecondDetectorGuiMon::create_event_display(SecondDetectorGuiMon *M, char *name, char *, char *, int, int)
{
    GtkHst1D *evt;
    HstHistogram *hst = hst_histogram_new(M->_ana->Ntot(), 0, M->_ana->Ntot(), 0);
    g_object_set(G_OBJECT(hst), "ptr", M->_ana->Event(), NULL);
    GtkWidget *w = gtk_hst1d_new( HST_FUNCTION(hst));
    evt = GTK_HST1D(w);
    gtk_hst1d_draw_x_axis(evt, HST_AXIS_NONE);
    gtk_hst1d_draw_y_axis(evt, HST_AXIS_MIDDLE);
    g_object_set(G_OBJECT(evt), "hymin", std::min(-50., M->_ana->get_range_min()) , "hymax", M->_ana->get_range_max(), 
    NULL);
    gtk_hst1d_sync_range(GTK_HST1D(w), TRUE);
    gtk_hst1d_set_colors(evt, "foreground", "firebrick", "background", "black", "axis", "darkseagreen", 
    NULL);
    gtk_hst1d_monitor_start(evt, hst_update_freq / 2);
    g_object_set_data(G_OBJECT(w), "hst", hst);
    M->hlist.insert(w);
    return w;
}

GtkWidget *SecondDetectorGuiMon::create_hitmap(SecondDetectorGuiMon *M, char *name, char *, char *, int, int)
{
    return M->create_2D_frame("Map", "Channel", "Hit map", "Pad X", "Pad Y");
}

GtkWidget *SecondDetectorGuiMon::create_tracer_cmmd(SecondDetectorGuiMon *M, char *name, char *, char *, int, int)
{
    HstTracer *tracer = hst_tracer_new(50);
    GtkWidget *w = gtk_hst1d_new( HST_FUNCTION(tracer));
    gtk_hst1d_draw_x_axis(GTK_HST1D(w), HST_AXIS_NONE);
    gtk_hst1d_draw_y_axis(GTK_HST1D(w), HST_AXIS_MIDDLE);
    gtk_hst1d_monitor_start(GTK_HST1D(w), hst_update_freq / 2);
    gtk_hst1d_sync_range(GTK_HST1D(w), TRUE);
    gtk_hst1d_set_colors( GTK_HST1D(w),
    "foreground", "firebrick",
    "background", "black",
    "axis", "darkseagreen",
    NULL);
    g_object_set_data(G_OBJECT(w), "hst", tracer);
    M->hlist.insert(w);
    return w;
}

GtkWidget *SecondDetectorGuiMon::create_gauge_rate(SecondDetectorGuiMon *M, char *, char *, char *, int, int)
{
    GtkWidget *w = gtk_gauge_new(0., 10000.);
    gtk_gauge_set_color(GTK_GAUGE(w), "green");
    gtk_gauge_set_title(GTK_GAUGE(w), "Rate (Hz)");
    return w;
}

GtkWidget *SecondDetectorGuiMon::create_gauge_eff(SecondDetectorGuiMon *M, char *, char *, char *, int, int)
{
    GtkWidget *w = gtk_gauge_new(0., 100.);
    gtk_gauge_set_title(GTK_GAUGE(w), "Efficiency");
    return w;
}
