#ifndef __SecondDetectorGuiMon_h__
#define __SecondDetectorGuiMon_h__
#include <iostream>
#include <iomanip>
#include <sstream>
#include <set>

#include <DAQ++/DAQid.h>
#include <data/Analysis.h>
#include <gui/WidgetUpdate.h>
#include <gui/GladeApp.h>


//////////////////////////////////////////////////////////////////////////
// class SecondDetectorGuiMon
//
//   Created: Fri Aug 04 14:00:45 2006        Author: Eric Cochran
//   Purpose:
//
//   Modified from:
// class Monitor
//
//   Created: Mon May 27 17:42:33 2002        Author: Carlos Lacasta
//   Purpose:
//
//////////////////////////////////////////////////////////////////////////

class SecondDetectorGuiMon : public GladeApp
{
private:
    DAQpp::DAQid _id;
    int       tmout;
    int       g_chip;
    Analysis *_ana;
    bool      updating;

    typedef std::set<GtkWidget *> HstList;
    HstList hlist;

    static void on_Quit(GtkWidget *w, SecondDetectorGuiMon *m);
    static void on_Reset(GtkWidget *w, SecondDetectorGuiMon *m);
    static void on_ResetPed(GtkWidget *w, SecondDetectorGuiMon *m);
    static void on_Help(GtkWidget *w, SecondDetectorGuiMon *m);
    static void on_NewChip(GtkWidget *, SecondDetectorGuiMon *m);
    static void on_ChangeView(GtkWidget *, SecondDetectorGuiMon *m);
    static void on_Change2Noise(GtkWidget *, SecondDetectorGuiMon *m);
    static void on_Btn2DSignal(GtkToggleButton *, SecondDetectorGuiMon *m);
    static void on_Btn2DMap(GtkWidget *, SecondDetectorGuiMon *m);
    static void on_BtnApplyCut(GtkWidget *, SecondDetectorGuiMon *m);
    static void on_BtnResetCut(GtkWidget *, SecondDetectorGuiMon *m);
    static gboolean on_WindowSelDown(GtkWidget *, GdkEventButton *, SecondDetectorGuiMon *);
    static gboolean on_WindowSelUp(GtkWidget *, GdkEventButton *, SecondDetectorGuiMon *);
    static int  on_Update(void *);

    static GtkWidget *create_hst_pedestals(SecondDetectorGuiMon *, char *, char *, char *, int , int);
    static GtkWidget *create_hst_noise(SecondDetectorGuiMon *, char *, char *, char *, int , int);
    static GtkWidget *create_hst_signal(SecondDetectorGuiMon *, char *, char *, char *, int , int);
    static GtkWidget *create_event_display(SecondDetectorGuiMon *, char *, char *, char *, int , int);
    static GtkWidget *create_hitmap(SecondDetectorGuiMon *, char *, char *, char *, int , int);
    static GtkWidget *create_tracer_cmmd(SecondDetectorGuiMon *, char *, char *, char *, int , int);

    static GtkWidget *create_gauge_rate(SecondDetectorGuiMon *, char *, char *, char *, int , int);
    static GtkWidget *create_gauge_eff(SecondDetectorGuiMon *, char *, char *, char *, int , int);

    GtkWidget *create_2D_frame( const std::string &prefix,
                                char *xtit1, char *ytit1,
                                char *xtit2, char *ytit2 );

    void stop_histograms();
    void update_histograms();
    void reset_histograms();

    void toggle2Dhst(const char *frame, GtkToggleButton *);
    void resize_hst1d(GtkWidget *, int, double, double, gpointer, double hmin = 0., double hmax = 0.);
    void resize_hst2d(GtkWidget *, int, double, double, int, double, double, gpointer);

public:
    SecondDetectorGuiMon(const DAQpp::DAQid &id);
    ~SecondDetectorGuiMon();

    void inLocal();
    void inGlobal();

    // GladeApp methods
    void user_init();
    void auto_connect();
    void set_handlers();
};

#endif
