#ifndef __WAVE_FORM__
#define __WAVE_FORM__

#include <iostream>

class WaveForm
{
private:
    int nSamples;
    double *data;

    double energy;
    double pedestal;
    double noise;

    // variables used to compute avgWF
    const double *avgWF;
    const double *simWF;
    const double *avgWFderiv;
    double deltaT;
    double tau;
    double tauRise;
    int Nquad;
    int Usample;
    int Mrise;
    int Lcheck;

    double Beta;
    double dataRange;

    // start of data, set by data_start
    int start;

    // time of interaction related parameters
    double ttime;
    int ti;
    int joff;

    // the waveform
    double *wf;

    int navg;

    // matrix elements
    double W;
    double WW;
    double WY;
    double Y;
    double S;

public:
    WaveForm(int _nSamples);
    ~WaveForm();

    void compute_avgWF();
    void sim_data();

    int data_start(int offset = 0);
    int basic_trigger_time() const;
    double energy_estimate(int iMaxDer = 0) const;
    double improve_trigger_time(int iMaxDer = 0) const;

    //double get_energy()        const {return energy;}
    //double get_pedestal()      const {return pedestal;}
    double get_noise() const
    {
        return noise;
    }
    const double* get_wf() const
    {
        return wf;
    }
    void set_wf(double* x);
    int get_nSamples() const
    {
        return nSamples;
    }
    int get_n_avg() const
    {
        return navg;
    }
    double get_ttime() const
    {
        return ttime;
    }
    void set_ttime(double _ttime);

    double getW1() const
    {
        return WY -Y*W / S;
    }
    double getW2() const
    {
        return WW -W*W / S;
    }
    double get_energy() const
    {
        return (WY -Y*W / S) / (WW - W*W / S);
    }
    double get_pedestal() const
    {
        return (Y / S - get_energy()*W / S);
    }

    double get_deltaT() const
    {
        return deltaT;
    }
    void set_deltaT(double x)
    {
        deltaT = x;
    }
    double get_tau() const
    {
        return tau;
    }
    void set_tau(double x)
    {
        tau = x;
    }
    double get_tauRise() const
    {
        return tauRise;
    }
    void set_tauRise(double x)
    {
        tauRise = x;
    }
    int get_Nquad() const
    {
        return Nquad;
    }
    void set_Nquad(int x)
    {
        Nquad = x;
    }
    int get_Usample() const
    {
        return Usample;
    }
    void set_Usample(int x)
    {
        Usample = x;
    }
    int get_Mrise() const
    {
        return Mrise;
    }
    void set_Mrise(int x)
    {
        Mrise = x;
    }

    double& operator[](int i)
    {
        return wf[i];
    }
    double operator[](int i) const
    {
        return wf[i];
    }

public:

    void set()
    {
        return;
    }
    void reset();

    void add_to_average(double toff);
    void reset_average();
    double energy_final(double beta, double& err) const;

    double awf_trigger_time(double atime, int Lcheck = 7) const;

    double energy_awf(double ttime, double& err, int i0, int i1) const;
    double chi2(double beta) const;

    void matrix_elements(double beta, int flag = 0);
    void set_weights();
    void set_weights(double beta);
};

#endif
