#include "ftdiUSB.h"
#include "ftdi.h"
#include <iostream>
#include <unistd.h>

using namespace std;

ftdiPort::ftdiPort(ftdi_interface interface,
                   struct usb_device *usbdev): ftdi(NULL)
{
    open(interface, usbdev);
}

ftdiPort::~ftdiPort()
{
    close();
}

void ftdiPort::open(ftdi_interface interface,
                    struct usb_device *usbdev)
{

    ftdi = new ftdi_context;
    ftdi_init(ftdi);
    ftdi_set_interface(ftdi, interface);

    int f = ftdi_usb_open_dev(ftdi, usbdev);
    if (f < 0)
    {
        cout << "unable to open ftdi[" << interface << "], f=" << f << endl;
        cout << "If f=-5 you may need to unload ftdi_sio driver; do (as root)\n"
        << "#>/sbin/modprobe -r ftdi_sio" << endl;
        throw ftdiPort_Error();
    }

    f = ftdi_enable_bitbang(ftdi, 0xFF);
    if (f == 0)
        cout << "bitbangMode is enabled." << endl;
    else
        cout << "bitbanMode is not avaiable" << endl;
}

void ftdiPort::close()
{
    if (!ftdi->usb_dev) return;
    ftdi_disable_bitbang(ftdi);
    ftdi_usb_close(ftdi);
    ftdi_deinit(ftdi);
    delete ftdi;
}

int ftdiPort::write(unsigned char word)
{

    usleep( 100 );
    int f = ftdi_write_data(ftdi, &word, 1);
    if (f < 1)
    {
        cout << "write failed; word=" << word << " f=" << f << endl;
        throw ftdiPort_Error();
    }
    return 0;
}

unsigned int ftdiPort::get_serial()
{
    unsigned char input[128];
    ftdi_read_eeprom(ftdi, input);
    return ftdiUSB::decode_serial(input);
}

void ftdiPort::write_eeprom(unsigned char output[], int check)
{
    ftdi_write_eeprom(ftdi, output);
    if (check)
    {
        unsigned char input[128];
        for (int j = 0;j < 128;j++) input[j] = 0;
        ftdi_read_eeprom(ftdi, input);
        for (int j = 0;j < 128;j++)
        {
            cout << j
            << " 0x" << hex << (int)output[j]//<<'('<<output[i]<<')'
            << " 0x" << hex << (int)input[j]//<<'('<<input[i]<<')'
            << dec << '\t' << flush;
            if (!((j + 1) % 4)) cout << endl;
        }
        cout << endl;
    }

}


