//#include <cstdio>
#include <cstring>
#include <unistd.h>
//usleep
//#include <usb.h>
//#include <ftdi.h>
#include "ftdiUSB.h"
#include <iostream>
#include <sys/types.h>
#include <usb.h>

using namespace std;

struct usb_device ** ftdiUSB::usbList = 0;
unsigned int *ftdiUSB::usbSerialNo = 0;
int ftdiUSB::nDevices = 0;

ftdiUSB::ftdiUSB(unsigned int serial)
{
    ftdiList.clear();
    open(serial);
}

ftdiUSB::~ftdiUSB()
{
    close();
}

void ftdiUSB::open(unsigned int serial)
{

    struct usb_device *usbdev = find_device(serial);
    ftdiList[INTERFACE_A] = new ftdiPort(INTERFACE_A, usbdev);
    ftdiList[INTERFACE_B] = new ftdiPort(INTERFACE_B, usbdev);
}

void ftdiUSB::close()
{
    map<ftdi_interface, ftdiPort *>::iterator it;
    for (it = ftdiList.begin();it != ftdiList.end();it++)
        delete (it->second);
}

ftdiPort *ftdiUSB::getPort(enum ftdi_interface interface)
{
    map<ftdi_interface, ftdiPort *>::iterator it;
    it = ftdiList.find(interface);
    if (it == ftdiList.end())
    {
        cout << "Interface " << interface << " not found" << endl;
        throw ftdiUSB_Error();
    }
    return it->second;
}

unsigned int ftdiUSB::decode_serial(const unsigned char *eeprom)
{
    int offset = eeprom[18] - 0x80;
    int size = (eeprom[offset] - 2) / 2;
    //cout<<"offset="<<offset<<" size="<<size<<endl;

    if (size > 4 || size == 0)
    {
        cout << "Serial number out of bounds; size=" << size << " expected 4" << endl;
        throw ftdiUSB_Error();
    }
    unsigned int data = 0;
    for (int i = 0;i < size;i++)
    {
        data = (eeprom[offset+2*i+2] << 8 * i) | data;
    }
    return data;
}

void ftdiUSB::code_serial(unsigned int serial, ftdi_eeprom *fEE)
{
    if (!fEE->serial) fEE->serial = new char[10];
    for (int i = 0;i < 4;i++)
    {
        fEE->serial[i] = (char) (serial & 0xff);
        serial = serial >> 8;
        //      fEE->serial[i]=(char)((serial >> (8*i) ) & 0xff);
    }
    fEE->serial[4] = 0;
    cout << "Serial: " << fEE->serial << dec << endl;
    if (strlen(fEE->serial) != 4)
    {
        cout << "Bad serial number; size=" << strlen(fEE->serial) << " expected 4" << endl;
        throw ftdiUSB_Error();
    }
}

void ftdiUSB::set_serial(unsigned int serial, int check)
{
    ftdi_eeprom fEE;
    ftdi_eeprom_initdefaults(&fEE);
    fEE.product_id = 0x6010;
    fEE.use_serial = 1;
    ftdiUSB::code_serial(serial, &fEE);
    unsigned char output[128];
    ftdi_eeprom_build(&fEE, output);
    cout << "serial=0x" << hex << ftdiUSB::decode_serial(output) << dec << endl;
    cout << "OK (Ctrl-C to exit)?" << endl;
    while (cin.get() != '\n');
    map<ftdi_interface, ftdiPort *>::iterator it;
    for (it = ftdiList.begin();it != ftdiList.end();it++)
        (it->second)->write_eeprom(output);
}

unsigned int ftdiUSB::get_serial()
{
    unsigned int serial[2];
    //unsigned char input[128];
    int i = 0;
    map<ftdi_interface, ftdiPort *>::iterator it;
    for (it = ftdiList.begin();it != ftdiList.end();it++)
        serial[i++] = (it->second)->get_serial();
    if (serial[0] != serial[1])
    {
        cout << "Serial numbers of sub-interfaces mis-match: "
        << "serial0=0x" << hex << serial[0] << " serial1=0x" << serial[1]
        << dec << endl;
        throw ftdiUSB_Error();
    }
    return serial[0];
}

struct usb_device *ftdiUSB::find_device(unsigned int serial)
{
    if (!usbList) set_usbList();
    if (nDevices == 0)
    {
        cout << "No usb devices found" << endl;
        throw ftdiUSB_Error();
    }
    int iDev = 0;
    while (true)
    {
        for (iDev = 0;iDev < nDevices;iDev++)
            if (usbSerialNo[iDev] == serial) break;
        if (iDev < nDevices) break;

        cout << "Device with serialNo=0x" << hex << serial << dec << " not found." << endl;
        cout << "Connect USB cable to correct board" << endl;
        clear_usbList();
        sleep(10);
        set_usbList();
        //throw ftdiUSB_Error();
    }
    return usbList[iDev];
}

void ftdiUSB::set_usbList()
{

    ftdi_context ftdi_foo;
    ftdi_init(&ftdi_foo);
    struct ftdi_device_list* devlist = NULL, *curdev;

    //find all ftdi chips on our USB bus
    int f = ftdi_usb_find_all(&ftdi_foo, &devlist, 0x0403, 0x6010);
    if (f < 1)
    {
        cout << "Error finding ftdi devices; no=" << f << endl;
        throw ftdiUSB_Error();
    }
    cout << "No of devices found:" << f << endl;
    usbSerialNo = new unsigned int[f];
    usbList = new struct usb_device *[f];
    nDevices = f;

    curdev = devlist;
    //while (curdev){
    for (int i = 0;curdev;i++)
    {
        usbList[i] = curdev->dev;
        ftdiPort *fPort = new ftdiPort(INTERFACE_ANY, usbList[i]);
        unsigned int my_serial = fPort->get_serial();
        delete fPort;//close port
        cout << "Device " << i << ": ";
        cout << "serial=0x" << hex << my_serial << dec << endl;
        usbSerialNo[i] = my_serial;
        //usbList[my_serial]=usbdev;
        curdev = curdev->next;
    }
}

void ftdiUSB::clear_usbList()
{
    delete []usbSerialNo;
    delete []usbList;
    nDevices = 0;
}
