#include "pmtUSB.h"
#include <iostream>
#include <cmath>

using namespace std;

unsigned char SetBit(int bit, bool value, unsigned char& word)
{

    if (value)
        word = word | (1 << bit);

    else
        word = word & ~(1 << bit);

    return word;
}

const int pmtUSB::nbits;
const int pmtUSB::ndata;
//const int dacUSB::nch;


pmtUSB::pmtUSB(unsigned int serial)
{
    fUSB = new ftdiUSB(serial);
    //dac=new dacUSB(fUSB->getPort(INTERFACE_B));
    //delay=new delayUSB(fUSB->getPort(INTERFACE_A));
}

pmtUSB::~pmtUSB()
{
    //delete dac;
    //delete delay;
    delete fUSB;
}


// 'hook' added just for compliance with vmedaq plug-in format
extern "C"
{
    pmtUSB *create_pmtUSB()
    {
        return 0;
    }
}


void pmtUSB::writeDAC(unsigned int data)
{

    ftdiPort *fPort = fUSB->getPort(INTERFACE_B);
    unsigned char word = 0;
    //set load to 1
    SetBit(LOAD, 1, word);
    SetBit(LDAC, 0, word);
    for (int i = 0;i < ndata;i++)
    {
        SetBit(CLOCK, 1, word);
        bool databit = (bool)(data & (1 << (ndata - i - 1)));
        SetBit(DATA, databit, word);
        cout << databit << flush;
        fPort->write(word);
        usleep(10*1000); // half clock cycle
        fPort->write(SetBit(CLOCK, 0, word));
        //write(port,word);
        usleep(10*1000); //other half clock cycle
    }

    //pulse load to set value
    fPort->write(SetBit(LOAD, 0, word));
    //write(port,word);
    usleep(10*1000); // half clock cycle
    fPort->write(SetBit(LOAD, 1, word));
    //write(port,word);
    cout << endl;
    //return 0;
}

void pmtUSB::setDACd(int ch, double value)
{
    if (value > 1)
    {
        cout << "Error: Value (" << value << ") above 1." << endl;
        throw pmtUSB_Error();
    }
    unsigned int val = ((unsigned int)(floor(value * (1 << nbits) - 1e-6))) & 0xff;
    cout << "davUSB:setDAC(" << ch << ',' << val << '[' << value << ']' << ')' << endl;
    setDACi(ch, val);
}

void pmtUSB::setDACi(int ch, unsigned int val)
{
    val = val | ((((unsigned int)ch) & 0x7) << 9);
    writeDAC(val);
}

void pmtUSB::set_delay(unsigned char delay)
{

    cout << "delay=" << (int)delay << endl;
    unsigned char word = 0;

    SetBit(LATCH, 0, word);
    fUSB->getPort(INTERFACE_B)->write(word);

    //while (cin.get()!='\n');

    fUSB->getPort(INTERFACE_A)->write(delay);

    //while (cin.get()!='\n');

    SetBit(LATCH, 1, word);
    fUSB->getPort(INTERFACE_B)->write(word);

}
