#include <iostream>
#include <iomanip>
#include <sstream>
#include <cstdio>
#include <DAQ++/DAQmanager.h>
#include <gui/gtkutils.h>
#include <gui/ParamSlider.h>
#include <data/Analysis.h>
#include <config/Config.h>
#include "VATAmodule.h"
#include "TAmanager.h"
#include "VATAmoduleGuiConfig.h"

class VATAmoduleDebug : public GladeApp
{
private:
    VATAmodule *md;
    int channel;

    static void onExecute(GtkWidget *, VATAmoduleDebug *);
public:
    VATAmoduleDebug(VATAmodule *);
    void auto_connect();
    void process_text(const std::string &s, GtkWidget *w);
    void user_init();
};

//=====================================================================
// Constructor
//=====================================================================
VATAmoduleGuiConfig::VATAmoduleGuiConfig(const DAQpp::DAQid &id, TAmanager *tt, const std::string &g_file) :
    _id(id), df(0), tav(tt)
{
}

void VATAmoduleGuiConfig::auto_connect()
{
    SET_CONNECT( onOK );
    SET_CONNECT( onApply );
    SET_CONNECT( onCancel );

    SET_CONNECT( onDelay );
    SET_CONNECT( onNadj );

    SET_CONNECT( onMaskAll );
    SET_CONNECT( onClearMask );
    SET_CONNECT( onTestAll );
    SET_CONNECT( onClearTest );

    SET_CONNECT( onGainToggled );
    SET_CONNECT( onTestToggled );
    SET_CONNECT( onProbeToggled );
    SET_CONNECT( onNsideToggled );
    SET_CONNECT( onCCToggled );
    SET_CONNECT( on_internal_cap);
    SET_CONNECT( on_internal_step);
    SET_CONNECT( on_test_channel);
    SET_CONNECT( on_slew);
    SET_CONNECT( on_dlt);
    SET_CONNECT( on_trim_dac_range);
    SET_CONNECT( on_chip_type);
    SET_CONNECT( on_change_page);
    SET_CONNECT( onDebug );
}

void VATAmoduleGuiConfig::user_init()
{
    std::ostringstream ostr;
    ostr << _id() << " - Module configuration";
    gtk_window_set_title(GTK_WINDOW(get_main_window()), ostr.str().c_str());

    VATAmodule *vata;
    vata = dynamic_cast<VATAmodule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(_id));

    //---------------------------------------------------------------------
    // Readout mode
    //---------------------------------------------------------------------
    update();

    if (vata)
    {
        double2entry("txtWped", vata->get_analysis()->get_ped_weight(), "%6.4f");
        double2entry("txtWnoise", vata->get_analysis()->get_noise_weight(), "%6.4f");
    }
    //---------------------------------------------------------------------
    // Sliders
    //---------------------------------------------------------------------
    if (vata )
    {
        GtkSpinButton *btn= GTK_SPIN_BUTTON( get_widget("spinThreshold") );
        GtkRange *scale= GTK_RANGE( get_widget("scaleThreshold") );
        GtkAdjustment *adj = gtk_range_get_adjustment(scale);
        gtk_spin_button_set_adjustment(btn, adj );
        ParamSlider::new_slider(_id, "threshold", adj)->set_value_silent(vata->get_threshold());
    }

    //---------------------------------------------------------------------
    // TA configuration
    //---------------------------------------------------------------------

    // Chip option menu
    GtkWidget *tmp, *btn = get_widget("optChip");
    int ichip, nchip = tav->Nchip();
    GtkWidget *menu = gtk_menu_new();
    gtk_option_menu_remove_menu(GTK_OPTION_MENU(btn));
    for (ichip = 0; ichip < nchip; ichip++)
    {
        std::ostringstream ostr;
        ostr << ichip;
        tmp = gtk_menu_item_new_with_label(ostr.str().c_str());
        gtk_widget_show(tmp);
        gtk_menu_append(GTK_MENU(menu), tmp);
    }
    gtk_option_menu_set_menu(GTK_OPTION_MENU(btn), menu);
    g_signal_connect(
                    G_OBJECT(menu),
                    "selection_done",
                    G_CALLBACK(onNewChip),
                    this);
    gtk_option_menu_set_history(GTK_OPTION_MENU(btn), 0);

    // Set the number of chips
    int2entry("txtNChips", nchip);
    
    // polarity option menu
    btn = get_widget("optPolarity");
    g_signal_connect(
                    G_OBJECT(gtk_option_menu_get_menu(GTK_OPTION_MENU(btn))),
                    "selection_done",
                    G_CALLBACK(onPolChanged),
                    this);

    // Gain
    btn = get_widget("optGain");
    g_signal_connect(
                    G_OBJECT(gtk_option_menu_get_menu(GTK_OPTION_MENU(btn))),
                    "selection_done",
                    G_CALLBACK(onGainChanged),
                    this);

    btn = get_widget("optChipType");
    gtk_option_menu_set_history(GTK_OPTION_MENU(btn), tav->GetChipType());
    
    onNewChip(0, this);
}

VATAmoduleGuiConfig::~VATAmoduleGuiConfig()
{
}

void VATAmoduleGuiConfig::onOK(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    if (d)
    {
        onApply(w, d);
        onCancel(w, d);
    }
}
void VATAmoduleGuiConfig::onApply(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    d->set_values();
}
void VATAmoduleGuiConfig::onCancel(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    delete d;
}
void VATAmoduleGuiConfig::update()
{
    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    if (daqm == 0)
        return;

    VATAmodule *dpx = dynamic_cast<VATAmodule *>(daqm->find_module(_id().c_str()));
    if (dpx == 0)
        return;

    int romode = dpx->get_rout();
    GtkWidget *tmp = get_widget("optROmode");
    std::cout << "romode "<< romode << std::endl;
    if (romode & VATAmodule::Serial)
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 0);
    if (romode & VATAmodule::Sparse)
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 1);
    if (romode & VATAmodule::SparseAdj)
        gtk_option_menu_set_history(GTK_OPTION_MENU(tmp), 2);

    gtk_spin_button_set_value( GTK_SPIN_BUTTON(get_widget("txtNadj")),
                               dpx->get_adj());
    gtk_spin_button_set_value( GTK_SPIN_BUTTON(get_widget("txtDelay"))
                               , dpx->get_delay());
}

void VATAmoduleGuiConfig::set_values()
{

    DAQpp::DAQmanager *daqm = DAQpp::DAQmanager::theDAQmanager();
    if (daqm == 0)
        return;

    VATAmodule *dpx = dynamic_cast<VATAmodule *>(daqm->find_module(_id().c_str()));
    if (dpx == 0)
        return;

    int nchips;
    if ( (nchips=entry2int("txtNChips"))!= tav->Nchip() )
    {
        // we have changed the number of chips
        int ichip, mxchips = std::min(nchips, tav->Nchip());
        TAmanager tmp = *tav;
        tav->Resize( nchips );
        
        // Copy the existing chips into the new chips list
        for (ichip=0;ichip<mxchips;ichip++)
        {
            *tav->chip(ichip) = *tmp.chip(ichip);
        }
    }
    dpx->set_ta( *tav );

    GtkWidget *tmp;
    tmp = get_widget("optROmode");
    int romode = gtk_option_menu_get_history(GTK_OPTION_MENU(tmp));
    int val = dpx->get_rout();
    std::cout << "ROmode Opt. menu selection  = "<< romode<< " romode "<< std::hex << val << std::dec << std::endl;
    val &= 0x11;
    std::cout << "..."<< std::hex << val << std::dec << std::endl;
    switch (romode)
    {
    case 0:
        val |= VATAmodule::Serial;
        break;
    case 1:
        val |= VATAmodule::Sparse;
        break;
    case 2:
        val |= VATAmodule::SparseAdj;
        break;
    }
    std::cout << "...setting "<< val << std::endl;
    dpx->set_rout(val);
    dpx->set_nchip(dpx->get_nchip());

    int nadj = entry2int("txtNadj", 2);
    std::cout << "Number of adjacent modules "<< nadj << std::endl;
    dpx->set_adj(nadj );
    dpx->set_delay(entry2double("txtDelay"));
    dpx->get_analysis()->SetWeights(entry2double("txtWped"), entry2double("txtWnoise"));
    dpx->load_register();
    dpx->Reset();
}

int VATAmoduleGuiConfig::WhichChip()
{
    GtkWidget *active;
    GtkOptionMenu *optChip= GTK_OPTION_MENU(get_widget("optChip"));

    g_return_val_if_fail(GTK_IS_OPTION_MENU(optChip), 0);

    active = gtk_menu_get_active(GTK_MENU(optChip->menu));
    if (active )
        return g_list_index(GTK_MENU_SHELL(optChip->menu)->children,
                            active);
    else
        return 0;
}

TAchip *VATAmoduleGuiConfig::current_chip()
{
    int ichip = WhichChip();
    return tav->chip(ichip);
}

void VATAmoduleGuiConfig::on_chip_type(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    int type = gtk_option_menu_get_history(GTK_OPTION_MENU(d->get_widget("optChipType")));
    if (type>=0 && type<TAmanager::last_type)
        d->tav->SetChipType( (TAmanager::ChipType)type);
}

void VATAmoduleGuiConfig::onPolChanged(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    int ipol;
    GtkWidget *active;
    TAchip *chip = d->current_chip();
    if (chip)
    {
        active = gtk_menu_get_active(GTK_MENU(w));
        ipol = g_list_index(GTK_MENU_SHELL(GTK_MENU(w))->children, active);
        chip->Polarity(ipol);
    }
}

void VATAmoduleGuiConfig::onGainToggled(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    d->current_chip()->EnableGain(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w)));
}

void VATAmoduleGuiConfig::onGainChanged(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    GtkWidget *active = gtk_menu_get_active(GTK_MENU(w));
    int igain = g_list_index(GTK_MENU_SHELL(GTK_MENU(w))->children, active);
    d->current_chip()->SetGain(igain);
}

void VATAmoduleGuiConfig::onTestToggled(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    d->current_chip()->Test(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w)));
}

void VATAmoduleGuiConfig::onBtnToggled(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    long ichan = long(gtk_object_get_data(GTK_OBJECT(w), "channel"));
    long itype = long(gtk_object_get_data(GTK_OBJECT(w), "type"));
    int lval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
    TAchip *chip = d->current_chip();
    switch (itype)
    {
    case 0:
        chip->mask(ichan, lval);
        break;
    case 1:
        chip->test(ichan, lval);
        break;
    default:
        g_warning("If you se this message you better quit the program...\n");
        break;
    }
}

void VATAmoduleGuiConfig::onProbeToggled(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    d->current_chip()->Probe_Enable(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w)));
}

void VATAmoduleGuiConfig::onNsideToggled(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    d->current_chip()->N_side(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w)));
}

void VATAmoduleGuiConfig::onCCToggled(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    d->current_chip()->CC_Enable(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w)));
}

void VATAmoduleGuiConfig::process_text(const std::string &s, GtkWidget *w)
{
    VATAmodule *dpx;
    dpx = dynamic_cast<VATAmodule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(_id));

    if (s == "txtChipAddr")
    {
        onChipIDChanged(w, this);
    }
    else if (s == "txtWped")
    {
        double x = entry2double("txtWped");
        dpx->get_analysis()->SetWeights(x, dpx->get_analysis()->get_noise_weight());
    }
    else if (s == "txtWnoise")
    {
        double x = entry2double("txtWnoise");
        dpx->get_analysis()->SetWeights(dpx->get_analysis()->get_ped_weight(), x);
    }
    else if (s=="txtThrsDAC")
    {
        int idac = entry2int(s);
        current_chip()->thrs_dac(idac);
    }
    else if (s=="txtCalStep")
    {
        int ival = entry2int(s);
        current_chip()->calib_dac(ival);
    }
    else if (s=="txtNChips")
    {
        int nchips = entry2int(s);
        std::cout << "number of chips: " << nchips << std::endl;
    }
}

void VATAmoduleGuiConfig::onChipIDChanged(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    int ival;
    const char *cval = gtk_entry_get_text(GTK_ENTRY(w));
    ival = atoi(cval);
    ival &= 0x1ff;
    d->current_chip()->Chip_Address(ival);
}

void VATAmoduleGuiConfig::onTxtChanged(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    int ival;
    long ichan = long( gtk_object_get_data(GTK_OBJECT(w), "channel") );
    const char *cval = gtk_entry_get_text(GTK_ENTRY(w));

    char *endp;
    ival = strtol(cval, &endp, 0);
    if (endp != cval )
    {
        ival &= 0x1ff;
        d->current_chip()->trim(ichan, ival);
    }
}

void VATAmoduleGuiConfig::on_change_page(GtkNotebook *nb, GtkNotebookPage *nbp, gint page, VATAmoduleGuiConfig *d)
{
    GtkWidget *table;
    switch (page)
    {
        case 0:
            table = d->get_widget("vbox-ta-mask");
            break;
        case 1:
            table = d->get_widget("vbox-ta-test");
            break;
        case 2:
            table = d->get_widget("vbox-threshold");
            break;
        default:
            return;
    }
    gtk_widget_queue_resize(table);
}

void VATAmoduleGuiConfig::onNewChip(GtkWidget *w, VATAmoduleGuiConfig *d)
{
    GtkWidget *chipOptPol = d->get_widget("optPolarity");
    GtkWidget *chipGain = d->get_widget("btnGain");
    GtkWidget *chipOptGain = d->get_widget("optGain");
    GtkWidget *chipTest = d->get_widget("btnTest");
    GtkWidget *chipAddr = d->get_widget("txtChipAddr");

    // Get the chip
    int ichip;
    TAchip *chip = d->current_chip();

    // Now set the chip Address
    std::ostringstream oadd;
    oadd << chip->Chip_Address();
    gtk_entry_set_text( GTK_ENTRY(chipAddr), oadd.str().c_str());

    // Now set the chip mode
    int bz = chip->Polarity();
    gtk_option_menu_set_history(GTK_OPTION_MENU(chipOptPol), bz);
    bz = chip->GainEnabled();
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chipGain), bz);
    bz = chip->Test();
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chipTest), bz);
    bz = chip->Gain();
    gtk_option_menu_set_history(GTK_OPTION_MENU(chipOptGain), bz);
    
    d->activate_button("btnSlew", chip->slew());
    d->activate_button("btnDLT", chip->dltc());
    d->activate_button("btnDACrange", chip->dac_range());
    d->activate_button("btnTestChannel", chip->test_channel());
    d->int2entry("txtThrsDAC",chip->thrs_dac());
    d->int2entry("txtCalStep",chip->calib_dac());
    d->activate_button("btnCapInt", !chip->calib_ctrl());
    d->activate_button("btnStepInt", chip->calib_gen());

    GtkTable *tbl[3];
    tbl[0] = GTK_TABLE(d->get_widget("tblMask"));
    tbl[1] = GTK_TABLE(d->get_widget("tblTest"));
    tbl[2] = GTK_TABLE(d->get_widget("tblTrim"));

    // Clear and resize the tables
    for (int k = 0; k < 3; k++)
    {
        d->draw_table(tbl[k], k);
    }

}

void VATAmoduleGuiConfig::draw_table(GtkTable *tbl, int k)
{
    // Now the heavy part... Resize the tables with the masks and test bits
    int i, ic, ir;
    int nchan = tav->Nchan();
    int ncol = 16;
    int nrow = nchan / ncol + (nchan % ncol ? 1 : 0);
    GtkWidget *tmp;
    char line[10];

    // Get the chip
    TAchip *chip = current_chip();

    GtkWidget *T = GTK_WIDGET(tbl);
    
    gtk_container_foreach(GTK_CONTAINER(T),
                          (GtkCallback) gtk_widget_destroy,
                          NULL);
    gtk_table_resize(tbl, nrow, ncol + 1);
    for (i = 0, ic = 0, ir = -1; i < nchan; i++, ic++)
    {
        if (!(i % ncol))
        {
            ir++;
            ic = 0;
            // add the label
            sprintf(line, "%d", i);
            tmp = gtk_label_new(line);
            gtk_widget_show(tmp);
            gtk_table_attach(tbl, tmp, ic, ic + 1, ir, ir + 1, (GtkAttachOptions) 0, (GtkAttachOptions) 0, 0, 0);
            ic++;
        }
        if (k < 2)
        {
            // create the check button
            tmp = gtk_check_button_new();
            gtk_widget_show(tmp);
            gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(tmp), FALSE);
            gtk_widget_set_size_request(tmp, 10, 10);
            gtk_table_attach(tbl, tmp, ic, ic + 1, ir, ir + 1, (GtkAttachOptions) (GTK_EXPAND), (GtkAttachOptions) (0), 0, 0);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp),
                                         k ? chip->test(i) : chip->mask(i));
            gtk_signal_connect(GTK_OBJECT(tmp), "toggled",
                               GTK_SIGNAL_FUNC(onBtnToggled), this);

        }
        else
        {
            tmp = gtk_entry_new();
            sprintf(line, "%d", chip->trim(i));
            gtk_entry_set_text(GTK_ENTRY(tmp), line);
            gtk_widget_set_size_request(tmp, 20, 20);
            gtk_widget_show(tmp);
            gtk_table_attach(tbl, tmp, ic, ic + 1, ir, ir + 1, (GtkAttachOptions) (GTK_EXPAND |GTK_FILL),
                             (GtkAttachOptions) (0), 0, 0);
            gtk_signal_connect(GTK_OBJECT(tmp), "changed",
                               GTK_SIGNAL_FUNC(onTxtChanged), this);

        }
        gtk_object_set_data(GTK_OBJECT(tmp), "channel", (gpointer) i);
        gtk_object_set_data(GTK_OBJECT(tmp), "type", (gpointer) k);
    }
    gtk_widget_queue_resize(T);
}

int VATAmoduleGuiConfig::onUpdate(VATAmoduleGuiConfig *d)
{
    return 1;
}

void VATAmoduleGuiConfig::onMaskAll(GtkWidget *w, VATAmoduleGuiConfig *md)
{
    VATAmodule *dpx;
    dpx = dynamic_cast<VATAmodule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(md->_id));
    if (dpx )
    {
        int i;
        TAchip *chip = md->current_chip();
        if (chip )
        {
            int nchan = chip->Nchan();
            for (i = 0; i < nchan; i++)
                chip->mask(i, true);
        }
        onNewChip(0, md);
    }
}
void VATAmoduleGuiConfig::onClearMask(GtkWidget *w, VATAmoduleGuiConfig *md)
{
    VATAmodule *dpx;
    dpx = dynamic_cast<VATAmodule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(md->_id));
    if (dpx )
    {
        int i;
        TAchip *chip = md->current_chip();
        if (chip )
        {
            int nchan = chip->Nchan();
            for (i = 0; i < nchan; i++)
                chip->mask(i, false);
        }
        onNewChip(0, md);
    }
}
void VATAmoduleGuiConfig::onTestAll(GtkWidget *w, VATAmoduleGuiConfig *md)
{
    VATAmodule *dpx;
    dpx = dynamic_cast<VATAmodule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(md->_id));
    if (dpx )
    {
        int i;
        TAchip *chip = md->current_chip();
        if (chip )
        {
            int nchan = chip->Nchan();
            for (i = 0; i < nchan; i++)
                chip->test(i, true);
        }
        onNewChip(0, md);
    }

}
void VATAmoduleGuiConfig::onClearTest(GtkWidget *w, VATAmoduleGuiConfig *md)
{
    VATAmodule *dpx;
    dpx = dynamic_cast<VATAmodule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(md->_id));
    if (dpx )
    {
        int i;
        TAchip *chip = md->current_chip();
        if (chip )
        {
            int nchan = chip->Nchan();
            for (i = 0; i < nchan; i++)
                chip->test(i, false);
        }
        onNewChip(0, md);
    }

}

void VATAmoduleGuiConfig::onNadj(GtkWidget *w, VATAmoduleGuiConfig *md)
{
    VATAmodule *dpx;
    dpx = dynamic_cast<VATAmodule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(md->_id));
    if (dpx )
    {
        double val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(w));
        dpx->set_adj( (int)val, false);
    }
}
void VATAmoduleGuiConfig::onDelay(GtkWidget *w, VATAmoduleGuiConfig *md)
{
    VATAmodule *dpx;
    dpx = dynamic_cast<VATAmodule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(md->_id));
    if (dpx )
    {
        double val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(w));
        dpx->set_delay(val );
    }
}

void VATAmoduleGuiConfig::on_internal_cap(GtkWidget *w, VATAmoduleGuiConfig *M)
{
    bool rc = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
    if (rc) {
      std::cout << "internal cap:  0" << std::endl;
      M->current_chip()->calib_ctrl(0);
    }
    else {
      std::cout << "external cap:  1" << std::endl;
      M->current_chip()->calib_ctrl(1);
    }
}
void VATAmoduleGuiConfig::on_internal_step(GtkWidget *w, VATAmoduleGuiConfig *M)
{
    bool rc = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
    std::cout << "internal step: " << rc << std::endl;
    M->current_chip()->calib_gen(rc);
}
void VATAmoduleGuiConfig::on_test_channel(GtkWidget *w, VATAmoduleGuiConfig *M)
{
    bool rc = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
    M->current_chip()->test_channel(rc);
}
void VATAmoduleGuiConfig::on_slew(GtkWidget *w, VATAmoduleGuiConfig *M)
{
    bool rc = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
    M->current_chip()->slew(rc);
}
void VATAmoduleGuiConfig::on_dlt(GtkWidget *w, VATAmoduleGuiConfig *M)
{
    bool rc = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
    M->current_chip()->dltc(rc);
}

void VATAmoduleGuiConfig::on_trim_dac_range(GtkWidget *w, VATAmoduleGuiConfig *M)
{
    bool rc = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
    M->current_chip()->dac_range(rc);
}


void VATAmoduleGuiConfig::onDebug(GtkWidget *w, VATAmoduleGuiConfig *M)
{
    VATAmodule *dpx;
    ConfigParser *cfg = ConfigParser::theConfig();
    dpx = dynamic_cast<VATAmodule *>(DAQpp::DAQmanager::theDAQmanager()->find_module(M->_id));
    if (!dpx)
        return;

    if ( !dpx->GetStatus().isReady() )
        return;
    ModuleParser *mp = dynamic_cast<ModuleParser *>(cfg->get_id_handler(dpx->get_id()));
    if (mp)
        mp->save_config_id(dpx->get_id());

    VATAmoduleDebug *dbg = new VATAmoduleDebug(dpx);
    dbg->init(M->get_file_name(), "ChannelDebug");
    gtk_dialog_run(GTK_DIALOG(dbg->get_main_window()));
    delete dbg;

    if (mp)
        mp->parse_node_id(dpx->get_id());
}

VATAmoduleDebug::VATAmoduleDebug(VATAmodule *m) :
    md(m), channel(10)
{
}



void VATAmoduleDebug::user_init()
{
    int2entry("txtChannel", channel);
}

void VATAmoduleDebug::auto_connect()
{
    SET_CONNECT( onExecute );
}

void VATAmoduleDebug::process_text(const std::string &s, GtkWidget *w)
{
    channel = entry2int("txtChannel", 10);
}

void VATAmoduleDebug::onExecute(GtkWidget *w, VATAmoduleDebug *D)
{
    D->md->test_channel(D->channel);
}

