#include <iostream>
#include <corbo.h>
#include <daq/VMEmanager.h>


using namespace std;


#define bitset(w,b) (w |= (1<<(b)))
#define bitclr(w,b) (w &= ~(1<<(b)))
#define bittst(w,b) (w & (1<<(b)) )

template < class T >
T byteset (T w, int start, int end, T v)
{
    int i, width = end - start + 1;
    T mask;
    for (i = 0, mask = 0; i < width; i++)
        mask |= 1 << i;

    v &= mask;
    for (i = 0; i < width; i++)
    {
        T bit = 1 << i;
        (v & bit) ? bitset (w, i + start) : bitclr (w, i + start);
    }
    return w;
}

template < class T >
T byteget (T w, int start, int end)
{
    int i, width = end - start + 1;
    T mask;
    for (i = 0, mask = 0; i < width; i++)
    {
        if (bittst (w, i + start))
            mask |= 1 << i;
    }
    return mask;
}

CSR & CSR::ChnEna (unsigned int x)
{
    (x ? bitset (v, 0) : bitclr (v, 0));
    Set (v);
    return *this;
}

CSR & CSR::BusyMode (unsigned int x)
{
    (x ? bitset (v, 1) : bitclr (v, 1));
    Set (v);
    return *this;
}

CSR & CSR::InpSel (unsigned int x)
{
    v = byteset < type > (v, 2, 3, x);
    Set (v);
    return *this;
}

CSR & CSR::BusyOut (unsigned int x)
{
    (x ? bitset (v, 4) : bitclr (v, 4));
    Set (v);
    return *this;
}

CSR & CSR::CntSel (unsigned int x)
{
    (x ? bitset (v, 5) : bitclr (v, 5));
    Set (v);
    return *this;
}

CSR & CSR::FastClr (unsigned int x)
{
    (x ? bitset (v, 6) : bitclr (v, 6));
    Set (v);
    return *this;
}

CSR & CSR::PushEna (unsigned int x)
{
    (x ? bitset (v, 7) : bitclr (v, 7));
    Set (v);
    return *this;
}

unsigned int CSR::ChnEna ()
{
    Get ();
    return bittst (v, 0);
}
unsigned int CSR::BusyMode ()
{
    Get ();
    return bittst (v, 1);
}
unsigned int CSR::InpSel ()
{
    int rc;
    Get ();
    rc = byteget < type > (v, 2, 3);
    return rc;
}
unsigned int CSR::BusyOut ()
{
    Get ();
    return bittst (v, 4);
}
unsigned int CSR::CntSel ()
{
    Get ();
    return bittst (v, 5);
}
unsigned int CSR::FastClr ()
{
    Get ();
    return bittst (v, 6);
}
unsigned int CSR::PushEna ()
{
    Get ();
    return bittst (v, 7);
}
unsigned int CSR::InpState ()
{
    Get ();
    return bittst (v, 8);
}
unsigned int CSR::LclBusy ()
{
    Get ();
    return bittst (v, 9);
}
unsigned int CSR::DifBusy ()
{
    Get ();
    return bittst (v, 10);
}
unsigned int CSR::IntPend ()
{
    Get ();
    return bittst (v, 11);
}





BIMCR & BIMCR::IRQ (unsigned int x)
{
    v = byteset < type > (v, 0, 2, x);
    Set (v);
    return *this;
}

BIMCR & BIMCR::IRAC (unsigned int x)
{
    (x ? bitset (v, 3) : bitclr (v, 3));
    Set (v);
    return *this;
}

BIMCR & BIMCR::IRE (unsigned int x)
{
    (x ? bitset (v, 4) : bitclr (v, 4));
    Set (v);
    return *this;
}

BIMCR & BIMCR::XIN (unsigned int x)
{
    (x ? bitset (v, 5) : bitclr (v, 5));
    Set (v);
    return *this;
}

BIMCR & BIMCR::FAC (unsigned int x)
{
    (x ? bitset (v, 6) : bitclr (v, 6));
    Set (v);
    return *this;
}

BIMCR & BIMCR::F (unsigned int x)
{
    (x ? bitset (v, 7) : bitclr (v, 7));
    Set (v);
    return *this;
}

unsigned int BIMCR::IRQ ()
{
    int rc;
    Get ();
    rc = byteget < type > (v, 0, 2);
    return rc;
}
unsigned int BIMCR::IRAC ()
{
    Get ();
    return bittst (v, 3);
}
unsigned int BIMCR::IRE ()
{
    Get ();
    return bittst (v, 4);
}
unsigned int BIMCR::XIN ()
{
    Get ();
    return bittst (v, 5);
}
unsigned int BIMCR::FAC ()
{
    Get ();
    return bittst (v, 6);
}
unsigned int BIMCR::F ()
{
    Get ();
    return bittst (v, 7);
}

void CharReg::write ()
{
    vme->write8 (addr, v);
}

CharReg::type CharReg::read ()
{
    return vme->read8 (addr);
}

void ShortReg::write ()
{
    vme->write16 (addr, v);
}

ShortReg::type ShortReg::read ()
{
    return vme->read16 (addr);
}

void LongReg::write ()
{
    unsigned short v1, v2;
    v1 = v & 0xffff;
    v2 = (v & 0xffff0000) >> 16;
    vme->write16 (addr, v2);
    vme->write16 (addr + 2, v1);
}

LongReg::type LongReg::read ()
{
    unsigned long v1, v2;
    v2 = vme->read16 (addr);
    v1 = vme->read16 (addr + 2);
    return (v2 << 16) | v1;
}




void ROcntl::SetAddress (unsigned long x)
{
    int i, stp = 0;
    addr = x;
    if (addr)
        stp = 2;

    for (i = 0; i < 4; i++, x += stp)
        _csr[i].SetAddress (x);

    if (addr)
        x = addr + 0x10;

    for (i = 0; i < 4; i++, x += 2 * stp)
        _evt[i].SetAddress (x);


    for (i = 0; i < 4; i++, x += stp)
        _dtim[i].SetAddress (x);

    if (addr)
        x = addr + 0x31;

    for (i = 0; i < 4; i++, x += stp)
        _evcr[i].SetAddress (x);

    for (i = 0; i < 4; i++, x += stp)
        _evvr[i].SetAddress (x);

    for (i = 0; i < 4; i++, x += stp)
        _tmcr[i].SetAddress (x);

    for (i = 0; i < 4; i++, x += stp)
        _tmvr[i].SetAddress (x);

    if (addr)
        x = addr + 0x50;

    for (i = 0; i < 4; i++, x += stp)
        _tst[i].SetAddress (x, false);

    for (i = 0; i < 4; i++, x += stp)
        _clr[i].SetAddress (x, false);

}

void ROcntl::Dump ()
{
    int i;
    cout << "Base address " << hex << this << " " << hex << GetAddress () << endl;
    for (i = 0; i < 4; i++)
        cout << "\tCSR" << dec << i + 1 << " " << hex << _csr[i].GetAddress ()
             << " " << hex << _csr[i].Get () << dec << endl;
    for (i = 0; i < 4; i++)
        cout << "\tEvtNum" << dec << i + 1 << " " << hex << _evt[i].GetAddress ()
             << " " << hex << _evt[i].Get () << dec << endl;
    for (i = 0; i < 4; i++)
        cout << "\tDeadTime" << dec << i + 1 << " " << hex << _dtim[i].GetAddress ()
             << " " << hex << _dtim[i].Get () << dec << endl;
    for (i = 0; i < 4; i++)
        cout << "\tBIM1 CR" << dec << i + 1 << " " << hex << _evcr[i].GetAddress ()
             << " " << hex << (int) _evcr[i].Get () << dec << endl;
    for (i = 0; i < 4; i++)
        cout << "\tBIM1 VR" << dec << i + 1 << " " << hex << _evvr[i].GetAddress ()
             << " " << hex << (int) _evvr[i].Get () << dec << endl;
    for (i = 0; i < 4; i++)
        cout << "\tBIM2 CR" << dec << i + 1 << " " << hex << _tmcr[i].GetAddress ()
             << " " << hex << (int) _tmcr[i].Get () << dec << endl;
    for (i = 0; i < 4; i++)
        cout << "\tBIM2 VR" << dec << i + 1 << " " << hex << _tmvr[i].GetAddress ()
             << " " << hex << (int) _tmvr[i].Get () << dec << endl;
}

