#ifndef __tdc488_h__
#define __tdc488_h__


struct dataWord {
      unsigned short val     :12;
      unsigned short channel : 3;
      unsigned short header  : 1;
};
#define TDC_DATA(u) ((*dataWord)(&u))

union _helper_ {
      unsigned short val;
      dataWord data;
};

//=====================================================================
// Header for the Caen 8 channel TDC mod V. 488 controler class
//=====================================================================
class tdc488 {
   public:
      /// Constructor
      tdc488(unsigned int base=0);
      /// Dummy destructor
      ~tdc488() {}

      enum modeType { start, stop };
      /// Sets the addresses w.r.t. base
      void setAddresses(unsigned int);
      
      /// Reset the module
      tdc488 *reset();
      
      /** Sets the High Threshold value.
	  The usable range of the analog input of the ADC is 0.15V to 3.75V.
	  The conversion of an analog input greater than the upper limit
	  could lead to an unpredictable output. The users should use the high
	  threshold to maintain the analog input under the upper limit.
	  The values are 8 bits long and the recomended values are less than
	  0xC7
      */
      tdc488 *HighThreshold(int val);
      
      /** Sets the low threshold value.
	  The value is 8 bits long and should be smaller than the high 
	  threshold value (see \Ref{HighThreshold}).
      */
      tdc488 *LowThreshold(int val);
      
      /// Prints the module identifier
      void ID();

      /// Dumps the contents of all the registers
      void Dump();

      /** Sets the modules in Half Full mode.
	  In this mode does not accept any trigger (module BUSY) when the 
	  number of data stored is greater than the half size of the FIFO.
      */
      tdc488 *HFmode();
      
      /** Sets the module in FIFO FULL mode.
	  The module will not accept any trigger when the FIFO is full. In this
	  mode it is possible to use all the memory locations, but it is 
	  possible to lose some data if the FIFO gets full during a conversion
	  sequence
      */
      tdc488 *FFmode();

      /** Sets the full scale time range.
	  The range can be set from 90ns to 770ns.
      */
      tdc488 *setRange(float);
//      tdc488 *setRange(unsigned short x) {*_range = (x & 0xff); return this; }

      /// Sets the module either in  Start or Stop mode      
      tdc488 *setMode( modeType );

      /// Enables channels. Each bit of the input is one channel.
      tdc488 *enableChan( unsigned short);

      /// Disables channels. Each bit of the input is one channel.
      tdc488 *disableChan( unsigned short);

      /// Checks if there is data
      bool empty();

      /// Checks if the FIFO is full
      bool full();

      /// returns data as it comes
      unsigned short data();

      /// range
      unsigned short range2dac(float r) const;
      float dac2range(unsigned short x) const;
   private:
      /// Interrupt register
      unsigned long _IR;        
      /// Version & series
      unsigned long _version;   
      /// Manufacturer & Module type
      unsigned long _manuf;     
      /// Fixed code
      unsigned long _code;      
      /// HF register
      unsigned long _hfreg;     
      /// reset register
      unsigned long _reset;     
      /// control register
      unsigned long _control;   
      /// output buffer
      unsigned long _outb;      
      /// FF register
      unsigned long _ffreg;     
      /// range register
      unsigned long _range;     
      /// THRH register
      unsigned long _thrh;      
      /// THRL register
      unsigned long _thrl;      
};

#endif
