#include <iostream>
#include <vector>
#include <daq/VMEmanager.h>
#include <data/Analysis.h>
#include <data/DataFormat.h>
#include "VelmexModule.h"
#include "VelmexModuleAna.h"


VelmexModule::VelmexModule( const DAQpp::DAQid &id, std::string address )
        : GenModule( id, 0 )
{
    velmexDriver = new VelmexDriver( address.c_str() );

    // Using DAQ++ parameters can be useful, especially if you want to
    // access them the DAQcorba
    // module parameters



    // reset pedestals and noise to sensible values
    /*
    fill_n(back_inserter( get_par_value< std::vector<double> >("pedestal") ), maxInputs*maxSamples, 0.);
    fill_n(back_inserter( get_par_value< std::vector<double> >("noise") ), maxInputs*maxSamples, 0.);
    */

    // now we should create the Analysis for the monitoring
    // never forget to do this !!!
    create_analysis();
}

VelmexModule::~VelmexModule()
{
    for ( int i = 0; i < 3; i++ ) move_to( i, 0 );
}

unsigned long VelmexModule::get_data_type() const
{
    return 0;
}

bool
VelmexModule::hasData()
{
	bool debug=0;
    if (debug)
	    std::cout << "\t+-> VelmexModule::hasData        [" << get_id() << "]"    << std::endl;
    return false;
}

char*
VelmexModule::getData( int &size )
{
    std::cout << "\t+-> VelmexModule::GetData        [" << get_id() << "]"
    << std::endl;
    return "nothing";
}

int
VelmexModule::PrepareForRun ()
{
    std::cout << "\t+-> VelmexModule::PrepareForRun  [" << get_id() << "]"
    << std::endl;
    return 0;
}

int
VelmexModule::Start ( int )
{
    std::cout << "\t+-> VelmexModule::Start          [" << get_id() << "]"
    << std::endl;
    return 0;
}

int
VelmexModule::Stop ( int )
{
    std::cout << "\t+-> VelmexModule::Stop           [" << get_id() << "]"
    << std::endl;
    return 0;
}

int
VelmexModule::Pause ()
{
    std::cout << "\t+-> VelmexModule::Pause          [" << get_id() << "]"
    << std::endl;
    return 0;
}

int
VelmexModule::Continue ()
{
    std::cout << "\t+-> VelmexModule::Continue       [" << get_id() << "]"
    << std::endl;
    return 0;
}

int
VelmexModule::Reset ()
{
    std::cout << "\t+-> VelmexModule::Reset          [" << get_id() << "]"
    << std::endl;
    return 0;
}

int
VelmexModule::GetReady ()
{
    std::cout << "\t+-> VelmexModule::GetReady       [" << get_id() << "]"
    << std::endl;
    return 0;
}

void
VelmexModule::set_scan_point( ScanPoint &scp )
{
  std::cout<<"Setting scan point"<<std::endl;

    // Set variables to the values in the current point
    // For all the modules
    Reset();
    std::vector<ScanVar>::iterator ip;

    if ( true )
    {
        std::cout << "\t+-> VelmexModule::set_scan_point [" << get_id() << "]"
        << std::endl;
    }

    for ( int i = 0; i < 3; i++ )
    {
        ip = scp.find_type( ScanVar::ScanType( get_motorScanType( i ) ) );
        if ( ip != scp.end() )
        {
            float fval = 0;
            if ( get_motorUseMap( i ) )
            {
                int ival = int( ip->value() );
                std::vector<float> mmap;
                get_motorMap( i, mmap );
                fval = mmap[ival];
                if ( true )
                {
                    std::cout << "\t        " << get_motor_s_ScanType( i ) << " = " << ival
                    << "    map value = " << fval
                    << std::endl;
                }
            }
            else
            {
                fval = ip->value();
                if ( true )
                {
                    std::cout << "\t        " << get_motor_s_ScanType( i ) << " = " << fval
                    << std::endl;
                }
            }
            move_to( i, fval );
        }
    }
}

void
VelmexModule::set_header_description( unsigned int &data )
{
    unsigned long id = get_id();
    //data = (get_nModules() & 0xff)<<20 | (SecDetData & 0xf0000) | (id & 0xffff);
    data = ( 3 & 0xff ) << 20 | ( 0 & 0xf0000 ) | ( id & 0xffff );
}

// *** module specific methods
void
VelmexModule::move_to( int mtr, float pos )
{
    float scale = get_motorScale( mtr );
    int ipos    = velmexDriver->GetPos( mtr );
    int steps   = int( pos * scale ) - ipos;

    velmexDriver->Move( mtr, steps );
}

void
VelmexModule::create_analysis( int nchip, int nchan )
{
    if ( this->ana )
        delete this->ana;

    this->ana = new VelmexModuleAna( nchip, nchan );
}
