#include <iostream>
#include <iomanip>
#include <sstream>
#include <cstdio>
#include <DAQ++/DAQmanager.h>
#include <gui/gtkutils.h>
#include <gui/ParamSlider.h>
#include "VelmexModule.h"
#include "VelmexModuleGuiConf.h"
#include <data/Analysis.h>
#include <config/ConfigParser.h>
#include <config/ModuleParser.h>


//=====================================================================
// Constructor
//=====================================================================
VelmexModuleGuiConf::VelmexModuleGuiConf(
    const DAQpp::DAQid &id )
        : _id( id ), df( 0 ), _monitor( 0 )
{
//---------------------------------------------------------------------
// Get the pointer to the module
//---------------------------------------------------------------------
    DAQpp::Module *dpx;
    dpx = DAQpp::DAQmanager::theDAQmanager()->find_module( _id );
    if ( dpx == 0 )
        return;

    module = dynamic_cast<VelmexModule *>( dpx );
    if ( !module )
        return;
}

void
VelmexModuleGuiConf::auto_connect()
{
    SET_CONNECT( onOK );
    SET_CONNECT( onApply );
    SET_CONNECT( onCancel );
    SET_CONNECT( on_checkALL );
    SET_CONNECT( on_checkEditThresholds );
}

void
VelmexModuleGuiConf::user_init()
{
    update();
}

VelmexModuleGuiConf::~VelmexModuleGuiConf()
{ }

void
VelmexModuleGuiConf::onOK( GtkWidget *w, VelmexModuleGuiConf *d )
{
    if ( d )
    {
        onApply( w, d );
        onCancel( w, d );
    }
}

void
VelmexModuleGuiConf::onApply( GtkWidget *w, VelmexModuleGuiConf *d )
{
    d->set_values();
    d->update();
}

void
VelmexModuleGuiConf::onCancel( GtkWidget *w, VelmexModuleGuiConf *d )
{
    delete d;
}

void
VelmexModuleGuiConf::update()
{
}

void
VelmexModuleGuiConf::set_values()
{
}

void
VelmexModuleGuiConf::toggleThresholds ()
{
    if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( get_widget( "checkALL" ) ) ) )
    {
        gtk_widget_set_sensitive( get_widget( "spinALL" ), true );
        gtk_widget_set_sensitive( get_widget( "spinthreshold1" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold2" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold3" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold4" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold5" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold6" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold7" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold8" ), false );
    }
    else
    {
        gtk_widget_set_sensitive( get_widget( "spinALL" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold1" ), true );
        gtk_widget_set_sensitive( get_widget( "spinthreshold2" ), true );
        gtk_widget_set_sensitive( get_widget( "spinthreshold3" ), true );
        gtk_widget_set_sensitive( get_widget( "spinthreshold4" ), true );
        gtk_widget_set_sensitive( get_widget( "spinthreshold5" ), true );
        gtk_widget_set_sensitive( get_widget( "spinthreshold6" ), true );
        gtk_widget_set_sensitive( get_widget( "spinthreshold7" ), true );
        gtk_widget_set_sensitive( get_widget( "spinthreshold8" ), true );
    }
}

void
VelmexModuleGuiConf::toggleEditThresholds ()
{
    if ( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( get_widget( "checkEditThresholds" ) ) ) )
    {
        gtk_widget_set_sensitive( get_widget( "checkALL" ), true );
        toggleThresholds();
    }
    else
    {
        gtk_widget_set_sensitive( get_widget( "checkALL" ), false );
        gtk_widget_set_sensitive( get_widget( "spinALL" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold1" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold2" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold3" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold4" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold5" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold6" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold7" ), false );
        gtk_widget_set_sensitive( get_widget( "spinthreshold8" ), false );
    }
}

void
VelmexModuleGuiConf::on_checkALL( GtkWidget *w, VelmexModuleGuiConf *d )
{
    d->toggleThresholds();
}

void
VelmexModuleGuiConf::on_checkEditThresholds( GtkWidget *w, VelmexModuleGuiConf *d )
{
    d->toggleEditThresholds();
}
