#include <termios.h> /* serial port contrl */
#include <cstdio>

/* 4 header files for file open  */
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <iostream>

#include <cstring>
#include <cerrno>
#include <cstdlib>

#include "serialComm.h"

using namespace std;

SerialComm::SerialComm( const char* serialPort,SerialComm::PortSettings ps): fd( -1 )
{
  if ( initSerialComm( serialPort,ps) )
    {
        cout << " Troubles intializing serial port" << endl;
        throw SerialComm_Err();
    }
};

int
SerialComm::initSerialComm( const char* port, SerialComm::PortSettings ps )
{
    if ( ( fd = open( port, O_RDWR | O_NOCTTY | O_NDELAY ) ) < 0 )
    {
        printf( "\n Serial port COM1 open error! \n" );
        return -1;
    }
    cout << "Open " << port << " fd=" << fd << endl;
    termios old_termios;
    if ( ( tcgetattr( fd, &old_termios ) ) != 0 )
    {
        printf( "\n tcgetattr error! \n" );
        printf( "old cflag=0x%08x \n", old_termios.c_cflag );
        printf( "old oflag=0x%08x \n", old_termios.c_oflag );
        printf( "old iflag=0x%08x \n", old_termios.c_iflag );
        printf( "old lflag=0x%08x \n", old_termios.c_lflag );
        printf( "old line=0x%08x \n", old_termios.c_line );
    }
    termios new_termios = old_termios;

    /* clear any input data received, but not yet read */
    flush();

//     /* 9600 baud rate, 7 bits, 2 stop bits */
    switch (ps){
    case DATA7_PAREVEN_STOP2:
      new_termios.c_cflag = B9600 | CS7 | CSTOPB | PARENB | CREAD | CLOCAL | HUPCL;
      new_termios.c_cflag &= ~PARODD; /* even parity */

      new_termios.c_lflag &= ~ICANON;
      new_termios.c_lflag &= ~ECHO;
      new_termios.c_lflag &= ~ISIG;
      break;
    case DATA8_PARNONE_STOP1:
      /* 9600 baud rate, 8 bits, no parity 1 stop bits */
      new_termios.c_cflag = B9600 | CS8 | CREAD | CLOCAL | HUPCL;
      new_termios.c_cflag &= ~PARENB; //disable parity
      new_termios.c_cflag &= ~CSTOPB; //send just one stop bit
      
      new_termios.c_lflag &= ~ICANON;
      new_termios.c_lflag &= ~ECHO;
      new_termios.c_lflag &= ~ISIG;
      break;
    default:
      printf("Settings for port were not set; wrong flag\n");
      throw SerialComm_Err();
    }
    

    new_termios.c_cc[VMIN] = 1;
    new_termios.c_cc[VTIME] = 1;

    if ( ( cfsetspeed( &new_termios, B9600 ) ) != 0 )
        printf( "\n cfsetspeed error! \n" );

    if ( ( tcsetattr( fd, TCSANOW, &new_termios ) ) != 0 )
        printf( "\n tcsetattr error! \n" );

    printf( "new cflag=0x%08x \n", new_termios.c_cflag );
    printf( "new oflag=0x%08x \n", new_termios.c_oflag );
    printf( "new iflag=0x%08x \n", new_termios.c_iflag );
    printf( "new lflag=0x%08x \n", new_termios.c_lflag );
    printf( "new line=0x%08x \n", new_termios.c_line );

    return 0;
};

SerialComm::~SerialComm()
{
    if ( fd > 0 ) close( fd );
}

int
SerialComm::write( const char *cmd ) const
{
    cout << "Sending " << cmd << endl;
    if ( fd < 0 ) return 0;
    if ( ( ::write( fd, cmd, strlen( cmd ) ) ) < 0 )
    {
        cout << "write error! " << endl;
        throw SerialComm_Err();
    }
    usleep( 500 );
    return 0;
}

char
SerialComm::getc() const
{
    if ( fd < 0 ) return 0;

    char c = '\0';

    int i = 0;
    for ( ;( c == '\0' );i++ )
    {
        ::read( fd, &c, 1 );
        //cout<<(int)c<<' '<<std::flush;
        if ( i && !( i % 100000 ) ) cout << '.' << std::flush;
    }
    cout << "i=" << i << endl;

    return c;
}

int
SerialComm::readline( char resp[], char end_marker ) const
{
    if ( fd < 0 ) return 0;

    //circular buffer to store output
    int myL = 40;
    char my_resp[myL];
    char c = 0;
    int i = 0;
    for ( ;( c != end_marker );i++ )
    {
        ::read( fd, &c, 1 );
        //cout<<(int)c<<' '<<std::flush;
        //if (i && !(i%100)) cout<<endl;
        my_resp[i%myL] = c;
    }
    //cout<<" i="<<i<<endl;

    int start = i < myL ? 0 : i - myL;
    int stop = i;

    //unwind circular buffer
    //cout<<"resp="<<std::flush;
    int k = 0;
    for ( i = start;i < stop;i++ )
    {
        //strip blanks
        if ( my_resp[i%myL] == '\0' ) continue;
        resp[k] = my_resp[i%myL];
        //cout<<(int) resp[k]<<' '<<std::flush;
        k++;
    }
    //cout<<endl;
    flush();
    return 0;
}

int
SerialComm::flush() const
{
    if ( fd < 0 ) return 0;
    if ( ( tcflush( fd, TCIOFLUSH ) ) != 0 )
    {
        cout << " tcflush error! " << endl;
        throw SerialComm_Err();
    }

    usleep( 50000 );
    return 0;
}

