#include "velmexDriver.h"
#include <cstdlib>
#include <cstdio>
#include <iostream>
#include <unistd.h>

using namespace std;

VelmexDriver::VelmexDriver( const char* serialPort,SerialComm::PortSettings ps): SerialComm( serialPort,ps )
{
    //set limits
    //start; no echo
    write( "F" ); //Enable on-line mode with echo off("F") on("E")
    flush();
    Clear();

    sendCommand( "O1,R\r" );//do tell when limit is exceeded

    //Set default motor speeds
    for ( int i = 0;i < 2;i++ ) SetSpeed( i, 2000 );
}

int
VelmexDriver::SetZero()
{
    return write( "N\r" );
}

int
VelmexDriver::Move( int mtr, int steps )
{
    if ( steps == 0 ) return 0;

    char cmd[20];
    sprintf( cmd, "I%iM%i,R\r", mtr + 1, steps );//command along with carriage return
    //if (write(cmd)) return -1;
    for ( int i = 0;i < 1;i++ )
    {
        char c = sendCommand( cmd );
        if ( c == '^' ) return 0;
        if ( c == 'O' ) return 1;
        usleep( 10000 );
    }
    cout << "Could not execute command " << cmd << endl;
    throw VelmexDriver_Error();
}

int
VelmexDriver::GoToZero( int mtr )
{
    char cmd[20];
    sprintf( cmd, "I%iM0,R\r", mtr + 1 );
    sendCommand( cmd );
    return 0;
}

int
VelmexDriver::GoToExtremum( bool max, int mtr ) const
{
    //find extreme number of steps along mtr. If max is true search for
    //maximum, else for minimum
    int stepSearch[3];
    stepSearch[0] = 5000;//coarse step
    stepSearch[1] = 100;
    stepSearch[2] = 10;//fine step
    char cmd[20];
    for ( int istep = 0;istep < 3;istep++ )
    {
        sprintf( cmd, "I%iM%i\n", mtr + 1, ( 2*max - 1 )*stepSearch[istep] );
        write( cmd );
        while ( true )
        {
            write( "R" );
            if ( getc() == 'O' ) break;
        }
        flush();
        Clear();
        usleep( 5000 );
    }
    return 0;
}

int
VelmexDriver::SetSpeed( int mtr, int speed )
{
    char cmd[20];
    sprintf( cmd, "S%iM%i,R\r", mtr + 1, speed );
    return sendCommand( cmd );
}


int
VelmexDriver::Clear() const
{
    return write( "C" );
}

char
VelmexDriver::sendCommand( const char* command ) const
{
    /* Send command */
    flush();

    //cout<<"Sending command: "<<command<<endl;
    write( command );

    //wait for completion
    //char iBuffer[256];
    //for (int i=0;i<256;i++) iBuffer[i]='\0';

    char c = getc();
    cout << "response=";//<<flush;
    if ( c == 'O' ) cout << "Oh";
    else cout << c;
    cout << endl;
    //while (1){
    //read(iBuffer,'^');
    //cout<<iBuffer<<endl;
    //if (strchr(iBuffer,'^')) break;
    //}
    flush();
    Clear();
    return c;
}

int
VelmexDriver::GetPos( int mtr ) const
{
    switch ( mtr )
    {
    case 0:
        return GetX();
    case 1:
        return GetY();
    case 2:
        return GetZ();
    default:
        throw VelmexDriver_Error();
    }

    return 0;
}

int
VelmexDriver::GetX() const
{
    flush();
    write( "X" );
    usleep( 50000 );
    char resp[40];
    readline( resp );
    return atoi( resp );
}

int
VelmexDriver::GetY() const
{
    flush();
    write( "Y" );
    usleep( 50000 );
    char resp[40];
    readline( resp );
    return atoi( resp );
}

int
VelmexDriver::GetZ() const
{
    flush();
    write( "Z" );
    usleep( 50000 );
    char resp[40];
    readline( resp );
    return atoi( resp );
}


