#include <iostream>
#include <sstream>
#include <poll.h>
#include "PythonLoader.h"

#if defined(HAS_PYTHON) && defined(HAS_PYDAQPP)

#include "pymodule_parser.h"
PythonLoader PythonLoader::__the_loader;

/**
 * TODO: WHY !?!?!?!?
 * We need to understand why declaring _PyGC_generation0 here
 * stops the problem wirh
 * vmedaq: Modules/gcmodule.c:338:
 * visit_reachable: Assertion `gc_refs > 0 || gc_refs == (-3) || gc_refs == (-2)' failed
 *
 */
#define NUM_GENERATIONS 3
#define GEN_HEAD(n) (&generations[n].head)
struct gc_generation {
    PyGC_Head head;
    int threshold; /* collection threshold */
    int count; /* count of allocations or collections of younger
              generations */
};
/* linked lists of container objects */
static struct gc_generation generations[NUM_GENERATIONS] = {
    /* PyGC_Head,               threshold,  count */
    {{{GEN_HEAD(0), GEN_HEAD(0), 0}},   700,        0},
    {{{GEN_HEAD(1), GEN_HEAD(1), 0}},   10,     0},
    {{{GEN_HEAD(2), GEN_HEAD(2), 0}},   10,     0},
};

PyGC_Head *_PyGC_generation0 = GEN_HEAD(0);




void execute_function_from_module(const char *module_name, const char *function_name)
{
    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();

    // load the python module
    PyObject *func;
    PyObject *ret;
    PyObject *mName = PyString_FromString(module_name);
    PyObject *module = PyImport_Import( mName );
    if (!module)
    {
        if (PyErr_Occurred())
        {
            daqpp_print_error();
            PyErr_Print();
        }
    }
    else
    {
        PyObject *dict = PyModule_GetDict(module);
        func = PyDict_GetItemString(dict, function_name);
        if (!func)
        {
            std::cout << "Function " << function_name << " not found in " << module_name << std::endl;
            if (PyErr_Occurred())
            {
                daqpp_print_error();
                PyErr_Print();
            }

            Py_DECREF(mName);
            PyGILState_Release(gilstate);
            return;
        }
        ret = PyObject_CallObject(func, 0);
        if (!ret)
        {
            if (PyErr_Occurred())
            {
                daqpp_print_error();
                PyErr_Print();
            }
        }
        else
            Py_DECREF(ret);

        Py_DECREF(mName);
    }

    PyGILState_Release(gilstate);
}

PythonLoader::PythonLoader() :
    _initialized(false), _save(0)
{
    std::cout << "Initializing Python" << std::endl;
    Py_Initialize();
    // Get prepared for the threading...
    if (! PyEval_ThreadsInitialized() )
    {
        PyThreadState *py_tstate = NULL;
        PyEval_InitThreads();
        py_tstate = PyGILState_GetThisThreadState();
        PyEval_ReleaseThread(py_tstate);
    }
}

void PythonLoader::initialize(int argc, char **argv)
{
    if (_initialized)
        return;
    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();

    _initialized = true;
    PySys_SetArgv(argc, argv);
    PyRun_SimpleString("import sys\n"
                       "sys.path.insert(0,\"" LIB_DIR "\")\n"
                       "import daqpp\n"
                       "import pyvmedaq\n"
                       );
    PyGILState_Release(gilstate);
}

void PythonLoader::start_server(int port)
{
    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();

    // Load the server module
    PyObject *dict, *func, *args, *val;
    PyObject *mName = PyString_FromString("DAQXMLrpcServer");
    PyObject *module = PyImport_Import( mName );
    Py_DECREF(mName);

    if (!module)
    {
        if (PyErr_Occurred())
        {
            daqpp_print_error();
            PyErr_Print();
        }
    }
    else
    {
        Py_INCREF(module);
        dict = PyModule_GetDict(module);
        func = PyDict_GetItemString(dict, "start_server");
        if ( func && PyCallable_Check(func))
        {
            // Call the start_server function
            args = PyTuple_New(1);
            val = PyInt_FromLong(port);
            PyTuple_SetItem(args, 0, val);
            server = PyObject_CallObject(func, args);
            Py_DECREF(args);

            if (!server)
            {
                if (PyErr_Occurred())
                {
                    daqpp_print_error();
                    PyErr_Print();
                }
            }
            else
            {
                Py_INCREF(server);
                // Now we setup for the glib loop
                args = PyTuple_New(0);

                func = PyObject_GetAttrString(server, "fileno");
                val = PyObject_Call(func, args, NULL);
                if (!val)
                {
                    if (PyErr_Occurred())
                    {
                        daqpp_print_error();
                        PyErr_Print();
                    }
                }
                else
                {
                    int fd = PyInt_AsLong(val);
                    GIOChannel *ioc = g_io_channel_unix_new(fd);
                    g_io_add_watch_full(ioc, G_PRIORITY_HIGH,
                                        (GIOCondition) (G_IO_IN | G_IO_PRI | G_IO_HUP),
                                        (GIOFunc) PythonLoader::server_loop,
                                        server,
                                        NULL);
                    Py_DECREF(val);
                }
                Py_DECREF(args);
            }
        }
    }
    PyGILState_Release(gilstate);
}

gboolean PythonLoader::server_loop(GIOChannel * channel, GIOCondition condition, PyObject *server)
{
    if ( condition != G_IO_IN)
        return TRUE;

    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();

    int fd = g_io_channel_unix_get_fd(channel);
    PyObject *func = PyObject_GetAttrString(server, "handle_request");
    PyObject *args = PyTuple_New(0);

    PyObject *ret = PyObject_Call(func, args, NULL);
    if (PyErr_Occurred())
    {
        daqpp_print_error();
        PyErr_Print();
    }

    if (ret)
    {
        Py_DECREF(ret);
    }

    Py_DECREF(args);

    PyGILState_Release(gilstate);
    return TRUE;
}

void PythonLoader::add_path(const std::string &p)
{
    std::ostringstream ostr;
    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();

    ostr << "sys.path.append('"<< p << "')\n";
    PyRun_SimpleString(ostr.str().c_str());
    if (PyErr_Occurred())
    {
        daqpp_print_error();
        PyErr_Print();
    }

    PyGILState_Release(gilstate);
}

PythonLoader::~PythonLoader()
{
    std::cout << "Finalizing Python" << std::endl;

    ObjectList::iterator ip;
    for (ip=MethodList.begin();ip!=MethodList.end();ip++)
        Py_DECREF(ip->second);

    for (ip=ModuleList.begin();ip!=ModuleList.end();ip++)
        Py_DECREF(ip->second);

    //execute_function_from_module("debug_gc", "dumpObjects");

    PyGILState_Ensure();
    Py_Finalize();
}

PythonLoader &PythonLoader::getInstance()
{
    return __the_loader;
}

PyObject *PythonLoader::load_module(const std::string &module, const std::string &method)
{
    PyObject *pymodule, *pymethod;

    // Load the module
    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();

    ObjectList::iterator ip = ModuleList.find(module);
    if (ip==ModuleList.end())
    {
        pymodule = PyImport_ImportModule(module.c_str());
        if (pymodule==0)
        {
            std::cout << "Could not load " << module << std::endl;
            if ( PyErr_Occurred())
            {
                daqpp_print_error();
                PyErr_Print();
            }
            return 0;
        }
        ModuleList[module] = pymodule;
        Py_INCREF(pymodule);
    }
    else
    {
        pymodule = ip->second;
        Py_INCREF(pymodule);
    }
    PyGILState_Release(gilstate);
    /*
     * Now look for the hook
     */
    std::string name = method + '@' + module;
    ip = MethodList.find(name);
    if (ip==MethodList.end())
    {
        gilstate = PyGILState_Ensure();
        pymethod = PyObject_GetAttrString(pymodule, method.c_str());
        if (!pymethod)
        {
            std::cout << "Could not find " << method << " in " << module << std::endl;
            PyGILState_Release(gilstate);
            return 0;
        }

        if ( !PyCallable_Check(pymethod))
        {
            std::cout << method << " in " << module << " is not a callable function" << std::endl;
            PyGILState_Release(gilstate);
            return 0;
        }
        MethodList[name] = pymethod;
        Py_INCREF(pymethod);
        PyGILState_Release(gilstate);
    }
    else
        pymethod = ip->second;

    return pymethod;
}

DAQpp::Module *PythonLoader::create_module(const DAQpp::DAQid &id,
                                           const std::string &module,
                                           const std::string &method)
{
    PyObject *rc;
    PyObject *pyfunc = load_module(module, method);
    if (!pyfunc)
        return 0;

    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();
    // Create the module
    rc = PyObject_CallFunction(pyfunc,"s",(const char *)id);
    PyGILState_Release(gilstate);

    if (!rc)
    {
        std::cout << "I could not call " << method << " in " << module << std::endl;
        return 0;
    }
    DAQpp::Module *daq_module = DAQpp::DAQmanager::theDAQmanager()->find_module(id);
    if (!daq_module)
    {
        std::cout << "Something failed when creating module " << id << std::endl;
        return 0;
    }

    return daq_module;
}

DAQpp::RunManager *PythonLoader::create_run_manager(const DAQpp::DAQid &id,
                                                    const std::string &module,
                                                    const std::string &method)
{
    PyObject *rc;
    PyObject *pyfunc = load_module(module, method);
    if (!pyfunc)
        return 0;

    // Create the module
    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();
    rc = PyObject_CallFunction(pyfunc,"s",(const char *)id);
    PyGILState_Release(gilstate);
    if (!rc)
    {
        std::cout << "I could not call " << method << " in " << module << std::endl;
        return 0;
    }
    DAQpp::RunManager *daq_run_manager = DAQpp::DAQmanager::theDAQmanager()->find_runmanager(id);
    if (!daq_run_manager)
    {
        std::cout << "Something failed when creating module " << id << std::endl;
        return 0;
    }

    return daq_run_manager;
}

ModuleParser *PythonLoader::create_module_parser(const std::string &id, const std::string &module, const std::string &method)
{
    PyObject *rc;
    PyObject *pyfunc = load_module(module, method);

    if (!pyfunc)
        return 0;

    // Create the parser
    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();
    rc = PyObject_CallFunction(pyfunc,"s",id.c_str());
    PyGILState_Release(gilstate);
    if (!rc)
    {
        std::cout << "I could not call " << method << " in " << module << std::endl;
        gilstate = PyGILState_Ensure();
        if ( PyErr_Occurred())
        {
            daqpp_print_error();
            PyErr_Print();
        }
        PyGILState_Release(gilstate);
        return 0;
    }
    return ((vmedaq_module_parser *)rc)->parser;
}

//void PythonLoader::allow_threads()
//{
//    _save = PyEval_SaveThread();
//}
//
//void PythonLoader::disable_threads()
//{
//    if (_save)
//    {
//        PyEval_RestoreThread(_save);
//        _save = 0;
//    }
//}
//void PythonLoader::wait(double secs)
//{
//    struct timeval t;
//    double frac;
//    frac = fmod(secs, 1.0);
//    secs = floor(secs);
//    t.tv_sec = (long)secs;
//    t.tv_usec = (long)(frac*1000000.0);
//    Py_BEGIN_ALLOW_THREADS
//    if (select(0, (fd_set *)0, (fd_set *)0, (fd_set *)0, &t) != 0) {
//        if (errno!=EINTR)
//            std::cout << "Error in python_wait: " << strerror(errno) << std::endl;
//    }
//    Py_END_ALLOW_THREADS
//}

#endif
