#ifndef PYTHONLOADER_H_
#define PYTHONLOADER_H_


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if defined(HAS_PYTHON) && defined(HAS_PYDAQPP)
#include <Python.h>
#include <glib.h>
#include <DAQ++/DAQmanager.h>
#include <DAQ++/Module.h>
#include <DAQ++/RunManager.h>

class ModuleParser;

/**
 * This class handles the interaction with python
 */
class PythonLoader
{
    private:
        bool _initialized;
        PyObject *server;
    protected:
        PythonLoader();
        PythonLoader(const PythonLoader &);

        typedef std::map<std::string, PyObject *> ObjectList;
        ObjectList ModuleList;
        ObjectList MethodList;
        static PythonLoader __the_loader;
        PyThreadState *_save;

        PyObject *load_module(const std::string &module, const std::string &method);

        static gboolean server_loop(GIOChannel *, GIOCondition, PyObject *);
    public:
        static PythonLoader &getInstance();
        virtual ~PythonLoader();

        DAQpp::Module *create_module(const DAQpp::DAQid &id, const std::string &module, const std::string &method);
        DAQpp::RunManager *create_run_manager(const DAQpp::DAQid &id, const std::string &module, const std::string &method);
        ModuleParser *create_module_parser(const std::string &id, const std::string &module, const std::string &method);

//        void allow_threads();
//        void disable_threads();
//        void wait(double seconds);

        void initialize(int argc, char **argv);
        void add_path(const std::string &p);

        // Starts the xmlrpc server
        void start_server(int port);
};

#endif

#endif /*PYTHONLOADER_H_*/
