#include <sstream>
#include "pygenmodule.h"
#include "pyanalysis.h"
#include <DAQ++/python/pydaqpp_importSequence.h>
#include <DAQ++/python/pydaqpp_buffer.h>
#include <DAQ++/python/pydaqpp_types.h>

//#define DAQpp_DEBUG
#if defined(DAQpp_DEBUG)
#include <cstdio>
#define info_msg(fmt,args...)                                           \
    printf("***Info [%s:%d] " fmt, __PRETTY_FUNCTION__,__LINE__ , ## args)

#define WHOAMI printf("***WhoAmI [%s:%d] thread[%lx]\n", __PRETTY_FUNCTION__,__LINE__ , PyThread_get_thread_ident());
#else
#define WHOAMI
#define info_msg(...)
#endif


pygenmodule::pygenmodule(const DAQpp::DAQid &id, unsigned long add):
    GenModule(id, add)
{
    WHOAMI;
    set_analysis( new Analysis(1,1));
    info_msg("...quitting constructor\n");
}

pygenmodule::~pygenmodule()
{
    WHOAMI;
}

unsigned long pygenmodule::get_data_type() const
{
    WHOAMI;
    int rc = const_cast<pygenmodule *>(this)->call_int_method("get_data_type", NULL);
    info_msg("...quitting rc=%d\n", rc);
    return rc;
}
// Prepare for a new run
int pygenmodule::PrepareForRun()
{
    WHOAMI;
    int rc = call_int_method("PrepareForRun", NULL);
    info_msg("...quitting rc=%d\n", rc);
    return rc;
}
// Start the run
int pygenmodule::Start(int mode)
{
    WHOAMI;
    int rc = call_int_method("Start", "i", mode, NULL);
    info_msg("...quitting rc=%d\n", rc);
    return rc;
}
// Stop the run
int pygenmodule::Stop(int mode)
{
    WHOAMI;
    int rc = call_int_method("Stop", "i", mode, NULL);
    info_msg("...quitting rc=%d\n", rc);
    return rc;
}
// Pause the run
int pygenmodule::Pause()
{
    WHOAMI;
    int rc = call_int_method("Pause", NULL);
    info_msg("...quitting rc=%d\n", rc);
    return rc;
}
// Resume the run after pause
int pygenmodule::Continue()
{
    WHOAMI;
    int rc = call_int_method("Continue", NULL);
    info_msg("...quitting rc=%d\n", rc);
    return rc;
}
// Send a reset to the module
int pygenmodule::Reset()
{
    WHOAMI;
    int rc = call_int_method("Reset", NULL);
    info_msg("...quitting rc=%d\n", rc);
    return rc;
}
// Initialize the Module
int pygenmodule::GetReady()
{
    WHOAMI;
    int rc = call_int_method("GetReady", NULL);
    info_msg("...quitting rc=%d\n", rc);
    return rc;
}
// Issues a trigger
int pygenmodule::Trigger()
{
    WHOAMI;
    if ( !GetStatus().isRunning() )
        return 1;
    int rc = call_int_method("Trigger", NULL);
    info_msg("...quitting rc=%d\n", rc);
    return rc;
}
// Flush any remaining data on the Module
void pygenmodule::flush()
{
    WHOAMI;
    call_void_method("flush", NULL);
    info_msg("...quitting\n");
    return;
}


bool pygenmodule::hasData()
{
    WHOAMI;
    int rc=call_int_method("hasData", NULL);
    info_msg("...quitting rc=%d\n", rc);
    return (bool)rc;
}

void pygenmodule::EndOfEvent()
{
    WHOAMI;
    call_void_method("EndOfEvent", NULL);
    info_msg("...quitting\n");
    return;
}




int pygenmodule::check_attributes()
{
    static const char *msg = "Module should define its own hasData, getData and get_data_type methods";
    WHOAMI;
    if (!has_object())
        return 1;

    if (!PyObject_HasAttrString(self(), "hasData") )
    {
        PyErr_SetString(DAQpp_Error, msg);
        return -1;
    }
    if (!PyObject_HasAttrString(self(), "getData") )
    {
        PyErr_SetString(DAQpp_Error, msg);
        return -1;
    }
    if (!PyObject_HasAttrString(self(), "get_data_type") )
    {
        PyErr_SetString(DAQpp_Error, msg);
        return -1;
    }
    return 0;
}

char *pygenmodule::getData(int &size)
{
    WHOAMI;
    PyObject *obj;
    Py_ssize_t ssize;
    WHOAMI;
    char *ptr= NULL;
    char *cbuff=0;
    size=0;
    if (python_call() || !has_object() )
        return ptr;

    daqpp_observer_start();
    /**
     * We call here the getData method.
     */
    obj = PyObject_CallMethod(self(), "getData", NULL);
    if (obj==0)
    {
        if (PyErr_Occurred())
        {
            daqpp_print_error();
            PyErr_Print();
        }
        goto out;
    }
    if (obj==Py_None)
        goto out;


    // we should get a buffer here or a Py_None
    if ( !PyObject_AsReadBuffer(obj, (const void **)&cbuff, &ssize) )
    {
        size = ssize;
        ptr = allocate(size);
        memcpy(ptr, cbuff, size);
    }
    else if (PySequence_Check(obj) )
    {
        char *str = 0;
        PyErr_Clear(); // this is set by PyObject_AsReadBuffer

        try
        {
            str = import_sequence(obj,ssize);
            size = ssize;
        }
        catch ( SeqException e )
        {
            std::ostringstream ostr;
            ostr << "Module.getData: " << e.what();
            std::cout << "\n***** " << ostr << std::endl;
            PyErr_SetString(DAQpp_Error,ostr.str().c_str());
            goto out;
        }
        ptr = allocate(size);
        memcpy(ptr, str, size);
        PyMem_Free(str);
    }
    else
    {
        PyErr_SetString(DAQpp_Error, "Module.getData: get data should provide a buffer or a sequence");
        size = 0;
        ptr = 0;
    }

    out:
    Py_XDECREF(obj);
    daqpp_observer_stop();
    return ptr;
}



static PyObject *module_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    vmedaq_Module *self;
    WHOAMI;
    self = (vmedaq_Module *)type->tp_alloc(type, 0);
    if (!self)
        return (PyObject *)0;

    self->md = 0;
    return (PyObject *)self;
}

static void module_dealloc(vmedaq_Module* self)
{
    WHOAMI;
    terminate_daqpp_module((daqpp_Module *)self);
    Analysis *ana = self->md->get_analysis();
    if (ana)
    {
        pyanalysis *pyana = dynamic_cast<pyanalysis *>(ana);
        if (pyana)
        {
            Py_DECREF(pyana->self());
        }
    }
    self->pyM.pH.ob_type->tp_free((PyObject *)self);
}


static int module_init(vmedaq_Module *self, PyObject *args, PyObject *kwargs)
{
    WHOAMI;
    PyObject *in=Py_None;
    unsigned long add;
    int rc = 0;


    if (!PyArg_ParseTuple(args, "Ok:vmedaq.Module.__init__", &in,&add))
        return -1;

    if (in!=Py_None)
    {
        PyObject *str = PyObject_Str(in);
        std::string v = PyString_AsString(str);
        pygenmodule *m = new pygenmodule(v,add);
        DAQpp::Module *dm = dynamic_cast<DAQpp::Module *>(m);
        rc = setup_daqpp_module(dm, (daqpp_Module *)self);
        if (!rc)
            self->md = m;

        Py_DECREF(str);
    }
    self->pyM.pH.dict = PyDict_New();
    return rc;
}

static PyObject *pygenmodule_set_analysis(vmedaq_Module *self, PyObject *args, PyObject *kwargs)
{
    WHOAMI;
    PyObject *in;
    if (!PyArg_ParseTuple(args, "O:GenModule.set_analysis", &in))
        return NULL;

    if (in && in!=Py_None)
    {
        Analysis *ana = ((vmedaq_analysis *)in)->ana;
        self->md->set_analysis(ana);
        // set pedestals and noise with proper vector size
        std::fill_n(std::back_inserter( self->md->get_par_value< std::vector<double> >("pedestal") ), ana->Ntot(), 0.);
        std::fill_n(std::back_inserter( self->md->get_par_value< std::vector<double> >("noise") ), ana->Ntot(), 0.);
        Py_INCREF(in);
    }
    Py_INCREF(Py_None);
    return Py_None;
}

static PyObject *
pygenmodule_endofevent(vmedaq_Module *self, PyObject *args, PyObject *kwargs)
{
    WHOAMI;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( self->md );
    if (!pm->external_call())
    {
        pm->set_python_call();
        self->md->EndOfEvent();
        pm->clear_python_call();
    }
    Py_INCREF(Py_None);
    return Py_None;
}


static PyObject *
pygenmodule_increase_evt(vmedaq_Module *self, PyObject *args, PyObject *kwargs)
{
    WHOAMI;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( self->md );
    if (!pm->external_call())
    {
        pm->set_python_call();
        self->md->increase_evt_counter();
        pm->clear_python_call();
    }
    Py_INCREF(Py_None);
    return Py_None;
}


static PyObject *
pygenmodule_reset_evt(vmedaq_Module *self, PyObject *args, PyObject *kwargs)
{
    WHOAMI;
    PyDAQpp *pm= dynamic_cast<PyDAQpp *>( self->md );
    if (!pm->external_call())
    {
        pm->set_python_call();
        self->md->reset_evt_counter();
        pm->clear_python_call();
    }
    Py_INCREF(Py_None);
    return Py_None;
}

static PyObject *
pygenmodule_get_nchip(vmedaq_Module *self, void *closure)
{
    WHOAMI;
    GenModule *md = dynamic_cast<GenModule *>(self->md);
    return PyInt_FromLong(md->get_analysis()->Nchip());
}

static PyObject *
pygenmodule_get_nchan(vmedaq_Module *self, void *closure)
{
    WHOAMI;
    GenModule *md = dynamic_cast<GenModule *>(self->md);
    return PyInt_FromLong(md->get_analysis()->Nchan());
}
static PyObject *
pygenmodule_get_address(vmedaq_Module *self, void *closure)
{
    WHOAMI;
    GenModule *md = dynamic_cast<GenModule *>(self->md);
    return PyInt_FromLong(md->get_address());
}
static PyObject *
pygenmodule_get_evt_counter(vmedaq_Module *self, void *closure)
{
    WHOAMI;
    GenModule *md = dynamic_cast<GenModule *>(self->md);
    return PyInt_FromLong(md->get_evt_counter());
}


static PyObject *
pygenmodule_get_analysis(vmedaq_Module *self, void *closure)
{
    WHOAMI;
    PyObject *ret=Py_None;
    GenModule *md = dynamic_cast<GenModule *>(self->md);
    Analysis *ana = md->get_analysis();
    pyanalysis *pyana = dynamic_cast<pyanalysis *>(ana);
    if (pyana)
    {
        ret = pyana->self();
    }
    Py_INCREF(ret);
    return ret;
}

static PyGetSetDef pygenmodule_getseters[] =
{
 {"nchip",(getter)pygenmodule_get_nchip, (setter)0, "Number of chips",NULL},
 {"nchan",(getter)pygenmodule_get_nchan, (setter)0, "Number of channels",NULL},
 {"address",(getter)pygenmodule_get_address, (setter)0, "Address",NULL},
 {"evt_counter",(getter)pygenmodule_get_evt_counter, (setter)0, "Event counter",NULL},
 {"analysis",(getter)pygenmodule_get_analysis,(setter)0,"The module analysis",NULL},
 {NULL}  /* Sentinel */
};

static PyMethodDef module_methods[] =
{
 {"EndOfEvent",(PyCFunction)pygenmodule_endofevent,METH_NOARGS,""},
 {"set_analysis", (PyCFunction)pygenmodule_set_analysis,METH_VARARGS,""},
 {"increase_evt_counter", (PyCFunction)pygenmodule_increase_evt,METH_NOARGS,""},
 {"reset_evt_counter", (PyCFunction)pygenmodule_reset_evt,METH_NOARGS,""},
 {NULL}  /* Sentinel */
};
#define DEFERRED_ADDRESS(ADDR) 0
static PyTypeObject GenModule_Type = {
   PyObject_HEAD_INIT(NULL)
   0,                         /*ob_size*/
   "pyvmedaq.GenModule",             /*tp_name*/
   sizeof(vmedaq_Module),       /*tp_basicsize*/
   0,                         /*tp_itemsize*/
   (destructor)module_dealloc,/*tp_dealloc*/
   0,                         /*tp_print*/
   0,                         /*tp_getattr*/
   0,                         /*tp_setattr*/
   0,                         /*tp_compare*/
   (reprfunc)0,                         /*tp_repr*/
   0,                         /*tp_as_number*/
   0,                         /*tp_as_sequence*/
   0,                         /*tp_as_mapping*/
   0,                         /*tp_hash */
   (ternaryfunc)0,            /*tp_call*/
   (reprfunc)0,               /*tp_str*/
   0,                         /*tp_getattro*/
   0,                         /*tp_setattro*/
   0,                         /*tp_as_buffer*/
   Py_TPFLAGS_DEFAULT| Py_TPFLAGS_BASETYPE,        /*tp_flags*/
   0,             /* tp_doc */
   0,                      /* tp_traverse */
   0,                      /* tp_clear */
   0,                      /* tp_richcompare */
   0,                      /* tp_weaklistoffset */
   0,                      /* tp_iter */
   0,                      /* tp_iternext */
   module_methods,             /* tp_methods */
   0,             /* tp_members */
   pygenmodule_getseters,                         /* tp_getset */
   DEFERRED_ADDRESS(&Module_Type), /* tp_base */
   0,                         /* tp_dict */
   0,                         /* tp_descr_get */
   0,                         /* tp_descr_set */
   0,                         /* tp_dictoffset */
   (initproc)module_init,      /* tp_init */
   0,                         /* tp_alloc */
   module_new,                 /* tp_new */
};

void register_genmodule(PyObject *dict)
{
    WHOAMI;
    GenModule_Type.tp_base = &Module_Type;
    daqpp_register_class(dict, "GenModule", &GenModule_Type, 0);
}

int VMEdaq_Module_Check(PyObject *O)
{
    WHOAMI;
    return PyType_IsSubtype(O->ob_type, &GenModule_Type);
}

