#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sstream>
#if defined(HAS_PYTHON) && defined(HAS_PYDAQPP)
#include <Python.h>
#include <pymodule_parser.h>
#ifdef HAS_PYGTK
#include <pygobject.h>
#include <pygtk/pygtk.h>
#endif
#include <gui/MonitorWindow.h>



#ifdef HAS_PYGTK
typedef struct _callbackData callbackData;
struct _callbackData
{
    PyObject *func;
    PyObject *args;
    PyObject *kwargs;
};

static void vmedaq_callback(GtkWidget *wdt, callbackData *data)
{
    PyObject *ret;
    std::cout << "Callback...." << std::endl;
    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();
    ret = PyObject_Call(data->func, data->args, data->kwargs);
    if (PyErr_Occurred())
    {
        daqpp_print_error();
        PyErr_Print();
    }
    if (!ret)
    {
            std::cout << "Could not call callback funtion" << std::endl;
    }

    PyGILState_Release(gilstate);
}

PyObject *
vmedaq_connect(PyObject *self, PyObject *args, PyObject *kwargs)
{
    int item, iarg, narg, nkarg, nitem;
    PyObject *widget;
    PyObject *pysignal;
    char     *signal;
    PyObject *callback;
    PyObject *rest;
    GObject  *obj;

    narg = PyTuple_Size(args);
    if ( narg < 3 )
    {
        std::ostringstream ostr;
        ostr << "vmedaq_connect needs at least 3 arguments ("
             << narg << " given)";
        PyErr_SetString(PyExc_TypeError, ostr.str().c_str());
        return NULL;
    }
    widget = PyTuple_GetItem(args, 0);
    pysignal = PyTuple_GetItem(args, 1);
    callback = PyTuple_GetItem(args, 2);

    signal = PyString_AsString(pysignal);

    obj = G_OBJECT(pygobject_get(widget));
    if (!obj)
        return NULL;

    callbackData *data = g_new0(callbackData, 1);
    data->func = callback;
    Py_INCREF(callback);

    nitem = 1;
    if (narg>3)
    {
        narg -= 3;
        nitem += narg;
    }
    data->args = PyTuple_New( nitem );

    item = 0;
    Py_INCREF(widget);
    PyTuple_SetItem(data->args, item++, widget);

    for (iarg=0;iarg<narg;iarg++)
    {
        PyObject *O = PyTuple_GetItem(args, 3+iarg);
        Py_INCREF(O);
        PyTuple_SetItem(data->args, item++, O);
    }

    data->kwargs  = 0;
    if ( kwargs && PyDict_Check(kwargs) )
    {
        nkarg = PyDict_Size(kwargs);
        if (nkarg>0)
        {
            nitem += nkarg;
        }
        else
        {
            nkarg = 0;
        }

        if (nkarg>0)
        {
            data->kwargs = PyDict_Copy(kwargs);
        }
    }

    g_signal_connect(obj, signal, G_CALLBACK(vmedaq_callback), data);

    Py_INCREF(Py_None);
    return Py_None;
}
#endif



#define PyxmlNode_Get(v) (((v) == Py_None) ? NULL : \
    (((PyxmlNode_Object *)(v))->obj))

typedef struct {
    PyObject_HEAD
    xmlNodePtr obj;
} PyxmlNode_Object;


PyObject *
libxml_xmlNodePtrWrap(xmlNodePtr node)
{
    PyObject *ret;

    if (node == NULL) {
        Py_INCREF(Py_None);
        return (Py_None);
    }
    ret =
        PyCObject_FromVoidPtrAndDesc((void *) node, (char *) "xmlNodePtr",
                                     NULL);
    return (ret);
}



void pymodule_parser::parse_node(xmlNodePtr node)
throw (ConfigException)
{
    PyObject *_node = libxml_xmlNodePtrWrap(node);
    call_void_method("parse_node", "O", _node);
    Py_DECREF(_node);
}

void pymodule_parser::save_config()
throw(ConfigException)
{
    call_void_method("save_config", NULL);
}

static void py_widget_destroy(GObject *obj, gpointer ptr)
{
    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();
    if (ptr)
        Py_DECREF((PyObject *)ptr);

    PyGILState_Release(gilstate);
}

GtkWidget *pymodule_parser::config(const std::string &id)
{
#if defined(HAS_PYGTK)
    GtkWidget *widget;

    PyObject *ret;

    if (python_call() || !has_object() )
        return 0;

    set_external_call();
    ret = PyObject_CallMethod(self(),"config","s",id.c_str());
    if (!ret)
    {
        // Something failed
        if (PyErr_Occurred())
        {
            daqpp_print_error();
            PyErr_Print();
        }
        return 0;
    }
    clear_external_call();
    if (ret==Py_None)
        return 0;

    Py_INCREF(ret);
    widget = GTK_WIDGET(pygobject_get(ret));
    g_signal_connect(G_OBJECT(widget),"destroy",G_CALLBACK(py_widget_destroy),ret);
    return widget;
#else
    return 0;
#endif
}

MonitorWindow *pymodule_parser::monitor(const std::string &id, const std::string &glade_file)
{
#if defined(HAS_PYGTK)
    GtkWidget *widget;

    PyObject *ret;

    if (python_call() || !has_object() )
        return 0;

    set_external_call();
    PyGILState_STATE gilstate;
    gilstate = PyGILState_Ensure();
    ret = PyObject_CallMethod(self(),"monitor","ss",id.c_str(), glade_file.c_str());
    PyGILState_Release(gilstate);
    gilstate = PyGILState_Ensure();
    if (PyErr_Occurred())
    {
        daqpp_print_error();
        PyErr_Print();
    }
    PyGILState_Release(gilstate);

    if (!ret)
    {
        // Something failed
        return 0;
    }
    clear_external_call();
    if (ret==Py_None)
        return 0;

    PyGC_Head *gc_head = _Py_AS_GC(ret);
    widget = GTK_WIDGET(pygobject_get(ret));
    g_object_set_data(G_OBJECT(widget),"__pygtk_vmedaq__", (gpointer)ret);
    MonitorWindow *mw = new MonitorWindow(widget);

    return mw;
#else
    return 0;
#endif
}


static PyObject *
vmedaq_module_parser_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    vmedaq_module_parser *self;
    self =  PyObject_New(vmedaq_module_parser,type);
    if (self!=NULL) {
        self->parser  = 0;
        self->dict = PyDict_New();
    }
    return (PyObject *)self;
}

static void
vmedaq_module_parser_dealloc(vmedaq_module_parser* self)
{
    delete self->parser;
    Py_DECREF(self->dict);
    self->parser = 0;
    self->ob_type->tp_free((PyObject*)self);
}

static int
vmedaq_module_parser_init(vmedaq_module_parser *self, PyObject *args, PyObject *kwargs) {
    self->parser = new pymodule_parser();
    self->parser->set_object((PyObject *)self);
    return 0;
}


static PyObject *vmedaq_module_parser_parse_node(vmedaq_module_parser *self, PyObject *args, PyObject *kwargs)
{
    PyObject *O;
    pymodule_parser *pm = dynamic_cast<pymodule_parser *>(self->parser);

    if (!PyArg_ParseTuple(args, "O:ModuleParser.parse_node", &O))
        return NULL;

    if (!pm->external_call())
    {
        xmlNodePtr node = PyxmlNode_Get(O);
        pm->parse_node(node);
    }

    Py_DECREF(O);

    Py_INCREF(Py_None);
    return Py_None;
}

static PyObject *vmedaq_module_parser_save_config(vmedaq_module_parser *self)
{
    pymodule_parser *pm = dynamic_cast<pymodule_parser *>(self->parser);

    if (!pm->external_call())
    {
        pm->save_config();
    }
    Py_INCREF(Py_None);
    return Py_None;
}

static PyObject *vmedaq_module_parser_config(vmedaq_module_parser *self, PyObject *args, PyObject *kwargs)
{
    PyObject *pyid;
    GtkWidget *w=0;
    pymodule_parser *pm = dynamic_cast<pymodule_parser *>(self->parser);

    if (!PyArg_ParseTuple(args, "O:ModuleParser.parse_node", &pyid))
            return NULL;

    const std::string id = PyString_AsString(pyid);
    if (!pm->external_call())
    {
        w = pm->config(id);
    }
    if (!w)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }
    else {
        return pygobject_new((GObject*) w);
    }
}

static PyObject *vmedaq_module_parser_monitor(vmedaq_module_parser *self, PyObject *args, PyObject *kwargs)
{
    char *id=0, *file=0;
    PyObject *ret=0;
    pymodule_parser *pm = dynamic_cast<pymodule_parser *>(self->parser);
    if (!PyArg_ParseTuple(args, "ss:ModuleParser.monitor", &id, &file))
        return NULL;

    if (!pm->external_call())
    {
        MonitorWindow *mw = pm->monitor(id, file);
        if (mw==0)
        {
            ret=Py_None;
            Py_INCREF(ret);
        }
        else {
            ret = pygobject_new((GObject*) mw->widget());
        }
    }
    if (!ret)
    {
        Py_INCREF(Py_None);
        return Py_None;
    }
    return ret;
}

static PyObject *
pyparser_get_id(vmedaq_module_parser *self, void *closure)
{
    pymodule_parser *pm = dynamic_cast<pymodule_parser *>(self->parser);
    return PyString_FromString(pm->get_id().c_str());
}
static int
pyparser_set_id(vmedaq_module_parser *self, PyObject *value, void *closure)
{
    pymodule_parser *pm = dynamic_cast<pymodule_parser *>(self->parser);
    if (value == NULL) {
        PyErr_SetString(PyExc_TypeError, "ModuleParser - Cannot delete the attribute");
        return -1;
    }
    if (! PyString_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "The ID value value must be a string");
        return -1;
    }
    std::string id = PyString_AsString(value);
    pm->set_id(id);
    return 0;

}
static PyGetSetDef pymodule_getseters[] =
{
 {"id",(getter)pyparser_get_id, (setter)pyparser_set_id, "Module ID",NULL},
 {NULL}  /* Sentinel */
};

static PyMethodDef vmedaq_module_parser_methods[] =
{
    {"parse_node",(PyCFunction)vmedaq_module_parser_parse_node,METH_VARARGS,""},
    {"save_config",(PyCFunction)vmedaq_module_parser_save_config,METH_NOARGS,""},
    {"config",(PyCFunction)vmedaq_module_parser_config,METH_VARARGS,""},
    {"monitor",(PyCFunction)vmedaq_module_parser_monitor,METH_VARARGS,""},
    {NULL}  /* Sentinel */
};


static PyTypeObject vmedaq_module_parser_Type =
{
 PyObject_HEAD_INIT(NULL)
 0,                         /*ob_size*/
 "vmedaq.ModuleParser",             /*tp_name*/
 sizeof(vmedaq_module_parser),       /*tp_basicsize*/
 0,                         /*tp_itemsize*/
 (destructor)vmedaq_module_parser_dealloc,                         /*tp_dealloc*/
 0,                         /*tp_print*/
 0,                         /*tp_getattr*/
 0,                         /*tp_setattr*/
 0,                         /*tp_compare*/
 (reprfunc)0,                         /*tp_repr*/
 0,                         /*tp_as_number*/
 0,                         /*tp_as_sequence*/
 0,                         /*tp_as_mapping*/
 0,                         /*tp_hash */
 (ternaryfunc)0,                         /*tp_call*/
 (reprfunc)0,                 /*tp_str*/
 PyObject_GenericGetAttr,                         /*tp_getattro*/
 PyObject_GenericSetAttr,                         /*tp_setattro*/
 0,                         /*tp_as_buffer*/
 Py_TPFLAGS_DEFAULT| Py_TPFLAGS_BASETYPE,        /*tp_flags*/
 "Module Parser",           /* tp_doc */
 0,                      /* tp_traverse */
 0,                     /* tp_clear */
 0,                     /* tp_richcompare */
 0,                     /* tp_weaklistoffset */
 0,                     /* tp_iter */
 0,                     /* tp_iternext */
 vmedaq_module_parser_methods,             /* tp_methods */
 0,             /* tp_members */
 0,                         /* tp_getset */
 0,                         /* tp_base */
 0,                         /* tp_dict */
 0,                         /* tp_descr_get */
 0,                         /* tp_descr_set */
 offsetof(vmedaq_module_parser, dict),                         /* tp_dictoffset */
 (initproc)vmedaq_module_parser_init,      /* tp_init */
 0,                         /* tp_alloc */
 vmedaq_module_parser_new,                 /* tp_new */
};

void register_module_parser(PyObject *dict) {
    daqpp_register_class(dict,"ModuleParser",
                         &vmedaq_module_parser_Type,
                         0);
}


#endif
