#include <iostream>
#include <sstream>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#if defined(HAS_PYTHON)
#include <Python.h>
#include <DAQ++/python/pydaqpp.h>
#include <data/DataFormat.h>
#include <config/Config.h>

#ifndef PyMODINIT_FUNC  /* declarations for DLL import/export */
#define PyMODINIT_FUNC void
#endif




void register_module_parser(PyObject *dict);
void register_genmodule(PyObject *dict);
void register_analysis(PyObject *dict);

extern PyObject *vmedaq_connect(PyObject *self, PyObject *args, PyObject *kwargs);

static PyObject *vmedaq_find_glade_file(PyObject *self, PyObject *args, PyObject *kwargs)
{
    PyObject *fnam;
    if (!PyArg_ParseTuple(args, "S", &fnam))
            return NULL;

    std::string path = PathConfig::find_glade_file(PyString_AsString(fnam));
    if (path.empty())
    {
        Py_INCREF(Py_None);
        return Py_None;
    }
    else
    {
        return PyString_FromString(path.c_str());
    }
}

static PyObject *vmedaq_get_ped_method(PyObject *self, PyObject *args, PyObject *kwargs)
{
    ConfigParser *cfg = ConfigParser::theConfig();
    GeneralConfig *gnrl = dynamic_cast<GeneralConfig *>(cfg->get_handler("general"));
    return PyInt_FromLong(gnrl->get_ped_method());
}



static PyMethodDef vmedaq_functions[] =
{
    {"find_glade_file",(PyCFunction)vmedaq_find_glade_file, METH_VARARGS,"finds a glade file"},
    {"get_ped_method",(PyCFunction)vmedaq_get_ped_method, METH_NOARGS,""},
#ifdef HAS_PYGTK
    {"connect",(PyCFunction)vmedaq_connect, METH_VARARGS,"Connects to a GTK widget signal"},
#endif
    { NULL, NULL, 0 }
};

#if PY_VERSION_HEX < 0x2030000
extern "C" void
#else
PyMODINIT_FUNC
#endif
initpyvmedaq(void)
{
    PyObject *m,*d;

    m = Py_InitModule3("pyvmedaq",vmedaq_functions,0);
    d = PyModule_GetDict(m);

    register_module_parser(d);
    register_genmodule(d);
    register_analysis(d);

    daqpp_add_int_constant(d,"DataHeader",DataHeader);
    daqpp_add_int_constant(d,"StartOfRunRecord",StartOfRunRecord);
    daqpp_add_int_constant(d,"DataRecord",DataRecord);
    daqpp_add_int_constant(d,"EndOfRunRecord",EndOfRunRecord);
    daqpp_add_int_constant(d,"PedestalNormal",GeneralConfig::normal);
    daqpp_add_int_constant(d,"PedestalFast",GeneralConfig::fast);
}
#endif
