#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <zlib.h>

#include "../data/DataFormat.h"



int main(int argc, char **argv)
{
    const char *ifile = argv[1];
    if ( !ifile )
    {
        std::cout << "I need an input file" << std::endl;
        return 1;
    }

    bool verbose = (argv[2] == 0);

    gzFile in = gzopen(ifile, "rb");
    if ( !in )
    {
        std::cout << "COuld not open " << ifile << std::endl;
        return 1;
    }

    unsigned long x;
    int i = 0, j;
    int iok;
    int buffer_size = -1;
    unsigned long *data = 0;
    unsigned long prtFlags = 0;
    enum print_flags {
        printPed = 0x01,
        printData = 0x02,
        printSP  = 0x04,
        printMap = 0x08,
        printEoR = 0x10,
        printScan = 0x20,
    };
    int  id, romode;
    int nevts = 0;
    int pevt = 0;
    bool is_double = false;

    while ( 1 )
    {
        /*
        Read the file until we get a new data block
        */
        do
        {
            iok = gzread(in, &x, sizeof(unsigned long));
            if ( iok <= 0 )
            {
                std::cout << "End of file after " << nevts << " events " << std::endl;
                exit(0);
            }
        }
        while ( !IsDataBlock(x) );

        // Get the data block size and reallocate the buffer
        int block_size = x & 0xfff;
        if ( block_size > buffer_size )
        {
            buffer_size = block_size;
            data = (unsigned long *)realloc(data,
                                            (buffer_size + 1) * sizeof(unsigned long));
        }
        // Read the rest of the data block
        data[0] = x;
        iok = gzread(in, data + 1, (block_size - 1) * sizeof(unsigned long));
        if ( iok < (block_size - 1) / sizeof(unsigned long) )
        {
            std::cout << "Problems reading the file... Truncated record... "
            << iok
            << " [block_size " << block_size << "]"
            << std::endl;
            exit(1);
        }
        nevts++;

        std::cout << "*** Record type: " << std::hex << (data[0]&0xf000) << std::dec << std::endl;

        if ( IsDataRecord(data[0]) )
        {
            if ( data[1] - pevt > 1 )
                std::cout << "Data loss: gap " << data[1] - pevt << " at " << pevt << std::endl;

            pevt = data[1];
        }

        if ( verbose )
        {
            for ( i = 0;i < block_size;i++)
            {
                if ( !i )
                {
                    std::cout << "+-<- ";
                }
                else if ( i == block_size - 1 )
                {
                    std::cout << "+--> ";
                }
                else
                    std::cout << "|    ";

                std::cout << std::setw(6) << i << ": " << std::setw(8) << std::hex << data[i] << std::dec;
                if ( i > 2 && i < block_size - 1 && (prtFlags & printPed) )
                {
                    double noise = (data[i] >> 16) / 50.;
                    double ped   = (data[i] & 0xffff) / 50.;
                    std::cout << "-> Ped " << ped << " noise " << noise;
                }
                if ( i > 2 && i < block_size - 1 && (prtFlags &printData) )
                {
                    if ( romode == 2 )
                    {
                        std::cout << " down " << (data[i]&0x3ff)
                        << " up " << ((data[i]&0xffc00) >> 10);
                    }
                    else
                    {
                        std::cout << " chan. " << ((data[i]&0x7ff00000) >> 20) << " "
                        << " sig down " << (data[i]&0x3ff)
                        << " up " << ((data[i]&0xffc00) >> 10)
                        ;
                    }
                }
                if (i >= 1 && (prtFlags & printSP) )
                {
                    if (i == 1)
                        std::cout << " N. of variables " << (data[i] & 0xff)
                        << " N. evts/point " << ((data[i] & 0xffffff00) >> 8);
                    else
                    {
                        std::cout << " Var type " << ((data[i] & 0xf0000000) >> 28)
                        << " value " << (data[i] & 0xfffffff)
                        ;
                    }
                }
                if (i > 2 && (prtFlags & printMap))
                {
                    std::cout << "     "
                    << std::setw(3) << (data[i] >> 24 & 0xff) << "."
                    << std::setw(3) << (data[i] >> 16 & 0xff)
                    << " - "
                    << std::setw(3) << (data[i] >> 8 & 0xff) << "."
                    << std::setw(3) << (data[i] & 0xff);
                }
                if (i > 2 && (prtFlags & printScan))
                {
                    int icase = (i - 3) % 3;
                    union {
                        unsigned long lval;
                        float fval;
                    } val;
                    switch (icase)
                    {
                    case 0:
                        is_double = data[i] & 0xf0000000;
                        std::cout << "   type " << ((data[i]&0xfff0000) >> 16)
                        << " npoints " << (data[i]&0xffff);
                        break;
                    case 1:
                        std::cout << "   first ";
                        break;
                    case 2:
                        std::cout << "   last  ";
                        break;
                    default:
                        std::cout << "   ????  ";
                        break;
                    }
                    if (icase)
                    {
                        val.lval = data[i];
                        if (is_double)
                            std::cout << val.fval;
                        else
                            std::cout << val.lval;
                    }
                }
                if (!i )
                {
                    std::cout << "  size " << block_size;
                    if ( IsStartOfRunRecord(data[0]) )
                    {
                        prtFlags = 0;
                        /*
                          The Start of Run Record contains the list of modules
                        */
                        std::cout << " Start of Run ";
                        std::cout << "Number of Modules " << data[3] << ": ";
                        for (j = 0;j < data[3];j++)
                        {
                            std::cout
                            << "| id " << (data[4+j]&0xffff)
                            <<  " tp " << ((data[4+j]&0xf0000) >> 16)
                            <<  " nc " << ((data[4+j]&0xff00000) >> 20)
                            ;
                        }
                    }
                    else if ( IsPedestalRecord(data[0]) )
                    {
                        prtFlags = printPed;
                        std::cout << " Pedestals. Module " << data[1]
                        << " Nchips " << (data[2] >> 16)
                        << " Nchan " << (data[2] & 0xffff);
                    }
                    else if ( IsMapRecord(data[0]) )
                    {
                        prtFlags = printMap;
                        std::cout << " Map. Module " << data[1]
                        << " nX " << (data[2] >> 16)
                        << " Ny " << (data[2] & 0xffff);
                    }
                    else if ( IsDataRecord(data[0]) )
                    {
                        prtFlags = printData;
                        romode = (data[2] & 0xf000) >> 12;
                        id = (data[2] & 0xfff00000) >> 20;
                        std::cout << " Data record ";
                        // Here is where you read the data.
                        // See Analysis.cc for an example
                        std::cout << "Module " << id << " "
                        << "ROmode " << romode << " ";
                        if ( romode != 2 )
                        {
                            std::cout << "channel " << ((data[3]&0x7ff00000) >> 20) << " "
                            << "v " << (data[6]&0x3ff);
                        }
                    }
                    else if ( IsEndOfRunRecord(data[0]) )
                    {
                        prtFlags = printEoR;
                        std::cout << " End of Run";
                    }
                    else if ( IsNewPointRecord(data[0]) )
                    {
                        prtFlags = printSP;

                        romode = (data[2] & 0xf000) >> 12;
                        id = (data[2] & 0xfff00000) >> 20;
                        std::cout << " New point in the scan";
                        std::cout << "Module " << id << " "
                        << "ROmode " << romode << " ";
                    }
                    else if ( IsScanRecord(data[0]) )
                    {
                        prtFlags = printScan;
                        std::cout << " This is a Scan Record: nvar = "
                        << data[1] << " nevt/point " << data[2];
                    }
                    else
                        prtFlags = 0;
                    /*
                    See DataFormat.h for a list of possible data records
                    */
                }
                std::cout << std::endl;
            }
        }
    }
    return 0;
}
